#define _CFFI_

/* We try to define Py_LIMITED_API before including Python.h.

   Mess: we can only define it if Py_DEBUG, Py_TRACE_REFS and
   Py_REF_DEBUG are not defined.  This is a best-effort approximation:
   we can learn about Py_DEBUG from pyconfig.h, but it is unclear if
   the same works for the other two macros.  Py_DEBUG implies them,
   but not the other way around.

   The implementation is messy (issue #350): on Windows, with _MSC_VER,
   we have to define Py_LIMITED_API even before including pyconfig.h.
   In that case, we guess what pyconfig.h will do to the macros above,
   and check our guess after the #include.

   Note that on Windows, with CPython 3.x, you need >= 3.5 and virtualenv
   version >= 16.0.0.  With older versions of either, you don't get a
   copy of PYTHON3.DLL in the virtualenv.  We can't check the version of
   CPython *before* we even include pyconfig.h.  ffi.set_source() puts
   a ``#define _CFFI_NO_LIMITED_API'' at the start of this file if it is
   running on Windows < 3.5, as an attempt at fixing it, but that's
   arguably wrong because it may not be the target version of Python.
   Still better than nothing I guess.  As another workaround, you can
   remove the definition of Py_LIMITED_API here.

   See also 'py_limited_api' in cffi/setuptools_ext.py.
*/
#if !defined(_CFFI_USE_EMBEDDING) && !defined(Py_LIMITED_API)
#  ifdef _MSC_VER
#    if !defined(_DEBUG) && !defined(Py_DEBUG) && !defined(Py_TRACE_REFS) && !defined(Py_REF_DEBUG) && !defined(_CFFI_NO_LIMITED_API)
#      define Py_LIMITED_API
#    endif
#    include <pyconfig.h>
     /* sanity-check: Py_LIMITED_API will cause crashes if any of these
        are also defined.  Normally, the Python file PC/pyconfig.h does not
        cause any of these to be defined, with the exception that _DEBUG
        causes Py_DEBUG.  Double-check that. */
#    ifdef Py_LIMITED_API
#      if defined(Py_DEBUG)
#        error "pyconfig.h unexpectedly defines Py_DEBUG, but Py_LIMITED_API is set"
#      endif
#      if defined(Py_TRACE_REFS)
#        error "pyconfig.h unexpectedly defines Py_TRACE_REFS, but Py_LIMITED_API is set"
#      endif
#      if defined(Py_REF_DEBUG)
#        error "pyconfig.h unexpectedly defines Py_REF_DEBUG, but Py_LIMITED_API is set"
#      endif
#    endif
#  else
#    include <pyconfig.h>
#    if !defined(Py_DEBUG) && !defined(Py_TRACE_REFS) && !defined(Py_REF_DEBUG) && !defined(_CFFI_NO_LIMITED_API)
#      define Py_LIMITED_API
#    endif
#  endif
#endif

#include <Python.h>
#ifdef __cplusplus
extern "C" {
#endif
#include <stddef.h>

/* This part is from file 'cffi/parse_c_type.h'.  It is copied at the
   beginning of C sources generated by CFFI's ffi.set_source(). */

typedef void *_cffi_opcode_t;

#define _CFFI_OP(opcode, arg)   (_cffi_opcode_t)(opcode | (((uintptr_t)(arg)) << 8))
#define _CFFI_GETOP(cffi_opcode)    ((unsigned char)(uintptr_t)cffi_opcode)
#define _CFFI_GETARG(cffi_opcode)   (((intptr_t)cffi_opcode) >> 8)

#define _CFFI_OP_PRIMITIVE       1
#define _CFFI_OP_POINTER         3
#define _CFFI_OP_ARRAY           5
#define _CFFI_OP_OPEN_ARRAY      7
#define _CFFI_OP_STRUCT_UNION    9
#define _CFFI_OP_ENUM           11
#define _CFFI_OP_FUNCTION       13
#define _CFFI_OP_FUNCTION_END   15
#define _CFFI_OP_NOOP           17
#define _CFFI_OP_BITFIELD       19
#define _CFFI_OP_TYPENAME       21
#define _CFFI_OP_CPYTHON_BLTN_V 23   // varargs
#define _CFFI_OP_CPYTHON_BLTN_N 25   // noargs
#define _CFFI_OP_CPYTHON_BLTN_O 27   // O  (i.e. a single arg)
#define _CFFI_OP_CONSTANT       29
#define _CFFI_OP_CONSTANT_INT   31
#define _CFFI_OP_GLOBAL_VAR     33
#define _CFFI_OP_DLOPEN_FUNC    35
#define _CFFI_OP_DLOPEN_CONST   37
#define _CFFI_OP_GLOBAL_VAR_F   39
#define _CFFI_OP_EXTERN_PYTHON  41

#define _CFFI_PRIM_VOID          0
#define _CFFI_PRIM_BOOL          1
#define _CFFI_PRIM_CHAR          2
#define _CFFI_PRIM_SCHAR         3
#define _CFFI_PRIM_UCHAR         4
#define _CFFI_PRIM_SHORT         5
#define _CFFI_PRIM_USHORT        6
#define _CFFI_PRIM_INT           7
#define _CFFI_PRIM_UINT          8
#define _CFFI_PRIM_LONG          9
#define _CFFI_PRIM_ULONG        10
#define _CFFI_PRIM_LONGLONG     11
#define _CFFI_PRIM_ULONGLONG    12
#define _CFFI_PRIM_FLOAT        13
#define _CFFI_PRIM_DOUBLE       14
#define _CFFI_PRIM_LONGDOUBLE   15

#define _CFFI_PRIM_WCHAR        16
#define _CFFI_PRIM_INT8         17
#define _CFFI_PRIM_UINT8        18
#define _CFFI_PRIM_INT16        19
#define _CFFI_PRIM_UINT16       20
#define _CFFI_PRIM_INT32        21
#define _CFFI_PRIM_UINT32       22
#define _CFFI_PRIM_INT64        23
#define _CFFI_PRIM_UINT64       24
#define _CFFI_PRIM_INTPTR       25
#define _CFFI_PRIM_UINTPTR      26
#define _CFFI_PRIM_PTRDIFF      27
#define _CFFI_PRIM_SIZE         28
#define _CFFI_PRIM_SSIZE        29
#define _CFFI_PRIM_INT_LEAST8   30
#define _CFFI_PRIM_UINT_LEAST8  31
#define _CFFI_PRIM_INT_LEAST16  32
#define _CFFI_PRIM_UINT_LEAST16 33
#define _CFFI_PRIM_INT_LEAST32  34
#define _CFFI_PRIM_UINT_LEAST32 35
#define _CFFI_PRIM_INT_LEAST64  36
#define _CFFI_PRIM_UINT_LEAST64 37
#define _CFFI_PRIM_INT_FAST8    38
#define _CFFI_PRIM_UINT_FAST8   39
#define _CFFI_PRIM_INT_FAST16   40
#define _CFFI_PRIM_UINT_FAST16  41
#define _CFFI_PRIM_INT_FAST32   42
#define _CFFI_PRIM_UINT_FAST32  43
#define _CFFI_PRIM_INT_FAST64   44
#define _CFFI_PRIM_UINT_FAST64  45
#define _CFFI_PRIM_INTMAX       46
#define _CFFI_PRIM_UINTMAX      47
#define _CFFI_PRIM_FLOATCOMPLEX 48
#define _CFFI_PRIM_DOUBLECOMPLEX 49
#define _CFFI_PRIM_CHAR16       50
#define _CFFI_PRIM_CHAR32       51

#define _CFFI__NUM_PRIM         52
#define _CFFI__UNKNOWN_PRIM           (-1)
#define _CFFI__UNKNOWN_FLOAT_PRIM     (-2)
#define _CFFI__UNKNOWN_LONG_DOUBLE    (-3)

#define _CFFI__IO_FILE_STRUCT         (-1)


struct _cffi_global_s {
    const char *name;
    void *address;
    _cffi_opcode_t type_op;
    void *size_or_direct_fn;  // OP_GLOBAL_VAR: size, or 0 if unknown
                              // OP_CPYTHON_BLTN_*: addr of direct function
};

struct _cffi_getconst_s {
    unsigned long long value;
    const struct _cffi_type_context_s *ctx;
    int gindex;
};

struct _cffi_struct_union_s {
    const char *name;
    int type_index;          // -> _cffi_types, on a OP_STRUCT_UNION
    int flags;               // _CFFI_F_* flags below
    size_t size;
    int alignment;
    int first_field_index;   // -> _cffi_fields array
    int num_fields;
};
#define _CFFI_F_UNION         0x01   // is a union, not a struct
#define _CFFI_F_CHECK_FIELDS  0x02   // complain if fields are not in the
                                     // "standard layout" or if some are missing
#define _CFFI_F_PACKED        0x04   // for CHECK_FIELDS, assume a packed struct
#define _CFFI_F_EXTERNAL      0x08   // in some other ffi.include()
#define _CFFI_F_OPAQUE        0x10   // opaque

struct _cffi_field_s {
    const char *name;
    size_t field_offset;
    size_t field_size;
    _cffi_opcode_t field_type_op;
};

struct _cffi_enum_s {
    const char *name;
    int type_index;          // -> _cffi_types, on a OP_ENUM
    int type_prim;           // _CFFI_PRIM_xxx
    const char *enumerators; // comma-delimited string
};

struct _cffi_typename_s {
    const char *name;
    int type_index;   /* if opaque, points to a possibly artificial
                         OP_STRUCT which is itself opaque */
};

struct _cffi_type_context_s {
    _cffi_opcode_t *types;
    const struct _cffi_global_s *globals;
    const struct _cffi_field_s *fields;
    const struct _cffi_struct_union_s *struct_unions;
    const struct _cffi_enum_s *enums;
    const struct _cffi_typename_s *typenames;
    int num_globals;
    int num_struct_unions;
    int num_enums;
    int num_typenames;
    const char *const *includes;
    int num_types;
    int flags;      /* future extension */
};

struct _cffi_parse_info_s {
    const struct _cffi_type_context_s *ctx;
    _cffi_opcode_t *output;
    unsigned int output_size;
    size_t error_location;
    const char *error_message;
};

struct _cffi_externpy_s {
    const char *name;
    size_t size_of_result;
    void *reserved1, *reserved2;
};

#ifdef _CFFI_INTERNAL
static int parse_c_type(struct _cffi_parse_info_s *info, const char *input);
static int search_in_globals(const struct _cffi_type_context_s *ctx,
                             const char *search, size_t search_len);
static int search_in_struct_unions(const struct _cffi_type_context_s *ctx,
                                   const char *search, size_t search_len);
#endif

/* this block of #ifs should be kept exactly identical between
   c/_cffi_backend.c, cffi/vengine_cpy.py, cffi/vengine_gen.py
   and cffi/_cffi_include.h */
#if defined(_MSC_VER)
# include <malloc.h>   /* for alloca() */
# if _MSC_VER < 1600   /* MSVC < 2010 */
   typedef __int8 int8_t;
   typedef __int16 int16_t;
   typedef __int32 int32_t;
   typedef __int64 int64_t;
   typedef unsigned __int8 uint8_t;
   typedef unsigned __int16 uint16_t;
   typedef unsigned __int32 uint32_t;
   typedef unsigned __int64 uint64_t;
   typedef __int8 int_least8_t;
   typedef __int16 int_least16_t;
   typedef __int32 int_least32_t;
   typedef __int64 int_least64_t;
   typedef unsigned __int8 uint_least8_t;
   typedef unsigned __int16 uint_least16_t;
   typedef unsigned __int32 uint_least32_t;
   typedef unsigned __int64 uint_least64_t;
   typedef __int8 int_fast8_t;
   typedef __int16 int_fast16_t;
   typedef __int32 int_fast32_t;
   typedef __int64 int_fast64_t;
   typedef unsigned __int8 uint_fast8_t;
   typedef unsigned __int16 uint_fast16_t;
   typedef unsigned __int32 uint_fast32_t;
   typedef unsigned __int64 uint_fast64_t;
   typedef __int64 intmax_t;
   typedef unsigned __int64 uintmax_t;
# else
#  include <stdint.h>
# endif
# if _MSC_VER < 1800   /* MSVC < 2013 */
#  ifndef __cplusplus
    typedef unsigned char _Bool;
#  endif
# endif
#else
# include <stdint.h>
# if (defined (__SVR4) && defined (__sun)) || defined(_AIX) || defined(__hpux)
#  include <alloca.h>
# endif
#endif

#ifdef __GNUC__
# define _CFFI_UNUSED_FN  __attribute__((unused))
#else
# define _CFFI_UNUSED_FN  /* nothing */
#endif

#ifdef __cplusplus
# ifndef _Bool
   typedef bool _Bool;   /* semi-hackish: C++ has no _Bool; bool is builtin */
# endif
#endif

/**********  CPython-specific section  **********/
#ifndef PYPY_VERSION


#if PY_MAJOR_VERSION >= 3
# define PyInt_FromLong PyLong_FromLong
#endif

#define _cffi_from_c_double PyFloat_FromDouble
#define _cffi_from_c_float PyFloat_FromDouble
#define _cffi_from_c_long PyInt_FromLong
#define _cffi_from_c_ulong PyLong_FromUnsignedLong
#define _cffi_from_c_longlong PyLong_FromLongLong
#define _cffi_from_c_ulonglong PyLong_FromUnsignedLongLong
#define _cffi_from_c__Bool PyBool_FromLong

#define _cffi_to_c_double PyFloat_AsDouble
#define _cffi_to_c_float PyFloat_AsDouble

#define _cffi_from_c_int(x, type)                                        \
    (((type)-1) > 0 ? /* unsigned */                                     \
        (sizeof(type) < sizeof(long) ?                                   \
            PyInt_FromLong((long)x) :                                    \
         sizeof(type) == sizeof(long) ?                                  \
            PyLong_FromUnsignedLong((unsigned long)x) :                  \
            PyLong_FromUnsignedLongLong((unsigned long long)x)) :        \
        (sizeof(type) <= sizeof(long) ?                                  \
            PyInt_FromLong((long)x) :                                    \
            PyLong_FromLongLong((long long)x)))

#define _cffi_to_c_int(o, type)                                          \
    ((type)(                                                             \
     sizeof(type) == 1 ? (((type)-1) > 0 ? (type)_cffi_to_c_u8(o)        \
                                         : (type)_cffi_to_c_i8(o)) :     \
     sizeof(type) == 2 ? (((type)-1) > 0 ? (type)_cffi_to_c_u16(o)       \
                                         : (type)_cffi_to_c_i16(o)) :    \
     sizeof(type) == 4 ? (((type)-1) > 0 ? (type)_cffi_to_c_u32(o)       \
                                         : (type)_cffi_to_c_i32(o)) :    \
     sizeof(type) == 8 ? (((type)-1) > 0 ? (type)_cffi_to_c_u64(o)       \
                                         : (type)_cffi_to_c_i64(o)) :    \
     (Py_FatalError("unsupported size for type " #type), (type)0)))

#define _cffi_to_c_i8                                                    \
                 ((int(*)(PyObject *))_cffi_exports[1])
#define _cffi_to_c_u8                                                    \
                 ((int(*)(PyObject *))_cffi_exports[2])
#define _cffi_to_c_i16                                                   \
                 ((int(*)(PyObject *))_cffi_exports[3])
#define _cffi_to_c_u16                                                   \
                 ((int(*)(PyObject *))_cffi_exports[4])
#define _cffi_to_c_i32                                                   \
                 ((int(*)(PyObject *))_cffi_exports[5])
#define _cffi_to_c_u32                                                   \
                 ((unsigned int(*)(PyObject *))_cffi_exports[6])
#define _cffi_to_c_i64                                                   \
                 ((long long(*)(PyObject *))_cffi_exports[7])
#define _cffi_to_c_u64                                                   \
                 ((unsigned long long(*)(PyObject *))_cffi_exports[8])
#define _cffi_to_c_char                                                  \
                 ((int(*)(PyObject *))_cffi_exports[9])
#define _cffi_from_c_pointer                                             \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[10])
#define _cffi_to_c_pointer                                               \
    ((char *(*)(PyObject *, struct _cffi_ctypedescr *))_cffi_exports[11])
#define _cffi_get_struct_layout                                          \
    not used any more
#define _cffi_restore_errno                                              \
    ((void(*)(void))_cffi_exports[13])
#define _cffi_save_errno                                                 \
    ((void(*)(void))_cffi_exports[14])
#define _cffi_from_c_char                                                \
    ((PyObject *(*)(char))_cffi_exports[15])
#define _cffi_from_c_deref                                               \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[16])
#define _cffi_to_c                                                       \
    ((int(*)(char *, struct _cffi_ctypedescr *, PyObject *))_cffi_exports[17])
#define _cffi_from_c_struct                                              \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[18])
#define _cffi_to_c_wchar_t                                               \
    ((_cffi_wchar_t(*)(PyObject *))_cffi_exports[19])
#define _cffi_from_c_wchar_t                                             \
    ((PyObject *(*)(_cffi_wchar_t))_cffi_exports[20])
#define _cffi_to_c_long_double                                           \
    ((long double(*)(PyObject *))_cffi_exports[21])
#define _cffi_to_c__Bool                                                 \
    ((_Bool(*)(PyObject *))_cffi_exports[22])
#define _cffi_prepare_pointer_call_argument                              \
    ((Py_ssize_t(*)(struct _cffi_ctypedescr *,                           \
                    PyObject *, char **))_cffi_exports[23])
#define _cffi_convert_array_from_object                                  \
    ((int(*)(char *, struct _cffi_ctypedescr *, PyObject *))_cffi_exports[24])
#define _CFFI_CPIDX  25
#define _cffi_call_python                                                \
    ((void(*)(struct _cffi_externpy_s *, char *))_cffi_exports[_CFFI_CPIDX])
#define _cffi_to_c_wchar3216_t                                           \
    ((int(*)(PyObject *))_cffi_exports[26])
#define _cffi_from_c_wchar3216_t                                         \
    ((PyObject *(*)(int))_cffi_exports[27])
#define _CFFI_NUM_EXPORTS 28

struct _cffi_ctypedescr;

static void *_cffi_exports[_CFFI_NUM_EXPORTS];

#define _cffi_type(index)   (                           \
    assert((((uintptr_t)_cffi_types[index]) & 1) == 0), \
    (struct _cffi_ctypedescr *)_cffi_types[index])

static PyObject *_cffi_init(const char *module_name, Py_ssize_t version,
                            const struct _cffi_type_context_s *ctx)
{
    PyObject *module, *o_arg, *new_module;
    void *raw[] = {
        (void *)module_name,
        (void *)version,
        (void *)_cffi_exports,
        (void *)ctx,
    };

    module = PyImport_ImportModule("_cffi_backend");
    if (module == NULL)
        goto failure;

    o_arg = PyLong_FromVoidPtr((void *)raw);
    if (o_arg == NULL)
        goto failure;

    new_module = PyObject_CallMethod(
        module, (char *)"_init_cffi_1_0_external_module", (char *)"O", o_arg);

    Py_DECREF(o_arg);
    Py_DECREF(module);
    return new_module;

  failure:
    Py_XDECREF(module);
    return NULL;
}


#ifdef HAVE_WCHAR_H
typedef wchar_t _cffi_wchar_t;
#else
typedef uint16_t _cffi_wchar_t;   /* same random pick as _cffi_backend.c */
#endif

_CFFI_UNUSED_FN static uint16_t _cffi_to_c_char16_t(PyObject *o)
{
    if (sizeof(_cffi_wchar_t) == 2)
        return (uint16_t)_cffi_to_c_wchar_t(o);
    else
        return (uint16_t)_cffi_to_c_wchar3216_t(o);
}

_CFFI_UNUSED_FN static PyObject *_cffi_from_c_char16_t(uint16_t x)
{
    if (sizeof(_cffi_wchar_t) == 2)
        return _cffi_from_c_wchar_t((_cffi_wchar_t)x);
    else
        return _cffi_from_c_wchar3216_t((int)x);
}

_CFFI_UNUSED_FN static int _cffi_to_c_char32_t(PyObject *o)
{
    if (sizeof(_cffi_wchar_t) == 4)
        return (int)_cffi_to_c_wchar_t(o);
    else
        return (int)_cffi_to_c_wchar3216_t(o);
}

_CFFI_UNUSED_FN static PyObject *_cffi_from_c_char32_t(unsigned int x)
{
    if (sizeof(_cffi_wchar_t) == 4)
        return _cffi_from_c_wchar_t((_cffi_wchar_t)x);
    else
        return _cffi_from_c_wchar3216_t((int)x);
}

union _cffi_union_alignment_u {
    unsigned char m_char;
    unsigned short m_short;
    unsigned int m_int;
    unsigned long m_long;
    unsigned long long m_longlong;
    float m_float;
    double m_double;
    long double m_longdouble;
};

struct _cffi_freeme_s {
    struct _cffi_freeme_s *next;
    union _cffi_union_alignment_u alignment;
};

_CFFI_UNUSED_FN static int
_cffi_convert_array_argument(struct _cffi_ctypedescr *ctptr, PyObject *arg,
                             char **output_data, Py_ssize_t datasize,
                             struct _cffi_freeme_s **freeme)
{
    char *p;
    if (datasize < 0)
        return -1;

    p = *output_data;
    if (p == NULL) {
        struct _cffi_freeme_s *fp = (struct _cffi_freeme_s *)PyObject_Malloc(
            offsetof(struct _cffi_freeme_s, alignment) + (size_t)datasize);
        if (fp == NULL)
            return -1;
        fp->next = *freeme;
        *freeme = fp;
        p = *output_data = (char *)&fp->alignment;
    }
    memset((void *)p, 0, (size_t)datasize);
    return _cffi_convert_array_from_object(p, ctptr, arg);
}

_CFFI_UNUSED_FN static void
_cffi_free_array_arguments(struct _cffi_freeme_s *freeme)
{
    do {
        void *p = (void *)freeme;
        freeme = freeme->next;
        PyObject_Free(p);
    } while (freeme != NULL);
}

/**********  end CPython-specific section  **********/
#else
_CFFI_UNUSED_FN
static void (*_cffi_call_python_org)(struct _cffi_externpy_s *, char *);
# define _cffi_call_python  _cffi_call_python_org
#endif


#define _cffi_array_len(array)   (sizeof(array) / sizeof((array)[0]))

#define _cffi_prim_int(size, sign)                                      \
    ((size) == 1 ? ((sign) ? _CFFI_PRIM_INT8  : _CFFI_PRIM_UINT8)  :    \
     (size) == 2 ? ((sign) ? _CFFI_PRIM_INT16 : _CFFI_PRIM_UINT16) :    \
     (size) == 4 ? ((sign) ? _CFFI_PRIM_INT32 : _CFFI_PRIM_UINT32) :    \
     (size) == 8 ? ((sign) ? _CFFI_PRIM_INT64 : _CFFI_PRIM_UINT64) :    \
     _CFFI__UNKNOWN_PRIM)

#define _cffi_prim_float(size)                                          \
    ((size) == sizeof(float) ? _CFFI_PRIM_FLOAT :                       \
     (size) == sizeof(double) ? _CFFI_PRIM_DOUBLE :                     \
     (size) == sizeof(long double) ? _CFFI__UNKNOWN_LONG_DOUBLE :       \
     _CFFI__UNKNOWN_FLOAT_PRIM)

#define _cffi_check_int(got, got_nonpos, expected)      \
    ((got_nonpos) == (expected <= 0) &&                 \
     (got) == (unsigned long long)expected)

#ifdef MS_WIN32
# define _cffi_stdcall  __stdcall
#else
# define _cffi_stdcall  /* nothing */
#endif

#ifdef __cplusplus
}
#endif

/************************************************************/

void f_ini_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[0] = -p[14]*x[0] - y[32] + u[10] + 1;

}
void g_ini_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[0] = 94.615102344455664*y[1] + 184.75059629806526*y[0] - 17.974903844364835*y[3] - 69.368640085185305*y[2] - 40.748687841669557*y[5] - 43.675799641021079*y[4] - 17.974903844364821*y[7] - 69.368640085185291*y[6] - 94.615102344455664*y[9] - 184.75059629806526*y[8] + 17.974903844364835*y[11] + 69.368640085185305*y[10] + 40.748687841669557*y[13] + 43.675799641021079*y[12] + 17.974903844364821*y[15] + 69.368640085185291*y[14] - y[16];
out[1] = 184.75059629806526*y[1] - 94.615102344455664*y[0] - 69.368640085185305*y[3] + 17.974903844364835*y[2] - 43.675799641021079*y[5] + 40.748687841669557*y[4] - 69.368640085185291*y[7] + 17.974903844364821*y[6] - 184.75059629806526*y[9] + 94.615102344455664*y[8] + 69.368640085185305*y[11] - 17.974903844364835*y[10] + 43.675799641021079*y[13] - 40.748687841669557*y[12] + 69.368640085185291*y[15] - 17.974903844364821*y[14] - y[20];
out[2] = -17.974903844364825*y[1] - 69.368640085185291*y[0] + 94.615102344455636*y[3] + 184.7505962980652*y[2] - 17.97490384436481*y[5] - 69.368640085185319*y[4] - 40.748687841669565*y[7] - 43.675799641021079*y[6] + 17.974903844364825*y[9] + 69.368640085185291*y[8] - 94.615102344455636*y[11] - 184.7505962980652*y[10] + 17.97490384436481*y[13] + 69.368640085185319*y[12] + 40.748687841669565*y[15] + 43.675799641021079*y[14] - y[17];
out[3] = -69.368640085185291*y[1] + 17.974903844364825*y[0] + 184.7505962980652*y[3] - 94.615102344455636*y[2] - 69.368640085185319*y[5] + 17.97490384436481*y[4] - 43.675799641021079*y[7] + 40.748687841669565*y[6] + 69.368640085185291*y[9] - 17.974903844364825*y[8] - 184.7505962980652*y[11] + 94.615102344455636*y[10] + 69.368640085185319*y[13] - 17.97490384436481*y[12] + 43.675799641021079*y[15] - 40.748687841669565*y[14] - y[21];
out[4] = -40.748687841669579*y[1] - 43.675799641021101*y[0] - 17.974903844364821*y[3] - 69.368640085185305*y[2] + 94.61510234445565*y[5] + 184.75059629806526*y[4] - 17.974903844364821*y[7] - 69.368640085185305*y[6] + 40.748687841669579*y[9] + 43.675799641021101*y[8] + 17.974903844364821*y[11] + 69.368640085185305*y[10] - 94.61510234445565*y[13] - 184.75059629806526*y[12] + 17.974903844364821*y[15] + 69.368640085185305*y[14] - y[18];
out[5] = -43.675799641021101*y[1] + 40.748687841669579*y[0] - 69.368640085185305*y[3] + 17.974903844364821*y[2] + 184.75059629806526*y[5] - 94.61510234445565*y[4] - 69.368640085185305*y[7] + 17.974903844364821*y[6] + 43.675799641021101*y[9] - 40.748687841669579*y[8] + 69.368640085185305*y[11] - 17.974903844364821*y[10] - 184.75059629806526*y[13] + 94.61510234445565*y[12] + 69.368640085185305*y[15] - 17.974903844364821*y[14] - y[22];
out[6] = -17.974903844364825*y[1] - 69.368640085185305*y[0] - 40.74868784166955*y[3] - 43.675799641021079*y[2] - 17.974903844364839*y[5] - 69.368640085185291*y[4] + 94.61510234445565*y[7] + 184.7505962980652*y[6] + 17.974903844364825*y[9] + 69.368640085185305*y[8] + 40.74868784166955*y[11] + 43.675799641021079*y[10] + 17.974903844364839*y[13] + 69.368640085185291*y[12] - 94.61510234445565*y[15] - 184.7505962980652*y[14] - y[19];
out[7] = -69.368640085185305*y[1] + 17.974903844364825*y[0] - 43.675799641021079*y[3] + 40.74868784166955*y[2] - 69.368640085185291*y[5] + 17.974903844364839*y[4] + 184.7505962980652*y[7] - 94.61510234445565*y[6] + 69.368640085185305*y[9] - 17.974903844364825*y[8] + 43.675799641021079*y[11] - 40.74868784166955*y[10] + 69.368640085185291*y[13] - 17.974903844364839*y[12] - 184.7505962980652*y[15] + 94.61510234445565*y[14] - y[23];
out[8] = -94.615102344455664*y[1] - 184.75059629806526*y[0] + 17.974903844364835*y[3] + 69.368640085185305*y[2] + 40.748687841669557*y[5] + 43.675799641021079*y[4] + 17.974903844364821*y[7] + 69.368640085185291*y[6] + 94.615102344455664*y[9] + 184.75059629806526*y[8] - 17.974903844364835*y[11] - 69.368640085185305*y[10] - 40.748687841669557*y[13] - 43.675799641021079*y[12] - 17.974903844364821*y[15] - 69.368640085185291*y[14] - y[24];
out[9] = -184.75059629806526*y[1] + 94.615102344455664*y[0] + 69.368640085185305*y[3] - 17.974903844364835*y[2] + 43.675799641021079*y[5] - 40.748687841669557*y[4] + 69.368640085185291*y[7] - 17.974903844364821*y[6] + 184.75059629806526*y[9] - 94.615102344455664*y[8] - 69.368640085185305*y[11] + 17.974903844364835*y[10] - 43.675799641021079*y[13] + 40.748687841669557*y[12] - 69.368640085185291*y[15] + 17.974903844364821*y[14] - y[28];
out[10] = 17.974903844364825*y[1] + 69.368640085185291*y[0] - 94.615102344455636*y[3] - 184.7505962980652*y[2] + 17.97490384436481*y[5] + 69.368640085185319*y[4] + 40.748687841669565*y[7] + 43.675799641021079*y[6] - 17.974903844364825*y[9] - 69.368640085185291*y[8] + 94.615102344455636*y[11] + 184.7505962980652*y[10] - 17.97490384436481*y[13] - 69.368640085185319*y[12] - 40.748687841669565*y[15] - 43.675799641021079*y[14] - y[25];
out[11] = 69.368640085185291*y[1] - 17.974903844364825*y[0] - 184.7505962980652*y[3] + 94.615102344455636*y[2] + 69.368640085185319*y[5] - 17.97490384436481*y[4] + 43.675799641021079*y[7] - 40.748687841669565*y[6] - 69.368640085185291*y[9] + 17.974903844364825*y[8] + 184.7505962980652*y[11] - 94.615102344455636*y[10] - 69.368640085185319*y[13] + 17.97490384436481*y[12] - 43.675799641021079*y[15] + 40.748687841669565*y[14] - y[29];
out[12] = 40.748687841669579*y[1] + 43.675799641021101*y[0] + 17.974903844364821*y[3] + 69.368640085185305*y[2] - 94.61510234445565*y[5] - 184.75059629806526*y[4] + 17.974903844364821*y[7] + 69.368640085185305*y[6] - 40.748687841669579*y[9] - 43.675799641021101*y[8] - 17.974903844364821*y[11] - 69.368640085185305*y[10] + 94.61510234445565*y[13] + 184.75059629806526*y[12] - 17.974903844364821*y[15] - 69.368640085185305*y[14] - y[26];
out[13] = 43.675799641021101*y[1] - 40.748687841669579*y[0] + 69.368640085185305*y[3] - 17.974903844364821*y[2] - 184.75059629806526*y[5] + 94.61510234445565*y[4] + 69.368640085185305*y[7] - 17.974903844364821*y[6] - 43.675799641021101*y[9] + 40.748687841669579*y[8] - 69.368640085185305*y[11] + 17.974903844364821*y[10] + 184.75059629806526*y[13] - 94.61510234445565*y[12] - 69.368640085185305*y[15] + 17.974903844364821*y[14] - y[30];
out[14] = 17.974903844364825*y[1] + 69.368640085185305*y[0] + 40.74868784166955*y[3] + 43.675799641021079*y[2] + 17.974903844364839*y[5] + 69.368640085185291*y[4] - 94.61510234445565*y[7] - 184.7505962980652*y[6] - 17.974903844364825*y[9] - 69.368640085185305*y[8] - 40.74868784166955*y[11] - 43.675799641021079*y[10] - 17.974903844364839*y[13] - 69.368640085185291*y[12] + 94.61510234445565*y[15] + 184.7505962980652*y[14] - y[27];
out[15] = 69.368640085185305*y[1] - 17.974903844364825*y[0] + 43.675799641021079*y[3] - 40.74868784166955*y[2] + 69.368640085185291*y[5] - 17.974903844364839*y[4] - 184.7505962980652*y[7] + 94.61510234445565*y[6] - 69.368640085185305*y[9] + 17.974903844364825*y[8] - 43.675799641021079*y[11] + 40.74868784166955*y[10] - 69.368640085185291*y[13] + 17.974903844364839*y[12] + 184.7505962980652*y[15] - 94.61510234445565*y[14] - y[31];
out[16] = -p[1]*y[16] - y[0] + 1.0*p[0]*y[20] + u[0]*cos(u[4]);
out[17] = -p[1]*y[17] - y[2] + 1.0*p[0]*y[21] + u[1]*cos(u[4] - 2.0943951023931953);
out[18] = -p[1]*y[18] - y[4] + 1.0*p[0]*y[22] + u[2]*cos(u[4] - 4.1887902047863905);
out[19] = -p[3]*y[19] - y[6] + 1.0*p[2]*y[23];
out[20] = -1.0*p[1]*y[20] - 1.0*y[1] - 1.0*p[0]*y[16] + 1.0*u[0]*sin(u[4]);
out[21] = -1.0*p[1]*y[21] - 1.0*y[3] - 1.0*p[0]*y[17] + 1.0*u[1]*sin(u[4] - 2.0943951023931953);
out[22] = -1.0*p[1]*y[22] - 1.0*y[5] - 1.0*p[0]*y[18] + 1.0*u[2]*sin(u[4] - 4.1887902047863905);
out[23] = -1.0*p[3]*y[23] - 1.0*y[7] - 1.0*p[2]*y[19];
out[24] = -p[7]*y[24] - y[8] + 1.0*p[6]*y[28] + u[5]*cos(u[9]);
out[25] = -p[7]*y[25] - y[10] + 1.0*p[6]*y[29] + u[6]*cos(u[9] - 2.0943951023931953);
out[26] = -p[7]*y[26] - y[12] + 1.0*p[6]*y[30] + u[7]*cos(u[9] - 4.1887902047863905);
out[27] = -p[9]*y[27] - y[14] + 1.0*p[8]*y[31];
out[28] = -1.0*p[7]*y[28] - 1.0*y[9] - 1.0*p[6]*y[24] + 1.0*u[5]*sin(u[9]);
out[29] = -1.0*p[7]*y[29] - 1.0*y[11] - 1.0*p[6]*y[25] + 1.0*u[6]*sin(u[9] - 2.0943951023931953);
out[30] = -1.0*p[7]*y[30] - 1.0*y[13] - 1.0*p[6]*y[26] + 1.0*u[7]*sin(u[9] - 4.1887902047863905);
out[31] = -1.0*p[9]*y[31] - 1.0*y[15] - 1.0*p[8]*y[27];
out[32] = 1.0 - y[32];
out[33] = p[13]*x[0] + p[12]*(1 - y[32]) - y[33];

}
void f_run_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[0] = -p[14]*x[0] - y[32] + u[10] + 1;

}
void g_run_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[0] = 94.615102344455664*y[1] + 184.75059629806526*y[0] - 17.974903844364835*y[3] - 69.368640085185305*y[2] - 40.748687841669557*y[5] - 43.675799641021079*y[4] - 17.974903844364821*y[7] - 69.368640085185291*y[6] - 94.615102344455664*y[9] - 184.75059629806526*y[8] + 17.974903844364835*y[11] + 69.368640085185305*y[10] + 40.748687841669557*y[13] + 43.675799641021079*y[12] + 17.974903844364821*y[15] + 69.368640085185291*y[14] - y[16];
out[1] = 184.75059629806526*y[1] - 94.615102344455664*y[0] - 69.368640085185305*y[3] + 17.974903844364835*y[2] - 43.675799641021079*y[5] + 40.748687841669557*y[4] - 69.368640085185291*y[7] + 17.974903844364821*y[6] - 184.75059629806526*y[9] + 94.615102344455664*y[8] + 69.368640085185305*y[11] - 17.974903844364835*y[10] + 43.675799641021079*y[13] - 40.748687841669557*y[12] + 69.368640085185291*y[15] - 17.974903844364821*y[14] - y[20];
out[2] = -17.974903844364825*y[1] - 69.368640085185291*y[0] + 94.615102344455636*y[3] + 184.7505962980652*y[2] - 17.97490384436481*y[5] - 69.368640085185319*y[4] - 40.748687841669565*y[7] - 43.675799641021079*y[6] + 17.974903844364825*y[9] + 69.368640085185291*y[8] - 94.615102344455636*y[11] - 184.7505962980652*y[10] + 17.97490384436481*y[13] + 69.368640085185319*y[12] + 40.748687841669565*y[15] + 43.675799641021079*y[14] - y[17];
out[3] = -69.368640085185291*y[1] + 17.974903844364825*y[0] + 184.7505962980652*y[3] - 94.615102344455636*y[2] - 69.368640085185319*y[5] + 17.97490384436481*y[4] - 43.675799641021079*y[7] + 40.748687841669565*y[6] + 69.368640085185291*y[9] - 17.974903844364825*y[8] - 184.7505962980652*y[11] + 94.615102344455636*y[10] + 69.368640085185319*y[13] - 17.97490384436481*y[12] + 43.675799641021079*y[15] - 40.748687841669565*y[14] - y[21];
out[4] = -40.748687841669579*y[1] - 43.675799641021101*y[0] - 17.974903844364821*y[3] - 69.368640085185305*y[2] + 94.61510234445565*y[5] + 184.75059629806526*y[4] - 17.974903844364821*y[7] - 69.368640085185305*y[6] + 40.748687841669579*y[9] + 43.675799641021101*y[8] + 17.974903844364821*y[11] + 69.368640085185305*y[10] - 94.61510234445565*y[13] - 184.75059629806526*y[12] + 17.974903844364821*y[15] + 69.368640085185305*y[14] - y[18];
out[5] = -43.675799641021101*y[1] + 40.748687841669579*y[0] - 69.368640085185305*y[3] + 17.974903844364821*y[2] + 184.75059629806526*y[5] - 94.61510234445565*y[4] - 69.368640085185305*y[7] + 17.974903844364821*y[6] + 43.675799641021101*y[9] - 40.748687841669579*y[8] + 69.368640085185305*y[11] - 17.974903844364821*y[10] - 184.75059629806526*y[13] + 94.61510234445565*y[12] + 69.368640085185305*y[15] - 17.974903844364821*y[14] - y[22];
out[6] = -17.974903844364825*y[1] - 69.368640085185305*y[0] - 40.74868784166955*y[3] - 43.675799641021079*y[2] - 17.974903844364839*y[5] - 69.368640085185291*y[4] + 94.61510234445565*y[7] + 184.7505962980652*y[6] + 17.974903844364825*y[9] + 69.368640085185305*y[8] + 40.74868784166955*y[11] + 43.675799641021079*y[10] + 17.974903844364839*y[13] + 69.368640085185291*y[12] - 94.61510234445565*y[15] - 184.7505962980652*y[14] - y[19];
out[7] = -69.368640085185305*y[1] + 17.974903844364825*y[0] - 43.675799641021079*y[3] + 40.74868784166955*y[2] - 69.368640085185291*y[5] + 17.974903844364839*y[4] + 184.7505962980652*y[7] - 94.61510234445565*y[6] + 69.368640085185305*y[9] - 17.974903844364825*y[8] + 43.675799641021079*y[11] - 40.74868784166955*y[10] + 69.368640085185291*y[13] - 17.974903844364839*y[12] - 184.7505962980652*y[15] + 94.61510234445565*y[14] - y[23];
out[8] = -94.615102344455664*y[1] - 184.75059629806526*y[0] + 17.974903844364835*y[3] + 69.368640085185305*y[2] + 40.748687841669557*y[5] + 43.675799641021079*y[4] + 17.974903844364821*y[7] + 69.368640085185291*y[6] + 94.615102344455664*y[9] + 184.75059629806526*y[8] - 17.974903844364835*y[11] - 69.368640085185305*y[10] - 40.748687841669557*y[13] - 43.675799641021079*y[12] - 17.974903844364821*y[15] - 69.368640085185291*y[14] - y[24];
out[9] = -184.75059629806526*y[1] + 94.615102344455664*y[0] + 69.368640085185305*y[3] - 17.974903844364835*y[2] + 43.675799641021079*y[5] - 40.748687841669557*y[4] + 69.368640085185291*y[7] - 17.974903844364821*y[6] + 184.75059629806526*y[9] - 94.615102344455664*y[8] - 69.368640085185305*y[11] + 17.974903844364835*y[10] - 43.675799641021079*y[13] + 40.748687841669557*y[12] - 69.368640085185291*y[15] + 17.974903844364821*y[14] - y[28];
out[10] = 17.974903844364825*y[1] + 69.368640085185291*y[0] - 94.615102344455636*y[3] - 184.7505962980652*y[2] + 17.97490384436481*y[5] + 69.368640085185319*y[4] + 40.748687841669565*y[7] + 43.675799641021079*y[6] - 17.974903844364825*y[9] - 69.368640085185291*y[8] + 94.615102344455636*y[11] + 184.7505962980652*y[10] - 17.97490384436481*y[13] - 69.368640085185319*y[12] - 40.748687841669565*y[15] - 43.675799641021079*y[14] - y[25];
out[11] = 69.368640085185291*y[1] - 17.974903844364825*y[0] - 184.7505962980652*y[3] + 94.615102344455636*y[2] + 69.368640085185319*y[5] - 17.97490384436481*y[4] + 43.675799641021079*y[7] - 40.748687841669565*y[6] - 69.368640085185291*y[9] + 17.974903844364825*y[8] + 184.7505962980652*y[11] - 94.615102344455636*y[10] - 69.368640085185319*y[13] + 17.97490384436481*y[12] - 43.675799641021079*y[15] + 40.748687841669565*y[14] - y[29];
out[12] = 40.748687841669579*y[1] + 43.675799641021101*y[0] + 17.974903844364821*y[3] + 69.368640085185305*y[2] - 94.61510234445565*y[5] - 184.75059629806526*y[4] + 17.974903844364821*y[7] + 69.368640085185305*y[6] - 40.748687841669579*y[9] - 43.675799641021101*y[8] - 17.974903844364821*y[11] - 69.368640085185305*y[10] + 94.61510234445565*y[13] + 184.75059629806526*y[12] - 17.974903844364821*y[15] - 69.368640085185305*y[14] - y[26];
out[13] = 43.675799641021101*y[1] - 40.748687841669579*y[0] + 69.368640085185305*y[3] - 17.974903844364821*y[2] - 184.75059629806526*y[5] + 94.61510234445565*y[4] + 69.368640085185305*y[7] - 17.974903844364821*y[6] - 43.675799641021101*y[9] + 40.748687841669579*y[8] - 69.368640085185305*y[11] + 17.974903844364821*y[10] + 184.75059629806526*y[13] - 94.61510234445565*y[12] - 69.368640085185305*y[15] + 17.974903844364821*y[14] - y[30];
out[14] = 17.974903844364825*y[1] + 69.368640085185305*y[0] + 40.74868784166955*y[3] + 43.675799641021079*y[2] + 17.974903844364839*y[5] + 69.368640085185291*y[4] - 94.61510234445565*y[7] - 184.7505962980652*y[6] - 17.974903844364825*y[9] - 69.368640085185305*y[8] - 40.74868784166955*y[11] - 43.675799641021079*y[10] - 17.974903844364839*y[13] - 69.368640085185291*y[12] + 94.61510234445565*y[15] + 184.7505962980652*y[14] - y[27];
out[15] = 69.368640085185305*y[1] - 17.974903844364825*y[0] + 43.675799641021079*y[3] - 40.74868784166955*y[2] + 69.368640085185291*y[5] - 17.974903844364839*y[4] - 184.7505962980652*y[7] + 94.61510234445565*y[6] - 69.368640085185305*y[9] + 17.974903844364825*y[8] - 43.675799641021079*y[11] + 40.74868784166955*y[10] - 69.368640085185291*y[13] + 17.974903844364839*y[12] + 184.7505962980652*y[15] - 94.61510234445565*y[14] - y[31];
out[16] = -p[1]*y[16] - y[0] + 1.0*p[0]*y[20] + u[0]*cos(u[4]);
out[17] = -p[1]*y[17] - y[2] + 1.0*p[0]*y[21] + u[1]*cos(u[4] - 2.0943951023931953);
out[18] = -p[1]*y[18] - y[4] + 1.0*p[0]*y[22] + u[2]*cos(u[4] - 4.1887902047863905);
out[19] = -p[3]*y[19] - y[6] + 1.0*p[2]*y[23];
out[20] = -1.0*p[1]*y[20] - 1.0*y[1] - 1.0*p[0]*y[16] + 1.0*u[0]*sin(u[4]);
out[21] = -1.0*p[1]*y[21] - 1.0*y[3] - 1.0*p[0]*y[17] + 1.0*u[1]*sin(u[4] - 2.0943951023931953);
out[22] = -1.0*p[1]*y[22] - 1.0*y[5] - 1.0*p[0]*y[18] + 1.0*u[2]*sin(u[4] - 4.1887902047863905);
out[23] = -1.0*p[3]*y[23] - 1.0*y[7] - 1.0*p[2]*y[19];
out[24] = -p[7]*y[24] - y[8] + 1.0*p[6]*y[28] + u[5]*cos(u[9]);
out[25] = -p[7]*y[25] - y[10] + 1.0*p[6]*y[29] + u[6]*cos(u[9] - 2.0943951023931953);
out[26] = -p[7]*y[26] - y[12] + 1.0*p[6]*y[30] + u[7]*cos(u[9] - 4.1887902047863905);
out[27] = -p[9]*y[27] - y[14] + 1.0*p[8]*y[31];
out[28] = -1.0*p[7]*y[28] - 1.0*y[9] - 1.0*p[6]*y[24] + 1.0*u[5]*sin(u[9]);
out[29] = -1.0*p[7]*y[29] - 1.0*y[11] - 1.0*p[6]*y[25] + 1.0*u[6]*sin(u[9] - 2.0943951023931953);
out[30] = -1.0*p[7]*y[30] - 1.0*y[13] - 1.0*p[6]*y[26] + 1.0*u[7]*sin(u[9] - 4.1887902047863905);
out[31] = -1.0*p[9]*y[31] - 1.0*y[15] - 1.0*p[8]*y[27];
out[32] = 1.0 - y[32];
out[33] = p[13]*x[0] + p[12]*(1 - y[32]) - y[33];

}
void h_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[0] = 94.615102344455664*y[1] + 184.75059629806526*y[0] - 17.974903844364835*y[3] - 69.368640085185305*y[2] - 40.748687841669557*y[5] - 43.675799641021079*y[4] - 17.974903844364821*y[7] - 69.368640085185291*y[6] - 94.615102344455664*y[9] - 184.75059629806526*y[8] + 17.974903844364835*y[11] + 69.368640085185305*y[10] + 40.748687841669557*y[13] + 43.675799641021079*y[12] + 17.974903844364821*y[15] + 69.368640085185291*y[14];
out[1] = 184.75059629806526*y[1] - 94.615102344455664*y[0] - 69.368640085185305*y[3] + 17.974903844364835*y[2] - 43.675799641021079*y[5] + 40.748687841669557*y[4] - 69.368640085185291*y[7] + 17.974903844364821*y[6] - 184.75059629806526*y[9] + 94.615102344455664*y[8] + 69.368640085185305*y[11] - 17.974903844364835*y[10] + 43.675799641021079*y[13] - 40.748687841669557*y[12] + 69.368640085185291*y[15] - 17.974903844364821*y[14];
out[2] = -17.974903844364825*y[1] - 69.368640085185291*y[0] + 94.615102344455636*y[3] + 184.7505962980652*y[2] - 17.97490384436481*y[5] - 69.368640085185319*y[4] - 40.748687841669565*y[7] - 43.675799641021079*y[6] + 17.974903844364825*y[9] + 69.368640085185291*y[8] - 94.615102344455636*y[11] - 184.7505962980652*y[10] + 17.97490384436481*y[13] + 69.368640085185319*y[12] + 40.748687841669565*y[15] + 43.675799641021079*y[14];
out[3] = -69.368640085185291*y[1] + 17.974903844364825*y[0] + 184.7505962980652*y[3] - 94.615102344455636*y[2] - 69.368640085185319*y[5] + 17.97490384436481*y[4] - 43.675799641021079*y[7] + 40.748687841669565*y[6] + 69.368640085185291*y[9] - 17.974903844364825*y[8] - 184.7505962980652*y[11] + 94.615102344455636*y[10] + 69.368640085185319*y[13] - 17.97490384436481*y[12] + 43.675799641021079*y[15] - 40.748687841669565*y[14];
out[4] = -40.748687841669579*y[1] - 43.675799641021101*y[0] - 17.974903844364821*y[3] - 69.368640085185305*y[2] + 94.61510234445565*y[5] + 184.75059629806526*y[4] - 17.974903844364821*y[7] - 69.368640085185305*y[6] + 40.748687841669579*y[9] + 43.675799641021101*y[8] + 17.974903844364821*y[11] + 69.368640085185305*y[10] - 94.61510234445565*y[13] - 184.75059629806526*y[12] + 17.974903844364821*y[15] + 69.368640085185305*y[14];
out[5] = -43.675799641021101*y[1] + 40.748687841669579*y[0] - 69.368640085185305*y[3] + 17.974903844364821*y[2] + 184.75059629806526*y[5] - 94.61510234445565*y[4] - 69.368640085185305*y[7] + 17.974903844364821*y[6] + 43.675799641021101*y[9] - 40.748687841669579*y[8] + 69.368640085185305*y[11] - 17.974903844364821*y[10] - 184.75059629806526*y[13] + 94.61510234445565*y[12] + 69.368640085185305*y[15] - 17.974903844364821*y[14];
out[6] = -17.974903844364825*y[1] - 69.368640085185305*y[0] - 40.74868784166955*y[3] - 43.675799641021079*y[2] - 17.974903844364839*y[5] - 69.368640085185291*y[4] + 94.61510234445565*y[7] + 184.7505962980652*y[6] + 17.974903844364825*y[9] + 69.368640085185305*y[8] + 40.74868784166955*y[11] + 43.675799641021079*y[10] + 17.974903844364839*y[13] + 69.368640085185291*y[12] - 94.61510234445565*y[15] - 184.7505962980652*y[14];
out[7] = -69.368640085185305*y[1] + 17.974903844364825*y[0] - 43.675799641021079*y[3] + 40.74868784166955*y[2] - 69.368640085185291*y[5] + 17.974903844364839*y[4] + 184.7505962980652*y[7] - 94.61510234445565*y[6] + 69.368640085185305*y[9] - 17.974903844364825*y[8] + 43.675799641021079*y[11] - 40.74868784166955*y[10] + 69.368640085185291*y[13] - 17.974903844364839*y[12] - 184.7505962980652*y[15] + 94.61510234445565*y[14];
out[8] = sqrt(1.0*pow(y[20], 2) + pow(y[16], 2));
out[9] = sqrt(1.0*pow(y[21], 2) + pow(y[17], 2));
out[10] = sqrt(1.0*pow(y[22], 2) + pow(y[18], 2));
out[11] = sqrt(1.0*pow(y[23], 2) + pow(y[19], 2));
out[12] = 1.0*y[1]*y[20] + y[0]*y[16] + 1.0*y[3]*y[21] + y[2]*y[17] + 1.0*y[5]*y[22] + y[4]*y[18];
out[13] = 1.0*y[1]*y[16] - 1.0*y[0]*y[20] + 1.0*y[3]*y[17] - 1.0*y[2]*y[21] + 1.0*y[5]*y[18] - 1.0*y[4]*y[22];
out[14] = sqrt(1.0*pow(y[28], 2) + pow(y[24], 2));
out[15] = sqrt(1.0*pow(y[29], 2) + pow(y[25], 2));
out[16] = sqrt(1.0*pow(y[30], 2) + pow(y[26], 2));
out[17] = sqrt(1.0*pow(y[31], 2) + pow(y[27], 2));
out[18] = 1.0*y[9]*y[28] + y[8]*y[24] + 1.0*y[11]*y[29] + y[10]*y[25] + 1.0*y[13]*y[30] + y[12]*y[26];
out[19] = 1.0*y[9]*y[24] - 1.0*y[8]*y[28] + 1.0*y[11]*y[25] - 1.0*y[10]*y[29] + 1.0*y[13]*y[26] - 1.0*y[12]*y[30];
out[20] = x[0];
out[21] = u[10];

}
void de_jac_ini_xy_eval(double *out,double *x,double *y,double *u,double *p,double Dt){


}

void de_jac_ini_up_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[0] = -p[14];
out[612] = -p[1];
out[616] = 1.0*p[0];
out[648] = -p[1];
out[652] = 1.0*p[0];
out[684] = -p[1];
out[688] = 1.0*p[0];
out[720] = -p[3];
out[724] = 1.0*p[2];
out[752] = -1.0*p[0];
out[756] = -1.0*p[1];
out[788] = -1.0*p[0];
out[792] = -1.0*p[1];
out[824] = -1.0*p[0];
out[828] = -1.0*p[1];
out[860] = -1.0*p[2];
out[864] = -1.0*p[3];
out[900] = -p[7];
out[904] = 1.0*p[6];
out[936] = -p[7];
out[940] = 1.0*p[6];
out[972] = -p[7];
out[976] = 1.0*p[6];
out[1008] = -p[9];
out[1012] = 1.0*p[8];
out[1040] = -1.0*p[6];
out[1044] = -1.0*p[7];
out[1076] = -1.0*p[6];
out[1080] = -1.0*p[7];
out[1112] = -1.0*p[6];
out[1116] = -1.0*p[7];
out[1148] = -1.0*p[8];
out[1152] = -1.0*p[9];
out[1190] = p[13];
out[1223] = -p[12];

}

void de_jac_ini_num_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[33] = -1.0;
out[36] = 184.75059629806526;
out[37] = 94.615102344455664;
out[38] = -69.368640085185305;
out[39] = -17.974903844364835;
out[40] = -43.675799641021079;
out[41] = -40.748687841669557;
out[42] = -69.368640085185291;
out[43] = -17.974903844364821;
out[44] = -184.75059629806526;
out[45] = -94.615102344455664;
out[46] = 69.368640085185305;
out[47] = 17.974903844364835;
out[48] = 43.675799641021079;
out[49] = 40.748687841669557;
out[50] = 69.368640085185291;
out[51] = 17.974903844364821;
out[52] = -1.0;
out[71] = -94.615102344455664;
out[72] = 184.75059629806526;
out[73] = 17.974903844364835;
out[74] = -69.368640085185305;
out[75] = 40.748687841669557;
out[76] = -43.675799641021079;
out[77] = 17.974903844364821;
out[78] = -69.368640085185291;
out[79] = 94.615102344455664;
out[80] = -184.75059629806526;
out[81] = -17.974903844364835;
out[82] = 69.368640085185305;
out[83] = -40.748687841669557;
out[84] = 43.675799641021079;
out[85] = -17.974903844364821;
out[86] = 69.368640085185291;
out[91] = -1.0;
out[106] = -69.368640085185291;
out[107] = -17.974903844364825;
out[108] = 184.7505962980652;
out[109] = 94.615102344455636;
out[110] = -69.368640085185319;
out[111] = -17.97490384436481;
out[112] = -43.675799641021079;
out[113] = -40.748687841669565;
out[114] = 69.368640085185291;
out[115] = 17.974903844364825;
out[116] = -184.7505962980652;
out[117] = -94.615102344455636;
out[118] = 69.368640085185319;
out[119] = 17.97490384436481;
out[120] = 43.675799641021079;
out[121] = 40.748687841669565;
out[123] = -1.0;
out[141] = 17.974903844364825;
out[142] = -69.368640085185291;
out[143] = -94.615102344455636;
out[144] = 184.7505962980652;
out[145] = 17.97490384436481;
out[146] = -69.368640085185319;
out[147] = 40.748687841669565;
out[148] = -43.675799641021079;
out[149] = -17.974903844364825;
out[150] = 69.368640085185291;
out[151] = 94.615102344455636;
out[152] = -184.7505962980652;
out[153] = -17.97490384436481;
out[154] = 69.368640085185319;
out[155] = -40.748687841669565;
out[156] = 43.675799641021079;
out[162] = -1.0;
out[176] = -43.675799641021101;
out[177] = -40.748687841669579;
out[178] = -69.368640085185305;
out[179] = -17.974903844364821;
out[180] = 184.75059629806526;
out[181] = 94.61510234445565;
out[182] = -69.368640085185305;
out[183] = -17.974903844364821;
out[184] = 43.675799641021101;
out[185] = 40.748687841669579;
out[186] = 69.368640085185305;
out[187] = 17.974903844364821;
out[188] = -184.75059629806526;
out[189] = -94.61510234445565;
out[190] = 69.368640085185305;
out[191] = 17.974903844364821;
out[194] = -1.0;
out[211] = 40.748687841669579;
out[212] = -43.675799641021101;
out[213] = 17.974903844364821;
out[214] = -69.368640085185305;
out[215] = -94.61510234445565;
out[216] = 184.75059629806526;
out[217] = 17.974903844364821;
out[218] = -69.368640085185305;
out[219] = -40.748687841669579;
out[220] = 43.675799641021101;
out[221] = -17.974903844364821;
out[222] = 69.368640085185305;
out[223] = 94.61510234445565;
out[224] = -184.75059629806526;
out[225] = -17.974903844364821;
out[226] = 69.368640085185305;
out[233] = -1.0;
out[246] = -69.368640085185305;
out[247] = -17.974903844364825;
out[248] = -43.675799641021079;
out[249] = -40.74868784166955;
out[250] = -69.368640085185291;
out[251] = -17.974903844364839;
out[252] = 184.7505962980652;
out[253] = 94.61510234445565;
out[254] = 69.368640085185305;
out[255] = 17.974903844364825;
out[256] = 43.675799641021079;
out[257] = 40.74868784166955;
out[258] = 69.368640085185291;
out[259] = 17.974903844364839;
out[260] = -184.7505962980652;
out[261] = -94.61510234445565;
out[265] = -1.0;
out[281] = 17.974903844364825;
out[282] = -69.368640085185305;
out[283] = 40.74868784166955;
out[284] = -43.675799641021079;
out[285] = 17.974903844364839;
out[286] = -69.368640085185291;
out[287] = -94.61510234445565;
out[288] = 184.7505962980652;
out[289] = -17.974903844364825;
out[290] = 69.368640085185305;
out[291] = -40.74868784166955;
out[292] = 43.675799641021079;
out[293] = -17.974903844364839;
out[294] = 69.368640085185291;
out[295] = 94.61510234445565;
out[296] = -184.7505962980652;
out[304] = -1.0;
out[316] = -184.75059629806526;
out[317] = -94.615102344455664;
out[318] = 69.368640085185305;
out[319] = 17.974903844364835;
out[320] = 43.675799641021079;
out[321] = 40.748687841669557;
out[322] = 69.368640085185291;
out[323] = 17.974903844364821;
out[324] = 184.75059629806526;
out[325] = 94.615102344455664;
out[326] = -69.368640085185305;
out[327] = -17.974903844364835;
out[328] = -43.675799641021079;
out[329] = -40.748687841669557;
out[330] = -69.368640085185291;
out[331] = -17.974903844364821;
out[340] = -1.0;
out[351] = 94.615102344455664;
out[352] = -184.75059629806526;
out[353] = -17.974903844364835;
out[354] = 69.368640085185305;
out[355] = -40.748687841669557;
out[356] = 43.675799641021079;
out[357] = -17.974903844364821;
out[358] = 69.368640085185291;
out[359] = -94.615102344455664;
out[360] = 184.75059629806526;
out[361] = 17.974903844364835;
out[362] = -69.368640085185305;
out[363] = 40.748687841669557;
out[364] = -43.675799641021079;
out[365] = 17.974903844364821;
out[366] = -69.368640085185291;
out[379] = -1.0;
out[386] = 69.368640085185291;
out[387] = 17.974903844364825;
out[388] = -184.7505962980652;
out[389] = -94.615102344455636;
out[390] = 69.368640085185319;
out[391] = 17.97490384436481;
out[392] = 43.675799641021079;
out[393] = 40.748687841669565;
out[394] = -69.368640085185291;
out[395] = -17.974903844364825;
out[396] = 184.7505962980652;
out[397] = 94.615102344455636;
out[398] = -69.368640085185319;
out[399] = -17.97490384436481;
out[400] = -43.675799641021079;
out[401] = -40.748687841669565;
out[411] = -1.0;
out[421] = -17.974903844364825;
out[422] = 69.368640085185291;
out[423] = 94.615102344455636;
out[424] = -184.7505962980652;
out[425] = -17.97490384436481;
out[426] = 69.368640085185319;
out[427] = -40.748687841669565;
out[428] = 43.675799641021079;
out[429] = 17.974903844364825;
out[430] = -69.368640085185291;
out[431] = -94.615102344455636;
out[432] = 184.7505962980652;
out[433] = 17.97490384436481;
out[434] = -69.368640085185319;
out[435] = 40.748687841669565;
out[436] = -43.675799641021079;
out[450] = -1.0;
out[456] = 43.675799641021101;
out[457] = 40.748687841669579;
out[458] = 69.368640085185305;
out[459] = 17.974903844364821;
out[460] = -184.75059629806526;
out[461] = -94.61510234445565;
out[462] = 69.368640085185305;
out[463] = 17.974903844364821;
out[464] = -43.675799641021101;
out[465] = -40.748687841669579;
out[466] = -69.368640085185305;
out[467] = -17.974903844364821;
out[468] = 184.75059629806526;
out[469] = 94.61510234445565;
out[470] = -69.368640085185305;
out[471] = -17.974903844364821;
out[482] = -1.0;
out[491] = -40.748687841669579;
out[492] = 43.675799641021101;
out[493] = -17.974903844364821;
out[494] = 69.368640085185305;
out[495] = 94.61510234445565;
out[496] = -184.75059629806526;
out[497] = -17.974903844364821;
out[498] = 69.368640085185305;
out[499] = 40.748687841669579;
out[500] = -43.675799641021101;
out[501] = 17.974903844364821;
out[502] = -69.368640085185305;
out[503] = -94.61510234445565;
out[504] = 184.75059629806526;
out[505] = 17.974903844364821;
out[506] = -69.368640085185305;
out[521] = -1.0;
out[526] = 69.368640085185305;
out[527] = 17.974903844364825;
out[528] = 43.675799641021079;
out[529] = 40.74868784166955;
out[530] = 69.368640085185291;
out[531] = 17.974903844364839;
out[532] = -184.7505962980652;
out[533] = -94.61510234445565;
out[534] = -69.368640085185305;
out[535] = -17.974903844364825;
out[536] = -43.675799641021079;
out[537] = -40.74868784166955;
out[538] = -69.368640085185291;
out[539] = -17.974903844364839;
out[540] = 184.7505962980652;
out[541] = 94.61510234445565;
out[553] = -1.0;
out[561] = -17.974903844364825;
out[562] = 69.368640085185305;
out[563] = -40.74868784166955;
out[564] = 43.675799641021079;
out[565] = -17.974903844364839;
out[566] = 69.368640085185291;
out[567] = 94.61510234445565;
out[568] = -184.7505962980652;
out[569] = 17.974903844364825;
out[570] = -69.368640085185305;
out[571] = 40.74868784166955;
out[572] = -43.675799641021079;
out[573] = 17.974903844364839;
out[574] = -69.368640085185291;
out[575] = -94.61510234445565;
out[576] = 184.7505962980652;
out[592] = -1.0;
out[596] = -1.0;
out[633] = -1.0;
out[670] = -1.0;
out[707] = -1.0;
out[737] = -1.0;
out[774] = -1.0;
out[811] = -1.0;
out[848] = -1.0;
out[884] = -1.0;
out[921] = -1.0;
out[958] = -1.0;
out[995] = -1.0;
out[1025] = -1.0;
out[1062] = -1.0;
out[1099] = -1.0;
out[1136] = -1.0;
out[1188] = -1.0;
out[1224] = -1.0;

}

void sp_jac_ini_xy_eval(double *out,double *x,double *y,double *u,double *p,double Dt){


}

void sp_jac_ini_up_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[0] = -p[14];
out[275] = -p[1];
out[276] = 1.0*p[0];
out[278] = -p[1];
out[279] = 1.0*p[0];
out[281] = -p[1];
out[282] = 1.0*p[0];
out[284] = -p[3];
out[285] = 1.0*p[2];
out[287] = -1.0*p[0];
out[288] = -1.0*p[1];
out[290] = -1.0*p[0];
out[291] = -1.0*p[1];
out[293] = -1.0*p[0];
out[294] = -1.0*p[1];
out[296] = -1.0*p[2];
out[297] = -1.0*p[3];
out[299] = -p[7];
out[300] = 1.0*p[6];
out[302] = -p[7];
out[303] = 1.0*p[6];
out[305] = -p[7];
out[306] = 1.0*p[6];
out[308] = -p[9];
out[309] = 1.0*p[8];
out[311] = -1.0*p[6];
out[312] = -1.0*p[7];
out[314] = -1.0*p[6];
out[315] = -1.0*p[7];
out[317] = -1.0*p[6];
out[318] = -1.0*p[7];
out[320] = -1.0*p[8];
out[321] = -1.0*p[9];
out[323] = p[13];
out[324] = -p[12];

}

void sp_jac_ini_num_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[1] = -1.0;
out[2] = 184.75059629806526;
out[3] = 94.615102344455664;
out[4] = -69.368640085185305;
out[5] = -17.974903844364835;
out[6] = -43.675799641021079;
out[7] = -40.748687841669557;
out[8] = -69.368640085185291;
out[9] = -17.974903844364821;
out[10] = -184.75059629806526;
out[11] = -94.615102344455664;
out[12] = 69.368640085185305;
out[13] = 17.974903844364835;
out[14] = 43.675799641021079;
out[15] = 40.748687841669557;
out[16] = 69.368640085185291;
out[17] = 17.974903844364821;
out[18] = -1.0;
out[19] = -94.615102344455664;
out[20] = 184.75059629806526;
out[21] = 17.974903844364835;
out[22] = -69.368640085185305;
out[23] = 40.748687841669557;
out[24] = -43.675799641021079;
out[25] = 17.974903844364821;
out[26] = -69.368640085185291;
out[27] = 94.615102344455664;
out[28] = -184.75059629806526;
out[29] = -17.974903844364835;
out[30] = 69.368640085185305;
out[31] = -40.748687841669557;
out[32] = 43.675799641021079;
out[33] = -17.974903844364821;
out[34] = 69.368640085185291;
out[35] = -1.0;
out[36] = -69.368640085185291;
out[37] = -17.974903844364825;
out[38] = 184.7505962980652;
out[39] = 94.615102344455636;
out[40] = -69.368640085185319;
out[41] = -17.97490384436481;
out[42] = -43.675799641021079;
out[43] = -40.748687841669565;
out[44] = 69.368640085185291;
out[45] = 17.974903844364825;
out[46] = -184.7505962980652;
out[47] = -94.615102344455636;
out[48] = 69.368640085185319;
out[49] = 17.97490384436481;
out[50] = 43.675799641021079;
out[51] = 40.748687841669565;
out[52] = -1.0;
out[53] = 17.974903844364825;
out[54] = -69.368640085185291;
out[55] = -94.615102344455636;
out[56] = 184.7505962980652;
out[57] = 17.97490384436481;
out[58] = -69.368640085185319;
out[59] = 40.748687841669565;
out[60] = -43.675799641021079;
out[61] = -17.974903844364825;
out[62] = 69.368640085185291;
out[63] = 94.615102344455636;
out[64] = -184.7505962980652;
out[65] = -17.97490384436481;
out[66] = 69.368640085185319;
out[67] = -40.748687841669565;
out[68] = 43.675799641021079;
out[69] = -1.0;
out[70] = -43.675799641021101;
out[71] = -40.748687841669579;
out[72] = -69.368640085185305;
out[73] = -17.974903844364821;
out[74] = 184.75059629806526;
out[75] = 94.61510234445565;
out[76] = -69.368640085185305;
out[77] = -17.974903844364821;
out[78] = 43.675799641021101;
out[79] = 40.748687841669579;
out[80] = 69.368640085185305;
out[81] = 17.974903844364821;
out[82] = -184.75059629806526;
out[83] = -94.61510234445565;
out[84] = 69.368640085185305;
out[85] = 17.974903844364821;
out[86] = -1.0;
out[87] = 40.748687841669579;
out[88] = -43.675799641021101;
out[89] = 17.974903844364821;
out[90] = -69.368640085185305;
out[91] = -94.61510234445565;
out[92] = 184.75059629806526;
out[93] = 17.974903844364821;
out[94] = -69.368640085185305;
out[95] = -40.748687841669579;
out[96] = 43.675799641021101;
out[97] = -17.974903844364821;
out[98] = 69.368640085185305;
out[99] = 94.61510234445565;
out[100] = -184.75059629806526;
out[101] = -17.974903844364821;
out[102] = 69.368640085185305;
out[103] = -1.0;
out[104] = -69.368640085185305;
out[105] = -17.974903844364825;
out[106] = -43.675799641021079;
out[107] = -40.74868784166955;
out[108] = -69.368640085185291;
out[109] = -17.974903844364839;
out[110] = 184.7505962980652;
out[111] = 94.61510234445565;
out[112] = 69.368640085185305;
out[113] = 17.974903844364825;
out[114] = 43.675799641021079;
out[115] = 40.74868784166955;
out[116] = 69.368640085185291;
out[117] = 17.974903844364839;
out[118] = -184.7505962980652;
out[119] = -94.61510234445565;
out[120] = -1.0;
out[121] = 17.974903844364825;
out[122] = -69.368640085185305;
out[123] = 40.74868784166955;
out[124] = -43.675799641021079;
out[125] = 17.974903844364839;
out[126] = -69.368640085185291;
out[127] = -94.61510234445565;
out[128] = 184.7505962980652;
out[129] = -17.974903844364825;
out[130] = 69.368640085185305;
out[131] = -40.74868784166955;
out[132] = 43.675799641021079;
out[133] = -17.974903844364839;
out[134] = 69.368640085185291;
out[135] = 94.61510234445565;
out[136] = -184.7505962980652;
out[137] = -1.0;
out[138] = -184.75059629806526;
out[139] = -94.615102344455664;
out[140] = 69.368640085185305;
out[141] = 17.974903844364835;
out[142] = 43.675799641021079;
out[143] = 40.748687841669557;
out[144] = 69.368640085185291;
out[145] = 17.974903844364821;
out[146] = 184.75059629806526;
out[147] = 94.615102344455664;
out[148] = -69.368640085185305;
out[149] = -17.974903844364835;
out[150] = -43.675799641021079;
out[151] = -40.748687841669557;
out[152] = -69.368640085185291;
out[153] = -17.974903844364821;
out[154] = -1.0;
out[155] = 94.615102344455664;
out[156] = -184.75059629806526;
out[157] = -17.974903844364835;
out[158] = 69.368640085185305;
out[159] = -40.748687841669557;
out[160] = 43.675799641021079;
out[161] = -17.974903844364821;
out[162] = 69.368640085185291;
out[163] = -94.615102344455664;
out[164] = 184.75059629806526;
out[165] = 17.974903844364835;
out[166] = -69.368640085185305;
out[167] = 40.748687841669557;
out[168] = -43.675799641021079;
out[169] = 17.974903844364821;
out[170] = -69.368640085185291;
out[171] = -1.0;
out[172] = 69.368640085185291;
out[173] = 17.974903844364825;
out[174] = -184.7505962980652;
out[175] = -94.615102344455636;
out[176] = 69.368640085185319;
out[177] = 17.97490384436481;
out[178] = 43.675799641021079;
out[179] = 40.748687841669565;
out[180] = -69.368640085185291;
out[181] = -17.974903844364825;
out[182] = 184.7505962980652;
out[183] = 94.615102344455636;
out[184] = -69.368640085185319;
out[185] = -17.97490384436481;
out[186] = -43.675799641021079;
out[187] = -40.748687841669565;
out[188] = -1.0;
out[189] = -17.974903844364825;
out[190] = 69.368640085185291;
out[191] = 94.615102344455636;
out[192] = -184.7505962980652;
out[193] = -17.97490384436481;
out[194] = 69.368640085185319;
out[195] = -40.748687841669565;
out[196] = 43.675799641021079;
out[197] = 17.974903844364825;
out[198] = -69.368640085185291;
out[199] = -94.615102344455636;
out[200] = 184.7505962980652;
out[201] = 17.97490384436481;
out[202] = -69.368640085185319;
out[203] = 40.748687841669565;
out[204] = -43.675799641021079;
out[205] = -1.0;
out[206] = 43.675799641021101;
out[207] = 40.748687841669579;
out[208] = 69.368640085185305;
out[209] = 17.974903844364821;
out[210] = -184.75059629806526;
out[211] = -94.61510234445565;
out[212] = 69.368640085185305;
out[213] = 17.974903844364821;
out[214] = -43.675799641021101;
out[215] = -40.748687841669579;
out[216] = -69.368640085185305;
out[217] = -17.974903844364821;
out[218] = 184.75059629806526;
out[219] = 94.61510234445565;
out[220] = -69.368640085185305;
out[221] = -17.974903844364821;
out[222] = -1.0;
out[223] = -40.748687841669579;
out[224] = 43.675799641021101;
out[225] = -17.974903844364821;
out[226] = 69.368640085185305;
out[227] = 94.61510234445565;
out[228] = -184.75059629806526;
out[229] = -17.974903844364821;
out[230] = 69.368640085185305;
out[231] = 40.748687841669579;
out[232] = -43.675799641021101;
out[233] = 17.974903844364821;
out[234] = -69.368640085185305;
out[235] = -94.61510234445565;
out[236] = 184.75059629806526;
out[237] = 17.974903844364821;
out[238] = -69.368640085185305;
out[239] = -1.0;
out[240] = 69.368640085185305;
out[241] = 17.974903844364825;
out[242] = 43.675799641021079;
out[243] = 40.74868784166955;
out[244] = 69.368640085185291;
out[245] = 17.974903844364839;
out[246] = -184.7505962980652;
out[247] = -94.61510234445565;
out[248] = -69.368640085185305;
out[249] = -17.974903844364825;
out[250] = -43.675799641021079;
out[251] = -40.74868784166955;
out[252] = -69.368640085185291;
out[253] = -17.974903844364839;
out[254] = 184.7505962980652;
out[255] = 94.61510234445565;
out[256] = -1.0;
out[257] = -17.974903844364825;
out[258] = 69.368640085185305;
out[259] = -40.74868784166955;
out[260] = 43.675799641021079;
out[261] = -17.974903844364839;
out[262] = 69.368640085185291;
out[263] = 94.61510234445565;
out[264] = -184.7505962980652;
out[265] = 17.974903844364825;
out[266] = -69.368640085185305;
out[267] = 40.74868784166955;
out[268] = -43.675799641021079;
out[269] = 17.974903844364839;
out[270] = -69.368640085185291;
out[271] = -94.61510234445565;
out[272] = 184.7505962980652;
out[273] = -1.0;
out[274] = -1.0;
out[277] = -1.0;
out[280] = -1.0;
out[283] = -1.0;
out[286] = -1.0;
out[289] = -1.0;
out[292] = -1.0;
out[295] = -1.0;
out[298] = -1.0;
out[301] = -1.0;
out[304] = -1.0;
out[307] = -1.0;
out[310] = -1.0;
out[313] = -1.0;
out[316] = -1.0;
out[319] = -1.0;
out[322] = -1.0;
out[325] = -1.0;

}

void de_jac_run_xy_eval(double *out,double *x,double *y,double *u,double *p,double Dt){


}

void de_jac_run_up_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[0] = -p[14];
out[612] = -p[1];
out[616] = 1.0*p[0];
out[648] = -p[1];
out[652] = 1.0*p[0];
out[684] = -p[1];
out[688] = 1.0*p[0];
out[720] = -p[3];
out[724] = 1.0*p[2];
out[752] = -1.0*p[0];
out[756] = -1.0*p[1];
out[788] = -1.0*p[0];
out[792] = -1.0*p[1];
out[824] = -1.0*p[0];
out[828] = -1.0*p[1];
out[860] = -1.0*p[2];
out[864] = -1.0*p[3];
out[900] = -p[7];
out[904] = 1.0*p[6];
out[936] = -p[7];
out[940] = 1.0*p[6];
out[972] = -p[7];
out[976] = 1.0*p[6];
out[1008] = -p[9];
out[1012] = 1.0*p[8];
out[1040] = -1.0*p[6];
out[1044] = -1.0*p[7];
out[1076] = -1.0*p[6];
out[1080] = -1.0*p[7];
out[1112] = -1.0*p[6];
out[1116] = -1.0*p[7];
out[1148] = -1.0*p[8];
out[1152] = -1.0*p[9];
out[1190] = p[13];
out[1223] = -p[12];

}

void de_jac_run_num_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[33] = -1.0;
out[36] = 184.75059629806526;
out[37] = 94.615102344455664;
out[38] = -69.368640085185305;
out[39] = -17.974903844364835;
out[40] = -43.675799641021079;
out[41] = -40.748687841669557;
out[42] = -69.368640085185291;
out[43] = -17.974903844364821;
out[44] = -184.75059629806526;
out[45] = -94.615102344455664;
out[46] = 69.368640085185305;
out[47] = 17.974903844364835;
out[48] = 43.675799641021079;
out[49] = 40.748687841669557;
out[50] = 69.368640085185291;
out[51] = 17.974903844364821;
out[52] = -1.0;
out[71] = -94.615102344455664;
out[72] = 184.75059629806526;
out[73] = 17.974903844364835;
out[74] = -69.368640085185305;
out[75] = 40.748687841669557;
out[76] = -43.675799641021079;
out[77] = 17.974903844364821;
out[78] = -69.368640085185291;
out[79] = 94.615102344455664;
out[80] = -184.75059629806526;
out[81] = -17.974903844364835;
out[82] = 69.368640085185305;
out[83] = -40.748687841669557;
out[84] = 43.675799641021079;
out[85] = -17.974903844364821;
out[86] = 69.368640085185291;
out[91] = -1.0;
out[106] = -69.368640085185291;
out[107] = -17.974903844364825;
out[108] = 184.7505962980652;
out[109] = 94.615102344455636;
out[110] = -69.368640085185319;
out[111] = -17.97490384436481;
out[112] = -43.675799641021079;
out[113] = -40.748687841669565;
out[114] = 69.368640085185291;
out[115] = 17.974903844364825;
out[116] = -184.7505962980652;
out[117] = -94.615102344455636;
out[118] = 69.368640085185319;
out[119] = 17.97490384436481;
out[120] = 43.675799641021079;
out[121] = 40.748687841669565;
out[123] = -1.0;
out[141] = 17.974903844364825;
out[142] = -69.368640085185291;
out[143] = -94.615102344455636;
out[144] = 184.7505962980652;
out[145] = 17.97490384436481;
out[146] = -69.368640085185319;
out[147] = 40.748687841669565;
out[148] = -43.675799641021079;
out[149] = -17.974903844364825;
out[150] = 69.368640085185291;
out[151] = 94.615102344455636;
out[152] = -184.7505962980652;
out[153] = -17.97490384436481;
out[154] = 69.368640085185319;
out[155] = -40.748687841669565;
out[156] = 43.675799641021079;
out[162] = -1.0;
out[176] = -43.675799641021101;
out[177] = -40.748687841669579;
out[178] = -69.368640085185305;
out[179] = -17.974903844364821;
out[180] = 184.75059629806526;
out[181] = 94.61510234445565;
out[182] = -69.368640085185305;
out[183] = -17.974903844364821;
out[184] = 43.675799641021101;
out[185] = 40.748687841669579;
out[186] = 69.368640085185305;
out[187] = 17.974903844364821;
out[188] = -184.75059629806526;
out[189] = -94.61510234445565;
out[190] = 69.368640085185305;
out[191] = 17.974903844364821;
out[194] = -1.0;
out[211] = 40.748687841669579;
out[212] = -43.675799641021101;
out[213] = 17.974903844364821;
out[214] = -69.368640085185305;
out[215] = -94.61510234445565;
out[216] = 184.75059629806526;
out[217] = 17.974903844364821;
out[218] = -69.368640085185305;
out[219] = -40.748687841669579;
out[220] = 43.675799641021101;
out[221] = -17.974903844364821;
out[222] = 69.368640085185305;
out[223] = 94.61510234445565;
out[224] = -184.75059629806526;
out[225] = -17.974903844364821;
out[226] = 69.368640085185305;
out[233] = -1.0;
out[246] = -69.368640085185305;
out[247] = -17.974903844364825;
out[248] = -43.675799641021079;
out[249] = -40.74868784166955;
out[250] = -69.368640085185291;
out[251] = -17.974903844364839;
out[252] = 184.7505962980652;
out[253] = 94.61510234445565;
out[254] = 69.368640085185305;
out[255] = 17.974903844364825;
out[256] = 43.675799641021079;
out[257] = 40.74868784166955;
out[258] = 69.368640085185291;
out[259] = 17.974903844364839;
out[260] = -184.7505962980652;
out[261] = -94.61510234445565;
out[265] = -1.0;
out[281] = 17.974903844364825;
out[282] = -69.368640085185305;
out[283] = 40.74868784166955;
out[284] = -43.675799641021079;
out[285] = 17.974903844364839;
out[286] = -69.368640085185291;
out[287] = -94.61510234445565;
out[288] = 184.7505962980652;
out[289] = -17.974903844364825;
out[290] = 69.368640085185305;
out[291] = -40.74868784166955;
out[292] = 43.675799641021079;
out[293] = -17.974903844364839;
out[294] = 69.368640085185291;
out[295] = 94.61510234445565;
out[296] = -184.7505962980652;
out[304] = -1.0;
out[316] = -184.75059629806526;
out[317] = -94.615102344455664;
out[318] = 69.368640085185305;
out[319] = 17.974903844364835;
out[320] = 43.675799641021079;
out[321] = 40.748687841669557;
out[322] = 69.368640085185291;
out[323] = 17.974903844364821;
out[324] = 184.75059629806526;
out[325] = 94.615102344455664;
out[326] = -69.368640085185305;
out[327] = -17.974903844364835;
out[328] = -43.675799641021079;
out[329] = -40.748687841669557;
out[330] = -69.368640085185291;
out[331] = -17.974903844364821;
out[340] = -1.0;
out[351] = 94.615102344455664;
out[352] = -184.75059629806526;
out[353] = -17.974903844364835;
out[354] = 69.368640085185305;
out[355] = -40.748687841669557;
out[356] = 43.675799641021079;
out[357] = -17.974903844364821;
out[358] = 69.368640085185291;
out[359] = -94.615102344455664;
out[360] = 184.75059629806526;
out[361] = 17.974903844364835;
out[362] = -69.368640085185305;
out[363] = 40.748687841669557;
out[364] = -43.675799641021079;
out[365] = 17.974903844364821;
out[366] = -69.368640085185291;
out[379] = -1.0;
out[386] = 69.368640085185291;
out[387] = 17.974903844364825;
out[388] = -184.7505962980652;
out[389] = -94.615102344455636;
out[390] = 69.368640085185319;
out[391] = 17.97490384436481;
out[392] = 43.675799641021079;
out[393] = 40.748687841669565;
out[394] = -69.368640085185291;
out[395] = -17.974903844364825;
out[396] = 184.7505962980652;
out[397] = 94.615102344455636;
out[398] = -69.368640085185319;
out[399] = -17.97490384436481;
out[400] = -43.675799641021079;
out[401] = -40.748687841669565;
out[411] = -1.0;
out[421] = -17.974903844364825;
out[422] = 69.368640085185291;
out[423] = 94.615102344455636;
out[424] = -184.7505962980652;
out[425] = -17.97490384436481;
out[426] = 69.368640085185319;
out[427] = -40.748687841669565;
out[428] = 43.675799641021079;
out[429] = 17.974903844364825;
out[430] = -69.368640085185291;
out[431] = -94.615102344455636;
out[432] = 184.7505962980652;
out[433] = 17.97490384436481;
out[434] = -69.368640085185319;
out[435] = 40.748687841669565;
out[436] = -43.675799641021079;
out[450] = -1.0;
out[456] = 43.675799641021101;
out[457] = 40.748687841669579;
out[458] = 69.368640085185305;
out[459] = 17.974903844364821;
out[460] = -184.75059629806526;
out[461] = -94.61510234445565;
out[462] = 69.368640085185305;
out[463] = 17.974903844364821;
out[464] = -43.675799641021101;
out[465] = -40.748687841669579;
out[466] = -69.368640085185305;
out[467] = -17.974903844364821;
out[468] = 184.75059629806526;
out[469] = 94.61510234445565;
out[470] = -69.368640085185305;
out[471] = -17.974903844364821;
out[482] = -1.0;
out[491] = -40.748687841669579;
out[492] = 43.675799641021101;
out[493] = -17.974903844364821;
out[494] = 69.368640085185305;
out[495] = 94.61510234445565;
out[496] = -184.75059629806526;
out[497] = -17.974903844364821;
out[498] = 69.368640085185305;
out[499] = 40.748687841669579;
out[500] = -43.675799641021101;
out[501] = 17.974903844364821;
out[502] = -69.368640085185305;
out[503] = -94.61510234445565;
out[504] = 184.75059629806526;
out[505] = 17.974903844364821;
out[506] = -69.368640085185305;
out[521] = -1.0;
out[526] = 69.368640085185305;
out[527] = 17.974903844364825;
out[528] = 43.675799641021079;
out[529] = 40.74868784166955;
out[530] = 69.368640085185291;
out[531] = 17.974903844364839;
out[532] = -184.7505962980652;
out[533] = -94.61510234445565;
out[534] = -69.368640085185305;
out[535] = -17.974903844364825;
out[536] = -43.675799641021079;
out[537] = -40.74868784166955;
out[538] = -69.368640085185291;
out[539] = -17.974903844364839;
out[540] = 184.7505962980652;
out[541] = 94.61510234445565;
out[553] = -1.0;
out[561] = -17.974903844364825;
out[562] = 69.368640085185305;
out[563] = -40.74868784166955;
out[564] = 43.675799641021079;
out[565] = -17.974903844364839;
out[566] = 69.368640085185291;
out[567] = 94.61510234445565;
out[568] = -184.7505962980652;
out[569] = 17.974903844364825;
out[570] = -69.368640085185305;
out[571] = 40.74868784166955;
out[572] = -43.675799641021079;
out[573] = 17.974903844364839;
out[574] = -69.368640085185291;
out[575] = -94.61510234445565;
out[576] = 184.7505962980652;
out[592] = -1.0;
out[596] = -1.0;
out[633] = -1.0;
out[670] = -1.0;
out[707] = -1.0;
out[737] = -1.0;
out[774] = -1.0;
out[811] = -1.0;
out[848] = -1.0;
out[884] = -1.0;
out[921] = -1.0;
out[958] = -1.0;
out[995] = -1.0;
out[1025] = -1.0;
out[1062] = -1.0;
out[1099] = -1.0;
out[1136] = -1.0;
out[1188] = -1.0;
out[1224] = -1.0;

}

void sp_jac_run_xy_eval(double *out,double *x,double *y,double *u,double *p,double Dt){


}

void sp_jac_run_up_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[0] = -p[14];
out[275] = -p[1];
out[276] = 1.0*p[0];
out[278] = -p[1];
out[279] = 1.0*p[0];
out[281] = -p[1];
out[282] = 1.0*p[0];
out[284] = -p[3];
out[285] = 1.0*p[2];
out[287] = -1.0*p[0];
out[288] = -1.0*p[1];
out[290] = -1.0*p[0];
out[291] = -1.0*p[1];
out[293] = -1.0*p[0];
out[294] = -1.0*p[1];
out[296] = -1.0*p[2];
out[297] = -1.0*p[3];
out[299] = -p[7];
out[300] = 1.0*p[6];
out[302] = -p[7];
out[303] = 1.0*p[6];
out[305] = -p[7];
out[306] = 1.0*p[6];
out[308] = -p[9];
out[309] = 1.0*p[8];
out[311] = -1.0*p[6];
out[312] = -1.0*p[7];
out[314] = -1.0*p[6];
out[315] = -1.0*p[7];
out[317] = -1.0*p[6];
out[318] = -1.0*p[7];
out[320] = -1.0*p[8];
out[321] = -1.0*p[9];
out[323] = p[13];
out[324] = -p[12];

}

void sp_jac_run_num_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[1] = -1.0;
out[2] = 184.75059629806526;
out[3] = 94.615102344455664;
out[4] = -69.368640085185305;
out[5] = -17.974903844364835;
out[6] = -43.675799641021079;
out[7] = -40.748687841669557;
out[8] = -69.368640085185291;
out[9] = -17.974903844364821;
out[10] = -184.75059629806526;
out[11] = -94.615102344455664;
out[12] = 69.368640085185305;
out[13] = 17.974903844364835;
out[14] = 43.675799641021079;
out[15] = 40.748687841669557;
out[16] = 69.368640085185291;
out[17] = 17.974903844364821;
out[18] = -1.0;
out[19] = -94.615102344455664;
out[20] = 184.75059629806526;
out[21] = 17.974903844364835;
out[22] = -69.368640085185305;
out[23] = 40.748687841669557;
out[24] = -43.675799641021079;
out[25] = 17.974903844364821;
out[26] = -69.368640085185291;
out[27] = 94.615102344455664;
out[28] = -184.75059629806526;
out[29] = -17.974903844364835;
out[30] = 69.368640085185305;
out[31] = -40.748687841669557;
out[32] = 43.675799641021079;
out[33] = -17.974903844364821;
out[34] = 69.368640085185291;
out[35] = -1.0;
out[36] = -69.368640085185291;
out[37] = -17.974903844364825;
out[38] = 184.7505962980652;
out[39] = 94.615102344455636;
out[40] = -69.368640085185319;
out[41] = -17.97490384436481;
out[42] = -43.675799641021079;
out[43] = -40.748687841669565;
out[44] = 69.368640085185291;
out[45] = 17.974903844364825;
out[46] = -184.7505962980652;
out[47] = -94.615102344455636;
out[48] = 69.368640085185319;
out[49] = 17.97490384436481;
out[50] = 43.675799641021079;
out[51] = 40.748687841669565;
out[52] = -1.0;
out[53] = 17.974903844364825;
out[54] = -69.368640085185291;
out[55] = -94.615102344455636;
out[56] = 184.7505962980652;
out[57] = 17.97490384436481;
out[58] = -69.368640085185319;
out[59] = 40.748687841669565;
out[60] = -43.675799641021079;
out[61] = -17.974903844364825;
out[62] = 69.368640085185291;
out[63] = 94.615102344455636;
out[64] = -184.7505962980652;
out[65] = -17.97490384436481;
out[66] = 69.368640085185319;
out[67] = -40.748687841669565;
out[68] = 43.675799641021079;
out[69] = -1.0;
out[70] = -43.675799641021101;
out[71] = -40.748687841669579;
out[72] = -69.368640085185305;
out[73] = -17.974903844364821;
out[74] = 184.75059629806526;
out[75] = 94.61510234445565;
out[76] = -69.368640085185305;
out[77] = -17.974903844364821;
out[78] = 43.675799641021101;
out[79] = 40.748687841669579;
out[80] = 69.368640085185305;
out[81] = 17.974903844364821;
out[82] = -184.75059629806526;
out[83] = -94.61510234445565;
out[84] = 69.368640085185305;
out[85] = 17.974903844364821;
out[86] = -1.0;
out[87] = 40.748687841669579;
out[88] = -43.675799641021101;
out[89] = 17.974903844364821;
out[90] = -69.368640085185305;
out[91] = -94.61510234445565;
out[92] = 184.75059629806526;
out[93] = 17.974903844364821;
out[94] = -69.368640085185305;
out[95] = -40.748687841669579;
out[96] = 43.675799641021101;
out[97] = -17.974903844364821;
out[98] = 69.368640085185305;
out[99] = 94.61510234445565;
out[100] = -184.75059629806526;
out[101] = -17.974903844364821;
out[102] = 69.368640085185305;
out[103] = -1.0;
out[104] = -69.368640085185305;
out[105] = -17.974903844364825;
out[106] = -43.675799641021079;
out[107] = -40.74868784166955;
out[108] = -69.368640085185291;
out[109] = -17.974903844364839;
out[110] = 184.7505962980652;
out[111] = 94.61510234445565;
out[112] = 69.368640085185305;
out[113] = 17.974903844364825;
out[114] = 43.675799641021079;
out[115] = 40.74868784166955;
out[116] = 69.368640085185291;
out[117] = 17.974903844364839;
out[118] = -184.7505962980652;
out[119] = -94.61510234445565;
out[120] = -1.0;
out[121] = 17.974903844364825;
out[122] = -69.368640085185305;
out[123] = 40.74868784166955;
out[124] = -43.675799641021079;
out[125] = 17.974903844364839;
out[126] = -69.368640085185291;
out[127] = -94.61510234445565;
out[128] = 184.7505962980652;
out[129] = -17.974903844364825;
out[130] = 69.368640085185305;
out[131] = -40.74868784166955;
out[132] = 43.675799641021079;
out[133] = -17.974903844364839;
out[134] = 69.368640085185291;
out[135] = 94.61510234445565;
out[136] = -184.7505962980652;
out[137] = -1.0;
out[138] = -184.75059629806526;
out[139] = -94.615102344455664;
out[140] = 69.368640085185305;
out[141] = 17.974903844364835;
out[142] = 43.675799641021079;
out[143] = 40.748687841669557;
out[144] = 69.368640085185291;
out[145] = 17.974903844364821;
out[146] = 184.75059629806526;
out[147] = 94.615102344455664;
out[148] = -69.368640085185305;
out[149] = -17.974903844364835;
out[150] = -43.675799641021079;
out[151] = -40.748687841669557;
out[152] = -69.368640085185291;
out[153] = -17.974903844364821;
out[154] = -1.0;
out[155] = 94.615102344455664;
out[156] = -184.75059629806526;
out[157] = -17.974903844364835;
out[158] = 69.368640085185305;
out[159] = -40.748687841669557;
out[160] = 43.675799641021079;
out[161] = -17.974903844364821;
out[162] = 69.368640085185291;
out[163] = -94.615102344455664;
out[164] = 184.75059629806526;
out[165] = 17.974903844364835;
out[166] = -69.368640085185305;
out[167] = 40.748687841669557;
out[168] = -43.675799641021079;
out[169] = 17.974903844364821;
out[170] = -69.368640085185291;
out[171] = -1.0;
out[172] = 69.368640085185291;
out[173] = 17.974903844364825;
out[174] = -184.7505962980652;
out[175] = -94.615102344455636;
out[176] = 69.368640085185319;
out[177] = 17.97490384436481;
out[178] = 43.675799641021079;
out[179] = 40.748687841669565;
out[180] = -69.368640085185291;
out[181] = -17.974903844364825;
out[182] = 184.7505962980652;
out[183] = 94.615102344455636;
out[184] = -69.368640085185319;
out[185] = -17.97490384436481;
out[186] = -43.675799641021079;
out[187] = -40.748687841669565;
out[188] = -1.0;
out[189] = -17.974903844364825;
out[190] = 69.368640085185291;
out[191] = 94.615102344455636;
out[192] = -184.7505962980652;
out[193] = -17.97490384436481;
out[194] = 69.368640085185319;
out[195] = -40.748687841669565;
out[196] = 43.675799641021079;
out[197] = 17.974903844364825;
out[198] = -69.368640085185291;
out[199] = -94.615102344455636;
out[200] = 184.7505962980652;
out[201] = 17.97490384436481;
out[202] = -69.368640085185319;
out[203] = 40.748687841669565;
out[204] = -43.675799641021079;
out[205] = -1.0;
out[206] = 43.675799641021101;
out[207] = 40.748687841669579;
out[208] = 69.368640085185305;
out[209] = 17.974903844364821;
out[210] = -184.75059629806526;
out[211] = -94.61510234445565;
out[212] = 69.368640085185305;
out[213] = 17.974903844364821;
out[214] = -43.675799641021101;
out[215] = -40.748687841669579;
out[216] = -69.368640085185305;
out[217] = -17.974903844364821;
out[218] = 184.75059629806526;
out[219] = 94.61510234445565;
out[220] = -69.368640085185305;
out[221] = -17.974903844364821;
out[222] = -1.0;
out[223] = -40.748687841669579;
out[224] = 43.675799641021101;
out[225] = -17.974903844364821;
out[226] = 69.368640085185305;
out[227] = 94.61510234445565;
out[228] = -184.75059629806526;
out[229] = -17.974903844364821;
out[230] = 69.368640085185305;
out[231] = 40.748687841669579;
out[232] = -43.675799641021101;
out[233] = 17.974903844364821;
out[234] = -69.368640085185305;
out[235] = -94.61510234445565;
out[236] = 184.75059629806526;
out[237] = 17.974903844364821;
out[238] = -69.368640085185305;
out[239] = -1.0;
out[240] = 69.368640085185305;
out[241] = 17.974903844364825;
out[242] = 43.675799641021079;
out[243] = 40.74868784166955;
out[244] = 69.368640085185291;
out[245] = 17.974903844364839;
out[246] = -184.7505962980652;
out[247] = -94.61510234445565;
out[248] = -69.368640085185305;
out[249] = -17.974903844364825;
out[250] = -43.675799641021079;
out[251] = -40.74868784166955;
out[252] = -69.368640085185291;
out[253] = -17.974903844364839;
out[254] = 184.7505962980652;
out[255] = 94.61510234445565;
out[256] = -1.0;
out[257] = -17.974903844364825;
out[258] = 69.368640085185305;
out[259] = -40.74868784166955;
out[260] = 43.675799641021079;
out[261] = -17.974903844364839;
out[262] = 69.368640085185291;
out[263] = 94.61510234445565;
out[264] = -184.7505962980652;
out[265] = 17.974903844364825;
out[266] = -69.368640085185305;
out[267] = 40.74868784166955;
out[268] = -43.675799641021079;
out[269] = 17.974903844364839;
out[270] = -69.368640085185291;
out[271] = -94.61510234445565;
out[272] = 184.7505962980652;
out[273] = -1.0;
out[274] = -1.0;
out[277] = -1.0;
out[280] = -1.0;
out[283] = -1.0;
out[286] = -1.0;
out[289] = -1.0;
out[292] = -1.0;
out[295] = -1.0;
out[298] = -1.0;
out[301] = -1.0;
out[304] = -1.0;
out[307] = -1.0;
out[310] = -1.0;
out[313] = -1.0;
out[316] = -1.0;
out[319] = -1.0;
out[322] = -1.0;
out[325] = -1.0;

}

void de_jac_trap_xy_eval(double *out,double *x,double *y,double *u,double *p,double Dt){


}

void de_jac_trap_up_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[0] = 0.5*Dt*p[14] + 1.0;
out[33] = 0.5*Dt;
out[612] = -p[1];
out[616] = 1.0*p[0];
out[648] = -p[1];
out[652] = 1.0*p[0];
out[684] = -p[1];
out[688] = 1.0*p[0];
out[720] = -p[3];
out[724] = 1.0*p[2];
out[752] = -1.0*p[0];
out[756] = -1.0*p[1];
out[788] = -1.0*p[0];
out[792] = -1.0*p[1];
out[824] = -1.0*p[0];
out[828] = -1.0*p[1];
out[860] = -1.0*p[2];
out[864] = -1.0*p[3];
out[900] = -p[7];
out[904] = 1.0*p[6];
out[936] = -p[7];
out[940] = 1.0*p[6];
out[972] = -p[7];
out[976] = 1.0*p[6];
out[1008] = -p[9];
out[1012] = 1.0*p[8];
out[1040] = -1.0*p[6];
out[1044] = -1.0*p[7];
out[1076] = -1.0*p[6];
out[1080] = -1.0*p[7];
out[1112] = -1.0*p[6];
out[1116] = -1.0*p[7];
out[1148] = -1.0*p[8];
out[1152] = -1.0*p[9];
out[1190] = p[13];
out[1223] = -p[12];

}

void de_jac_trap_num_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[36] = 184.75059629806526;
out[37] = 94.615102344455664;
out[38] = -69.368640085185305;
out[39] = -17.974903844364835;
out[40] = -43.675799641021079;
out[41] = -40.748687841669557;
out[42] = -69.368640085185291;
out[43] = -17.974903844364821;
out[44] = -184.75059629806526;
out[45] = -94.615102344455664;
out[46] = 69.368640085185305;
out[47] = 17.974903844364835;
out[48] = 43.675799641021079;
out[49] = 40.748687841669557;
out[50] = 69.368640085185291;
out[51] = 17.974903844364821;
out[52] = -1.0;
out[71] = -94.615102344455664;
out[72] = 184.75059629806526;
out[73] = 17.974903844364835;
out[74] = -69.368640085185305;
out[75] = 40.748687841669557;
out[76] = -43.675799641021079;
out[77] = 17.974903844364821;
out[78] = -69.368640085185291;
out[79] = 94.615102344455664;
out[80] = -184.75059629806526;
out[81] = -17.974903844364835;
out[82] = 69.368640085185305;
out[83] = -40.748687841669557;
out[84] = 43.675799641021079;
out[85] = -17.974903844364821;
out[86] = 69.368640085185291;
out[91] = -1.0;
out[106] = -69.368640085185291;
out[107] = -17.974903844364825;
out[108] = 184.7505962980652;
out[109] = 94.615102344455636;
out[110] = -69.368640085185319;
out[111] = -17.97490384436481;
out[112] = -43.675799641021079;
out[113] = -40.748687841669565;
out[114] = 69.368640085185291;
out[115] = 17.974903844364825;
out[116] = -184.7505962980652;
out[117] = -94.615102344455636;
out[118] = 69.368640085185319;
out[119] = 17.97490384436481;
out[120] = 43.675799641021079;
out[121] = 40.748687841669565;
out[123] = -1.0;
out[141] = 17.974903844364825;
out[142] = -69.368640085185291;
out[143] = -94.615102344455636;
out[144] = 184.7505962980652;
out[145] = 17.97490384436481;
out[146] = -69.368640085185319;
out[147] = 40.748687841669565;
out[148] = -43.675799641021079;
out[149] = -17.974903844364825;
out[150] = 69.368640085185291;
out[151] = 94.615102344455636;
out[152] = -184.7505962980652;
out[153] = -17.97490384436481;
out[154] = 69.368640085185319;
out[155] = -40.748687841669565;
out[156] = 43.675799641021079;
out[162] = -1.0;
out[176] = -43.675799641021101;
out[177] = -40.748687841669579;
out[178] = -69.368640085185305;
out[179] = -17.974903844364821;
out[180] = 184.75059629806526;
out[181] = 94.61510234445565;
out[182] = -69.368640085185305;
out[183] = -17.974903844364821;
out[184] = 43.675799641021101;
out[185] = 40.748687841669579;
out[186] = 69.368640085185305;
out[187] = 17.974903844364821;
out[188] = -184.75059629806526;
out[189] = -94.61510234445565;
out[190] = 69.368640085185305;
out[191] = 17.974903844364821;
out[194] = -1.0;
out[211] = 40.748687841669579;
out[212] = -43.675799641021101;
out[213] = 17.974903844364821;
out[214] = -69.368640085185305;
out[215] = -94.61510234445565;
out[216] = 184.75059629806526;
out[217] = 17.974903844364821;
out[218] = -69.368640085185305;
out[219] = -40.748687841669579;
out[220] = 43.675799641021101;
out[221] = -17.974903844364821;
out[222] = 69.368640085185305;
out[223] = 94.61510234445565;
out[224] = -184.75059629806526;
out[225] = -17.974903844364821;
out[226] = 69.368640085185305;
out[233] = -1.0;
out[246] = -69.368640085185305;
out[247] = -17.974903844364825;
out[248] = -43.675799641021079;
out[249] = -40.74868784166955;
out[250] = -69.368640085185291;
out[251] = -17.974903844364839;
out[252] = 184.7505962980652;
out[253] = 94.61510234445565;
out[254] = 69.368640085185305;
out[255] = 17.974903844364825;
out[256] = 43.675799641021079;
out[257] = 40.74868784166955;
out[258] = 69.368640085185291;
out[259] = 17.974903844364839;
out[260] = -184.7505962980652;
out[261] = -94.61510234445565;
out[265] = -1.0;
out[281] = 17.974903844364825;
out[282] = -69.368640085185305;
out[283] = 40.74868784166955;
out[284] = -43.675799641021079;
out[285] = 17.974903844364839;
out[286] = -69.368640085185291;
out[287] = -94.61510234445565;
out[288] = 184.7505962980652;
out[289] = -17.974903844364825;
out[290] = 69.368640085185305;
out[291] = -40.74868784166955;
out[292] = 43.675799641021079;
out[293] = -17.974903844364839;
out[294] = 69.368640085185291;
out[295] = 94.61510234445565;
out[296] = -184.7505962980652;
out[304] = -1.0;
out[316] = -184.75059629806526;
out[317] = -94.615102344455664;
out[318] = 69.368640085185305;
out[319] = 17.974903844364835;
out[320] = 43.675799641021079;
out[321] = 40.748687841669557;
out[322] = 69.368640085185291;
out[323] = 17.974903844364821;
out[324] = 184.75059629806526;
out[325] = 94.615102344455664;
out[326] = -69.368640085185305;
out[327] = -17.974903844364835;
out[328] = -43.675799641021079;
out[329] = -40.748687841669557;
out[330] = -69.368640085185291;
out[331] = -17.974903844364821;
out[340] = -1.0;
out[351] = 94.615102344455664;
out[352] = -184.75059629806526;
out[353] = -17.974903844364835;
out[354] = 69.368640085185305;
out[355] = -40.748687841669557;
out[356] = 43.675799641021079;
out[357] = -17.974903844364821;
out[358] = 69.368640085185291;
out[359] = -94.615102344455664;
out[360] = 184.75059629806526;
out[361] = 17.974903844364835;
out[362] = -69.368640085185305;
out[363] = 40.748687841669557;
out[364] = -43.675799641021079;
out[365] = 17.974903844364821;
out[366] = -69.368640085185291;
out[379] = -1.0;
out[386] = 69.368640085185291;
out[387] = 17.974903844364825;
out[388] = -184.7505962980652;
out[389] = -94.615102344455636;
out[390] = 69.368640085185319;
out[391] = 17.97490384436481;
out[392] = 43.675799641021079;
out[393] = 40.748687841669565;
out[394] = -69.368640085185291;
out[395] = -17.974903844364825;
out[396] = 184.7505962980652;
out[397] = 94.615102344455636;
out[398] = -69.368640085185319;
out[399] = -17.97490384436481;
out[400] = -43.675799641021079;
out[401] = -40.748687841669565;
out[411] = -1.0;
out[421] = -17.974903844364825;
out[422] = 69.368640085185291;
out[423] = 94.615102344455636;
out[424] = -184.7505962980652;
out[425] = -17.97490384436481;
out[426] = 69.368640085185319;
out[427] = -40.748687841669565;
out[428] = 43.675799641021079;
out[429] = 17.974903844364825;
out[430] = -69.368640085185291;
out[431] = -94.615102344455636;
out[432] = 184.7505962980652;
out[433] = 17.97490384436481;
out[434] = -69.368640085185319;
out[435] = 40.748687841669565;
out[436] = -43.675799641021079;
out[450] = -1.0;
out[456] = 43.675799641021101;
out[457] = 40.748687841669579;
out[458] = 69.368640085185305;
out[459] = 17.974903844364821;
out[460] = -184.75059629806526;
out[461] = -94.61510234445565;
out[462] = 69.368640085185305;
out[463] = 17.974903844364821;
out[464] = -43.675799641021101;
out[465] = -40.748687841669579;
out[466] = -69.368640085185305;
out[467] = -17.974903844364821;
out[468] = 184.75059629806526;
out[469] = 94.61510234445565;
out[470] = -69.368640085185305;
out[471] = -17.974903844364821;
out[482] = -1.0;
out[491] = -40.748687841669579;
out[492] = 43.675799641021101;
out[493] = -17.974903844364821;
out[494] = 69.368640085185305;
out[495] = 94.61510234445565;
out[496] = -184.75059629806526;
out[497] = -17.974903844364821;
out[498] = 69.368640085185305;
out[499] = 40.748687841669579;
out[500] = -43.675799641021101;
out[501] = 17.974903844364821;
out[502] = -69.368640085185305;
out[503] = -94.61510234445565;
out[504] = 184.75059629806526;
out[505] = 17.974903844364821;
out[506] = -69.368640085185305;
out[521] = -1.0;
out[526] = 69.368640085185305;
out[527] = 17.974903844364825;
out[528] = 43.675799641021079;
out[529] = 40.74868784166955;
out[530] = 69.368640085185291;
out[531] = 17.974903844364839;
out[532] = -184.7505962980652;
out[533] = -94.61510234445565;
out[534] = -69.368640085185305;
out[535] = -17.974903844364825;
out[536] = -43.675799641021079;
out[537] = -40.74868784166955;
out[538] = -69.368640085185291;
out[539] = -17.974903844364839;
out[540] = 184.7505962980652;
out[541] = 94.61510234445565;
out[553] = -1.0;
out[561] = -17.974903844364825;
out[562] = 69.368640085185305;
out[563] = -40.74868784166955;
out[564] = 43.675799641021079;
out[565] = -17.974903844364839;
out[566] = 69.368640085185291;
out[567] = 94.61510234445565;
out[568] = -184.7505962980652;
out[569] = 17.974903844364825;
out[570] = -69.368640085185305;
out[571] = 40.74868784166955;
out[572] = -43.675799641021079;
out[573] = 17.974903844364839;
out[574] = -69.368640085185291;
out[575] = -94.61510234445565;
out[576] = 184.7505962980652;
out[592] = -1.0;
out[596] = -1.0;
out[633] = -1.0;
out[670] = -1.0;
out[707] = -1.0;
out[737] = -1.0;
out[774] = -1.0;
out[811] = -1.0;
out[848] = -1.0;
out[884] = -1.0;
out[921] = -1.0;
out[958] = -1.0;
out[995] = -1.0;
out[1025] = -1.0;
out[1062] = -1.0;
out[1099] = -1.0;
out[1136] = -1.0;
out[1188] = -1.0;
out[1224] = -1.0;

}

void sp_jac_trap_xy_eval(double *out,double *x,double *y,double *u,double *p,double Dt){


}

void sp_jac_trap_up_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[0] = 0.5*Dt*p[14] + 1.0;
out[1] = 0.5*Dt;
out[275] = -p[1];
out[276] = 1.0*p[0];
out[278] = -p[1];
out[279] = 1.0*p[0];
out[281] = -p[1];
out[282] = 1.0*p[0];
out[284] = -p[3];
out[285] = 1.0*p[2];
out[287] = -1.0*p[0];
out[288] = -1.0*p[1];
out[290] = -1.0*p[0];
out[291] = -1.0*p[1];
out[293] = -1.0*p[0];
out[294] = -1.0*p[1];
out[296] = -1.0*p[2];
out[297] = -1.0*p[3];
out[299] = -p[7];
out[300] = 1.0*p[6];
out[302] = -p[7];
out[303] = 1.0*p[6];
out[305] = -p[7];
out[306] = 1.0*p[6];
out[308] = -p[9];
out[309] = 1.0*p[8];
out[311] = -1.0*p[6];
out[312] = -1.0*p[7];
out[314] = -1.0*p[6];
out[315] = -1.0*p[7];
out[317] = -1.0*p[6];
out[318] = -1.0*p[7];
out[320] = -1.0*p[8];
out[321] = -1.0*p[9];
out[323] = p[13];
out[324] = -p[12];

}

void sp_jac_trap_num_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[2] = 184.75059629806526;
out[3] = 94.615102344455664;
out[4] = -69.368640085185305;
out[5] = -17.974903844364835;
out[6] = -43.675799641021079;
out[7] = -40.748687841669557;
out[8] = -69.368640085185291;
out[9] = -17.974903844364821;
out[10] = -184.75059629806526;
out[11] = -94.615102344455664;
out[12] = 69.368640085185305;
out[13] = 17.974903844364835;
out[14] = 43.675799641021079;
out[15] = 40.748687841669557;
out[16] = 69.368640085185291;
out[17] = 17.974903844364821;
out[18] = -1.0;
out[19] = -94.615102344455664;
out[20] = 184.75059629806526;
out[21] = 17.974903844364835;
out[22] = -69.368640085185305;
out[23] = 40.748687841669557;
out[24] = -43.675799641021079;
out[25] = 17.974903844364821;
out[26] = -69.368640085185291;
out[27] = 94.615102344455664;
out[28] = -184.75059629806526;
out[29] = -17.974903844364835;
out[30] = 69.368640085185305;
out[31] = -40.748687841669557;
out[32] = 43.675799641021079;
out[33] = -17.974903844364821;
out[34] = 69.368640085185291;
out[35] = -1.0;
out[36] = -69.368640085185291;
out[37] = -17.974903844364825;
out[38] = 184.7505962980652;
out[39] = 94.615102344455636;
out[40] = -69.368640085185319;
out[41] = -17.97490384436481;
out[42] = -43.675799641021079;
out[43] = -40.748687841669565;
out[44] = 69.368640085185291;
out[45] = 17.974903844364825;
out[46] = -184.7505962980652;
out[47] = -94.615102344455636;
out[48] = 69.368640085185319;
out[49] = 17.97490384436481;
out[50] = 43.675799641021079;
out[51] = 40.748687841669565;
out[52] = -1.0;
out[53] = 17.974903844364825;
out[54] = -69.368640085185291;
out[55] = -94.615102344455636;
out[56] = 184.7505962980652;
out[57] = 17.97490384436481;
out[58] = -69.368640085185319;
out[59] = 40.748687841669565;
out[60] = -43.675799641021079;
out[61] = -17.974903844364825;
out[62] = 69.368640085185291;
out[63] = 94.615102344455636;
out[64] = -184.7505962980652;
out[65] = -17.97490384436481;
out[66] = 69.368640085185319;
out[67] = -40.748687841669565;
out[68] = 43.675799641021079;
out[69] = -1.0;
out[70] = -43.675799641021101;
out[71] = -40.748687841669579;
out[72] = -69.368640085185305;
out[73] = -17.974903844364821;
out[74] = 184.75059629806526;
out[75] = 94.61510234445565;
out[76] = -69.368640085185305;
out[77] = -17.974903844364821;
out[78] = 43.675799641021101;
out[79] = 40.748687841669579;
out[80] = 69.368640085185305;
out[81] = 17.974903844364821;
out[82] = -184.75059629806526;
out[83] = -94.61510234445565;
out[84] = 69.368640085185305;
out[85] = 17.974903844364821;
out[86] = -1.0;
out[87] = 40.748687841669579;
out[88] = -43.675799641021101;
out[89] = 17.974903844364821;
out[90] = -69.368640085185305;
out[91] = -94.61510234445565;
out[92] = 184.75059629806526;
out[93] = 17.974903844364821;
out[94] = -69.368640085185305;
out[95] = -40.748687841669579;
out[96] = 43.675799641021101;
out[97] = -17.974903844364821;
out[98] = 69.368640085185305;
out[99] = 94.61510234445565;
out[100] = -184.75059629806526;
out[101] = -17.974903844364821;
out[102] = 69.368640085185305;
out[103] = -1.0;
out[104] = -69.368640085185305;
out[105] = -17.974903844364825;
out[106] = -43.675799641021079;
out[107] = -40.74868784166955;
out[108] = -69.368640085185291;
out[109] = -17.974903844364839;
out[110] = 184.7505962980652;
out[111] = 94.61510234445565;
out[112] = 69.368640085185305;
out[113] = 17.974903844364825;
out[114] = 43.675799641021079;
out[115] = 40.74868784166955;
out[116] = 69.368640085185291;
out[117] = 17.974903844364839;
out[118] = -184.7505962980652;
out[119] = -94.61510234445565;
out[120] = -1.0;
out[121] = 17.974903844364825;
out[122] = -69.368640085185305;
out[123] = 40.74868784166955;
out[124] = -43.675799641021079;
out[125] = 17.974903844364839;
out[126] = -69.368640085185291;
out[127] = -94.61510234445565;
out[128] = 184.7505962980652;
out[129] = -17.974903844364825;
out[130] = 69.368640085185305;
out[131] = -40.74868784166955;
out[132] = 43.675799641021079;
out[133] = -17.974903844364839;
out[134] = 69.368640085185291;
out[135] = 94.61510234445565;
out[136] = -184.7505962980652;
out[137] = -1.0;
out[138] = -184.75059629806526;
out[139] = -94.615102344455664;
out[140] = 69.368640085185305;
out[141] = 17.974903844364835;
out[142] = 43.675799641021079;
out[143] = 40.748687841669557;
out[144] = 69.368640085185291;
out[145] = 17.974903844364821;
out[146] = 184.75059629806526;
out[147] = 94.615102344455664;
out[148] = -69.368640085185305;
out[149] = -17.974903844364835;
out[150] = -43.675799641021079;
out[151] = -40.748687841669557;
out[152] = -69.368640085185291;
out[153] = -17.974903844364821;
out[154] = -1.0;
out[155] = 94.615102344455664;
out[156] = -184.75059629806526;
out[157] = -17.974903844364835;
out[158] = 69.368640085185305;
out[159] = -40.748687841669557;
out[160] = 43.675799641021079;
out[161] = -17.974903844364821;
out[162] = 69.368640085185291;
out[163] = -94.615102344455664;
out[164] = 184.75059629806526;
out[165] = 17.974903844364835;
out[166] = -69.368640085185305;
out[167] = 40.748687841669557;
out[168] = -43.675799641021079;
out[169] = 17.974903844364821;
out[170] = -69.368640085185291;
out[171] = -1.0;
out[172] = 69.368640085185291;
out[173] = 17.974903844364825;
out[174] = -184.7505962980652;
out[175] = -94.615102344455636;
out[176] = 69.368640085185319;
out[177] = 17.97490384436481;
out[178] = 43.675799641021079;
out[179] = 40.748687841669565;
out[180] = -69.368640085185291;
out[181] = -17.974903844364825;
out[182] = 184.7505962980652;
out[183] = 94.615102344455636;
out[184] = -69.368640085185319;
out[185] = -17.97490384436481;
out[186] = -43.675799641021079;
out[187] = -40.748687841669565;
out[188] = -1.0;
out[189] = -17.974903844364825;
out[190] = 69.368640085185291;
out[191] = 94.615102344455636;
out[192] = -184.7505962980652;
out[193] = -17.97490384436481;
out[194] = 69.368640085185319;
out[195] = -40.748687841669565;
out[196] = 43.675799641021079;
out[197] = 17.974903844364825;
out[198] = -69.368640085185291;
out[199] = -94.615102344455636;
out[200] = 184.7505962980652;
out[201] = 17.97490384436481;
out[202] = -69.368640085185319;
out[203] = 40.748687841669565;
out[204] = -43.675799641021079;
out[205] = -1.0;
out[206] = 43.675799641021101;
out[207] = 40.748687841669579;
out[208] = 69.368640085185305;
out[209] = 17.974903844364821;
out[210] = -184.75059629806526;
out[211] = -94.61510234445565;
out[212] = 69.368640085185305;
out[213] = 17.974903844364821;
out[214] = -43.675799641021101;
out[215] = -40.748687841669579;
out[216] = -69.368640085185305;
out[217] = -17.974903844364821;
out[218] = 184.75059629806526;
out[219] = 94.61510234445565;
out[220] = -69.368640085185305;
out[221] = -17.974903844364821;
out[222] = -1.0;
out[223] = -40.748687841669579;
out[224] = 43.675799641021101;
out[225] = -17.974903844364821;
out[226] = 69.368640085185305;
out[227] = 94.61510234445565;
out[228] = -184.75059629806526;
out[229] = -17.974903844364821;
out[230] = 69.368640085185305;
out[231] = 40.748687841669579;
out[232] = -43.675799641021101;
out[233] = 17.974903844364821;
out[234] = -69.368640085185305;
out[235] = -94.61510234445565;
out[236] = 184.75059629806526;
out[237] = 17.974903844364821;
out[238] = -69.368640085185305;
out[239] = -1.0;
out[240] = 69.368640085185305;
out[241] = 17.974903844364825;
out[242] = 43.675799641021079;
out[243] = 40.74868784166955;
out[244] = 69.368640085185291;
out[245] = 17.974903844364839;
out[246] = -184.7505962980652;
out[247] = -94.61510234445565;
out[248] = -69.368640085185305;
out[249] = -17.974903844364825;
out[250] = -43.675799641021079;
out[251] = -40.74868784166955;
out[252] = -69.368640085185291;
out[253] = -17.974903844364839;
out[254] = 184.7505962980652;
out[255] = 94.61510234445565;
out[256] = -1.0;
out[257] = -17.974903844364825;
out[258] = 69.368640085185305;
out[259] = -40.74868784166955;
out[260] = 43.675799641021079;
out[261] = -17.974903844364839;
out[262] = 69.368640085185291;
out[263] = 94.61510234445565;
out[264] = -184.7505962980652;
out[265] = 17.974903844364825;
out[266] = -69.368640085185305;
out[267] = 40.74868784166955;
out[268] = -43.675799641021079;
out[269] = 17.974903844364839;
out[270] = -69.368640085185291;
out[271] = -94.61510234445565;
out[272] = 184.7505962980652;
out[273] = -1.0;
out[274] = -1.0;
out[277] = -1.0;
out[280] = -1.0;
out[283] = -1.0;
out[286] = -1.0;
out[289] = -1.0;
out[292] = -1.0;
out[295] = -1.0;
out[298] = -1.0;
out[301] = -1.0;
out[304] = -1.0;
out[307] = -1.0;
out[310] = -1.0;
out[313] = -1.0;
out[316] = -1.0;
out[319] = -1.0;
out[322] = -1.0;
out[325] = -1.0;

}

void de_Fu_run_xy_eval(double *out,double *x,double *y,double *u,double *p,double Dt){


}

void de_Fu_run_up_eval(double *out,double *x,double *y,double *u,double *p,double Dt){


}

void de_Fu_run_num_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[10] = 1.0;

}

void de_Gu_run_xy_eval(double *out,double *x,double *y,double *u,double *p,double Dt){


}

void de_Gu_run_up_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[176] = cos(u[4]);
out[180] = -u[0]*sin(u[4]);
out[188] = cos(u[4] - 2.0943951023931953);
out[191] = -u[1]*sin(u[4] - 2.0943951023931953);
out[200] = cos(u[4] - 4.1887902047863905);
out[202] = -u[2]*sin(u[4] - 4.1887902047863905);
out[220] = 1.0*sin(u[4]);
out[224] = 1.0*u[0]*cos(u[4]);
out[232] = 1.0*sin(u[4] - 2.0943951023931953);
out[235] = 1.0*u[1]*cos(u[4] - 2.0943951023931953);
out[244] = 1.0*sin(u[4] - 4.1887902047863905);
out[246] = 1.0*u[2]*cos(u[4] - 4.1887902047863905);
out[269] = cos(u[9]);
out[273] = -u[5]*sin(u[9]);
out[281] = cos(u[9] - 2.0943951023931953);
out[284] = -u[6]*sin(u[9] - 2.0943951023931953);
out[293] = cos(u[9] - 4.1887902047863905);
out[295] = -u[7]*sin(u[9] - 4.1887902047863905);
out[313] = 1.0*sin(u[9]);
out[317] = 1.0*u[5]*cos(u[9]);
out[325] = 1.0*sin(u[9] - 2.0943951023931953);
out[328] = 1.0*u[6]*cos(u[9] - 2.0943951023931953);
out[337] = 1.0*sin(u[9] - 4.1887902047863905);
out[339] = 1.0*u[7]*cos(u[9] - 4.1887902047863905);

}

void de_Gu_run_num_eval(double *out,double *x,double *y,double *u,double *p,double Dt){


}

void de_Hx_run_xy_eval(double *out,double *x,double *y,double *u,double *p,double Dt){


}

void de_Hx_run_up_eval(double *out,double *x,double *y,double *u,double *p,double Dt){


}

void de_Hx_run_num_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[20] = 1.0;

}

void de_Hy_run_xy_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[288] = y[16]*pow(1.0*pow(y[20], 2) + pow(y[16], 2), -0.5);
out[292] = 1.0*y[20]*pow(1.0*pow(y[20], 2) + pow(y[16], 2), -0.5);
out[323] = y[17]*pow(1.0*pow(y[21], 2) + pow(y[17], 2), -0.5);
out[327] = 1.0*y[21]*pow(1.0*pow(y[21], 2) + pow(y[17], 2), -0.5);
out[358] = y[18]*pow(1.0*pow(y[22], 2) + pow(y[18], 2), -0.5);
out[362] = 1.0*y[22]*pow(1.0*pow(y[22], 2) + pow(y[18], 2), -0.5);
out[393] = y[19]*pow(1.0*pow(y[23], 2) + pow(y[19], 2), -0.5);
out[397] = 1.0*y[23]*pow(1.0*pow(y[23], 2) + pow(y[19], 2), -0.5);
out[408] = y[16];
out[409] = 1.0*y[20];
out[410] = y[17];
out[411] = 1.0*y[21];
out[412] = y[18];
out[413] = 1.0*y[22];
out[424] = y[0];
out[425] = y[2];
out[426] = y[4];
out[428] = 1.0*y[1];
out[429] = 1.0*y[3];
out[430] = 1.0*y[5];
out[442] = -1.0*y[20];
out[443] = 1.0*y[16];
out[444] = -1.0*y[21];
out[445] = 1.0*y[17];
out[446] = -1.0*y[22];
out[447] = 1.0*y[18];
out[458] = 1.0*y[1];
out[459] = 1.0*y[3];
out[460] = 1.0*y[5];
out[462] = -1.0*y[0];
out[463] = -1.0*y[2];
out[464] = -1.0*y[4];
out[500] = y[24]*pow(1.0*pow(y[28], 2) + pow(y[24], 2), -0.5);
out[504] = 1.0*y[28]*pow(1.0*pow(y[28], 2) + pow(y[24], 2), -0.5);
out[535] = y[25]*pow(1.0*pow(y[29], 2) + pow(y[25], 2), -0.5);
out[539] = 1.0*y[29]*pow(1.0*pow(y[29], 2) + pow(y[25], 2), -0.5);
out[570] = y[26]*pow(1.0*pow(y[30], 2) + pow(y[26], 2), -0.5);
out[574] = 1.0*y[30]*pow(1.0*pow(y[30], 2) + pow(y[26], 2), -0.5);
out[605] = y[27]*pow(1.0*pow(y[31], 2) + pow(y[27], 2), -0.5);
out[609] = 1.0*y[31]*pow(1.0*pow(y[31], 2) + pow(y[27], 2), -0.5);
out[620] = y[24];
out[621] = 1.0*y[28];
out[622] = y[25];
out[623] = 1.0*y[29];
out[624] = y[26];
out[625] = 1.0*y[30];
out[636] = y[8];
out[637] = y[10];
out[638] = y[12];
out[640] = 1.0*y[9];
out[641] = 1.0*y[11];
out[642] = 1.0*y[13];
out[654] = -1.0*y[28];
out[655] = 1.0*y[24];
out[656] = -1.0*y[29];
out[657] = 1.0*y[25];
out[658] = -1.0*y[30];
out[659] = 1.0*y[26];
out[670] = 1.0*y[9];
out[671] = 1.0*y[11];
out[672] = 1.0*y[13];
out[674] = -1.0*y[8];
out[675] = -1.0*y[10];
out[676] = -1.0*y[12];

}

void de_Hy_run_up_eval(double *out,double *x,double *y,double *u,double *p,double Dt){


}

void de_Hy_run_num_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[0] = 184.75059629806526;
out[1] = 94.615102344455664;
out[2] = -69.368640085185305;
out[3] = -17.974903844364835;
out[4] = -43.675799641021079;
out[5] = -40.748687841669557;
out[6] = -69.368640085185291;
out[7] = -17.974903844364821;
out[8] = -184.75059629806526;
out[9] = -94.615102344455664;
out[10] = 69.368640085185305;
out[11] = 17.974903844364835;
out[12] = 43.675799641021079;
out[13] = 40.748687841669557;
out[14] = 69.368640085185291;
out[15] = 17.974903844364821;
out[34] = -94.615102344455664;
out[35] = 184.75059629806526;
out[36] = 17.974903844364835;
out[37] = -69.368640085185305;
out[38] = 40.748687841669557;
out[39] = -43.675799641021079;
out[40] = 17.974903844364821;
out[41] = -69.368640085185291;
out[42] = 94.615102344455664;
out[43] = -184.75059629806526;
out[44] = -17.974903844364835;
out[45] = 69.368640085185305;
out[46] = -40.748687841669557;
out[47] = 43.675799641021079;
out[48] = -17.974903844364821;
out[49] = 69.368640085185291;
out[68] = -69.368640085185291;
out[69] = -17.974903844364825;
out[70] = 184.7505962980652;
out[71] = 94.615102344455636;
out[72] = -69.368640085185319;
out[73] = -17.97490384436481;
out[74] = -43.675799641021079;
out[75] = -40.748687841669565;
out[76] = 69.368640085185291;
out[77] = 17.974903844364825;
out[78] = -184.7505962980652;
out[79] = -94.615102344455636;
out[80] = 69.368640085185319;
out[81] = 17.97490384436481;
out[82] = 43.675799641021079;
out[83] = 40.748687841669565;
out[102] = 17.974903844364825;
out[103] = -69.368640085185291;
out[104] = -94.615102344455636;
out[105] = 184.7505962980652;
out[106] = 17.97490384436481;
out[107] = -69.368640085185319;
out[108] = 40.748687841669565;
out[109] = -43.675799641021079;
out[110] = -17.974903844364825;
out[111] = 69.368640085185291;
out[112] = 94.615102344455636;
out[113] = -184.7505962980652;
out[114] = -17.97490384436481;
out[115] = 69.368640085185319;
out[116] = -40.748687841669565;
out[117] = 43.675799641021079;
out[136] = -43.675799641021101;
out[137] = -40.748687841669579;
out[138] = -69.368640085185305;
out[139] = -17.974903844364821;
out[140] = 184.75059629806526;
out[141] = 94.61510234445565;
out[142] = -69.368640085185305;
out[143] = -17.974903844364821;
out[144] = 43.675799641021101;
out[145] = 40.748687841669579;
out[146] = 69.368640085185305;
out[147] = 17.974903844364821;
out[148] = -184.75059629806526;
out[149] = -94.61510234445565;
out[150] = 69.368640085185305;
out[151] = 17.974903844364821;
out[170] = 40.748687841669579;
out[171] = -43.675799641021101;
out[172] = 17.974903844364821;
out[173] = -69.368640085185305;
out[174] = -94.61510234445565;
out[175] = 184.75059629806526;
out[176] = 17.974903844364821;
out[177] = -69.368640085185305;
out[178] = -40.748687841669579;
out[179] = 43.675799641021101;
out[180] = -17.974903844364821;
out[181] = 69.368640085185305;
out[182] = 94.61510234445565;
out[183] = -184.75059629806526;
out[184] = -17.974903844364821;
out[185] = 69.368640085185305;
out[204] = -69.368640085185305;
out[205] = -17.974903844364825;
out[206] = -43.675799641021079;
out[207] = -40.74868784166955;
out[208] = -69.368640085185291;
out[209] = -17.974903844364839;
out[210] = 184.7505962980652;
out[211] = 94.61510234445565;
out[212] = 69.368640085185305;
out[213] = 17.974903844364825;
out[214] = 43.675799641021079;
out[215] = 40.74868784166955;
out[216] = 69.368640085185291;
out[217] = 17.974903844364839;
out[218] = -184.7505962980652;
out[219] = -94.61510234445565;
out[238] = 17.974903844364825;
out[239] = -69.368640085185305;
out[240] = 40.74868784166955;
out[241] = -43.675799641021079;
out[242] = 17.974903844364839;
out[243] = -69.368640085185291;
out[244] = -94.61510234445565;
out[245] = 184.7505962980652;
out[246] = -17.974903844364825;
out[247] = 69.368640085185305;
out[248] = -40.74868784166955;
out[249] = 43.675799641021079;
out[250] = -17.974903844364839;
out[251] = 69.368640085185291;
out[252] = 94.61510234445565;
out[253] = -184.7505962980652;

}

void de_Hu_run_xy_eval(double *out,double *x,double *y,double *u,double *p,double Dt){


}

void de_Hu_run_up_eval(double *out,double *x,double *y,double *u,double *p,double Dt){


}

void de_Hu_run_num_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[241] = 1.0;

}

void sp_Fu_run_xy_eval(double *out,double *x,double *y,double *u,double *p,double Dt){


}

void sp_Fu_run_up_eval(double *out,double *x,double *y,double *u,double *p,double Dt){


}

void sp_Fu_run_num_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[0] = 1.0;

}

void sp_Gu_run_xy_eval(double *out,double *x,double *y,double *u,double *p,double Dt){


}

void sp_Gu_run_up_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[0] = cos(u[4]);
out[1] = -u[0]*sin(u[4]);
out[2] = cos(u[4] - 2.0943951023931953);
out[3] = -u[1]*sin(u[4] - 2.0943951023931953);
out[4] = cos(u[4] - 4.1887902047863905);
out[5] = -u[2]*sin(u[4] - 4.1887902047863905);
out[6] = 1.0*sin(u[4]);
out[7] = 1.0*u[0]*cos(u[4]);
out[8] = 1.0*sin(u[4] - 2.0943951023931953);
out[9] = 1.0*u[1]*cos(u[4] - 2.0943951023931953);
out[10] = 1.0*sin(u[4] - 4.1887902047863905);
out[11] = 1.0*u[2]*cos(u[4] - 4.1887902047863905);
out[12] = cos(u[9]);
out[13] = -u[5]*sin(u[9]);
out[14] = cos(u[9] - 2.0943951023931953);
out[15] = -u[6]*sin(u[9] - 2.0943951023931953);
out[16] = cos(u[9] - 4.1887902047863905);
out[17] = -u[7]*sin(u[9] - 4.1887902047863905);
out[18] = 1.0*sin(u[9]);
out[19] = 1.0*u[5]*cos(u[9]);
out[20] = 1.0*sin(u[9] - 2.0943951023931953);
out[21] = 1.0*u[6]*cos(u[9] - 2.0943951023931953);
out[22] = 1.0*sin(u[9] - 4.1887902047863905);
out[23] = 1.0*u[7]*cos(u[9] - 4.1887902047863905);

}

void sp_Gu_run_num_eval(double *out,double *x,double *y,double *u,double *p,double Dt){


}

void sp_Hx_run_xy_eval(double *out,double *x,double *y,double *u,double *p,double Dt){


}

void sp_Hx_run_up_eval(double *out,double *x,double *y,double *u,double *p,double Dt){


}

void sp_Hx_run_num_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[0] = 1.0;

}

void sp_Hy_run_xy_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[128] = y[16]*pow(1.0*pow(y[20], 2) + pow(y[16], 2), -0.5);
out[129] = 1.0*y[20]*pow(1.0*pow(y[20], 2) + pow(y[16], 2), -0.5);
out[130] = y[17]*pow(1.0*pow(y[21], 2) + pow(y[17], 2), -0.5);
out[131] = 1.0*y[21]*pow(1.0*pow(y[21], 2) + pow(y[17], 2), -0.5);
out[132] = y[18]*pow(1.0*pow(y[22], 2) + pow(y[18], 2), -0.5);
out[133] = 1.0*y[22]*pow(1.0*pow(y[22], 2) + pow(y[18], 2), -0.5);
out[134] = y[19]*pow(1.0*pow(y[23], 2) + pow(y[19], 2), -0.5);
out[135] = 1.0*y[23]*pow(1.0*pow(y[23], 2) + pow(y[19], 2), -0.5);
out[136] = y[16];
out[137] = 1.0*y[20];
out[138] = y[17];
out[139] = 1.0*y[21];
out[140] = y[18];
out[141] = 1.0*y[22];
out[142] = y[0];
out[143] = y[2];
out[144] = y[4];
out[145] = 1.0*y[1];
out[146] = 1.0*y[3];
out[147] = 1.0*y[5];
out[148] = -1.0*y[20];
out[149] = 1.0*y[16];
out[150] = -1.0*y[21];
out[151] = 1.0*y[17];
out[152] = -1.0*y[22];
out[153] = 1.0*y[18];
out[154] = 1.0*y[1];
out[155] = 1.0*y[3];
out[156] = 1.0*y[5];
out[157] = -1.0*y[0];
out[158] = -1.0*y[2];
out[159] = -1.0*y[4];
out[160] = y[24]*pow(1.0*pow(y[28], 2) + pow(y[24], 2), -0.5);
out[161] = 1.0*y[28]*pow(1.0*pow(y[28], 2) + pow(y[24], 2), -0.5);
out[162] = y[25]*pow(1.0*pow(y[29], 2) + pow(y[25], 2), -0.5);
out[163] = 1.0*y[29]*pow(1.0*pow(y[29], 2) + pow(y[25], 2), -0.5);
out[164] = y[26]*pow(1.0*pow(y[30], 2) + pow(y[26], 2), -0.5);
out[165] = 1.0*y[30]*pow(1.0*pow(y[30], 2) + pow(y[26], 2), -0.5);
out[166] = y[27]*pow(1.0*pow(y[31], 2) + pow(y[27], 2), -0.5);
out[167] = 1.0*y[31]*pow(1.0*pow(y[31], 2) + pow(y[27], 2), -0.5);
out[168] = y[24];
out[169] = 1.0*y[28];
out[170] = y[25];
out[171] = 1.0*y[29];
out[172] = y[26];
out[173] = 1.0*y[30];
out[174] = y[8];
out[175] = y[10];
out[176] = y[12];
out[177] = 1.0*y[9];
out[178] = 1.0*y[11];
out[179] = 1.0*y[13];
out[180] = -1.0*y[28];
out[181] = 1.0*y[24];
out[182] = -1.0*y[29];
out[183] = 1.0*y[25];
out[184] = -1.0*y[30];
out[185] = 1.0*y[26];
out[186] = 1.0*y[9];
out[187] = 1.0*y[11];
out[188] = 1.0*y[13];
out[189] = -1.0*y[8];
out[190] = -1.0*y[10];
out[191] = -1.0*y[12];

}

void sp_Hy_run_up_eval(double *out,double *x,double *y,double *u,double *p,double Dt){


}

void sp_Hy_run_num_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[0] = 184.75059629806526;
out[1] = 94.615102344455664;
out[2] = -69.368640085185305;
out[3] = -17.974903844364835;
out[4] = -43.675799641021079;
out[5] = -40.748687841669557;
out[6] = -69.368640085185291;
out[7] = -17.974903844364821;
out[8] = -184.75059629806526;
out[9] = -94.615102344455664;
out[10] = 69.368640085185305;
out[11] = 17.974903844364835;
out[12] = 43.675799641021079;
out[13] = 40.748687841669557;
out[14] = 69.368640085185291;
out[15] = 17.974903844364821;
out[16] = -94.615102344455664;
out[17] = 184.75059629806526;
out[18] = 17.974903844364835;
out[19] = -69.368640085185305;
out[20] = 40.748687841669557;
out[21] = -43.675799641021079;
out[22] = 17.974903844364821;
out[23] = -69.368640085185291;
out[24] = 94.615102344455664;
out[25] = -184.75059629806526;
out[26] = -17.974903844364835;
out[27] = 69.368640085185305;
out[28] = -40.748687841669557;
out[29] = 43.675799641021079;
out[30] = -17.974903844364821;
out[31] = 69.368640085185291;
out[32] = -69.368640085185291;
out[33] = -17.974903844364825;
out[34] = 184.7505962980652;
out[35] = 94.615102344455636;
out[36] = -69.368640085185319;
out[37] = -17.97490384436481;
out[38] = -43.675799641021079;
out[39] = -40.748687841669565;
out[40] = 69.368640085185291;
out[41] = 17.974903844364825;
out[42] = -184.7505962980652;
out[43] = -94.615102344455636;
out[44] = 69.368640085185319;
out[45] = 17.97490384436481;
out[46] = 43.675799641021079;
out[47] = 40.748687841669565;
out[48] = 17.974903844364825;
out[49] = -69.368640085185291;
out[50] = -94.615102344455636;
out[51] = 184.7505962980652;
out[52] = 17.97490384436481;
out[53] = -69.368640085185319;
out[54] = 40.748687841669565;
out[55] = -43.675799641021079;
out[56] = -17.974903844364825;
out[57] = 69.368640085185291;
out[58] = 94.615102344455636;
out[59] = -184.7505962980652;
out[60] = -17.97490384436481;
out[61] = 69.368640085185319;
out[62] = -40.748687841669565;
out[63] = 43.675799641021079;
out[64] = -43.675799641021101;
out[65] = -40.748687841669579;
out[66] = -69.368640085185305;
out[67] = -17.974903844364821;
out[68] = 184.75059629806526;
out[69] = 94.61510234445565;
out[70] = -69.368640085185305;
out[71] = -17.974903844364821;
out[72] = 43.675799641021101;
out[73] = 40.748687841669579;
out[74] = 69.368640085185305;
out[75] = 17.974903844364821;
out[76] = -184.75059629806526;
out[77] = -94.61510234445565;
out[78] = 69.368640085185305;
out[79] = 17.974903844364821;
out[80] = 40.748687841669579;
out[81] = -43.675799641021101;
out[82] = 17.974903844364821;
out[83] = -69.368640085185305;
out[84] = -94.61510234445565;
out[85] = 184.75059629806526;
out[86] = 17.974903844364821;
out[87] = -69.368640085185305;
out[88] = -40.748687841669579;
out[89] = 43.675799641021101;
out[90] = -17.974903844364821;
out[91] = 69.368640085185305;
out[92] = 94.61510234445565;
out[93] = -184.75059629806526;
out[94] = -17.974903844364821;
out[95] = 69.368640085185305;
out[96] = -69.368640085185305;
out[97] = -17.974903844364825;
out[98] = -43.675799641021079;
out[99] = -40.74868784166955;
out[100] = -69.368640085185291;
out[101] = -17.974903844364839;
out[102] = 184.7505962980652;
out[103] = 94.61510234445565;
out[104] = 69.368640085185305;
out[105] = 17.974903844364825;
out[106] = 43.675799641021079;
out[107] = 40.74868784166955;
out[108] = 69.368640085185291;
out[109] = 17.974903844364839;
out[110] = -184.7505962980652;
out[111] = -94.61510234445565;
out[112] = 17.974903844364825;
out[113] = -69.368640085185305;
out[114] = 40.74868784166955;
out[115] = -43.675799641021079;
out[116] = 17.974903844364839;
out[117] = -69.368640085185291;
out[118] = -94.61510234445565;
out[119] = 184.7505962980652;
out[120] = -17.974903844364825;
out[121] = 69.368640085185305;
out[122] = -40.74868784166955;
out[123] = 43.675799641021079;
out[124] = -17.974903844364839;
out[125] = 69.368640085185291;
out[126] = 94.61510234445565;
out[127] = -184.7505962980652;

}

void sp_Hu_run_xy_eval(double *out,double *x,double *y,double *u,double *p,double Dt){


}

void sp_Hu_run_up_eval(double *out,double *x,double *y,double *u,double *p,double Dt){


}

void sp_Hu_run_num_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[0] = 1.0;

}



/************************************************************/

static void *_cffi_types[] = {
/*  0 */ _CFFI_OP(_CFFI_OP_FUNCTION, 8), // void()(double *, double *, double *, double *, double *, double)
/*  1 */ _CFFI_OP(_CFFI_OP_POINTER, 6), // double *
/*  2 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/*  3 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/*  4 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/*  5 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/*  6 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 14), // double
/*  7 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/*  8 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 0), // void
};

static void _cffi_d_de_Fu_run_num_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_Fu_run_num_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_Fu_run_num_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_Fu_run_num_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_Fu_run_num_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_Fu_run_num_eval _cffi_d_de_Fu_run_num_eval
#endif

static void _cffi_d_de_Fu_run_up_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_Fu_run_up_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_Fu_run_up_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_Fu_run_up_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_Fu_run_up_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_Fu_run_up_eval _cffi_d_de_Fu_run_up_eval
#endif

static void _cffi_d_de_Fu_run_xy_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_Fu_run_xy_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_Fu_run_xy_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_Fu_run_xy_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_Fu_run_xy_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_Fu_run_xy_eval _cffi_d_de_Fu_run_xy_eval
#endif

static void _cffi_d_de_Gu_run_num_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_Gu_run_num_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_Gu_run_num_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_Gu_run_num_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_Gu_run_num_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_Gu_run_num_eval _cffi_d_de_Gu_run_num_eval
#endif

static void _cffi_d_de_Gu_run_up_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_Gu_run_up_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_Gu_run_up_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_Gu_run_up_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_Gu_run_up_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_Gu_run_up_eval _cffi_d_de_Gu_run_up_eval
#endif

static void _cffi_d_de_Gu_run_xy_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_Gu_run_xy_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_Gu_run_xy_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_Gu_run_xy_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_Gu_run_xy_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_Gu_run_xy_eval _cffi_d_de_Gu_run_xy_eval
#endif

static void _cffi_d_de_Hu_run_num_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_Hu_run_num_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_Hu_run_num_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_Hu_run_num_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_Hu_run_num_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_Hu_run_num_eval _cffi_d_de_Hu_run_num_eval
#endif

static void _cffi_d_de_Hu_run_up_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_Hu_run_up_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_Hu_run_up_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_Hu_run_up_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_Hu_run_up_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_Hu_run_up_eval _cffi_d_de_Hu_run_up_eval
#endif

static void _cffi_d_de_Hu_run_xy_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_Hu_run_xy_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_Hu_run_xy_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_Hu_run_xy_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_Hu_run_xy_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_Hu_run_xy_eval _cffi_d_de_Hu_run_xy_eval
#endif

static void _cffi_d_de_Hx_run_num_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_Hx_run_num_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_Hx_run_num_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_Hx_run_num_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_Hx_run_num_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_Hx_run_num_eval _cffi_d_de_Hx_run_num_eval
#endif

static void _cffi_d_de_Hx_run_up_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_Hx_run_up_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_Hx_run_up_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_Hx_run_up_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_Hx_run_up_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_Hx_run_up_eval _cffi_d_de_Hx_run_up_eval
#endif

static void _cffi_d_de_Hx_run_xy_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_Hx_run_xy_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_Hx_run_xy_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_Hx_run_xy_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_Hx_run_xy_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_Hx_run_xy_eval _cffi_d_de_Hx_run_xy_eval
#endif

static void _cffi_d_de_Hy_run_num_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_Hy_run_num_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_Hy_run_num_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_Hy_run_num_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_Hy_run_num_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_Hy_run_num_eval _cffi_d_de_Hy_run_num_eval
#endif

static void _cffi_d_de_Hy_run_up_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_Hy_run_up_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_Hy_run_up_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_Hy_run_up_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_Hy_run_up_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_Hy_run_up_eval _cffi_d_de_Hy_run_up_eval
#endif

static void _cffi_d_de_Hy_run_xy_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_Hy_run_xy_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_Hy_run_xy_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_Hy_run_xy_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_Hy_run_xy_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_Hy_run_xy_eval _cffi_d_de_Hy_run_xy_eval
#endif

static void _cffi_d_de_jac_ini_num_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_jac_ini_num_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_jac_ini_num_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_jac_ini_num_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_jac_ini_num_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_jac_ini_num_eval _cffi_d_de_jac_ini_num_eval
#endif

static void _cffi_d_de_jac_ini_up_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_jac_ini_up_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_jac_ini_up_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_jac_ini_up_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_jac_ini_up_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_jac_ini_up_eval _cffi_d_de_jac_ini_up_eval
#endif

static void _cffi_d_de_jac_ini_xy_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_jac_ini_xy_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_jac_ini_xy_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_jac_ini_xy_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_jac_ini_xy_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_jac_ini_xy_eval _cffi_d_de_jac_ini_xy_eval
#endif

static void _cffi_d_de_jac_run_num_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_jac_run_num_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_jac_run_num_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_jac_run_num_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_jac_run_num_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_jac_run_num_eval _cffi_d_de_jac_run_num_eval
#endif

static void _cffi_d_de_jac_run_up_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_jac_run_up_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_jac_run_up_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_jac_run_up_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_jac_run_up_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_jac_run_up_eval _cffi_d_de_jac_run_up_eval
#endif

static void _cffi_d_de_jac_run_xy_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_jac_run_xy_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_jac_run_xy_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_jac_run_xy_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_jac_run_xy_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_jac_run_xy_eval _cffi_d_de_jac_run_xy_eval
#endif

static void _cffi_d_de_jac_trap_num_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_jac_trap_num_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_jac_trap_num_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_jac_trap_num_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_jac_trap_num_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_jac_trap_num_eval _cffi_d_de_jac_trap_num_eval
#endif

static void _cffi_d_de_jac_trap_up_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_jac_trap_up_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_jac_trap_up_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_jac_trap_up_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_jac_trap_up_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_jac_trap_up_eval _cffi_d_de_jac_trap_up_eval
#endif

static void _cffi_d_de_jac_trap_xy_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_jac_trap_xy_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_jac_trap_xy_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_jac_trap_xy_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_jac_trap_xy_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_jac_trap_xy_eval _cffi_d_de_jac_trap_xy_eval
#endif

static void _cffi_d_f_ini_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  f_ini_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_f_ini_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "f_ini_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { f_ini_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_f_ini_eval _cffi_d_f_ini_eval
#endif

static void _cffi_d_f_run_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  f_run_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_f_run_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "f_run_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { f_run_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_f_run_eval _cffi_d_f_run_eval
#endif

static void _cffi_d_g_ini_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  g_ini_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_g_ini_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "g_ini_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { g_ini_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_g_ini_eval _cffi_d_g_ini_eval
#endif

static void _cffi_d_g_run_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  g_run_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_g_run_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "g_run_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { g_run_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_g_run_eval _cffi_d_g_run_eval
#endif

static void _cffi_d_h_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  h_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_h_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "h_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { h_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_h_eval _cffi_d_h_eval
#endif

static void _cffi_d_sp_Fu_run_num_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  sp_Fu_run_num_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_sp_Fu_run_num_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "sp_Fu_run_num_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { sp_Fu_run_num_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_sp_Fu_run_num_eval _cffi_d_sp_Fu_run_num_eval
#endif

static void _cffi_d_sp_Fu_run_up_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  sp_Fu_run_up_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_sp_Fu_run_up_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "sp_Fu_run_up_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { sp_Fu_run_up_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_sp_Fu_run_up_eval _cffi_d_sp_Fu_run_up_eval
#endif

static void _cffi_d_sp_Fu_run_xy_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  sp_Fu_run_xy_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_sp_Fu_run_xy_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "sp_Fu_run_xy_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { sp_Fu_run_xy_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_sp_Fu_run_xy_eval _cffi_d_sp_Fu_run_xy_eval
#endif

static void _cffi_d_sp_Gu_run_num_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  sp_Gu_run_num_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_sp_Gu_run_num_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "sp_Gu_run_num_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { sp_Gu_run_num_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_sp_Gu_run_num_eval _cffi_d_sp_Gu_run_num_eval
#endif

static void _cffi_d_sp_Gu_run_up_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  sp_Gu_run_up_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_sp_Gu_run_up_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "sp_Gu_run_up_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { sp_Gu_run_up_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_sp_Gu_run_up_eval _cffi_d_sp_Gu_run_up_eval
#endif

static void _cffi_d_sp_Gu_run_xy_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  sp_Gu_run_xy_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_sp_Gu_run_xy_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "sp_Gu_run_xy_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { sp_Gu_run_xy_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_sp_Gu_run_xy_eval _cffi_d_sp_Gu_run_xy_eval
#endif

static void _cffi_d_sp_Hu_run_num_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  sp_Hu_run_num_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_sp_Hu_run_num_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "sp_Hu_run_num_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { sp_Hu_run_num_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_sp_Hu_run_num_eval _cffi_d_sp_Hu_run_num_eval
#endif

static void _cffi_d_sp_Hu_run_up_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  sp_Hu_run_up_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_sp_Hu_run_up_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "sp_Hu_run_up_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { sp_Hu_run_up_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_sp_Hu_run_up_eval _cffi_d_sp_Hu_run_up_eval
#endif

static void _cffi_d_sp_Hu_run_xy_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  sp_Hu_run_xy_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_sp_Hu_run_xy_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "sp_Hu_run_xy_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { sp_Hu_run_xy_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_sp_Hu_run_xy_eval _cffi_d_sp_Hu_run_xy_eval
#endif

static void _cffi_d_sp_Hx_run_num_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  sp_Hx_run_num_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_sp_Hx_run_num_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "sp_Hx_run_num_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { sp_Hx_run_num_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_sp_Hx_run_num_eval _cffi_d_sp_Hx_run_num_eval
#endif

static void _cffi_d_sp_Hx_run_up_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  sp_Hx_run_up_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_sp_Hx_run_up_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "sp_Hx_run_up_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { sp_Hx_run_up_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_sp_Hx_run_up_eval _cffi_d_sp_Hx_run_up_eval
#endif

static void _cffi_d_sp_Hx_run_xy_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  sp_Hx_run_xy_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_sp_Hx_run_xy_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "sp_Hx_run_xy_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { sp_Hx_run_xy_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_sp_Hx_run_xy_eval _cffi_d_sp_Hx_run_xy_eval
#endif

static void _cffi_d_sp_Hy_run_num_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  sp_Hy_run_num_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_sp_Hy_run_num_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "sp_Hy_run_num_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { sp_Hy_run_num_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_sp_Hy_run_num_eval _cffi_d_sp_Hy_run_num_eval
#endif

static void _cffi_d_sp_Hy_run_up_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  sp_Hy_run_up_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_sp_Hy_run_up_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "sp_Hy_run_up_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { sp_Hy_run_up_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_sp_Hy_run_up_eval _cffi_d_sp_Hy_run_up_eval
#endif

static void _cffi_d_sp_Hy_run_xy_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  sp_Hy_run_xy_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_sp_Hy_run_xy_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "sp_Hy_run_xy_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { sp_Hy_run_xy_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_sp_Hy_run_xy_eval _cffi_d_sp_Hy_run_xy_eval
#endif

static void _cffi_d_sp_jac_ini_num_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  sp_jac_ini_num_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_sp_jac_ini_num_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "sp_jac_ini_num_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { sp_jac_ini_num_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_sp_jac_ini_num_eval _cffi_d_sp_jac_ini_num_eval
#endif

static void _cffi_d_sp_jac_ini_up_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  sp_jac_ini_up_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_sp_jac_ini_up_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "sp_jac_ini_up_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { sp_jac_ini_up_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_sp_jac_ini_up_eval _cffi_d_sp_jac_ini_up_eval
#endif

static void _cffi_d_sp_jac_ini_xy_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  sp_jac_ini_xy_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_sp_jac_ini_xy_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "sp_jac_ini_xy_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { sp_jac_ini_xy_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_sp_jac_ini_xy_eval _cffi_d_sp_jac_ini_xy_eval
#endif

static void _cffi_d_sp_jac_run_num_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  sp_jac_run_num_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_sp_jac_run_num_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "sp_jac_run_num_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { sp_jac_run_num_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_sp_jac_run_num_eval _cffi_d_sp_jac_run_num_eval
#endif

static void _cffi_d_sp_jac_run_up_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  sp_jac_run_up_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_sp_jac_run_up_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "sp_jac_run_up_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { sp_jac_run_up_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_sp_jac_run_up_eval _cffi_d_sp_jac_run_up_eval
#endif

static void _cffi_d_sp_jac_run_xy_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  sp_jac_run_xy_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_sp_jac_run_xy_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "sp_jac_run_xy_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { sp_jac_run_xy_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_sp_jac_run_xy_eval _cffi_d_sp_jac_run_xy_eval
#endif

static void _cffi_d_sp_jac_trap_num_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  sp_jac_trap_num_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_sp_jac_trap_num_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "sp_jac_trap_num_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { sp_jac_trap_num_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_sp_jac_trap_num_eval _cffi_d_sp_jac_trap_num_eval
#endif

static void _cffi_d_sp_jac_trap_up_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  sp_jac_trap_up_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_sp_jac_trap_up_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "sp_jac_trap_up_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { sp_jac_trap_up_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_sp_jac_trap_up_eval _cffi_d_sp_jac_trap_up_eval
#endif

static void _cffi_d_sp_jac_trap_xy_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  sp_jac_trap_xy_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_sp_jac_trap_xy_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "sp_jac_trap_xy_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { sp_jac_trap_xy_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_sp_jac_trap_xy_eval _cffi_d_sp_jac_trap_xy_eval
#endif

static const struct _cffi_global_s _cffi_globals[] = {
  { "de_Fu_run_num_eval", (void *)_cffi_f_de_Fu_run_num_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_Fu_run_num_eval },
  { "de_Fu_run_up_eval", (void *)_cffi_f_de_Fu_run_up_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_Fu_run_up_eval },
  { "de_Fu_run_xy_eval", (void *)_cffi_f_de_Fu_run_xy_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_Fu_run_xy_eval },
  { "de_Gu_run_num_eval", (void *)_cffi_f_de_Gu_run_num_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_Gu_run_num_eval },
  { "de_Gu_run_up_eval", (void *)_cffi_f_de_Gu_run_up_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_Gu_run_up_eval },
  { "de_Gu_run_xy_eval", (void *)_cffi_f_de_Gu_run_xy_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_Gu_run_xy_eval },
  { "de_Hu_run_num_eval", (void *)_cffi_f_de_Hu_run_num_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_Hu_run_num_eval },
  { "de_Hu_run_up_eval", (void *)_cffi_f_de_Hu_run_up_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_Hu_run_up_eval },
  { "de_Hu_run_xy_eval", (void *)_cffi_f_de_Hu_run_xy_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_Hu_run_xy_eval },
  { "de_Hx_run_num_eval", (void *)_cffi_f_de_Hx_run_num_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_Hx_run_num_eval },
  { "de_Hx_run_up_eval", (void *)_cffi_f_de_Hx_run_up_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_Hx_run_up_eval },
  { "de_Hx_run_xy_eval", (void *)_cffi_f_de_Hx_run_xy_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_Hx_run_xy_eval },
  { "de_Hy_run_num_eval", (void *)_cffi_f_de_Hy_run_num_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_Hy_run_num_eval },
  { "de_Hy_run_up_eval", (void *)_cffi_f_de_Hy_run_up_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_Hy_run_up_eval },
  { "de_Hy_run_xy_eval", (void *)_cffi_f_de_Hy_run_xy_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_Hy_run_xy_eval },
  { "de_jac_ini_num_eval", (void *)_cffi_f_de_jac_ini_num_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_jac_ini_num_eval },
  { "de_jac_ini_up_eval", (void *)_cffi_f_de_jac_ini_up_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_jac_ini_up_eval },
  { "de_jac_ini_xy_eval", (void *)_cffi_f_de_jac_ini_xy_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_jac_ini_xy_eval },
  { "de_jac_run_num_eval", (void *)_cffi_f_de_jac_run_num_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_jac_run_num_eval },
  { "de_jac_run_up_eval", (void *)_cffi_f_de_jac_run_up_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_jac_run_up_eval },
  { "de_jac_run_xy_eval", (void *)_cffi_f_de_jac_run_xy_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_jac_run_xy_eval },
  { "de_jac_trap_num_eval", (void *)_cffi_f_de_jac_trap_num_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_jac_trap_num_eval },
  { "de_jac_trap_up_eval", (void *)_cffi_f_de_jac_trap_up_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_jac_trap_up_eval },
  { "de_jac_trap_xy_eval", (void *)_cffi_f_de_jac_trap_xy_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_jac_trap_xy_eval },
  { "f_ini_eval", (void *)_cffi_f_f_ini_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_f_ini_eval },
  { "f_run_eval", (void *)_cffi_f_f_run_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_f_run_eval },
  { "g_ini_eval", (void *)_cffi_f_g_ini_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_g_ini_eval },
  { "g_run_eval", (void *)_cffi_f_g_run_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_g_run_eval },
  { "h_eval", (void *)_cffi_f_h_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_h_eval },
  { "sp_Fu_run_num_eval", (void *)_cffi_f_sp_Fu_run_num_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_sp_Fu_run_num_eval },
  { "sp_Fu_run_up_eval", (void *)_cffi_f_sp_Fu_run_up_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_sp_Fu_run_up_eval },
  { "sp_Fu_run_xy_eval", (void *)_cffi_f_sp_Fu_run_xy_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_sp_Fu_run_xy_eval },
  { "sp_Gu_run_num_eval", (void *)_cffi_f_sp_Gu_run_num_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_sp_Gu_run_num_eval },
  { "sp_Gu_run_up_eval", (void *)_cffi_f_sp_Gu_run_up_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_sp_Gu_run_up_eval },
  { "sp_Gu_run_xy_eval", (void *)_cffi_f_sp_Gu_run_xy_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_sp_Gu_run_xy_eval },
  { "sp_Hu_run_num_eval", (void *)_cffi_f_sp_Hu_run_num_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_sp_Hu_run_num_eval },
  { "sp_Hu_run_up_eval", (void *)_cffi_f_sp_Hu_run_up_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_sp_Hu_run_up_eval },
  { "sp_Hu_run_xy_eval", (void *)_cffi_f_sp_Hu_run_xy_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_sp_Hu_run_xy_eval },
  { "sp_Hx_run_num_eval", (void *)_cffi_f_sp_Hx_run_num_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_sp_Hx_run_num_eval },
  { "sp_Hx_run_up_eval", (void *)_cffi_f_sp_Hx_run_up_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_sp_Hx_run_up_eval },
  { "sp_Hx_run_xy_eval", (void *)_cffi_f_sp_Hx_run_xy_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_sp_Hx_run_xy_eval },
  { "sp_Hy_run_num_eval", (void *)_cffi_f_sp_Hy_run_num_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_sp_Hy_run_num_eval },
  { "sp_Hy_run_up_eval", (void *)_cffi_f_sp_Hy_run_up_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_sp_Hy_run_up_eval },
  { "sp_Hy_run_xy_eval", (void *)_cffi_f_sp_Hy_run_xy_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_sp_Hy_run_xy_eval },
  { "sp_jac_ini_num_eval", (void *)_cffi_f_sp_jac_ini_num_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_sp_jac_ini_num_eval },
  { "sp_jac_ini_up_eval", (void *)_cffi_f_sp_jac_ini_up_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_sp_jac_ini_up_eval },
  { "sp_jac_ini_xy_eval", (void *)_cffi_f_sp_jac_ini_xy_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_sp_jac_ini_xy_eval },
  { "sp_jac_run_num_eval", (void *)_cffi_f_sp_jac_run_num_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_sp_jac_run_num_eval },
  { "sp_jac_run_up_eval", (void *)_cffi_f_sp_jac_run_up_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_sp_jac_run_up_eval },
  { "sp_jac_run_xy_eval", (void *)_cffi_f_sp_jac_run_xy_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_sp_jac_run_xy_eval },
  { "sp_jac_trap_num_eval", (void *)_cffi_f_sp_jac_trap_num_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_sp_jac_trap_num_eval },
  { "sp_jac_trap_up_eval", (void *)_cffi_f_sp_jac_trap_up_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_sp_jac_trap_up_eval },
  { "sp_jac_trap_xy_eval", (void *)_cffi_f_sp_jac_trap_xy_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_sp_jac_trap_xy_eval },
};

static const struct _cffi_type_context_s _cffi_type_context = {
  _cffi_types,
  _cffi_globals,
  NULL,  /* no fields */
  NULL,  /* no struct_unions */
  NULL,  /* no enums */
  NULL,  /* no typenames */
  53,  /* num_globals */
  0,  /* num_struct_unions */
  0,  /* num_enums */
  0,  /* num_typenames */
  NULL,  /* no includes */
  9,  /* num_types */
  0,  /* flags */
};

#ifdef __GNUC__
#  pragma GCC visibility push(default)  /* for -fvisibility= */
#endif

#ifdef PYPY_VERSION
PyMODINIT_FUNC
_cffi_pypyinit_temp_cffi(const void *p[])
{
    p[0] = (const void *)0x2601;
    p[1] = &_cffi_type_context;
#if PY_MAJOR_VERSION >= 3
    return NULL;
#endif
}
#  ifdef _MSC_VER
     PyMODINIT_FUNC
#  if PY_MAJOR_VERSION >= 3
     PyInit_temp_cffi(void) { return NULL; }
#  else
     inittemp_cffi(void) { }
#  endif
#  endif
#elif PY_MAJOR_VERSION >= 3
PyMODINIT_FUNC
PyInit_temp_cffi(void)
{
  return _cffi_init("temp_cffi", 0x2601, &_cffi_type_context);
}
#else
PyMODINIT_FUNC
inittemp_cffi(void)
{
  _cffi_init("temp_cffi", 0x2601, &_cffi_type_context);
}
#endif

#ifdef __GNUC__
#  pragma GCC visibility pop
#endif
