#define _CFFI_

/* We try to define Py_LIMITED_API before including Python.h.

   Mess: we can only define it if Py_DEBUG, Py_TRACE_REFS and
   Py_REF_DEBUG are not defined.  This is a best-effort approximation:
   we can learn about Py_DEBUG from pyconfig.h, but it is unclear if
   the same works for the other two macros.  Py_DEBUG implies them,
   but not the other way around.

   The implementation is messy (issue #350): on Windows, with _MSC_VER,
   we have to define Py_LIMITED_API even before including pyconfig.h.
   In that case, we guess what pyconfig.h will do to the macros above,
   and check our guess after the #include.

   Note that on Windows, with CPython 3.x, you need >= 3.5 and virtualenv
   version >= 16.0.0.  With older versions of either, you don't get a
   copy of PYTHON3.DLL in the virtualenv.  We can't check the version of
   CPython *before* we even include pyconfig.h.  ffi.set_source() puts
   a ``#define _CFFI_NO_LIMITED_API'' at the start of this file if it is
   running on Windows < 3.5, as an attempt at fixing it, but that's
   arguably wrong because it may not be the target version of Python.
   Still better than nothing I guess.  As another workaround, you can
   remove the definition of Py_LIMITED_API here.

   See also 'py_limited_api' in cffi/setuptools_ext.py.
*/
#if !defined(_CFFI_USE_EMBEDDING) && !defined(Py_LIMITED_API)
#  ifdef _MSC_VER
#    if !defined(_DEBUG) && !defined(Py_DEBUG) && !defined(Py_TRACE_REFS) && !defined(Py_REF_DEBUG) && !defined(_CFFI_NO_LIMITED_API)
#      define Py_LIMITED_API
#    endif
#    include <pyconfig.h>
     /* sanity-check: Py_LIMITED_API will cause crashes if any of these
        are also defined.  Normally, the Python file PC/pyconfig.h does not
        cause any of these to be defined, with the exception that _DEBUG
        causes Py_DEBUG.  Double-check that. */
#    ifdef Py_LIMITED_API
#      if defined(Py_DEBUG)
#        error "pyconfig.h unexpectedly defines Py_DEBUG, but Py_LIMITED_API is set"
#      endif
#      if defined(Py_TRACE_REFS)
#        error "pyconfig.h unexpectedly defines Py_TRACE_REFS, but Py_LIMITED_API is set"
#      endif
#      if defined(Py_REF_DEBUG)
#        error "pyconfig.h unexpectedly defines Py_REF_DEBUG, but Py_LIMITED_API is set"
#      endif
#    endif
#  else
#    include <pyconfig.h>
#    if !defined(Py_DEBUG) && !defined(Py_TRACE_REFS) && !defined(Py_REF_DEBUG) && !defined(_CFFI_NO_LIMITED_API)
#      define Py_LIMITED_API
#    endif
#  endif
#endif

#include <Python.h>
#ifdef __cplusplus
extern "C" {
#endif
#include <stddef.h>

/* This part is from file 'cffi/parse_c_type.h'.  It is copied at the
   beginning of C sources generated by CFFI's ffi.set_source(). */

typedef void *_cffi_opcode_t;

#define _CFFI_OP(opcode, arg)   (_cffi_opcode_t)(opcode | (((uintptr_t)(arg)) << 8))
#define _CFFI_GETOP(cffi_opcode)    ((unsigned char)(uintptr_t)cffi_opcode)
#define _CFFI_GETARG(cffi_opcode)   (((intptr_t)cffi_opcode) >> 8)

#define _CFFI_OP_PRIMITIVE       1
#define _CFFI_OP_POINTER         3
#define _CFFI_OP_ARRAY           5
#define _CFFI_OP_OPEN_ARRAY      7
#define _CFFI_OP_STRUCT_UNION    9
#define _CFFI_OP_ENUM           11
#define _CFFI_OP_FUNCTION       13
#define _CFFI_OP_FUNCTION_END   15
#define _CFFI_OP_NOOP           17
#define _CFFI_OP_BITFIELD       19
#define _CFFI_OP_TYPENAME       21
#define _CFFI_OP_CPYTHON_BLTN_V 23   // varargs
#define _CFFI_OP_CPYTHON_BLTN_N 25   // noargs
#define _CFFI_OP_CPYTHON_BLTN_O 27   // O  (i.e. a single arg)
#define _CFFI_OP_CONSTANT       29
#define _CFFI_OP_CONSTANT_INT   31
#define _CFFI_OP_GLOBAL_VAR     33
#define _CFFI_OP_DLOPEN_FUNC    35
#define _CFFI_OP_DLOPEN_CONST   37
#define _CFFI_OP_GLOBAL_VAR_F   39
#define _CFFI_OP_EXTERN_PYTHON  41

#define _CFFI_PRIM_VOID          0
#define _CFFI_PRIM_BOOL          1
#define _CFFI_PRIM_CHAR          2
#define _CFFI_PRIM_SCHAR         3
#define _CFFI_PRIM_UCHAR         4
#define _CFFI_PRIM_SHORT         5
#define _CFFI_PRIM_USHORT        6
#define _CFFI_PRIM_INT           7
#define _CFFI_PRIM_UINT          8
#define _CFFI_PRIM_LONG          9
#define _CFFI_PRIM_ULONG        10
#define _CFFI_PRIM_LONGLONG     11
#define _CFFI_PRIM_ULONGLONG    12
#define _CFFI_PRIM_FLOAT        13
#define _CFFI_PRIM_DOUBLE       14
#define _CFFI_PRIM_LONGDOUBLE   15

#define _CFFI_PRIM_WCHAR        16
#define _CFFI_PRIM_INT8         17
#define _CFFI_PRIM_UINT8        18
#define _CFFI_PRIM_INT16        19
#define _CFFI_PRIM_UINT16       20
#define _CFFI_PRIM_INT32        21
#define _CFFI_PRIM_UINT32       22
#define _CFFI_PRIM_INT64        23
#define _CFFI_PRIM_UINT64       24
#define _CFFI_PRIM_INTPTR       25
#define _CFFI_PRIM_UINTPTR      26
#define _CFFI_PRIM_PTRDIFF      27
#define _CFFI_PRIM_SIZE         28
#define _CFFI_PRIM_SSIZE        29
#define _CFFI_PRIM_INT_LEAST8   30
#define _CFFI_PRIM_UINT_LEAST8  31
#define _CFFI_PRIM_INT_LEAST16  32
#define _CFFI_PRIM_UINT_LEAST16 33
#define _CFFI_PRIM_INT_LEAST32  34
#define _CFFI_PRIM_UINT_LEAST32 35
#define _CFFI_PRIM_INT_LEAST64  36
#define _CFFI_PRIM_UINT_LEAST64 37
#define _CFFI_PRIM_INT_FAST8    38
#define _CFFI_PRIM_UINT_FAST8   39
#define _CFFI_PRIM_INT_FAST16   40
#define _CFFI_PRIM_UINT_FAST16  41
#define _CFFI_PRIM_INT_FAST32   42
#define _CFFI_PRIM_UINT_FAST32  43
#define _CFFI_PRIM_INT_FAST64   44
#define _CFFI_PRIM_UINT_FAST64  45
#define _CFFI_PRIM_INTMAX       46
#define _CFFI_PRIM_UINTMAX      47
#define _CFFI_PRIM_FLOATCOMPLEX 48
#define _CFFI_PRIM_DOUBLECOMPLEX 49
#define _CFFI_PRIM_CHAR16       50
#define _CFFI_PRIM_CHAR32       51

#define _CFFI__NUM_PRIM         52
#define _CFFI__UNKNOWN_PRIM           (-1)
#define _CFFI__UNKNOWN_FLOAT_PRIM     (-2)
#define _CFFI__UNKNOWN_LONG_DOUBLE    (-3)

#define _CFFI__IO_FILE_STRUCT         (-1)


struct _cffi_global_s {
    const char *name;
    void *address;
    _cffi_opcode_t type_op;
    void *size_or_direct_fn;  // OP_GLOBAL_VAR: size, or 0 if unknown
                              // OP_CPYTHON_BLTN_*: addr of direct function
};

struct _cffi_getconst_s {
    unsigned long long value;
    const struct _cffi_type_context_s *ctx;
    int gindex;
};

struct _cffi_struct_union_s {
    const char *name;
    int type_index;          // -> _cffi_types, on a OP_STRUCT_UNION
    int flags;               // _CFFI_F_* flags below
    size_t size;
    int alignment;
    int first_field_index;   // -> _cffi_fields array
    int num_fields;
};
#define _CFFI_F_UNION         0x01   // is a union, not a struct
#define _CFFI_F_CHECK_FIELDS  0x02   // complain if fields are not in the
                                     // "standard layout" or if some are missing
#define _CFFI_F_PACKED        0x04   // for CHECK_FIELDS, assume a packed struct
#define _CFFI_F_EXTERNAL      0x08   // in some other ffi.include()
#define _CFFI_F_OPAQUE        0x10   // opaque

struct _cffi_field_s {
    const char *name;
    size_t field_offset;
    size_t field_size;
    _cffi_opcode_t field_type_op;
};

struct _cffi_enum_s {
    const char *name;
    int type_index;          // -> _cffi_types, on a OP_ENUM
    int type_prim;           // _CFFI_PRIM_xxx
    const char *enumerators; // comma-delimited string
};

struct _cffi_typename_s {
    const char *name;
    int type_index;   /* if opaque, points to a possibly artificial
                         OP_STRUCT which is itself opaque */
};

struct _cffi_type_context_s {
    _cffi_opcode_t *types;
    const struct _cffi_global_s *globals;
    const struct _cffi_field_s *fields;
    const struct _cffi_struct_union_s *struct_unions;
    const struct _cffi_enum_s *enums;
    const struct _cffi_typename_s *typenames;
    int num_globals;
    int num_struct_unions;
    int num_enums;
    int num_typenames;
    const char *const *includes;
    int num_types;
    int flags;      /* future extension */
};

struct _cffi_parse_info_s {
    const struct _cffi_type_context_s *ctx;
    _cffi_opcode_t *output;
    unsigned int output_size;
    size_t error_location;
    const char *error_message;
};

struct _cffi_externpy_s {
    const char *name;
    size_t size_of_result;
    void *reserved1, *reserved2;
};

#ifdef _CFFI_INTERNAL
static int parse_c_type(struct _cffi_parse_info_s *info, const char *input);
static int search_in_globals(const struct _cffi_type_context_s *ctx,
                             const char *search, size_t search_len);
static int search_in_struct_unions(const struct _cffi_type_context_s *ctx,
                                   const char *search, size_t search_len);
#endif

/* this block of #ifs should be kept exactly identical between
   c/_cffi_backend.c, cffi/vengine_cpy.py, cffi/vengine_gen.py
   and cffi/_cffi_include.h */
#if defined(_MSC_VER)
# include <malloc.h>   /* for alloca() */
# if _MSC_VER < 1600   /* MSVC < 2010 */
   typedef __int8 int8_t;
   typedef __int16 int16_t;
   typedef __int32 int32_t;
   typedef __int64 int64_t;
   typedef unsigned __int8 uint8_t;
   typedef unsigned __int16 uint16_t;
   typedef unsigned __int32 uint32_t;
   typedef unsigned __int64 uint64_t;
   typedef __int8 int_least8_t;
   typedef __int16 int_least16_t;
   typedef __int32 int_least32_t;
   typedef __int64 int_least64_t;
   typedef unsigned __int8 uint_least8_t;
   typedef unsigned __int16 uint_least16_t;
   typedef unsigned __int32 uint_least32_t;
   typedef unsigned __int64 uint_least64_t;
   typedef __int8 int_fast8_t;
   typedef __int16 int_fast16_t;
   typedef __int32 int_fast32_t;
   typedef __int64 int_fast64_t;
   typedef unsigned __int8 uint_fast8_t;
   typedef unsigned __int16 uint_fast16_t;
   typedef unsigned __int32 uint_fast32_t;
   typedef unsigned __int64 uint_fast64_t;
   typedef __int64 intmax_t;
   typedef unsigned __int64 uintmax_t;
# else
#  include <stdint.h>
# endif
# if _MSC_VER < 1800   /* MSVC < 2013 */
#  ifndef __cplusplus
    typedef unsigned char _Bool;
#  endif
# endif
#else
# include <stdint.h>
# if (defined (__SVR4) && defined (__sun)) || defined(_AIX) || defined(__hpux)
#  include <alloca.h>
# endif
#endif

#ifdef __GNUC__
# define _CFFI_UNUSED_FN  __attribute__((unused))
#else
# define _CFFI_UNUSED_FN  /* nothing */
#endif

#ifdef __cplusplus
# ifndef _Bool
   typedef bool _Bool;   /* semi-hackish: C++ has no _Bool; bool is builtin */
# endif
#endif

/**********  CPython-specific section  **********/
#ifndef PYPY_VERSION


#if PY_MAJOR_VERSION >= 3
# define PyInt_FromLong PyLong_FromLong
#endif

#define _cffi_from_c_double PyFloat_FromDouble
#define _cffi_from_c_float PyFloat_FromDouble
#define _cffi_from_c_long PyInt_FromLong
#define _cffi_from_c_ulong PyLong_FromUnsignedLong
#define _cffi_from_c_longlong PyLong_FromLongLong
#define _cffi_from_c_ulonglong PyLong_FromUnsignedLongLong
#define _cffi_from_c__Bool PyBool_FromLong

#define _cffi_to_c_double PyFloat_AsDouble
#define _cffi_to_c_float PyFloat_AsDouble

#define _cffi_from_c_int(x, type)                                        \
    (((type)-1) > 0 ? /* unsigned */                                     \
        (sizeof(type) < sizeof(long) ?                                   \
            PyInt_FromLong((long)x) :                                    \
         sizeof(type) == sizeof(long) ?                                  \
            PyLong_FromUnsignedLong((unsigned long)x) :                  \
            PyLong_FromUnsignedLongLong((unsigned long long)x)) :        \
        (sizeof(type) <= sizeof(long) ?                                  \
            PyInt_FromLong((long)x) :                                    \
            PyLong_FromLongLong((long long)x)))

#define _cffi_to_c_int(o, type)                                          \
    ((type)(                                                             \
     sizeof(type) == 1 ? (((type)-1) > 0 ? (type)_cffi_to_c_u8(o)        \
                                         : (type)_cffi_to_c_i8(o)) :     \
     sizeof(type) == 2 ? (((type)-1) > 0 ? (type)_cffi_to_c_u16(o)       \
                                         : (type)_cffi_to_c_i16(o)) :    \
     sizeof(type) == 4 ? (((type)-1) > 0 ? (type)_cffi_to_c_u32(o)       \
                                         : (type)_cffi_to_c_i32(o)) :    \
     sizeof(type) == 8 ? (((type)-1) > 0 ? (type)_cffi_to_c_u64(o)       \
                                         : (type)_cffi_to_c_i64(o)) :    \
     (Py_FatalError("unsupported size for type " #type), (type)0)))

#define _cffi_to_c_i8                                                    \
                 ((int(*)(PyObject *))_cffi_exports[1])
#define _cffi_to_c_u8                                                    \
                 ((int(*)(PyObject *))_cffi_exports[2])
#define _cffi_to_c_i16                                                   \
                 ((int(*)(PyObject *))_cffi_exports[3])
#define _cffi_to_c_u16                                                   \
                 ((int(*)(PyObject *))_cffi_exports[4])
#define _cffi_to_c_i32                                                   \
                 ((int(*)(PyObject *))_cffi_exports[5])
#define _cffi_to_c_u32                                                   \
                 ((unsigned int(*)(PyObject *))_cffi_exports[6])
#define _cffi_to_c_i64                                                   \
                 ((long long(*)(PyObject *))_cffi_exports[7])
#define _cffi_to_c_u64                                                   \
                 ((unsigned long long(*)(PyObject *))_cffi_exports[8])
#define _cffi_to_c_char                                                  \
                 ((int(*)(PyObject *))_cffi_exports[9])
#define _cffi_from_c_pointer                                             \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[10])
#define _cffi_to_c_pointer                                               \
    ((char *(*)(PyObject *, struct _cffi_ctypedescr *))_cffi_exports[11])
#define _cffi_get_struct_layout                                          \
    not used any more
#define _cffi_restore_errno                                              \
    ((void(*)(void))_cffi_exports[13])
#define _cffi_save_errno                                                 \
    ((void(*)(void))_cffi_exports[14])
#define _cffi_from_c_char                                                \
    ((PyObject *(*)(char))_cffi_exports[15])
#define _cffi_from_c_deref                                               \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[16])
#define _cffi_to_c                                                       \
    ((int(*)(char *, struct _cffi_ctypedescr *, PyObject *))_cffi_exports[17])
#define _cffi_from_c_struct                                              \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[18])
#define _cffi_to_c_wchar_t                                               \
    ((_cffi_wchar_t(*)(PyObject *))_cffi_exports[19])
#define _cffi_from_c_wchar_t                                             \
    ((PyObject *(*)(_cffi_wchar_t))_cffi_exports[20])
#define _cffi_to_c_long_double                                           \
    ((long double(*)(PyObject *))_cffi_exports[21])
#define _cffi_to_c__Bool                                                 \
    ((_Bool(*)(PyObject *))_cffi_exports[22])
#define _cffi_prepare_pointer_call_argument                              \
    ((Py_ssize_t(*)(struct _cffi_ctypedescr *,                           \
                    PyObject *, char **))_cffi_exports[23])
#define _cffi_convert_array_from_object                                  \
    ((int(*)(char *, struct _cffi_ctypedescr *, PyObject *))_cffi_exports[24])
#define _CFFI_CPIDX  25
#define _cffi_call_python                                                \
    ((void(*)(struct _cffi_externpy_s *, char *))_cffi_exports[_CFFI_CPIDX])
#define _cffi_to_c_wchar3216_t                                           \
    ((int(*)(PyObject *))_cffi_exports[26])
#define _cffi_from_c_wchar3216_t                                         \
    ((PyObject *(*)(int))_cffi_exports[27])
#define _CFFI_NUM_EXPORTS 28

struct _cffi_ctypedescr;

static void *_cffi_exports[_CFFI_NUM_EXPORTS];

#define _cffi_type(index)   (                           \
    assert((((uintptr_t)_cffi_types[index]) & 1) == 0), \
    (struct _cffi_ctypedescr *)_cffi_types[index])

static PyObject *_cffi_init(const char *module_name, Py_ssize_t version,
                            const struct _cffi_type_context_s *ctx)
{
    PyObject *module, *o_arg, *new_module;
    void *raw[] = {
        (void *)module_name,
        (void *)version,
        (void *)_cffi_exports,
        (void *)ctx,
    };

    module = PyImport_ImportModule("_cffi_backend");
    if (module == NULL)
        goto failure;

    o_arg = PyLong_FromVoidPtr((void *)raw);
    if (o_arg == NULL)
        goto failure;

    new_module = PyObject_CallMethod(
        module, (char *)"_init_cffi_1_0_external_module", (char *)"O", o_arg);

    Py_DECREF(o_arg);
    Py_DECREF(module);
    return new_module;

  failure:
    Py_XDECREF(module);
    return NULL;
}


#ifdef HAVE_WCHAR_H
typedef wchar_t _cffi_wchar_t;
#else
typedef uint16_t _cffi_wchar_t;   /* same random pick as _cffi_backend.c */
#endif

_CFFI_UNUSED_FN static uint16_t _cffi_to_c_char16_t(PyObject *o)
{
    if (sizeof(_cffi_wchar_t) == 2)
        return (uint16_t)_cffi_to_c_wchar_t(o);
    else
        return (uint16_t)_cffi_to_c_wchar3216_t(o);
}

_CFFI_UNUSED_FN static PyObject *_cffi_from_c_char16_t(uint16_t x)
{
    if (sizeof(_cffi_wchar_t) == 2)
        return _cffi_from_c_wchar_t((_cffi_wchar_t)x);
    else
        return _cffi_from_c_wchar3216_t((int)x);
}

_CFFI_UNUSED_FN static int _cffi_to_c_char32_t(PyObject *o)
{
    if (sizeof(_cffi_wchar_t) == 4)
        return (int)_cffi_to_c_wchar_t(o);
    else
        return (int)_cffi_to_c_wchar3216_t(o);
}

_CFFI_UNUSED_FN static PyObject *_cffi_from_c_char32_t(unsigned int x)
{
    if (sizeof(_cffi_wchar_t) == 4)
        return _cffi_from_c_wchar_t((_cffi_wchar_t)x);
    else
        return _cffi_from_c_wchar3216_t((int)x);
}

union _cffi_union_alignment_u {
    unsigned char m_char;
    unsigned short m_short;
    unsigned int m_int;
    unsigned long m_long;
    unsigned long long m_longlong;
    float m_float;
    double m_double;
    long double m_longdouble;
};

struct _cffi_freeme_s {
    struct _cffi_freeme_s *next;
    union _cffi_union_alignment_u alignment;
};

_CFFI_UNUSED_FN static int
_cffi_convert_array_argument(struct _cffi_ctypedescr *ctptr, PyObject *arg,
                             char **output_data, Py_ssize_t datasize,
                             struct _cffi_freeme_s **freeme)
{
    char *p;
    if (datasize < 0)
        return -1;

    p = *output_data;
    if (p == NULL) {
        struct _cffi_freeme_s *fp = (struct _cffi_freeme_s *)PyObject_Malloc(
            offsetof(struct _cffi_freeme_s, alignment) + (size_t)datasize);
        if (fp == NULL)
            return -1;
        fp->next = *freeme;
        *freeme = fp;
        p = *output_data = (char *)&fp->alignment;
    }
    memset((void *)p, 0, (size_t)datasize);
    return _cffi_convert_array_from_object(p, ctptr, arg);
}

_CFFI_UNUSED_FN static void
_cffi_free_array_arguments(struct _cffi_freeme_s *freeme)
{
    do {
        void *p = (void *)freeme;
        freeme = freeme->next;
        PyObject_Free(p);
    } while (freeme != NULL);
}

/**********  end CPython-specific section  **********/
#else
_CFFI_UNUSED_FN
static void (*_cffi_call_python_org)(struct _cffi_externpy_s *, char *);
# define _cffi_call_python  _cffi_call_python_org
#endif


#define _cffi_array_len(array)   (sizeof(array) / sizeof((array)[0]))

#define _cffi_prim_int(size, sign)                                      \
    ((size) == 1 ? ((sign) ? _CFFI_PRIM_INT8  : _CFFI_PRIM_UINT8)  :    \
     (size) == 2 ? ((sign) ? _CFFI_PRIM_INT16 : _CFFI_PRIM_UINT16) :    \
     (size) == 4 ? ((sign) ? _CFFI_PRIM_INT32 : _CFFI_PRIM_UINT32) :    \
     (size) == 8 ? ((sign) ? _CFFI_PRIM_INT64 : _CFFI_PRIM_UINT64) :    \
     _CFFI__UNKNOWN_PRIM)

#define _cffi_prim_float(size)                                          \
    ((size) == sizeof(float) ? _CFFI_PRIM_FLOAT :                       \
     (size) == sizeof(double) ? _CFFI_PRIM_DOUBLE :                     \
     (size) == sizeof(long double) ? _CFFI__UNKNOWN_LONG_DOUBLE :       \
     _CFFI__UNKNOWN_FLOAT_PRIM)

#define _cffi_check_int(got, got_nonpos, expected)      \
    ((got_nonpos) == (expected <= 0) &&                 \
     (got) == (unsigned long long)expected)

#ifdef MS_WIN32
# define _cffi_stdcall  __stdcall
#else
# define _cffi_stdcall  /* nothing */
#endif

#ifdef __cplusplus
}
#endif

/************************************************************/

void f_ini_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[0] = -p[17]*x[0] - y[64] + u[23] + 1;

}
void g_ini_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[0] = 94.615102344455664*y[1] + 184.75059629806526*y[0] - 17.974903844364835*y[3] - 69.368640085185305*y[2] - 40.748687841669557*y[5] - 43.675799641021079*y[4] - 17.974903844364821*y[7] - 69.368640085185291*y[6] - 94.615102344455664*y[9] - 184.75059629806526*y[8] + 17.974903844364835*y[11] + 69.368640085185305*y[10] + 40.748687841669557*y[13] + 43.675799641021079*y[12] + 17.974903844364821*y[15] + 69.368640085185291*y[14] - y[48];
out[1] = 184.75059629806526*y[1] - 94.615102344455664*y[0] - 69.368640085185305*y[3] + 17.974903844364835*y[2] - 43.675799641021079*y[5] + 40.748687841669557*y[4] - 69.368640085185291*y[7] + 17.974903844364821*y[6] - 184.75059629806526*y[9] + 94.615102344455664*y[8] + 69.368640085185305*y[11] - 17.974903844364835*y[10] + 43.675799641021079*y[13] - 40.748687841669557*y[12] + 69.368640085185291*y[15] - 17.974903844364821*y[14] - y[52];
out[2] = -17.974903844364825*y[1] - 69.368640085185291*y[0] + 94.615102344455636*y[3] + 184.7505962980652*y[2] - 17.97490384436481*y[5] - 69.368640085185319*y[4] - 40.748687841669565*y[7] - 43.675799641021079*y[6] + 17.974903844364825*y[9] + 69.368640085185291*y[8] - 94.615102344455636*y[11] - 184.7505962980652*y[10] + 17.97490384436481*y[13] + 69.368640085185319*y[12] + 40.748687841669565*y[15] + 43.675799641021079*y[14] - y[49];
out[3] = -69.368640085185291*y[1] + 17.974903844364825*y[0] + 184.7505962980652*y[3] - 94.615102344455636*y[2] - 69.368640085185319*y[5] + 17.97490384436481*y[4] - 43.675799641021079*y[7] + 40.748687841669565*y[6] + 69.368640085185291*y[9] - 17.974903844364825*y[8] - 184.7505962980652*y[11] + 94.615102344455636*y[10] + 69.368640085185319*y[13] - 17.97490384436481*y[12] + 43.675799641021079*y[15] - 40.748687841669565*y[14] - y[53];
out[4] = -40.748687841669579*y[1] - 43.675799641021101*y[0] - 17.974903844364821*y[3] - 69.368640085185305*y[2] + 94.61510234445565*y[5] + 184.75059629806526*y[4] - 17.974903844364821*y[7] - 69.368640085185305*y[6] + 40.748687841669579*y[9] + 43.675799641021101*y[8] + 17.974903844364821*y[11] + 69.368640085185305*y[10] - 94.61510234445565*y[13] - 184.75059629806526*y[12] + 17.974903844364821*y[15] + 69.368640085185305*y[14] - y[50];
out[5] = -43.675799641021101*y[1] + 40.748687841669579*y[0] - 69.368640085185305*y[3] + 17.974903844364821*y[2] + 184.75059629806526*y[5] - 94.61510234445565*y[4] - 69.368640085185305*y[7] + 17.974903844364821*y[6] + 43.675799641021101*y[9] - 40.748687841669579*y[8] + 69.368640085185305*y[11] - 17.974903844364821*y[10] - 184.75059629806526*y[13] + 94.61510234445565*y[12] + 69.368640085185305*y[15] - 17.974903844364821*y[14] - y[54];
out[6] = -17.974903844364825*y[1] - 69.368640085185305*y[0] - 40.74868784166955*y[3] - 43.675799641021079*y[2] - 17.974903844364839*y[5] - 69.368640085185291*y[4] + 94.61510234445565*y[7] + 184.7505962980652*y[6] + 17.974903844364825*y[9] + 69.368640085185305*y[8] + 40.74868784166955*y[11] + 43.675799641021079*y[10] + 17.974903844364839*y[13] + 69.368640085185291*y[12] - 94.61510234445565*y[15] - 184.7505962980652*y[14] - y[51];
out[7] = -69.368640085185305*y[1] + 17.974903844364825*y[0] - 43.675799641021079*y[3] + 40.74868784166955*y[2] - 69.368640085185291*y[5] + 17.974903844364839*y[4] + 184.7505962980652*y[7] - 94.61510234445565*y[6] + 69.368640085185305*y[9] - 17.974903844364825*y[8] + 43.675799641021079*y[11] - 40.74868784166955*y[10] + 69.368640085185291*y[13] - 17.974903844364839*y[12] - 184.7505962980652*y[15] + 94.61510234445565*y[14] - y[55];
out[8] = -94.615102344455664*y[1] - 184.75059629806526*y[0] + 17.974903844364835*y[3] + 69.368640085185305*y[2] + 40.748687841669557*y[5] + 43.675799641021079*y[4] + 17.974903844364821*y[7] + 69.368640085185291*y[6] + 94.615102344455664*y[9] + 184.75059629806526*y[8] - 17.974903844364835*y[11] - 69.368640085185305*y[10] - 40.748687841669557*y[13] - 43.675799641021079*y[12] - 17.974903844364821*y[15] - 69.368640085185291*y[14] - y[40]*u[12] - y[32];
out[9] = -184.75059629806526*y[1] + 94.615102344455664*y[0] + 69.368640085185305*y[3] - 17.974903844364835*y[2] + 43.675799641021079*y[5] - 40.748687841669557*y[4] + 69.368640085185291*y[7] - 17.974903844364821*y[6] + 184.75059629806526*y[9] - 94.615102344455664*y[8] - 69.368640085185305*y[11] + 17.974903844364835*y[10] - 43.675799641021079*y[13] + 40.748687841669557*y[12] - 69.368640085185291*y[15] + 17.974903844364821*y[14] - y[44]*u[12] - y[33];
out[10] = 17.974903844364825*y[1] + 69.368640085185291*y[0] - 94.615102344455636*y[3] - 184.7505962980652*y[2] + 17.97490384436481*y[5] + 69.368640085185319*y[4] + 40.748687841669565*y[7] + 43.675799641021079*y[6] - 17.974903844364825*y[9] - 69.368640085185291*y[8] + 94.615102344455636*y[11] + 184.7505962980652*y[10] - 17.97490384436481*y[13] - 69.368640085185319*y[12] - 40.748687841669565*y[15] - 43.675799641021079*y[14] - y[41]*u[12] - y[34];
out[11] = 69.368640085185291*y[1] - 17.974903844364825*y[0] - 184.7505962980652*y[3] + 94.615102344455636*y[2] + 69.368640085185319*y[5] - 17.97490384436481*y[4] + 43.675799641021079*y[7] - 40.748687841669565*y[6] - 69.368640085185291*y[9] + 17.974903844364825*y[8] + 184.7505962980652*y[11] - 94.615102344455636*y[10] - 69.368640085185319*y[13] + 17.97490384436481*y[12] - 43.675799641021079*y[15] + 40.748687841669565*y[14] - y[45]*u[12] - y[35];
out[12] = 40.748687841669579*y[1] + 43.675799641021101*y[0] + 17.974903844364821*y[3] + 69.368640085185305*y[2] - 94.61510234445565*y[5] - 184.75059629806526*y[4] + 17.974903844364821*y[7] + 69.368640085185305*y[6] - 40.748687841669579*y[9] - 43.675799641021101*y[8] - 17.974903844364821*y[11] - 69.368640085185305*y[10] + 94.61510234445565*y[13] + 184.75059629806526*y[12] - 17.974903844364821*y[15] - 69.368640085185305*y[14] - y[42]*u[12] - y[36];
out[13] = 43.675799641021101*y[1] - 40.748687841669579*y[0] + 69.368640085185305*y[3] - 17.974903844364821*y[2] - 184.75059629806526*y[5] + 94.61510234445565*y[4] + 69.368640085185305*y[7] - 17.974903844364821*y[6] - 43.675799641021101*y[9] + 40.748687841669579*y[8] - 69.368640085185305*y[11] + 17.974903844364821*y[10] + 184.75059629806526*y[13] - 94.61510234445565*y[12] - 69.368640085185305*y[15] + 17.974903844364821*y[14] - y[46]*u[12] - y[37];
out[14] = 17.974903844364825*y[1] + 69.368640085185305*y[0] + 40.74868784166955*y[3] + 43.675799641021079*y[2] + 17.974903844364839*y[5] + 69.368640085185291*y[4] - 94.61510234445565*y[7] - 184.7505962980652*y[6] - 17.974903844364825*y[9] - 69.368640085185305*y[8] - 40.74868784166955*y[11] - 43.675799641021079*y[10] - 17.974903844364839*y[13] - 69.368640085185291*y[12] + 94.61510234445565*y[15] + 184.7505962980652*y[14] - y[43]*u[12] - y[38];
out[15] = 69.368640085185305*y[1] - 17.974903844364825*y[0] + 43.675799641021079*y[3] - 40.74868784166955*y[2] + 69.368640085185291*y[5] - 17.974903844364839*y[4] - 184.7505962980652*y[7] + 94.61510234445565*y[6] - 69.368640085185305*y[9] + 17.974903844364825*y[8] - 43.675799641021079*y[11] + 40.74868784166955*y[10] - 69.368640085185291*y[13] + 17.974903844364839*y[12] + 184.7505962980652*y[15] - 94.61510234445565*y[14] - y[47]*u[12] - y[39];
out[16] = 94.615102344455664*y[17] + 184.75059629806526*y[16] - 17.974903844364835*y[19] - 69.368640085185305*y[18] - 40.748687841669557*y[21] - 43.675799641021079*y[20] - 17.974903844364821*y[23] - 69.368640085185291*y[22] - 94.615102344455664*y[25] - 184.75059629806526*y[24] + 17.974903844364835*y[27] + 69.368640085185305*y[26] + 40.748687841669557*y[29] + 43.675799641021079*y[28] + 17.974903844364821*y[31] + 69.368640085185291*y[30] + y[40]*u[12];
out[17] = 184.75059629806526*y[17] - 94.615102344455664*y[16] - 69.368640085185305*y[19] + 17.974903844364835*y[18] - 43.675799641021079*y[21] + 40.748687841669557*y[20] - 69.368640085185291*y[23] + 17.974903844364821*y[22] - 184.75059629806526*y[25] + 94.615102344455664*y[24] + 69.368640085185305*y[27] - 17.974903844364835*y[26] + 43.675799641021079*y[29] - 40.748687841669557*y[28] + 69.368640085185291*y[31] - 17.974903844364821*y[30] + y[44]*u[12];
out[18] = -17.974903844364825*y[17] - 69.368640085185291*y[16] + 94.615102344455636*y[19] + 184.7505962980652*y[18] - 17.97490384436481*y[21] - 69.368640085185319*y[20] - 40.748687841669565*y[23] - 43.675799641021079*y[22] + 17.974903844364825*y[25] + 69.368640085185291*y[24] - 94.615102344455636*y[27] - 184.7505962980652*y[26] + 17.97490384436481*y[29] + 69.368640085185319*y[28] + 40.748687841669565*y[31] + 43.675799641021079*y[30] + y[41]*u[12];
out[19] = -69.368640085185291*y[17] + 17.974903844364825*y[16] + 184.7505962980652*y[19] - 94.615102344455636*y[18] - 69.368640085185319*y[21] + 17.97490384436481*y[20] - 43.675799641021079*y[23] + 40.748687841669565*y[22] + 69.368640085185291*y[25] - 17.974903844364825*y[24] - 184.7505962980652*y[27] + 94.615102344455636*y[26] + 69.368640085185319*y[29] - 17.97490384436481*y[28] + 43.675799641021079*y[31] - 40.748687841669565*y[30] + y[45]*u[12];
out[20] = -40.748687841669579*y[17] - 43.675799641021101*y[16] - 17.974903844364821*y[19] - 69.368640085185305*y[18] + 94.61510234445565*y[21] + 184.75059629806526*y[20] - 17.974903844364821*y[23] - 69.368640085185305*y[22] + 40.748687841669579*y[25] + 43.675799641021101*y[24] + 17.974903844364821*y[27] + 69.368640085185305*y[26] - 94.61510234445565*y[29] - 184.75059629806526*y[28] + 17.974903844364821*y[31] + 69.368640085185305*y[30] + y[42]*u[12];
out[21] = -43.675799641021101*y[17] + 40.748687841669579*y[16] - 69.368640085185305*y[19] + 17.974903844364821*y[18] + 184.75059629806526*y[21] - 94.61510234445565*y[20] - 69.368640085185305*y[23] + 17.974903844364821*y[22] + 43.675799641021101*y[25] - 40.748687841669579*y[24] + 69.368640085185305*y[27] - 17.974903844364821*y[26] - 184.75059629806526*y[29] + 94.61510234445565*y[28] + 69.368640085185305*y[31] - 17.974903844364821*y[30] + y[46]*u[12];
out[22] = -17.974903844364825*y[17] - 69.368640085185305*y[16] - 40.74868784166955*y[19] - 43.675799641021079*y[18] - 17.974903844364839*y[21] - 69.368640085185291*y[20] + 94.61510234445565*y[23] + 184.7505962980652*y[22] + 17.974903844364825*y[25] + 69.368640085185305*y[24] + 40.74868784166955*y[27] + 43.675799641021079*y[26] + 17.974903844364839*y[29] + 69.368640085185291*y[28] - 94.61510234445565*y[31] - 184.7505962980652*y[30] + y[43]*u[12];
out[23] = -69.368640085185305*y[17] + 17.974903844364825*y[16] - 43.675799641021079*y[19] + 40.74868784166955*y[18] - 69.368640085185291*y[21] + 17.974903844364839*y[20] + 184.7505962980652*y[23] - 94.61510234445565*y[22] + 69.368640085185305*y[25] - 17.974903844364825*y[24] + 43.675799641021079*y[27] - 40.74868784166955*y[26] + 69.368640085185291*y[29] - 17.974903844364839*y[28] - 184.7505962980652*y[31] + 94.61510234445565*y[30] + y[47]*u[12];
out[24] = -94.615102344455664*y[17] - 184.75059629806526*y[16] + 17.974903844364835*y[19] + 69.368640085185305*y[18] + 40.748687841669557*y[21] + 43.675799641021079*y[20] + 17.974903844364821*y[23] + 69.368640085185291*y[22] + 94.615102344455664*y[25] + 184.75059629806526*y[24] - 17.974903844364835*y[27] - 69.368640085185305*y[26] - 40.748687841669557*y[29] - 43.675799641021079*y[28] - 17.974903844364821*y[31] - 69.368640085185291*y[30] - y[56];
out[25] = -184.75059629806526*y[17] + 94.615102344455664*y[16] + 69.368640085185305*y[19] - 17.974903844364835*y[18] + 43.675799641021079*y[21] - 40.748687841669557*y[20] + 69.368640085185291*y[23] - 17.974903844364821*y[22] + 184.75059629806526*y[25] - 94.615102344455664*y[24] - 69.368640085185305*y[27] + 17.974903844364835*y[26] - 43.675799641021079*y[29] + 40.748687841669557*y[28] - 69.368640085185291*y[31] + 17.974903844364821*y[30] - y[60];
out[26] = 17.974903844364825*y[17] + 69.368640085185291*y[16] - 94.615102344455636*y[19] - 184.7505962980652*y[18] + 17.97490384436481*y[21] + 69.368640085185319*y[20] + 40.748687841669565*y[23] + 43.675799641021079*y[22] - 17.974903844364825*y[25] - 69.368640085185291*y[24] + 94.615102344455636*y[27] + 184.7505962980652*y[26] - 17.97490384436481*y[29] - 69.368640085185319*y[28] - 40.748687841669565*y[31] - 43.675799641021079*y[30] - y[57];
out[27] = 69.368640085185291*y[17] - 17.974903844364825*y[16] - 184.7505962980652*y[19] + 94.615102344455636*y[18] + 69.368640085185319*y[21] - 17.97490384436481*y[20] + 43.675799641021079*y[23] - 40.748687841669565*y[22] - 69.368640085185291*y[25] + 17.974903844364825*y[24] + 184.7505962980652*y[27] - 94.615102344455636*y[26] - 69.368640085185319*y[29] + 17.97490384436481*y[28] - 43.675799641021079*y[31] + 40.748687841669565*y[30] - y[61];
out[28] = 40.748687841669579*y[17] + 43.675799641021101*y[16] + 17.974903844364821*y[19] + 69.368640085185305*y[18] - 94.61510234445565*y[21] - 184.75059629806526*y[20] + 17.974903844364821*y[23] + 69.368640085185305*y[22] - 40.748687841669579*y[25] - 43.675799641021101*y[24] - 17.974903844364821*y[27] - 69.368640085185305*y[26] + 94.61510234445565*y[29] + 184.75059629806526*y[28] - 17.974903844364821*y[31] - 69.368640085185305*y[30] - y[58];
out[29] = 43.675799641021101*y[17] - 40.748687841669579*y[16] + 69.368640085185305*y[19] - 17.974903844364821*y[18] - 184.75059629806526*y[21] + 94.61510234445565*y[20] + 69.368640085185305*y[23] - 17.974903844364821*y[22] - 43.675799641021101*y[25] + 40.748687841669579*y[24] - 69.368640085185305*y[27] + 17.974903844364821*y[26] + 184.75059629806526*y[29] - 94.61510234445565*y[28] - 69.368640085185305*y[31] + 17.974903844364821*y[30] - y[62];
out[30] = 17.974903844364825*y[17] + 69.368640085185305*y[16] + 40.74868784166955*y[19] + 43.675799641021079*y[18] + 17.974903844364839*y[21] + 69.368640085185291*y[20] - 94.61510234445565*y[23] - 184.7505962980652*y[22] - 17.974903844364825*y[25] - 69.368640085185305*y[24] - 40.74868784166955*y[27] - 43.675799641021079*y[26] - 17.974903844364839*y[29] - 69.368640085185291*y[28] + 94.61510234445565*y[31] + 184.7505962980652*y[30] - y[59];
out[31] = 69.368640085185305*y[17] - 17.974903844364825*y[16] + 43.675799641021079*y[19] - 40.74868784166955*y[18] + 69.368640085185291*y[21] - 17.974903844364839*y[20] - 184.7505962980652*y[23] + 94.61510234445565*y[22] - 69.368640085185305*y[25] + 17.974903844364825*y[24] - 43.675799641021079*y[27] + 40.74868784166955*y[26] - 69.368640085185291*y[29] + 17.974903844364839*y[28] + 184.7505962980652*y[31] - 94.61510234445565*y[30] - y[63];
out[32] = p[0]*(pow(y[9], 2)*u[2] - 2*y[9]*y[15]*u[2] + pow(y[8], 2)*u[2] - 2*y[8]*y[14]*u[2] + pow(y[15], 2)*u[2] + pow(y[14], 2)*u[2] + u[0] + (y[9]*y[33] + y[8]*y[32] - y[15]*y[33] - y[14]*y[32])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[33] = p[0]*(pow(y[11], 2)*u[6] - 2*y[11]*y[15]*u[6] + pow(y[10], 2)*u[6] - 2*y[10]*y[14]*u[6] + pow(y[15], 2)*u[6] + pow(y[14], 2)*u[6] + u[4] + (y[11]*y[35] + y[10]*y[34] - y[15]*y[35] - y[14]*y[34])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[34] = p[0]*(pow(y[13], 2)*u[10] - 2*y[13]*y[15]*u[10] + pow(y[12], 2)*u[10] - 2*y[12]*y[14]*u[10] + pow(y[15], 2)*u[10] + pow(y[14], 2)*u[10] + u[8] + (y[13]*y[37] + y[12]*y[36] - y[15]*y[37] - y[14]*y[36])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[35] = p[0]*(-1.0*pow(y[9], 2)*u[3] + 2.0*y[9]*y[15]*u[3] - 1.0*pow(y[8], 2)*u[3] + 2.0*y[8]*y[14]*u[3] - 1.0*pow(y[15], 2)*u[3] - 1.0*pow(y[14], 2)*u[3] + u[1] + (y[9]*y[32] - y[8]*y[33] - y[15]*y[32] + y[14]*y[33])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[36] = p[0]*(-1.0*pow(y[11], 2)*u[7] + 2.0*y[11]*y[15]*u[7] - 1.0*pow(y[10], 2)*u[7] + 2.0*y[10]*y[14]*u[7] - 1.0*pow(y[15], 2)*u[7] - 1.0*pow(y[14], 2)*u[7] + u[5] + (y[11]*y[34] - y[10]*y[35] - y[15]*y[34] + y[14]*y[35])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[37] = p[0]*(-1.0*pow(y[13], 2)*u[11] + 2.0*y[13]*y[15]*u[11] - 1.0*pow(y[12], 2)*u[11] + 2.0*y[12]*y[14]*u[11] - 1.0*pow(y[15], 2)*u[11] - 1.0*pow(y[14], 2)*u[11] + u[9] + (y[13]*y[36] - y[12]*y[37] - y[15]*y[36] + y[14]*y[37])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[38] = y[32] + y[34] + y[36] + y[38];
out[39] = y[33] + y[35] + y[37] + y[39];
out[40] = -p[1]*y[40] + y[8] - y[16] + 1.0*p[2]*y[44];
out[41] = -p[1]*y[41] + y[10] - y[18] + 1.0*p[2]*y[45];
out[42] = -p[1]*y[42] + y[12] - y[20] + 1.0*p[2]*y[46];
out[43] = -p[1]*y[43] + y[14] - y[22] + 1.0*p[2]*y[47];
out[44] = -1.0*p[1]*y[44] + 1.0*y[9] - 1.0*y[17] - 1.0*p[2]*y[40];
out[45] = -1.0*p[1]*y[45] + 1.0*y[11] - 1.0*y[19] - 1.0*p[2]*y[41];
out[46] = -1.0*p[1]*y[46] + 1.0*y[13] - 1.0*y[21] - 1.0*p[2]*y[42];
out[47] = -1.0*p[1]*y[47] + 1.0*y[15] - 1.0*y[23] - 1.0*p[2]*y[43];
out[48] = -p[4]*y[48] - y[0] + 1.0*p[3]*y[52] + u[13]*cos(u[17]);
out[49] = -p[4]*y[49] - y[2] + 1.0*p[3]*y[53] + u[14]*cos(u[17] - 2.0943951023931953);
out[50] = -p[4]*y[50] - y[4] + 1.0*p[3]*y[54] + u[15]*cos(u[17] - 4.1887902047863905);
out[51] = -p[6]*y[51] - y[6] + 1.0*p[5]*y[55];
out[52] = -1.0*p[4]*y[52] - 1.0*y[1] - 1.0*p[3]*y[48] + 1.0*u[13]*sin(u[17]);
out[53] = -1.0*p[4]*y[53] - 1.0*y[3] - 1.0*p[3]*y[49] + 1.0*u[14]*sin(u[17] - 2.0943951023931953);
out[54] = -1.0*p[4]*y[54] - 1.0*y[5] - 1.0*p[3]*y[50] + 1.0*u[15]*sin(u[17] - 4.1887902047863905);
out[55] = -1.0*p[6]*y[55] - 1.0*y[7] - 1.0*p[5]*y[51];
out[56] = -p[10]*y[56] - y[24] + 1.0*p[9]*y[60] + u[18]*cos(u[22]);
out[57] = -p[10]*y[57] - y[26] + 1.0*p[9]*y[61] + u[19]*cos(u[22] - 2.0943951023931953);
out[58] = -p[10]*y[58] - y[28] + 1.0*p[9]*y[62] + u[20]*cos(u[22] - 4.1887902047863905);
out[59] = -p[12]*y[59] - y[30] + 1.0*p[11]*y[63];
out[60] = -1.0*p[10]*y[60] - 1.0*y[25] - 1.0*p[9]*y[56] + 1.0*u[18]*sin(u[22]);
out[61] = -1.0*p[10]*y[61] - 1.0*y[27] - 1.0*p[9]*y[57] + 1.0*u[19]*sin(u[22] - 2.0943951023931953);
out[62] = -1.0*p[10]*y[62] - 1.0*y[29] - 1.0*p[9]*y[58] + 1.0*u[20]*sin(u[22] - 4.1887902047863905);
out[63] = -1.0*p[12]*y[63] - 1.0*y[31] - 1.0*p[11]*y[59];
out[64] = 1.0 - y[64];
out[65] = p[16]*x[0] + p[15]*(1 - y[64]) - y[65];

}
void f_run_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[0] = -p[17]*x[0] - y[64] + u[23] + 1;

}
void g_run_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[0] = 94.615102344455664*y[1] + 184.75059629806526*y[0] - 17.974903844364835*y[3] - 69.368640085185305*y[2] - 40.748687841669557*y[5] - 43.675799641021079*y[4] - 17.974903844364821*y[7] - 69.368640085185291*y[6] - 94.615102344455664*y[9] - 184.75059629806526*y[8] + 17.974903844364835*y[11] + 69.368640085185305*y[10] + 40.748687841669557*y[13] + 43.675799641021079*y[12] + 17.974903844364821*y[15] + 69.368640085185291*y[14] - y[48];
out[1] = 184.75059629806526*y[1] - 94.615102344455664*y[0] - 69.368640085185305*y[3] + 17.974903844364835*y[2] - 43.675799641021079*y[5] + 40.748687841669557*y[4] - 69.368640085185291*y[7] + 17.974903844364821*y[6] - 184.75059629806526*y[9] + 94.615102344455664*y[8] + 69.368640085185305*y[11] - 17.974903844364835*y[10] + 43.675799641021079*y[13] - 40.748687841669557*y[12] + 69.368640085185291*y[15] - 17.974903844364821*y[14] - y[52];
out[2] = -17.974903844364825*y[1] - 69.368640085185291*y[0] + 94.615102344455636*y[3] + 184.7505962980652*y[2] - 17.97490384436481*y[5] - 69.368640085185319*y[4] - 40.748687841669565*y[7] - 43.675799641021079*y[6] + 17.974903844364825*y[9] + 69.368640085185291*y[8] - 94.615102344455636*y[11] - 184.7505962980652*y[10] + 17.97490384436481*y[13] + 69.368640085185319*y[12] + 40.748687841669565*y[15] + 43.675799641021079*y[14] - y[49];
out[3] = -69.368640085185291*y[1] + 17.974903844364825*y[0] + 184.7505962980652*y[3] - 94.615102344455636*y[2] - 69.368640085185319*y[5] + 17.97490384436481*y[4] - 43.675799641021079*y[7] + 40.748687841669565*y[6] + 69.368640085185291*y[9] - 17.974903844364825*y[8] - 184.7505962980652*y[11] + 94.615102344455636*y[10] + 69.368640085185319*y[13] - 17.97490384436481*y[12] + 43.675799641021079*y[15] - 40.748687841669565*y[14] - y[53];
out[4] = -40.748687841669579*y[1] - 43.675799641021101*y[0] - 17.974903844364821*y[3] - 69.368640085185305*y[2] + 94.61510234445565*y[5] + 184.75059629806526*y[4] - 17.974903844364821*y[7] - 69.368640085185305*y[6] + 40.748687841669579*y[9] + 43.675799641021101*y[8] + 17.974903844364821*y[11] + 69.368640085185305*y[10] - 94.61510234445565*y[13] - 184.75059629806526*y[12] + 17.974903844364821*y[15] + 69.368640085185305*y[14] - y[50];
out[5] = -43.675799641021101*y[1] + 40.748687841669579*y[0] - 69.368640085185305*y[3] + 17.974903844364821*y[2] + 184.75059629806526*y[5] - 94.61510234445565*y[4] - 69.368640085185305*y[7] + 17.974903844364821*y[6] + 43.675799641021101*y[9] - 40.748687841669579*y[8] + 69.368640085185305*y[11] - 17.974903844364821*y[10] - 184.75059629806526*y[13] + 94.61510234445565*y[12] + 69.368640085185305*y[15] - 17.974903844364821*y[14] - y[54];
out[6] = -17.974903844364825*y[1] - 69.368640085185305*y[0] - 40.74868784166955*y[3] - 43.675799641021079*y[2] - 17.974903844364839*y[5] - 69.368640085185291*y[4] + 94.61510234445565*y[7] + 184.7505962980652*y[6] + 17.974903844364825*y[9] + 69.368640085185305*y[8] + 40.74868784166955*y[11] + 43.675799641021079*y[10] + 17.974903844364839*y[13] + 69.368640085185291*y[12] - 94.61510234445565*y[15] - 184.7505962980652*y[14] - y[51];
out[7] = -69.368640085185305*y[1] + 17.974903844364825*y[0] - 43.675799641021079*y[3] + 40.74868784166955*y[2] - 69.368640085185291*y[5] + 17.974903844364839*y[4] + 184.7505962980652*y[7] - 94.61510234445565*y[6] + 69.368640085185305*y[9] - 17.974903844364825*y[8] + 43.675799641021079*y[11] - 40.74868784166955*y[10] + 69.368640085185291*y[13] - 17.974903844364839*y[12] - 184.7505962980652*y[15] + 94.61510234445565*y[14] - y[55];
out[8] = -94.615102344455664*y[1] - 184.75059629806526*y[0] + 17.974903844364835*y[3] + 69.368640085185305*y[2] + 40.748687841669557*y[5] + 43.675799641021079*y[4] + 17.974903844364821*y[7] + 69.368640085185291*y[6] + 94.615102344455664*y[9] + 184.75059629806526*y[8] - 17.974903844364835*y[11] - 69.368640085185305*y[10] - 40.748687841669557*y[13] - 43.675799641021079*y[12] - 17.974903844364821*y[15] - 69.368640085185291*y[14] - y[40]*u[12] - y[32];
out[9] = -184.75059629806526*y[1] + 94.615102344455664*y[0] + 69.368640085185305*y[3] - 17.974903844364835*y[2] + 43.675799641021079*y[5] - 40.748687841669557*y[4] + 69.368640085185291*y[7] - 17.974903844364821*y[6] + 184.75059629806526*y[9] - 94.615102344455664*y[8] - 69.368640085185305*y[11] + 17.974903844364835*y[10] - 43.675799641021079*y[13] + 40.748687841669557*y[12] - 69.368640085185291*y[15] + 17.974903844364821*y[14] - y[44]*u[12] - y[33];
out[10] = 17.974903844364825*y[1] + 69.368640085185291*y[0] - 94.615102344455636*y[3] - 184.7505962980652*y[2] + 17.97490384436481*y[5] + 69.368640085185319*y[4] + 40.748687841669565*y[7] + 43.675799641021079*y[6] - 17.974903844364825*y[9] - 69.368640085185291*y[8] + 94.615102344455636*y[11] + 184.7505962980652*y[10] - 17.97490384436481*y[13] - 69.368640085185319*y[12] - 40.748687841669565*y[15] - 43.675799641021079*y[14] - y[41]*u[12] - y[34];
out[11] = 69.368640085185291*y[1] - 17.974903844364825*y[0] - 184.7505962980652*y[3] + 94.615102344455636*y[2] + 69.368640085185319*y[5] - 17.97490384436481*y[4] + 43.675799641021079*y[7] - 40.748687841669565*y[6] - 69.368640085185291*y[9] + 17.974903844364825*y[8] + 184.7505962980652*y[11] - 94.615102344455636*y[10] - 69.368640085185319*y[13] + 17.97490384436481*y[12] - 43.675799641021079*y[15] + 40.748687841669565*y[14] - y[45]*u[12] - y[35];
out[12] = 40.748687841669579*y[1] + 43.675799641021101*y[0] + 17.974903844364821*y[3] + 69.368640085185305*y[2] - 94.61510234445565*y[5] - 184.75059629806526*y[4] + 17.974903844364821*y[7] + 69.368640085185305*y[6] - 40.748687841669579*y[9] - 43.675799641021101*y[8] - 17.974903844364821*y[11] - 69.368640085185305*y[10] + 94.61510234445565*y[13] + 184.75059629806526*y[12] - 17.974903844364821*y[15] - 69.368640085185305*y[14] - y[42]*u[12] - y[36];
out[13] = 43.675799641021101*y[1] - 40.748687841669579*y[0] + 69.368640085185305*y[3] - 17.974903844364821*y[2] - 184.75059629806526*y[5] + 94.61510234445565*y[4] + 69.368640085185305*y[7] - 17.974903844364821*y[6] - 43.675799641021101*y[9] + 40.748687841669579*y[8] - 69.368640085185305*y[11] + 17.974903844364821*y[10] + 184.75059629806526*y[13] - 94.61510234445565*y[12] - 69.368640085185305*y[15] + 17.974903844364821*y[14] - y[46]*u[12] - y[37];
out[14] = 17.974903844364825*y[1] + 69.368640085185305*y[0] + 40.74868784166955*y[3] + 43.675799641021079*y[2] + 17.974903844364839*y[5] + 69.368640085185291*y[4] - 94.61510234445565*y[7] - 184.7505962980652*y[6] - 17.974903844364825*y[9] - 69.368640085185305*y[8] - 40.74868784166955*y[11] - 43.675799641021079*y[10] - 17.974903844364839*y[13] - 69.368640085185291*y[12] + 94.61510234445565*y[15] + 184.7505962980652*y[14] - y[43]*u[12] - y[38];
out[15] = 69.368640085185305*y[1] - 17.974903844364825*y[0] + 43.675799641021079*y[3] - 40.74868784166955*y[2] + 69.368640085185291*y[5] - 17.974903844364839*y[4] - 184.7505962980652*y[7] + 94.61510234445565*y[6] - 69.368640085185305*y[9] + 17.974903844364825*y[8] - 43.675799641021079*y[11] + 40.74868784166955*y[10] - 69.368640085185291*y[13] + 17.974903844364839*y[12] + 184.7505962980652*y[15] - 94.61510234445565*y[14] - y[47]*u[12] - y[39];
out[16] = 94.615102344455664*y[17] + 184.75059629806526*y[16] - 17.974903844364835*y[19] - 69.368640085185305*y[18] - 40.748687841669557*y[21] - 43.675799641021079*y[20] - 17.974903844364821*y[23] - 69.368640085185291*y[22] - 94.615102344455664*y[25] - 184.75059629806526*y[24] + 17.974903844364835*y[27] + 69.368640085185305*y[26] + 40.748687841669557*y[29] + 43.675799641021079*y[28] + 17.974903844364821*y[31] + 69.368640085185291*y[30] + y[40]*u[12];
out[17] = 184.75059629806526*y[17] - 94.615102344455664*y[16] - 69.368640085185305*y[19] + 17.974903844364835*y[18] - 43.675799641021079*y[21] + 40.748687841669557*y[20] - 69.368640085185291*y[23] + 17.974903844364821*y[22] - 184.75059629806526*y[25] + 94.615102344455664*y[24] + 69.368640085185305*y[27] - 17.974903844364835*y[26] + 43.675799641021079*y[29] - 40.748687841669557*y[28] + 69.368640085185291*y[31] - 17.974903844364821*y[30] + y[44]*u[12];
out[18] = -17.974903844364825*y[17] - 69.368640085185291*y[16] + 94.615102344455636*y[19] + 184.7505962980652*y[18] - 17.97490384436481*y[21] - 69.368640085185319*y[20] - 40.748687841669565*y[23] - 43.675799641021079*y[22] + 17.974903844364825*y[25] + 69.368640085185291*y[24] - 94.615102344455636*y[27] - 184.7505962980652*y[26] + 17.97490384436481*y[29] + 69.368640085185319*y[28] + 40.748687841669565*y[31] + 43.675799641021079*y[30] + y[41]*u[12];
out[19] = -69.368640085185291*y[17] + 17.974903844364825*y[16] + 184.7505962980652*y[19] - 94.615102344455636*y[18] - 69.368640085185319*y[21] + 17.97490384436481*y[20] - 43.675799641021079*y[23] + 40.748687841669565*y[22] + 69.368640085185291*y[25] - 17.974903844364825*y[24] - 184.7505962980652*y[27] + 94.615102344455636*y[26] + 69.368640085185319*y[29] - 17.97490384436481*y[28] + 43.675799641021079*y[31] - 40.748687841669565*y[30] + y[45]*u[12];
out[20] = -40.748687841669579*y[17] - 43.675799641021101*y[16] - 17.974903844364821*y[19] - 69.368640085185305*y[18] + 94.61510234445565*y[21] + 184.75059629806526*y[20] - 17.974903844364821*y[23] - 69.368640085185305*y[22] + 40.748687841669579*y[25] + 43.675799641021101*y[24] + 17.974903844364821*y[27] + 69.368640085185305*y[26] - 94.61510234445565*y[29] - 184.75059629806526*y[28] + 17.974903844364821*y[31] + 69.368640085185305*y[30] + y[42]*u[12];
out[21] = -43.675799641021101*y[17] + 40.748687841669579*y[16] - 69.368640085185305*y[19] + 17.974903844364821*y[18] + 184.75059629806526*y[21] - 94.61510234445565*y[20] - 69.368640085185305*y[23] + 17.974903844364821*y[22] + 43.675799641021101*y[25] - 40.748687841669579*y[24] + 69.368640085185305*y[27] - 17.974903844364821*y[26] - 184.75059629806526*y[29] + 94.61510234445565*y[28] + 69.368640085185305*y[31] - 17.974903844364821*y[30] + y[46]*u[12];
out[22] = -17.974903844364825*y[17] - 69.368640085185305*y[16] - 40.74868784166955*y[19] - 43.675799641021079*y[18] - 17.974903844364839*y[21] - 69.368640085185291*y[20] + 94.61510234445565*y[23] + 184.7505962980652*y[22] + 17.974903844364825*y[25] + 69.368640085185305*y[24] + 40.74868784166955*y[27] + 43.675799641021079*y[26] + 17.974903844364839*y[29] + 69.368640085185291*y[28] - 94.61510234445565*y[31] - 184.7505962980652*y[30] + y[43]*u[12];
out[23] = -69.368640085185305*y[17] + 17.974903844364825*y[16] - 43.675799641021079*y[19] + 40.74868784166955*y[18] - 69.368640085185291*y[21] + 17.974903844364839*y[20] + 184.7505962980652*y[23] - 94.61510234445565*y[22] + 69.368640085185305*y[25] - 17.974903844364825*y[24] + 43.675799641021079*y[27] - 40.74868784166955*y[26] + 69.368640085185291*y[29] - 17.974903844364839*y[28] - 184.7505962980652*y[31] + 94.61510234445565*y[30] + y[47]*u[12];
out[24] = -94.615102344455664*y[17] - 184.75059629806526*y[16] + 17.974903844364835*y[19] + 69.368640085185305*y[18] + 40.748687841669557*y[21] + 43.675799641021079*y[20] + 17.974903844364821*y[23] + 69.368640085185291*y[22] + 94.615102344455664*y[25] + 184.75059629806526*y[24] - 17.974903844364835*y[27] - 69.368640085185305*y[26] - 40.748687841669557*y[29] - 43.675799641021079*y[28] - 17.974903844364821*y[31] - 69.368640085185291*y[30] - y[56];
out[25] = -184.75059629806526*y[17] + 94.615102344455664*y[16] + 69.368640085185305*y[19] - 17.974903844364835*y[18] + 43.675799641021079*y[21] - 40.748687841669557*y[20] + 69.368640085185291*y[23] - 17.974903844364821*y[22] + 184.75059629806526*y[25] - 94.615102344455664*y[24] - 69.368640085185305*y[27] + 17.974903844364835*y[26] - 43.675799641021079*y[29] + 40.748687841669557*y[28] - 69.368640085185291*y[31] + 17.974903844364821*y[30] - y[60];
out[26] = 17.974903844364825*y[17] + 69.368640085185291*y[16] - 94.615102344455636*y[19] - 184.7505962980652*y[18] + 17.97490384436481*y[21] + 69.368640085185319*y[20] + 40.748687841669565*y[23] + 43.675799641021079*y[22] - 17.974903844364825*y[25] - 69.368640085185291*y[24] + 94.615102344455636*y[27] + 184.7505962980652*y[26] - 17.97490384436481*y[29] - 69.368640085185319*y[28] - 40.748687841669565*y[31] - 43.675799641021079*y[30] - y[57];
out[27] = 69.368640085185291*y[17] - 17.974903844364825*y[16] - 184.7505962980652*y[19] + 94.615102344455636*y[18] + 69.368640085185319*y[21] - 17.97490384436481*y[20] + 43.675799641021079*y[23] - 40.748687841669565*y[22] - 69.368640085185291*y[25] + 17.974903844364825*y[24] + 184.7505962980652*y[27] - 94.615102344455636*y[26] - 69.368640085185319*y[29] + 17.97490384436481*y[28] - 43.675799641021079*y[31] + 40.748687841669565*y[30] - y[61];
out[28] = 40.748687841669579*y[17] + 43.675799641021101*y[16] + 17.974903844364821*y[19] + 69.368640085185305*y[18] - 94.61510234445565*y[21] - 184.75059629806526*y[20] + 17.974903844364821*y[23] + 69.368640085185305*y[22] - 40.748687841669579*y[25] - 43.675799641021101*y[24] - 17.974903844364821*y[27] - 69.368640085185305*y[26] + 94.61510234445565*y[29] + 184.75059629806526*y[28] - 17.974903844364821*y[31] - 69.368640085185305*y[30] - y[58];
out[29] = 43.675799641021101*y[17] - 40.748687841669579*y[16] + 69.368640085185305*y[19] - 17.974903844364821*y[18] - 184.75059629806526*y[21] + 94.61510234445565*y[20] + 69.368640085185305*y[23] - 17.974903844364821*y[22] - 43.675799641021101*y[25] + 40.748687841669579*y[24] - 69.368640085185305*y[27] + 17.974903844364821*y[26] + 184.75059629806526*y[29] - 94.61510234445565*y[28] - 69.368640085185305*y[31] + 17.974903844364821*y[30] - y[62];
out[30] = 17.974903844364825*y[17] + 69.368640085185305*y[16] + 40.74868784166955*y[19] + 43.675799641021079*y[18] + 17.974903844364839*y[21] + 69.368640085185291*y[20] - 94.61510234445565*y[23] - 184.7505962980652*y[22] - 17.974903844364825*y[25] - 69.368640085185305*y[24] - 40.74868784166955*y[27] - 43.675799641021079*y[26] - 17.974903844364839*y[29] - 69.368640085185291*y[28] + 94.61510234445565*y[31] + 184.7505962980652*y[30] - y[59];
out[31] = 69.368640085185305*y[17] - 17.974903844364825*y[16] + 43.675799641021079*y[19] - 40.74868784166955*y[18] + 69.368640085185291*y[21] - 17.974903844364839*y[20] - 184.7505962980652*y[23] + 94.61510234445565*y[22] - 69.368640085185305*y[25] + 17.974903844364825*y[24] - 43.675799641021079*y[27] + 40.74868784166955*y[26] - 69.368640085185291*y[29] + 17.974903844364839*y[28] + 184.7505962980652*y[31] - 94.61510234445565*y[30] - y[63];
out[32] = p[0]*(pow(y[9], 2)*u[2] - 2*y[9]*y[15]*u[2] + pow(y[8], 2)*u[2] - 2*y[8]*y[14]*u[2] + pow(y[15], 2)*u[2] + pow(y[14], 2)*u[2] + u[0] + (y[9]*y[33] + y[8]*y[32] - y[15]*y[33] - y[14]*y[32])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[33] = p[0]*(pow(y[11], 2)*u[6] - 2*y[11]*y[15]*u[6] + pow(y[10], 2)*u[6] - 2*y[10]*y[14]*u[6] + pow(y[15], 2)*u[6] + pow(y[14], 2)*u[6] + u[4] + (y[11]*y[35] + y[10]*y[34] - y[15]*y[35] - y[14]*y[34])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[34] = p[0]*(pow(y[13], 2)*u[10] - 2*y[13]*y[15]*u[10] + pow(y[12], 2)*u[10] - 2*y[12]*y[14]*u[10] + pow(y[15], 2)*u[10] + pow(y[14], 2)*u[10] + u[8] + (y[13]*y[37] + y[12]*y[36] - y[15]*y[37] - y[14]*y[36])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[35] = p[0]*(-1.0*pow(y[9], 2)*u[3] + 2.0*y[9]*y[15]*u[3] - 1.0*pow(y[8], 2)*u[3] + 2.0*y[8]*y[14]*u[3] - 1.0*pow(y[15], 2)*u[3] - 1.0*pow(y[14], 2)*u[3] + u[1] + (y[9]*y[32] - y[8]*y[33] - y[15]*y[32] + y[14]*y[33])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[36] = p[0]*(-1.0*pow(y[11], 2)*u[7] + 2.0*y[11]*y[15]*u[7] - 1.0*pow(y[10], 2)*u[7] + 2.0*y[10]*y[14]*u[7] - 1.0*pow(y[15], 2)*u[7] - 1.0*pow(y[14], 2)*u[7] + u[5] + (y[11]*y[34] - y[10]*y[35] - y[15]*y[34] + y[14]*y[35])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[37] = p[0]*(-1.0*pow(y[13], 2)*u[11] + 2.0*y[13]*y[15]*u[11] - 1.0*pow(y[12], 2)*u[11] + 2.0*y[12]*y[14]*u[11] - 1.0*pow(y[15], 2)*u[11] - 1.0*pow(y[14], 2)*u[11] + u[9] + (y[13]*y[36] - y[12]*y[37] - y[15]*y[36] + y[14]*y[37])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[38] = y[32] + y[34] + y[36] + y[38];
out[39] = y[33] + y[35] + y[37] + y[39];
out[40] = -p[1]*y[40] + y[8] - y[16] + 1.0*p[2]*y[44];
out[41] = -p[1]*y[41] + y[10] - y[18] + 1.0*p[2]*y[45];
out[42] = -p[1]*y[42] + y[12] - y[20] + 1.0*p[2]*y[46];
out[43] = -p[1]*y[43] + y[14] - y[22] + 1.0*p[2]*y[47];
out[44] = -1.0*p[1]*y[44] + 1.0*y[9] - 1.0*y[17] - 1.0*p[2]*y[40];
out[45] = -1.0*p[1]*y[45] + 1.0*y[11] - 1.0*y[19] - 1.0*p[2]*y[41];
out[46] = -1.0*p[1]*y[46] + 1.0*y[13] - 1.0*y[21] - 1.0*p[2]*y[42];
out[47] = -1.0*p[1]*y[47] + 1.0*y[15] - 1.0*y[23] - 1.0*p[2]*y[43];
out[48] = -p[4]*y[48] - y[0] + 1.0*p[3]*y[52] + u[13]*cos(u[17]);
out[49] = -p[4]*y[49] - y[2] + 1.0*p[3]*y[53] + u[14]*cos(u[17] - 2.0943951023931953);
out[50] = -p[4]*y[50] - y[4] + 1.0*p[3]*y[54] + u[15]*cos(u[17] - 4.1887902047863905);
out[51] = -p[6]*y[51] - y[6] + 1.0*p[5]*y[55];
out[52] = -1.0*p[4]*y[52] - 1.0*y[1] - 1.0*p[3]*y[48] + 1.0*u[13]*sin(u[17]);
out[53] = -1.0*p[4]*y[53] - 1.0*y[3] - 1.0*p[3]*y[49] + 1.0*u[14]*sin(u[17] - 2.0943951023931953);
out[54] = -1.0*p[4]*y[54] - 1.0*y[5] - 1.0*p[3]*y[50] + 1.0*u[15]*sin(u[17] - 4.1887902047863905);
out[55] = -1.0*p[6]*y[55] - 1.0*y[7] - 1.0*p[5]*y[51];
out[56] = -p[10]*y[56] - y[24] + 1.0*p[9]*y[60] + u[18]*cos(u[22]);
out[57] = -p[10]*y[57] - y[26] + 1.0*p[9]*y[61] + u[19]*cos(u[22] - 2.0943951023931953);
out[58] = -p[10]*y[58] - y[28] + 1.0*p[9]*y[62] + u[20]*cos(u[22] - 4.1887902047863905);
out[59] = -p[12]*y[59] - y[30] + 1.0*p[11]*y[63];
out[60] = -1.0*p[10]*y[60] - 1.0*y[25] - 1.0*p[9]*y[56] + 1.0*u[18]*sin(u[22]);
out[61] = -1.0*p[10]*y[61] - 1.0*y[27] - 1.0*p[9]*y[57] + 1.0*u[19]*sin(u[22] - 2.0943951023931953);
out[62] = -1.0*p[10]*y[62] - 1.0*y[29] - 1.0*p[9]*y[58] + 1.0*u[20]*sin(u[22] - 4.1887902047863905);
out[63] = -1.0*p[12]*y[63] - 1.0*y[31] - 1.0*p[11]*y[59];
out[64] = 1.0 - y[64];
out[65] = p[16]*x[0] + p[15]*(1 - y[64]) - y[65];

}
void h_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[0] = 94.615102344455664*y[1] + 184.75059629806526*y[0] - 17.974903844364835*y[3] - 69.368640085185305*y[2] - 40.748687841669557*y[5] - 43.675799641021079*y[4] - 17.974903844364821*y[7] - 69.368640085185291*y[6] - 94.615102344455664*y[9] - 184.75059629806526*y[8] + 17.974903844364835*y[11] + 69.368640085185305*y[10] + 40.748687841669557*y[13] + 43.675799641021079*y[12] + 17.974903844364821*y[15] + 69.368640085185291*y[14];
out[1] = 184.75059629806526*y[1] - 94.615102344455664*y[0] - 69.368640085185305*y[3] + 17.974903844364835*y[2] - 43.675799641021079*y[5] + 40.748687841669557*y[4] - 69.368640085185291*y[7] + 17.974903844364821*y[6] - 184.75059629806526*y[9] + 94.615102344455664*y[8] + 69.368640085185305*y[11] - 17.974903844364835*y[10] + 43.675799641021079*y[13] - 40.748687841669557*y[12] + 69.368640085185291*y[15] - 17.974903844364821*y[14];
out[2] = -17.974903844364825*y[1] - 69.368640085185291*y[0] + 94.615102344455636*y[3] + 184.7505962980652*y[2] - 17.97490384436481*y[5] - 69.368640085185319*y[4] - 40.748687841669565*y[7] - 43.675799641021079*y[6] + 17.974903844364825*y[9] + 69.368640085185291*y[8] - 94.615102344455636*y[11] - 184.7505962980652*y[10] + 17.97490384436481*y[13] + 69.368640085185319*y[12] + 40.748687841669565*y[15] + 43.675799641021079*y[14];
out[3] = -69.368640085185291*y[1] + 17.974903844364825*y[0] + 184.7505962980652*y[3] - 94.615102344455636*y[2] - 69.368640085185319*y[5] + 17.97490384436481*y[4] - 43.675799641021079*y[7] + 40.748687841669565*y[6] + 69.368640085185291*y[9] - 17.974903844364825*y[8] - 184.7505962980652*y[11] + 94.615102344455636*y[10] + 69.368640085185319*y[13] - 17.97490384436481*y[12] + 43.675799641021079*y[15] - 40.748687841669565*y[14];
out[4] = -40.748687841669579*y[1] - 43.675799641021101*y[0] - 17.974903844364821*y[3] - 69.368640085185305*y[2] + 94.61510234445565*y[5] + 184.75059629806526*y[4] - 17.974903844364821*y[7] - 69.368640085185305*y[6] + 40.748687841669579*y[9] + 43.675799641021101*y[8] + 17.974903844364821*y[11] + 69.368640085185305*y[10] - 94.61510234445565*y[13] - 184.75059629806526*y[12] + 17.974903844364821*y[15] + 69.368640085185305*y[14];
out[5] = -43.675799641021101*y[1] + 40.748687841669579*y[0] - 69.368640085185305*y[3] + 17.974903844364821*y[2] + 184.75059629806526*y[5] - 94.61510234445565*y[4] - 69.368640085185305*y[7] + 17.974903844364821*y[6] + 43.675799641021101*y[9] - 40.748687841669579*y[8] + 69.368640085185305*y[11] - 17.974903844364821*y[10] - 184.75059629806526*y[13] + 94.61510234445565*y[12] + 69.368640085185305*y[15] - 17.974903844364821*y[14];
out[6] = -17.974903844364825*y[1] - 69.368640085185305*y[0] - 40.74868784166955*y[3] - 43.675799641021079*y[2] - 17.974903844364839*y[5] - 69.368640085185291*y[4] + 94.61510234445565*y[7] + 184.7505962980652*y[6] + 17.974903844364825*y[9] + 69.368640085185305*y[8] + 40.74868784166955*y[11] + 43.675799641021079*y[10] + 17.974903844364839*y[13] + 69.368640085185291*y[12] - 94.61510234445565*y[15] - 184.7505962980652*y[14];
out[7] = -69.368640085185305*y[1] + 17.974903844364825*y[0] - 43.675799641021079*y[3] + 40.74868784166955*y[2] - 69.368640085185291*y[5] + 17.974903844364839*y[4] + 184.7505962980652*y[7] - 94.61510234445565*y[6] + 69.368640085185305*y[9] - 17.974903844364825*y[8] + 43.675799641021079*y[11] - 40.74868784166955*y[10] + 69.368640085185291*y[13] - 17.974903844364839*y[12] - 184.7505962980652*y[15] + 94.61510234445565*y[14];
out[8] = 94.615102344455664*y[17] + 184.75059629806526*y[16] - 17.974903844364835*y[19] - 69.368640085185305*y[18] - 40.748687841669557*y[21] - 43.675799641021079*y[20] - 17.974903844364821*y[23] - 69.368640085185291*y[22] - 94.615102344455664*y[25] - 184.75059629806526*y[24] + 17.974903844364835*y[27] + 69.368640085185305*y[26] + 40.748687841669557*y[29] + 43.675799641021079*y[28] + 17.974903844364821*y[31] + 69.368640085185291*y[30];
out[9] = 184.75059629806526*y[17] - 94.615102344455664*y[16] - 69.368640085185305*y[19] + 17.974903844364835*y[18] - 43.675799641021079*y[21] + 40.748687841669557*y[20] - 69.368640085185291*y[23] + 17.974903844364821*y[22] - 184.75059629806526*y[25] + 94.615102344455664*y[24] + 69.368640085185305*y[27] - 17.974903844364835*y[26] + 43.675799641021079*y[29] - 40.748687841669557*y[28] + 69.368640085185291*y[31] - 17.974903844364821*y[30];
out[10] = -17.974903844364825*y[17] - 69.368640085185291*y[16] + 94.615102344455636*y[19] + 184.7505962980652*y[18] - 17.97490384436481*y[21] - 69.368640085185319*y[20] - 40.748687841669565*y[23] - 43.675799641021079*y[22] + 17.974903844364825*y[25] + 69.368640085185291*y[24] - 94.615102344455636*y[27] - 184.7505962980652*y[26] + 17.97490384436481*y[29] + 69.368640085185319*y[28] + 40.748687841669565*y[31] + 43.675799641021079*y[30];
out[11] = -69.368640085185291*y[17] + 17.974903844364825*y[16] + 184.7505962980652*y[19] - 94.615102344455636*y[18] - 69.368640085185319*y[21] + 17.97490384436481*y[20] - 43.675799641021079*y[23] + 40.748687841669565*y[22] + 69.368640085185291*y[25] - 17.974903844364825*y[24] - 184.7505962980652*y[27] + 94.615102344455636*y[26] + 69.368640085185319*y[29] - 17.97490384436481*y[28] + 43.675799641021079*y[31] - 40.748687841669565*y[30];
out[12] = -40.748687841669579*y[17] - 43.675799641021101*y[16] - 17.974903844364821*y[19] - 69.368640085185305*y[18] + 94.61510234445565*y[21] + 184.75059629806526*y[20] - 17.974903844364821*y[23] - 69.368640085185305*y[22] + 40.748687841669579*y[25] + 43.675799641021101*y[24] + 17.974903844364821*y[27] + 69.368640085185305*y[26] - 94.61510234445565*y[29] - 184.75059629806526*y[28] + 17.974903844364821*y[31] + 69.368640085185305*y[30];
out[13] = -43.675799641021101*y[17] + 40.748687841669579*y[16] - 69.368640085185305*y[19] + 17.974903844364821*y[18] + 184.75059629806526*y[21] - 94.61510234445565*y[20] - 69.368640085185305*y[23] + 17.974903844364821*y[22] + 43.675799641021101*y[25] - 40.748687841669579*y[24] + 69.368640085185305*y[27] - 17.974903844364821*y[26] - 184.75059629806526*y[29] + 94.61510234445565*y[28] + 69.368640085185305*y[31] - 17.974903844364821*y[30];
out[14] = -17.974903844364825*y[17] - 69.368640085185305*y[16] - 40.74868784166955*y[19] - 43.675799641021079*y[18] - 17.974903844364839*y[21] - 69.368640085185291*y[20] + 94.61510234445565*y[23] + 184.7505962980652*y[22] + 17.974903844364825*y[25] + 69.368640085185305*y[24] + 40.74868784166955*y[27] + 43.675799641021079*y[26] + 17.974903844364839*y[29] + 69.368640085185291*y[28] - 94.61510234445565*y[31] - 184.7505962980652*y[30];
out[15] = -69.368640085185305*y[17] + 17.974903844364825*y[16] - 43.675799641021079*y[19] + 40.74868784166955*y[18] - 69.368640085185291*y[21] + 17.974903844364839*y[20] + 184.7505962980652*y[23] - 94.61510234445565*y[22] + 69.368640085185305*y[25] - 17.974903844364825*y[24] + 43.675799641021079*y[27] - 40.74868784166955*y[26] + 69.368640085185291*y[29] - 17.974903844364839*y[28] - 184.7505962980652*y[31] + 94.61510234445565*y[30];
out[16] = sqrt(1.0*pow(y[44], 2) + pow(y[40], 2));
out[17] = sqrt(1.0*pow(y[45], 2) + pow(y[41], 2));
out[18] = sqrt(1.0*pow(y[46], 2) + pow(y[42], 2));
out[19] = sqrt(1.0*pow(y[47], 2) + pow(y[43], 2));
out[20] = sqrt(1.0*pow(y[44], 2) + pow(y[40], 2));
out[21] = sqrt(1.0*pow(y[45], 2) + pow(y[41], 2));
out[22] = sqrt(1.0*pow(y[46], 2) + pow(y[42], 2));
out[23] = sqrt(1.0*pow(y[47], 2) + pow(y[43], 2));
out[24] = sqrt(1.0*pow(y[52], 2) + pow(y[48], 2));
out[25] = sqrt(1.0*pow(y[53], 2) + pow(y[49], 2));
out[26] = sqrt(1.0*pow(y[54], 2) + pow(y[50], 2));
out[27] = sqrt(1.0*pow(y[55], 2) + pow(y[51], 2));
out[28] = 1.0*y[1]*y[52] + y[0]*y[48] + 1.0*y[3]*y[53] + y[2]*y[49] + 1.0*y[5]*y[54] + y[4]*y[50];
out[29] = 1.0*y[1]*y[48] - 1.0*y[0]*y[52] + 1.0*y[3]*y[49] - 1.0*y[2]*y[53] + 1.0*y[5]*y[50] - 1.0*y[4]*y[54];
out[30] = sqrt(1.0*pow(y[60], 2) + pow(y[56], 2));
out[31] = sqrt(1.0*pow(y[61], 2) + pow(y[57], 2));
out[32] = sqrt(1.0*pow(y[62], 2) + pow(y[58], 2));
out[33] = sqrt(1.0*pow(y[63], 2) + pow(y[59], 2));
out[34] = 1.0*y[25]*y[60] + y[24]*y[56] + 1.0*y[27]*y[61] + y[26]*y[57] + 1.0*y[29]*y[62] + y[28]*y[58];
out[35] = 1.0*y[25]*y[56] - 1.0*y[24]*y[60] + 1.0*y[27]*y[57] - 1.0*y[26]*y[61] + 1.0*y[29]*y[58] - 1.0*y[28]*y[62];
out[36] = x[0];
out[37] = u[23];

}
void de_jac_ini_xy_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[2220] = p[0]*(2.0*y[8]*u[2] - 2.0*y[14]*u[2] + y[32]*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[9]*y[33] + y[8]*y[32] - y[15]*y[33] - y[14]*y[32])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   -1.0*y[8]*pow(pow(y[9], 2) + pow(y[8], 2), -0.5)/pow(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[2221] = p[0]*(2.0*y[9]*u[2] - 2.0*y[15]*u[2] + y[33]*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[9]*y[33] + y[8]*y[32] - y[15]*y[33] - y[14]*y[32])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   -1.0*y[9]*pow(pow(y[9], 2) + pow(y[8], 2), -0.5)/pow(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[2226] = p[0]*(-2.0*y[8]*u[2] + 2.0*y[14]*u[2] - y[32]*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[2227] = p[0]*(-2.0*y[9]*u[2] + 2.0*y[15]*u[2] - y[33]*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[2244] = p[0]*(y[8] - y[14])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[2245] = p[0]*(y[9] - y[15])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[2289] = p[0]*(2.0*y[10]*u[6] - 2.0*y[14]*u[6] + y[34]*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[11]*y[35] + y[10]*y[34] - y[15]*y[35] - y[14]*y[34])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   -1.0*y[10]*pow(pow(y[11], 2) + pow(y[10], 2), -0.5)/pow(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[2290] = p[0]*(2.0*y[11]*u[6] - 2.0*y[15]*u[6] + y[35]*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[11]*y[35] + y[10]*y[34] - y[15]*y[35] - y[14]*y[34])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   -1.0*y[11]*pow(pow(y[11], 2) + pow(y[10], 2), -0.5)/pow(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[2293] = p[0]*(-2.0*y[10]*u[6] + 2.0*y[14]*u[6] - y[34]*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[2294] = p[0]*(-2.0*y[11]*u[6] + 2.0*y[15]*u[6] - y[35]*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[2313] = p[0]*(y[10] - y[14])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[2314] = p[0]*(y[11] - y[15])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[2358] = p[0]*(2.0*y[12]*u[10] - 2.0*y[14]*u[10] + y[36]*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[13]*y[37] + y[12]*y[36] - y[15]*y[37] - y[14]*y[36])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   -1.0*y[12]*pow(pow(y[13], 2) + pow(y[12], 2), -0.5)/pow(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[2359] = p[0]*(2.0*y[13]*u[10] - 2.0*y[15]*u[10] + y[37]*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[13]*y[37] + y[12]*y[36] - y[15]*y[37] - y[14]*y[36])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   -1.0*y[13]*pow(pow(y[13], 2) + pow(y[12], 2), -0.5)/pow(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[2360] = p[0]*(-2.0*y[12]*u[10] + 2.0*y[14]*u[10] - y[36]*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[2361] = p[0]*(-2.0*y[13]*u[10] + 2.0*y[15]*u[10] - y[37]*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[2382] = p[0]*(y[12] - y[14])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[2383] = p[0]*(y[13] - y[15])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[2421] = p[0]*(-2.0*y[8]*u[3] + 2.0*y[14]*u[3] - y[33]*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[9]*y[32] - y[8]*y[33] - y[15]*y[32] + y[14]*y[33])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   -1.0*y[8]*pow(pow(y[9], 2) + pow(y[8], 2), -0.5)/pow(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[2422] = p[0]*(-2.0*y[9]*u[3] + 2.0*y[15]*u[3] + y[32]*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[9]*y[32] - y[8]*y[33] - y[15]*y[32] + y[14]*y[33])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   -1.0*y[9]*pow(pow(y[9], 2) + pow(y[8], 2), -0.5)/pow(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[2427] = p[0]*(2.0*y[8]*u[3] - 2.0*y[14]*u[3] + y[33]*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[2428] = p[0]*(2.0*y[9]*u[3] - 2.0*y[15]*u[3] - y[32]*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[2445] = p[0]*(y[9] - y[15])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[2446] = p[0]*(-y[8] + y[14])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[2490] = p[0]*(-2.0*y[10]*u[7] + 2.0*y[14]*u[7] - y[35]*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[11]*y[34] - y[10]*y[35] - y[15]*y[34] + y[14]*y[35])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   -1.0*y[10]*pow(pow(y[11], 2) + pow(y[10], 2), -0.5)/pow(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[2491] = p[0]*(-2.0*y[11]*u[7] + 2.0*y[15]*u[7] + y[34]*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[11]*y[34] - y[10]*y[35] - y[15]*y[34] + y[14]*y[35])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   -1.0*y[11]*pow(pow(y[11], 2) + pow(y[10], 2), -0.5)/pow(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[2494] = p[0]*(2.0*y[10]*u[7] - 2.0*y[14]*u[7] + y[35]*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[2495] = p[0]*(2.0*y[11]*u[7] - 2.0*y[15]*u[7] - y[34]*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[2514] = p[0]*(y[11] - y[15])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[2515] = p[0]*(-y[10] + y[14])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[2559] = p[0]*(-2.0*y[12]*u[11] + 2.0*y[14]*u[11] - y[37]*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[13]*y[36] - y[12]*y[37] - y[15]*y[36] + y[14]*y[37])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   -1.0*y[12]*pow(pow(y[13], 2) + pow(y[12], 2), -0.5)/pow(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[2560] = p[0]*(-2.0*y[13]*u[11] + 2.0*y[15]*u[11] + y[36]*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[13]*y[36] - y[12]*y[37] - y[15]*y[36] + y[14]*y[37])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   -1.0*y[13]*pow(pow(y[13], 2) + pow(y[12], 2), -0.5)/pow(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[2561] = p[0]*(2.0*y[12]*u[11] - 2.0*y[14]*u[11] + y[37]*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[2562] = p[0]*(2.0*y[13]*u[11] - 2.0*y[15]*u[11] - y[36]*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[2583] = p[0]*(y[13] - y[15])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[2584] = p[0]*(-y[12] + y[14])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
));

}

void de_jac_ini_up_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[0] = -p[17];
out[644] = -u[12];
out[715] = -u[12];
out[779] = -u[12];
out[850] = -u[12];
out[914] = -u[12];
out[985] = -u[12];
out[1049] = -u[12];
out[1120] = -u[12];
out[1180] = u[12];
out[1251] = u[12];
out[1315] = u[12];
out[1386] = u[12];
out[1450] = u[12];
out[1521] = u[12];
out[1585] = u[12];
out[1656] = u[12];
out[2788] = -p[1];
out[2792] = 1.0*p[2];
out[2856] = -p[1];
out[2860] = 1.0*p[2];
out[2924] = -p[1];
out[2928] = 1.0*p[2];
out[2992] = -p[1];
out[2996] = 1.0*p[2];
out[3056] = -1.0*p[2];
out[3060] = -1.0*p[1];
out[3124] = -1.0*p[2];
out[3128] = -1.0*p[1];
out[3192] = -1.0*p[2];
out[3196] = -1.0*p[1];
out[3260] = -1.0*p[2];
out[3264] = -1.0*p[1];
out[3332] = -p[4];
out[3336] = 1.0*p[3];
out[3400] = -p[4];
out[3404] = 1.0*p[3];
out[3468] = -p[4];
out[3472] = 1.0*p[3];
out[3536] = -p[6];
out[3540] = 1.0*p[5];
out[3600] = -1.0*p[3];
out[3604] = -1.0*p[4];
out[3668] = -1.0*p[3];
out[3672] = -1.0*p[4];
out[3736] = -1.0*p[3];
out[3740] = -1.0*p[4];
out[3804] = -1.0*p[5];
out[3808] = -1.0*p[6];
out[3876] = -p[10];
out[3880] = 1.0*p[9];
out[3944] = -p[10];
out[3948] = 1.0*p[9];
out[4012] = -p[10];
out[4016] = 1.0*p[9];
out[4080] = -p[12];
out[4084] = 1.0*p[11];
out[4144] = -1.0*p[9];
out[4148] = -1.0*p[10];
out[4212] = -1.0*p[9];
out[4216] = -1.0*p[10];
out[4280] = -1.0*p[9];
out[4284] = -1.0*p[10];
out[4348] = -1.0*p[11];
out[4352] = -1.0*p[12];
out[4422] = p[16];
out[4487] = -p[15];

}

void de_jac_ini_num_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[65] = -1.0;
out[68] = 184.75059629806526;
out[69] = 94.615102344455664;
out[70] = -69.368640085185305;
out[71] = -17.974903844364835;
out[72] = -43.675799641021079;
out[73] = -40.748687841669557;
out[74] = -69.368640085185291;
out[75] = -17.974903844364821;
out[76] = -184.75059629806526;
out[77] = -94.615102344455664;
out[78] = 69.368640085185305;
out[79] = 17.974903844364835;
out[80] = 43.675799641021079;
out[81] = 40.748687841669557;
out[82] = 69.368640085185291;
out[83] = 17.974903844364821;
out[116] = -1.0;
out[135] = -94.615102344455664;
out[136] = 184.75059629806526;
out[137] = 17.974903844364835;
out[138] = -69.368640085185305;
out[139] = 40.748687841669557;
out[140] = -43.675799641021079;
out[141] = 17.974903844364821;
out[142] = -69.368640085185291;
out[143] = 94.615102344455664;
out[144] = -184.75059629806526;
out[145] = -17.974903844364835;
out[146] = 69.368640085185305;
out[147] = -40.748687841669557;
out[148] = 43.675799641021079;
out[149] = -17.974903844364821;
out[150] = 69.368640085185291;
out[187] = -1.0;
out[202] = -69.368640085185291;
out[203] = -17.974903844364825;
out[204] = 184.7505962980652;
out[205] = 94.615102344455636;
out[206] = -69.368640085185319;
out[207] = -17.97490384436481;
out[208] = -43.675799641021079;
out[209] = -40.748687841669565;
out[210] = 69.368640085185291;
out[211] = 17.974903844364825;
out[212] = -184.7505962980652;
out[213] = -94.615102344455636;
out[214] = 69.368640085185319;
out[215] = 17.97490384436481;
out[216] = 43.675799641021079;
out[217] = 40.748687841669565;
out[251] = -1.0;
out[269] = 17.974903844364825;
out[270] = -69.368640085185291;
out[271] = -94.615102344455636;
out[272] = 184.7505962980652;
out[273] = 17.97490384436481;
out[274] = -69.368640085185319;
out[275] = 40.748687841669565;
out[276] = -43.675799641021079;
out[277] = -17.974903844364825;
out[278] = 69.368640085185291;
out[279] = 94.615102344455636;
out[280] = -184.7505962980652;
out[281] = -17.97490384436481;
out[282] = 69.368640085185319;
out[283] = -40.748687841669565;
out[284] = 43.675799641021079;
out[322] = -1.0;
out[336] = -43.675799641021101;
out[337] = -40.748687841669579;
out[338] = -69.368640085185305;
out[339] = -17.974903844364821;
out[340] = 184.75059629806526;
out[341] = 94.61510234445565;
out[342] = -69.368640085185305;
out[343] = -17.974903844364821;
out[344] = 43.675799641021101;
out[345] = 40.748687841669579;
out[346] = 69.368640085185305;
out[347] = 17.974903844364821;
out[348] = -184.75059629806526;
out[349] = -94.61510234445565;
out[350] = 69.368640085185305;
out[351] = 17.974903844364821;
out[386] = -1.0;
out[403] = 40.748687841669579;
out[404] = -43.675799641021101;
out[405] = 17.974903844364821;
out[406] = -69.368640085185305;
out[407] = -94.61510234445565;
out[408] = 184.75059629806526;
out[409] = 17.974903844364821;
out[410] = -69.368640085185305;
out[411] = -40.748687841669579;
out[412] = 43.675799641021101;
out[413] = -17.974903844364821;
out[414] = 69.368640085185305;
out[415] = 94.61510234445565;
out[416] = -184.75059629806526;
out[417] = -17.974903844364821;
out[418] = 69.368640085185305;
out[457] = -1.0;
out[470] = -69.368640085185305;
out[471] = -17.974903844364825;
out[472] = -43.675799641021079;
out[473] = -40.74868784166955;
out[474] = -69.368640085185291;
out[475] = -17.974903844364839;
out[476] = 184.7505962980652;
out[477] = 94.61510234445565;
out[478] = 69.368640085185305;
out[479] = 17.974903844364825;
out[480] = 43.675799641021079;
out[481] = 40.74868784166955;
out[482] = 69.368640085185291;
out[483] = 17.974903844364839;
out[484] = -184.7505962980652;
out[485] = -94.61510234445565;
out[521] = -1.0;
out[537] = 17.974903844364825;
out[538] = -69.368640085185305;
out[539] = 40.74868784166955;
out[540] = -43.675799641021079;
out[541] = 17.974903844364839;
out[542] = -69.368640085185291;
out[543] = -94.61510234445565;
out[544] = 184.7505962980652;
out[545] = -17.974903844364825;
out[546] = 69.368640085185305;
out[547] = -40.74868784166955;
out[548] = 43.675799641021079;
out[549] = -17.974903844364839;
out[550] = 69.368640085185291;
out[551] = 94.61510234445565;
out[552] = -184.7505962980652;
out[592] = -1.0;
out[604] = -184.75059629806526;
out[605] = -94.615102344455664;
out[606] = 69.368640085185305;
out[607] = 17.974903844364835;
out[608] = 43.675799641021079;
out[609] = 40.748687841669557;
out[610] = 69.368640085185291;
out[611] = 17.974903844364821;
out[612] = 184.75059629806526;
out[613] = 94.615102344455664;
out[614] = -69.368640085185305;
out[615] = -17.974903844364835;
out[616] = -43.675799641021079;
out[617] = -40.748687841669557;
out[618] = -69.368640085185291;
out[619] = -17.974903844364821;
out[636] = -1.0;
out[671] = 94.615102344455664;
out[672] = -184.75059629806526;
out[673] = -17.974903844364835;
out[674] = 69.368640085185305;
out[675] = -40.748687841669557;
out[676] = 43.675799641021079;
out[677] = -17.974903844364821;
out[678] = 69.368640085185291;
out[679] = -94.615102344455664;
out[680] = 184.75059629806526;
out[681] = 17.974903844364835;
out[682] = -69.368640085185305;
out[683] = 40.748687841669557;
out[684] = -43.675799641021079;
out[685] = 17.974903844364821;
out[686] = -69.368640085185291;
out[704] = -1.0;
out[738] = 69.368640085185291;
out[739] = 17.974903844364825;
out[740] = -184.7505962980652;
out[741] = -94.615102344455636;
out[742] = 69.368640085185319;
out[743] = 17.97490384436481;
out[744] = 43.675799641021079;
out[745] = 40.748687841669565;
out[746] = -69.368640085185291;
out[747] = -17.974903844364825;
out[748] = 184.7505962980652;
out[749] = 94.615102344455636;
out[750] = -69.368640085185319;
out[751] = -17.97490384436481;
out[752] = -43.675799641021079;
out[753] = -40.748687841669565;
out[772] = -1.0;
out[805] = -17.974903844364825;
out[806] = 69.368640085185291;
out[807] = 94.615102344455636;
out[808] = -184.7505962980652;
out[809] = -17.97490384436481;
out[810] = 69.368640085185319;
out[811] = -40.748687841669565;
out[812] = 43.675799641021079;
out[813] = 17.974903844364825;
out[814] = -69.368640085185291;
out[815] = -94.615102344455636;
out[816] = 184.7505962980652;
out[817] = 17.97490384436481;
out[818] = -69.368640085185319;
out[819] = 40.748687841669565;
out[820] = -43.675799641021079;
out[840] = -1.0;
out[872] = 43.675799641021101;
out[873] = 40.748687841669579;
out[874] = 69.368640085185305;
out[875] = 17.974903844364821;
out[876] = -184.75059629806526;
out[877] = -94.61510234445565;
out[878] = 69.368640085185305;
out[879] = 17.974903844364821;
out[880] = -43.675799641021101;
out[881] = -40.748687841669579;
out[882] = -69.368640085185305;
out[883] = -17.974903844364821;
out[884] = 184.75059629806526;
out[885] = 94.61510234445565;
out[886] = -69.368640085185305;
out[887] = -17.974903844364821;
out[908] = -1.0;
out[939] = -40.748687841669579;
out[940] = 43.675799641021101;
out[941] = -17.974903844364821;
out[942] = 69.368640085185305;
out[943] = 94.61510234445565;
out[944] = -184.75059629806526;
out[945] = -17.974903844364821;
out[946] = 69.368640085185305;
out[947] = 40.748687841669579;
out[948] = -43.675799641021101;
out[949] = 17.974903844364821;
out[950] = -69.368640085185305;
out[951] = -94.61510234445565;
out[952] = 184.75059629806526;
out[953] = 17.974903844364821;
out[954] = -69.368640085185305;
out[976] = -1.0;
out[1006] = 69.368640085185305;
out[1007] = 17.974903844364825;
out[1008] = 43.675799641021079;
out[1009] = 40.74868784166955;
out[1010] = 69.368640085185291;
out[1011] = 17.974903844364839;
out[1012] = -184.7505962980652;
out[1013] = -94.61510234445565;
out[1014] = -69.368640085185305;
out[1015] = -17.974903844364825;
out[1016] = -43.675799641021079;
out[1017] = -40.74868784166955;
out[1018] = -69.368640085185291;
out[1019] = -17.974903844364839;
out[1020] = 184.7505962980652;
out[1021] = 94.61510234445565;
out[1044] = -1.0;
out[1073] = -17.974903844364825;
out[1074] = 69.368640085185305;
out[1075] = -40.74868784166955;
out[1076] = 43.675799641021079;
out[1077] = -17.974903844364839;
out[1078] = 69.368640085185291;
out[1079] = 94.61510234445565;
out[1080] = -184.7505962980652;
out[1081] = 17.974903844364825;
out[1082] = -69.368640085185305;
out[1083] = 40.74868784166955;
out[1084] = -43.675799641021079;
out[1085] = 17.974903844364839;
out[1086] = -69.368640085185291;
out[1087] = -94.61510234445565;
out[1088] = 184.7505962980652;
out[1112] = -1.0;
out[1156] = 184.75059629806526;
out[1157] = 94.615102344455664;
out[1158] = -69.368640085185305;
out[1159] = -17.974903844364835;
out[1160] = -43.675799641021079;
out[1161] = -40.748687841669557;
out[1162] = -69.368640085185291;
out[1163] = -17.974903844364821;
out[1164] = -184.75059629806526;
out[1165] = -94.615102344455664;
out[1166] = 69.368640085185305;
out[1167] = 17.974903844364835;
out[1168] = 43.675799641021079;
out[1169] = 40.748687841669557;
out[1170] = 69.368640085185291;
out[1171] = 17.974903844364821;
out[1223] = -94.615102344455664;
out[1224] = 184.75059629806526;
out[1225] = 17.974903844364835;
out[1226] = -69.368640085185305;
out[1227] = 40.748687841669557;
out[1228] = -43.675799641021079;
out[1229] = 17.974903844364821;
out[1230] = -69.368640085185291;
out[1231] = 94.615102344455664;
out[1232] = -184.75059629806526;
out[1233] = -17.974903844364835;
out[1234] = 69.368640085185305;
out[1235] = -40.748687841669557;
out[1236] = 43.675799641021079;
out[1237] = -17.974903844364821;
out[1238] = 69.368640085185291;
out[1290] = -69.368640085185291;
out[1291] = -17.974903844364825;
out[1292] = 184.7505962980652;
out[1293] = 94.615102344455636;
out[1294] = -69.368640085185319;
out[1295] = -17.97490384436481;
out[1296] = -43.675799641021079;
out[1297] = -40.748687841669565;
out[1298] = 69.368640085185291;
out[1299] = 17.974903844364825;
out[1300] = -184.7505962980652;
out[1301] = -94.615102344455636;
out[1302] = 69.368640085185319;
out[1303] = 17.97490384436481;
out[1304] = 43.675799641021079;
out[1305] = 40.748687841669565;
out[1357] = 17.974903844364825;
out[1358] = -69.368640085185291;
out[1359] = -94.615102344455636;
out[1360] = 184.7505962980652;
out[1361] = 17.97490384436481;
out[1362] = -69.368640085185319;
out[1363] = 40.748687841669565;
out[1364] = -43.675799641021079;
out[1365] = -17.974903844364825;
out[1366] = 69.368640085185291;
out[1367] = 94.615102344455636;
out[1368] = -184.7505962980652;
out[1369] = -17.97490384436481;
out[1370] = 69.368640085185319;
out[1371] = -40.748687841669565;
out[1372] = 43.675799641021079;
out[1424] = -43.675799641021101;
out[1425] = -40.748687841669579;
out[1426] = -69.368640085185305;
out[1427] = -17.974903844364821;
out[1428] = 184.75059629806526;
out[1429] = 94.61510234445565;
out[1430] = -69.368640085185305;
out[1431] = -17.974903844364821;
out[1432] = 43.675799641021101;
out[1433] = 40.748687841669579;
out[1434] = 69.368640085185305;
out[1435] = 17.974903844364821;
out[1436] = -184.75059629806526;
out[1437] = -94.61510234445565;
out[1438] = 69.368640085185305;
out[1439] = 17.974903844364821;
out[1491] = 40.748687841669579;
out[1492] = -43.675799641021101;
out[1493] = 17.974903844364821;
out[1494] = -69.368640085185305;
out[1495] = -94.61510234445565;
out[1496] = 184.75059629806526;
out[1497] = 17.974903844364821;
out[1498] = -69.368640085185305;
out[1499] = -40.748687841669579;
out[1500] = 43.675799641021101;
out[1501] = -17.974903844364821;
out[1502] = 69.368640085185305;
out[1503] = 94.61510234445565;
out[1504] = -184.75059629806526;
out[1505] = -17.974903844364821;
out[1506] = 69.368640085185305;
out[1558] = -69.368640085185305;
out[1559] = -17.974903844364825;
out[1560] = -43.675799641021079;
out[1561] = -40.74868784166955;
out[1562] = -69.368640085185291;
out[1563] = -17.974903844364839;
out[1564] = 184.7505962980652;
out[1565] = 94.61510234445565;
out[1566] = 69.368640085185305;
out[1567] = 17.974903844364825;
out[1568] = 43.675799641021079;
out[1569] = 40.74868784166955;
out[1570] = 69.368640085185291;
out[1571] = 17.974903844364839;
out[1572] = -184.7505962980652;
out[1573] = -94.61510234445565;
out[1625] = 17.974903844364825;
out[1626] = -69.368640085185305;
out[1627] = 40.74868784166955;
out[1628] = -43.675799641021079;
out[1629] = 17.974903844364839;
out[1630] = -69.368640085185291;
out[1631] = -94.61510234445565;
out[1632] = 184.7505962980652;
out[1633] = -17.974903844364825;
out[1634] = 69.368640085185305;
out[1635] = -40.74868784166955;
out[1636] = 43.675799641021079;
out[1637] = -17.974903844364839;
out[1638] = 69.368640085185291;
out[1639] = 94.61510234445565;
out[1640] = -184.7505962980652;
out[1692] = -184.75059629806526;
out[1693] = -94.615102344455664;
out[1694] = 69.368640085185305;
out[1695] = 17.974903844364835;
out[1696] = 43.675799641021079;
out[1697] = 40.748687841669557;
out[1698] = 69.368640085185291;
out[1699] = 17.974903844364821;
out[1700] = 184.75059629806526;
out[1701] = 94.615102344455664;
out[1702] = -69.368640085185305;
out[1703] = -17.974903844364835;
out[1704] = -43.675799641021079;
out[1705] = -40.748687841669557;
out[1706] = -69.368640085185291;
out[1707] = -17.974903844364821;
out[1732] = -1.0;
out[1759] = 94.615102344455664;
out[1760] = -184.75059629806526;
out[1761] = -17.974903844364835;
out[1762] = 69.368640085185305;
out[1763] = -40.748687841669557;
out[1764] = 43.675799641021079;
out[1765] = -17.974903844364821;
out[1766] = 69.368640085185291;
out[1767] = -94.615102344455664;
out[1768] = 184.75059629806526;
out[1769] = 17.974903844364835;
out[1770] = -69.368640085185305;
out[1771] = 40.748687841669557;
out[1772] = -43.675799641021079;
out[1773] = 17.974903844364821;
out[1774] = -69.368640085185291;
out[1803] = -1.0;
out[1826] = 69.368640085185291;
out[1827] = 17.974903844364825;
out[1828] = -184.7505962980652;
out[1829] = -94.615102344455636;
out[1830] = 69.368640085185319;
out[1831] = 17.97490384436481;
out[1832] = 43.675799641021079;
out[1833] = 40.748687841669565;
out[1834] = -69.368640085185291;
out[1835] = -17.974903844364825;
out[1836] = 184.7505962980652;
out[1837] = 94.615102344455636;
out[1838] = -69.368640085185319;
out[1839] = -17.97490384436481;
out[1840] = -43.675799641021079;
out[1841] = -40.748687841669565;
out[1867] = -1.0;
out[1893] = -17.974903844364825;
out[1894] = 69.368640085185291;
out[1895] = 94.615102344455636;
out[1896] = -184.7505962980652;
out[1897] = -17.97490384436481;
out[1898] = 69.368640085185319;
out[1899] = -40.748687841669565;
out[1900] = 43.675799641021079;
out[1901] = 17.974903844364825;
out[1902] = -69.368640085185291;
out[1903] = -94.615102344455636;
out[1904] = 184.7505962980652;
out[1905] = 17.97490384436481;
out[1906] = -69.368640085185319;
out[1907] = 40.748687841669565;
out[1908] = -43.675799641021079;
out[1938] = -1.0;
out[1960] = 43.675799641021101;
out[1961] = 40.748687841669579;
out[1962] = 69.368640085185305;
out[1963] = 17.974903844364821;
out[1964] = -184.75059629806526;
out[1965] = -94.61510234445565;
out[1966] = 69.368640085185305;
out[1967] = 17.974903844364821;
out[1968] = -43.675799641021101;
out[1969] = -40.748687841669579;
out[1970] = -69.368640085185305;
out[1971] = -17.974903844364821;
out[1972] = 184.75059629806526;
out[1973] = 94.61510234445565;
out[1974] = -69.368640085185305;
out[1975] = -17.974903844364821;
out[2002] = -1.0;
out[2027] = -40.748687841669579;
out[2028] = 43.675799641021101;
out[2029] = -17.974903844364821;
out[2030] = 69.368640085185305;
out[2031] = 94.61510234445565;
out[2032] = -184.75059629806526;
out[2033] = -17.974903844364821;
out[2034] = 69.368640085185305;
out[2035] = 40.748687841669579;
out[2036] = -43.675799641021101;
out[2037] = 17.974903844364821;
out[2038] = -69.368640085185305;
out[2039] = -94.61510234445565;
out[2040] = 184.75059629806526;
out[2041] = 17.974903844364821;
out[2042] = -69.368640085185305;
out[2073] = -1.0;
out[2094] = 69.368640085185305;
out[2095] = 17.974903844364825;
out[2096] = 43.675799641021079;
out[2097] = 40.74868784166955;
out[2098] = 69.368640085185291;
out[2099] = 17.974903844364839;
out[2100] = -184.7505962980652;
out[2101] = -94.61510234445565;
out[2102] = -69.368640085185305;
out[2103] = -17.974903844364825;
out[2104] = -43.675799641021079;
out[2105] = -40.74868784166955;
out[2106] = -69.368640085185291;
out[2107] = -17.974903844364839;
out[2108] = 184.7505962980652;
out[2109] = 94.61510234445565;
out[2137] = -1.0;
out[2161] = -17.974903844364825;
out[2162] = 69.368640085185305;
out[2163] = -40.74868784166955;
out[2164] = 43.675799641021079;
out[2165] = -17.974903844364839;
out[2166] = 69.368640085185291;
out[2167] = 94.61510234445565;
out[2168] = -184.7505962980652;
out[2169] = 17.974903844364825;
out[2170] = -69.368640085185305;
out[2171] = 40.74868784166955;
out[2172] = -43.675799641021079;
out[2173] = 17.974903844364839;
out[2174] = -69.368640085185291;
out[2175] = -94.61510234445565;
out[2176] = 184.7505962980652;
out[2208] = -1.0;
out[2646] = 1.0;
out[2648] = 1.0;
out[2650] = 1.0;
out[2652] = 1.0;
out[2714] = 1.0;
out[2716] = 1.0;
out[2718] = 1.0;
out[2720] = 1.0;
out[2756] = 1.0;
out[2764] = -1.0;
out[2825] = 1.0;
out[2833] = -1.0;
out[2894] = 1.0;
out[2902] = -1.0;
out[2963] = 1.0;
out[2971] = -1.0;
out[3025] = 1.0;
out[3033] = -1.0;
out[3094] = 1.0;
out[3102] = -1.0;
out[3163] = 1.0;
out[3171] = -1.0;
out[3232] = 1.0;
out[3240] = -1.0;
out[3284] = -1.0;
out[3353] = -1.0;
out[3422] = -1.0;
out[3491] = -1.0;
out[3553] = -1.0;
out[3622] = -1.0;
out[3691] = -1.0;
out[3760] = -1.0;
out[3844] = -1.0;
out[3913] = -1.0;
out[3982] = -1.0;
out[4051] = -1.0;
out[4113] = -1.0;
out[4182] = -1.0;
out[4251] = -1.0;
out[4320] = -1.0;
out[4420] = -1.0;
out[4488] = -1.0;

}

void sp_jac_ini_xy_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[554] = p[0]*(2.0*y[8]*u[2] - 2.0*y[14]*u[2] + y[32]*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[9]*y[33] + y[8]*y[32] - y[15]*y[33] - y[14]*y[32])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   -1.0*y[8]*pow(pow(y[9], 2) + pow(y[8], 2), -0.5)/pow(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[555] = p[0]*(2.0*y[9]*u[2] - 2.0*y[15]*u[2] + y[33]*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[9]*y[33] + y[8]*y[32] - y[15]*y[33] - y[14]*y[32])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   -1.0*y[9]*pow(pow(y[9], 2) + pow(y[8], 2), -0.5)/pow(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[556] = p[0]*(-2.0*y[8]*u[2] + 2.0*y[14]*u[2] - y[32]*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[557] = p[0]*(-2.0*y[9]*u[2] + 2.0*y[15]*u[2] - y[33]*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[558] = p[0]*(y[8] - y[14])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[559] = p[0]*(y[9] - y[15])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[560] = p[0]*(2.0*y[10]*u[6] - 2.0*y[14]*u[6] + y[34]*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[11]*y[35] + y[10]*y[34] - y[15]*y[35] - y[14]*y[34])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   -1.0*y[10]*pow(pow(y[11], 2) + pow(y[10], 2), -0.5)/pow(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[561] = p[0]*(2.0*y[11]*u[6] - 2.0*y[15]*u[6] + y[35]*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[11]*y[35] + y[10]*y[34] - y[15]*y[35] - y[14]*y[34])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   -1.0*y[11]*pow(pow(y[11], 2) + pow(y[10], 2), -0.5)/pow(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[562] = p[0]*(-2.0*y[10]*u[6] + 2.0*y[14]*u[6] - y[34]*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[563] = p[0]*(-2.0*y[11]*u[6] + 2.0*y[15]*u[6] - y[35]*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[564] = p[0]*(y[10] - y[14])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[565] = p[0]*(y[11] - y[15])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[566] = p[0]*(2.0*y[12]*u[10] - 2.0*y[14]*u[10] + y[36]*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[13]*y[37] + y[12]*y[36] - y[15]*y[37] - y[14]*y[36])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   -1.0*y[12]*pow(pow(y[13], 2) + pow(y[12], 2), -0.5)/pow(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[567] = p[0]*(2.0*y[13]*u[10] - 2.0*y[15]*u[10] + y[37]*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[13]*y[37] + y[12]*y[36] - y[15]*y[37] - y[14]*y[36])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   -1.0*y[13]*pow(pow(y[13], 2) + pow(y[12], 2), -0.5)/pow(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[568] = p[0]*(-2.0*y[12]*u[10] + 2.0*y[14]*u[10] - y[36]*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[569] = p[0]*(-2.0*y[13]*u[10] + 2.0*y[15]*u[10] - y[37]*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[570] = p[0]*(y[12] - y[14])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[571] = p[0]*(y[13] - y[15])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[572] = p[0]*(-2.0*y[8]*u[3] + 2.0*y[14]*u[3] - y[33]*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[9]*y[32] - y[8]*y[33] - y[15]*y[32] + y[14]*y[33])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   -1.0*y[8]*pow(pow(y[9], 2) + pow(y[8], 2), -0.5)/pow(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[573] = p[0]*(-2.0*y[9]*u[3] + 2.0*y[15]*u[3] + y[32]*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[9]*y[32] - y[8]*y[33] - y[15]*y[32] + y[14]*y[33])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   -1.0*y[9]*pow(pow(y[9], 2) + pow(y[8], 2), -0.5)/pow(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[574] = p[0]*(2.0*y[8]*u[3] - 2.0*y[14]*u[3] + y[33]*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[575] = p[0]*(2.0*y[9]*u[3] - 2.0*y[15]*u[3] - y[32]*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[576] = p[0]*(y[9] - y[15])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[577] = p[0]*(-y[8] + y[14])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[578] = p[0]*(-2.0*y[10]*u[7] + 2.0*y[14]*u[7] - y[35]*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[11]*y[34] - y[10]*y[35] - y[15]*y[34] + y[14]*y[35])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   -1.0*y[10]*pow(pow(y[11], 2) + pow(y[10], 2), -0.5)/pow(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[579] = p[0]*(-2.0*y[11]*u[7] + 2.0*y[15]*u[7] + y[34]*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[11]*y[34] - y[10]*y[35] - y[15]*y[34] + y[14]*y[35])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   -1.0*y[11]*pow(pow(y[11], 2) + pow(y[10], 2), -0.5)/pow(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[580] = p[0]*(2.0*y[10]*u[7] - 2.0*y[14]*u[7] + y[35]*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[581] = p[0]*(2.0*y[11]*u[7] - 2.0*y[15]*u[7] - y[34]*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[582] = p[0]*(y[11] - y[15])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[583] = p[0]*(-y[10] + y[14])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[584] = p[0]*(-2.0*y[12]*u[11] + 2.0*y[14]*u[11] - y[37]*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[13]*y[36] - y[12]*y[37] - y[15]*y[36] + y[14]*y[37])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   -1.0*y[12]*pow(pow(y[13], 2) + pow(y[12], 2), -0.5)/pow(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[585] = p[0]*(-2.0*y[13]*u[11] + 2.0*y[15]*u[11] + y[36]*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[13]*y[36] - y[12]*y[37] - y[15]*y[36] + y[14]*y[37])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   -1.0*y[13]*pow(pow(y[13], 2) + pow(y[12], 2), -0.5)/pow(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[586] = p[0]*(2.0*y[12]*u[11] - 2.0*y[14]*u[11] + y[37]*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[587] = p[0]*(2.0*y[13]*u[11] - 2.0*y[15]*u[11] - y[36]*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[588] = p[0]*(y[13] - y[15])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[589] = p[0]*(-y[12] + y[14])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
));

}

void sp_jac_ini_up_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[0] = -p[17];
out[155] = -u[12];
out[173] = -u[12];
out[191] = -u[12];
out[209] = -u[12];
out[227] = -u[12];
out[245] = -u[12];
out[263] = -u[12];
out[281] = -u[12];
out[298] = u[12];
out[315] = u[12];
out[332] = u[12];
out[349] = u[12];
out[366] = u[12];
out[383] = u[12];
out[400] = u[12];
out[417] = u[12];
out[600] = -p[1];
out[601] = 1.0*p[2];
out[604] = -p[1];
out[605] = 1.0*p[2];
out[608] = -p[1];
out[609] = 1.0*p[2];
out[612] = -p[1];
out[613] = 1.0*p[2];
out[616] = -1.0*p[2];
out[617] = -1.0*p[1];
out[620] = -1.0*p[2];
out[621] = -1.0*p[1];
out[624] = -1.0*p[2];
out[625] = -1.0*p[1];
out[628] = -1.0*p[2];
out[629] = -1.0*p[1];
out[631] = -p[4];
out[632] = 1.0*p[3];
out[634] = -p[4];
out[635] = 1.0*p[3];
out[637] = -p[4];
out[638] = 1.0*p[3];
out[640] = -p[6];
out[641] = 1.0*p[5];
out[643] = -1.0*p[3];
out[644] = -1.0*p[4];
out[646] = -1.0*p[3];
out[647] = -1.0*p[4];
out[649] = -1.0*p[3];
out[650] = -1.0*p[4];
out[652] = -1.0*p[5];
out[653] = -1.0*p[6];
out[655] = -p[10];
out[656] = 1.0*p[9];
out[658] = -p[10];
out[659] = 1.0*p[9];
out[661] = -p[10];
out[662] = 1.0*p[9];
out[664] = -p[12];
out[665] = 1.0*p[11];
out[667] = -1.0*p[9];
out[668] = -1.0*p[10];
out[670] = -1.0*p[9];
out[671] = -1.0*p[10];
out[673] = -1.0*p[9];
out[674] = -1.0*p[10];
out[676] = -1.0*p[11];
out[677] = -1.0*p[12];
out[679] = p[16];
out[680] = -p[15];

}

void sp_jac_ini_num_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[1] = -1.0;
out[2] = 184.75059629806526;
out[3] = 94.615102344455664;
out[4] = -69.368640085185305;
out[5] = -17.974903844364835;
out[6] = -43.675799641021079;
out[7] = -40.748687841669557;
out[8] = -69.368640085185291;
out[9] = -17.974903844364821;
out[10] = -184.75059629806526;
out[11] = -94.615102344455664;
out[12] = 69.368640085185305;
out[13] = 17.974903844364835;
out[14] = 43.675799641021079;
out[15] = 40.748687841669557;
out[16] = 69.368640085185291;
out[17] = 17.974903844364821;
out[18] = -1.0;
out[19] = -94.615102344455664;
out[20] = 184.75059629806526;
out[21] = 17.974903844364835;
out[22] = -69.368640085185305;
out[23] = 40.748687841669557;
out[24] = -43.675799641021079;
out[25] = 17.974903844364821;
out[26] = -69.368640085185291;
out[27] = 94.615102344455664;
out[28] = -184.75059629806526;
out[29] = -17.974903844364835;
out[30] = 69.368640085185305;
out[31] = -40.748687841669557;
out[32] = 43.675799641021079;
out[33] = -17.974903844364821;
out[34] = 69.368640085185291;
out[35] = -1.0;
out[36] = -69.368640085185291;
out[37] = -17.974903844364825;
out[38] = 184.7505962980652;
out[39] = 94.615102344455636;
out[40] = -69.368640085185319;
out[41] = -17.97490384436481;
out[42] = -43.675799641021079;
out[43] = -40.748687841669565;
out[44] = 69.368640085185291;
out[45] = 17.974903844364825;
out[46] = -184.7505962980652;
out[47] = -94.615102344455636;
out[48] = 69.368640085185319;
out[49] = 17.97490384436481;
out[50] = 43.675799641021079;
out[51] = 40.748687841669565;
out[52] = -1.0;
out[53] = 17.974903844364825;
out[54] = -69.368640085185291;
out[55] = -94.615102344455636;
out[56] = 184.7505962980652;
out[57] = 17.97490384436481;
out[58] = -69.368640085185319;
out[59] = 40.748687841669565;
out[60] = -43.675799641021079;
out[61] = -17.974903844364825;
out[62] = 69.368640085185291;
out[63] = 94.615102344455636;
out[64] = -184.7505962980652;
out[65] = -17.97490384436481;
out[66] = 69.368640085185319;
out[67] = -40.748687841669565;
out[68] = 43.675799641021079;
out[69] = -1.0;
out[70] = -43.675799641021101;
out[71] = -40.748687841669579;
out[72] = -69.368640085185305;
out[73] = -17.974903844364821;
out[74] = 184.75059629806526;
out[75] = 94.61510234445565;
out[76] = -69.368640085185305;
out[77] = -17.974903844364821;
out[78] = 43.675799641021101;
out[79] = 40.748687841669579;
out[80] = 69.368640085185305;
out[81] = 17.974903844364821;
out[82] = -184.75059629806526;
out[83] = -94.61510234445565;
out[84] = 69.368640085185305;
out[85] = 17.974903844364821;
out[86] = -1.0;
out[87] = 40.748687841669579;
out[88] = -43.675799641021101;
out[89] = 17.974903844364821;
out[90] = -69.368640085185305;
out[91] = -94.61510234445565;
out[92] = 184.75059629806526;
out[93] = 17.974903844364821;
out[94] = -69.368640085185305;
out[95] = -40.748687841669579;
out[96] = 43.675799641021101;
out[97] = -17.974903844364821;
out[98] = 69.368640085185305;
out[99] = 94.61510234445565;
out[100] = -184.75059629806526;
out[101] = -17.974903844364821;
out[102] = 69.368640085185305;
out[103] = -1.0;
out[104] = -69.368640085185305;
out[105] = -17.974903844364825;
out[106] = -43.675799641021079;
out[107] = -40.74868784166955;
out[108] = -69.368640085185291;
out[109] = -17.974903844364839;
out[110] = 184.7505962980652;
out[111] = 94.61510234445565;
out[112] = 69.368640085185305;
out[113] = 17.974903844364825;
out[114] = 43.675799641021079;
out[115] = 40.74868784166955;
out[116] = 69.368640085185291;
out[117] = 17.974903844364839;
out[118] = -184.7505962980652;
out[119] = -94.61510234445565;
out[120] = -1.0;
out[121] = 17.974903844364825;
out[122] = -69.368640085185305;
out[123] = 40.74868784166955;
out[124] = -43.675799641021079;
out[125] = 17.974903844364839;
out[126] = -69.368640085185291;
out[127] = -94.61510234445565;
out[128] = 184.7505962980652;
out[129] = -17.974903844364825;
out[130] = 69.368640085185305;
out[131] = -40.74868784166955;
out[132] = 43.675799641021079;
out[133] = -17.974903844364839;
out[134] = 69.368640085185291;
out[135] = 94.61510234445565;
out[136] = -184.7505962980652;
out[137] = -1.0;
out[138] = -184.75059629806526;
out[139] = -94.615102344455664;
out[140] = 69.368640085185305;
out[141] = 17.974903844364835;
out[142] = 43.675799641021079;
out[143] = 40.748687841669557;
out[144] = 69.368640085185291;
out[145] = 17.974903844364821;
out[146] = 184.75059629806526;
out[147] = 94.615102344455664;
out[148] = -69.368640085185305;
out[149] = -17.974903844364835;
out[150] = -43.675799641021079;
out[151] = -40.748687841669557;
out[152] = -69.368640085185291;
out[153] = -17.974903844364821;
out[154] = -1.0;
out[156] = 94.615102344455664;
out[157] = -184.75059629806526;
out[158] = -17.974903844364835;
out[159] = 69.368640085185305;
out[160] = -40.748687841669557;
out[161] = 43.675799641021079;
out[162] = -17.974903844364821;
out[163] = 69.368640085185291;
out[164] = -94.615102344455664;
out[165] = 184.75059629806526;
out[166] = 17.974903844364835;
out[167] = -69.368640085185305;
out[168] = 40.748687841669557;
out[169] = -43.675799641021079;
out[170] = 17.974903844364821;
out[171] = -69.368640085185291;
out[172] = -1.0;
out[174] = 69.368640085185291;
out[175] = 17.974903844364825;
out[176] = -184.7505962980652;
out[177] = -94.615102344455636;
out[178] = 69.368640085185319;
out[179] = 17.97490384436481;
out[180] = 43.675799641021079;
out[181] = 40.748687841669565;
out[182] = -69.368640085185291;
out[183] = -17.974903844364825;
out[184] = 184.7505962980652;
out[185] = 94.615102344455636;
out[186] = -69.368640085185319;
out[187] = -17.97490384436481;
out[188] = -43.675799641021079;
out[189] = -40.748687841669565;
out[190] = -1.0;
out[192] = -17.974903844364825;
out[193] = 69.368640085185291;
out[194] = 94.615102344455636;
out[195] = -184.7505962980652;
out[196] = -17.97490384436481;
out[197] = 69.368640085185319;
out[198] = -40.748687841669565;
out[199] = 43.675799641021079;
out[200] = 17.974903844364825;
out[201] = -69.368640085185291;
out[202] = -94.615102344455636;
out[203] = 184.7505962980652;
out[204] = 17.97490384436481;
out[205] = -69.368640085185319;
out[206] = 40.748687841669565;
out[207] = -43.675799641021079;
out[208] = -1.0;
out[210] = 43.675799641021101;
out[211] = 40.748687841669579;
out[212] = 69.368640085185305;
out[213] = 17.974903844364821;
out[214] = -184.75059629806526;
out[215] = -94.61510234445565;
out[216] = 69.368640085185305;
out[217] = 17.974903844364821;
out[218] = -43.675799641021101;
out[219] = -40.748687841669579;
out[220] = -69.368640085185305;
out[221] = -17.974903844364821;
out[222] = 184.75059629806526;
out[223] = 94.61510234445565;
out[224] = -69.368640085185305;
out[225] = -17.974903844364821;
out[226] = -1.0;
out[228] = -40.748687841669579;
out[229] = 43.675799641021101;
out[230] = -17.974903844364821;
out[231] = 69.368640085185305;
out[232] = 94.61510234445565;
out[233] = -184.75059629806526;
out[234] = -17.974903844364821;
out[235] = 69.368640085185305;
out[236] = 40.748687841669579;
out[237] = -43.675799641021101;
out[238] = 17.974903844364821;
out[239] = -69.368640085185305;
out[240] = -94.61510234445565;
out[241] = 184.75059629806526;
out[242] = 17.974903844364821;
out[243] = -69.368640085185305;
out[244] = -1.0;
out[246] = 69.368640085185305;
out[247] = 17.974903844364825;
out[248] = 43.675799641021079;
out[249] = 40.74868784166955;
out[250] = 69.368640085185291;
out[251] = 17.974903844364839;
out[252] = -184.7505962980652;
out[253] = -94.61510234445565;
out[254] = -69.368640085185305;
out[255] = -17.974903844364825;
out[256] = -43.675799641021079;
out[257] = -40.74868784166955;
out[258] = -69.368640085185291;
out[259] = -17.974903844364839;
out[260] = 184.7505962980652;
out[261] = 94.61510234445565;
out[262] = -1.0;
out[264] = -17.974903844364825;
out[265] = 69.368640085185305;
out[266] = -40.74868784166955;
out[267] = 43.675799641021079;
out[268] = -17.974903844364839;
out[269] = 69.368640085185291;
out[270] = 94.61510234445565;
out[271] = -184.7505962980652;
out[272] = 17.974903844364825;
out[273] = -69.368640085185305;
out[274] = 40.74868784166955;
out[275] = -43.675799641021079;
out[276] = 17.974903844364839;
out[277] = -69.368640085185291;
out[278] = -94.61510234445565;
out[279] = 184.7505962980652;
out[280] = -1.0;
out[282] = 184.75059629806526;
out[283] = 94.615102344455664;
out[284] = -69.368640085185305;
out[285] = -17.974903844364835;
out[286] = -43.675799641021079;
out[287] = -40.748687841669557;
out[288] = -69.368640085185291;
out[289] = -17.974903844364821;
out[290] = -184.75059629806526;
out[291] = -94.615102344455664;
out[292] = 69.368640085185305;
out[293] = 17.974903844364835;
out[294] = 43.675799641021079;
out[295] = 40.748687841669557;
out[296] = 69.368640085185291;
out[297] = 17.974903844364821;
out[299] = -94.615102344455664;
out[300] = 184.75059629806526;
out[301] = 17.974903844364835;
out[302] = -69.368640085185305;
out[303] = 40.748687841669557;
out[304] = -43.675799641021079;
out[305] = 17.974903844364821;
out[306] = -69.368640085185291;
out[307] = 94.615102344455664;
out[308] = -184.75059629806526;
out[309] = -17.974903844364835;
out[310] = 69.368640085185305;
out[311] = -40.748687841669557;
out[312] = 43.675799641021079;
out[313] = -17.974903844364821;
out[314] = 69.368640085185291;
out[316] = -69.368640085185291;
out[317] = -17.974903844364825;
out[318] = 184.7505962980652;
out[319] = 94.615102344455636;
out[320] = -69.368640085185319;
out[321] = -17.97490384436481;
out[322] = -43.675799641021079;
out[323] = -40.748687841669565;
out[324] = 69.368640085185291;
out[325] = 17.974903844364825;
out[326] = -184.7505962980652;
out[327] = -94.615102344455636;
out[328] = 69.368640085185319;
out[329] = 17.97490384436481;
out[330] = 43.675799641021079;
out[331] = 40.748687841669565;
out[333] = 17.974903844364825;
out[334] = -69.368640085185291;
out[335] = -94.615102344455636;
out[336] = 184.7505962980652;
out[337] = 17.97490384436481;
out[338] = -69.368640085185319;
out[339] = 40.748687841669565;
out[340] = -43.675799641021079;
out[341] = -17.974903844364825;
out[342] = 69.368640085185291;
out[343] = 94.615102344455636;
out[344] = -184.7505962980652;
out[345] = -17.97490384436481;
out[346] = 69.368640085185319;
out[347] = -40.748687841669565;
out[348] = 43.675799641021079;
out[350] = -43.675799641021101;
out[351] = -40.748687841669579;
out[352] = -69.368640085185305;
out[353] = -17.974903844364821;
out[354] = 184.75059629806526;
out[355] = 94.61510234445565;
out[356] = -69.368640085185305;
out[357] = -17.974903844364821;
out[358] = 43.675799641021101;
out[359] = 40.748687841669579;
out[360] = 69.368640085185305;
out[361] = 17.974903844364821;
out[362] = -184.75059629806526;
out[363] = -94.61510234445565;
out[364] = 69.368640085185305;
out[365] = 17.974903844364821;
out[367] = 40.748687841669579;
out[368] = -43.675799641021101;
out[369] = 17.974903844364821;
out[370] = -69.368640085185305;
out[371] = -94.61510234445565;
out[372] = 184.75059629806526;
out[373] = 17.974903844364821;
out[374] = -69.368640085185305;
out[375] = -40.748687841669579;
out[376] = 43.675799641021101;
out[377] = -17.974903844364821;
out[378] = 69.368640085185305;
out[379] = 94.61510234445565;
out[380] = -184.75059629806526;
out[381] = -17.974903844364821;
out[382] = 69.368640085185305;
out[384] = -69.368640085185305;
out[385] = -17.974903844364825;
out[386] = -43.675799641021079;
out[387] = -40.74868784166955;
out[388] = -69.368640085185291;
out[389] = -17.974903844364839;
out[390] = 184.7505962980652;
out[391] = 94.61510234445565;
out[392] = 69.368640085185305;
out[393] = 17.974903844364825;
out[394] = 43.675799641021079;
out[395] = 40.74868784166955;
out[396] = 69.368640085185291;
out[397] = 17.974903844364839;
out[398] = -184.7505962980652;
out[399] = -94.61510234445565;
out[401] = 17.974903844364825;
out[402] = -69.368640085185305;
out[403] = 40.74868784166955;
out[404] = -43.675799641021079;
out[405] = 17.974903844364839;
out[406] = -69.368640085185291;
out[407] = -94.61510234445565;
out[408] = 184.7505962980652;
out[409] = -17.974903844364825;
out[410] = 69.368640085185305;
out[411] = -40.74868784166955;
out[412] = 43.675799641021079;
out[413] = -17.974903844364839;
out[414] = 69.368640085185291;
out[415] = 94.61510234445565;
out[416] = -184.7505962980652;
out[418] = -184.75059629806526;
out[419] = -94.615102344455664;
out[420] = 69.368640085185305;
out[421] = 17.974903844364835;
out[422] = 43.675799641021079;
out[423] = 40.748687841669557;
out[424] = 69.368640085185291;
out[425] = 17.974903844364821;
out[426] = 184.75059629806526;
out[427] = 94.615102344455664;
out[428] = -69.368640085185305;
out[429] = -17.974903844364835;
out[430] = -43.675799641021079;
out[431] = -40.748687841669557;
out[432] = -69.368640085185291;
out[433] = -17.974903844364821;
out[434] = -1.0;
out[435] = 94.615102344455664;
out[436] = -184.75059629806526;
out[437] = -17.974903844364835;
out[438] = 69.368640085185305;
out[439] = -40.748687841669557;
out[440] = 43.675799641021079;
out[441] = -17.974903844364821;
out[442] = 69.368640085185291;
out[443] = -94.615102344455664;
out[444] = 184.75059629806526;
out[445] = 17.974903844364835;
out[446] = -69.368640085185305;
out[447] = 40.748687841669557;
out[448] = -43.675799641021079;
out[449] = 17.974903844364821;
out[450] = -69.368640085185291;
out[451] = -1.0;
out[452] = 69.368640085185291;
out[453] = 17.974903844364825;
out[454] = -184.7505962980652;
out[455] = -94.615102344455636;
out[456] = 69.368640085185319;
out[457] = 17.97490384436481;
out[458] = 43.675799641021079;
out[459] = 40.748687841669565;
out[460] = -69.368640085185291;
out[461] = -17.974903844364825;
out[462] = 184.7505962980652;
out[463] = 94.615102344455636;
out[464] = -69.368640085185319;
out[465] = -17.97490384436481;
out[466] = -43.675799641021079;
out[467] = -40.748687841669565;
out[468] = -1.0;
out[469] = -17.974903844364825;
out[470] = 69.368640085185291;
out[471] = 94.615102344455636;
out[472] = -184.7505962980652;
out[473] = -17.97490384436481;
out[474] = 69.368640085185319;
out[475] = -40.748687841669565;
out[476] = 43.675799641021079;
out[477] = 17.974903844364825;
out[478] = -69.368640085185291;
out[479] = -94.615102344455636;
out[480] = 184.7505962980652;
out[481] = 17.97490384436481;
out[482] = -69.368640085185319;
out[483] = 40.748687841669565;
out[484] = -43.675799641021079;
out[485] = -1.0;
out[486] = 43.675799641021101;
out[487] = 40.748687841669579;
out[488] = 69.368640085185305;
out[489] = 17.974903844364821;
out[490] = -184.75059629806526;
out[491] = -94.61510234445565;
out[492] = 69.368640085185305;
out[493] = 17.974903844364821;
out[494] = -43.675799641021101;
out[495] = -40.748687841669579;
out[496] = -69.368640085185305;
out[497] = -17.974903844364821;
out[498] = 184.75059629806526;
out[499] = 94.61510234445565;
out[500] = -69.368640085185305;
out[501] = -17.974903844364821;
out[502] = -1.0;
out[503] = -40.748687841669579;
out[504] = 43.675799641021101;
out[505] = -17.974903844364821;
out[506] = 69.368640085185305;
out[507] = 94.61510234445565;
out[508] = -184.75059629806526;
out[509] = -17.974903844364821;
out[510] = 69.368640085185305;
out[511] = 40.748687841669579;
out[512] = -43.675799641021101;
out[513] = 17.974903844364821;
out[514] = -69.368640085185305;
out[515] = -94.61510234445565;
out[516] = 184.75059629806526;
out[517] = 17.974903844364821;
out[518] = -69.368640085185305;
out[519] = -1.0;
out[520] = 69.368640085185305;
out[521] = 17.974903844364825;
out[522] = 43.675799641021079;
out[523] = 40.74868784166955;
out[524] = 69.368640085185291;
out[525] = 17.974903844364839;
out[526] = -184.7505962980652;
out[527] = -94.61510234445565;
out[528] = -69.368640085185305;
out[529] = -17.974903844364825;
out[530] = -43.675799641021079;
out[531] = -40.74868784166955;
out[532] = -69.368640085185291;
out[533] = -17.974903844364839;
out[534] = 184.7505962980652;
out[535] = 94.61510234445565;
out[536] = -1.0;
out[537] = -17.974903844364825;
out[538] = 69.368640085185305;
out[539] = -40.74868784166955;
out[540] = 43.675799641021079;
out[541] = -17.974903844364839;
out[542] = 69.368640085185291;
out[543] = 94.61510234445565;
out[544] = -184.7505962980652;
out[545] = 17.974903844364825;
out[546] = -69.368640085185305;
out[547] = 40.74868784166955;
out[548] = -43.675799641021079;
out[549] = 17.974903844364839;
out[550] = -69.368640085185291;
out[551] = -94.61510234445565;
out[552] = 184.7505962980652;
out[553] = -1.0;
out[590] = 1.0;
out[591] = 1.0;
out[592] = 1.0;
out[593] = 1.0;
out[594] = 1.0;
out[595] = 1.0;
out[596] = 1.0;
out[597] = 1.0;
out[598] = 1.0;
out[599] = -1.0;
out[602] = 1.0;
out[603] = -1.0;
out[606] = 1.0;
out[607] = -1.0;
out[610] = 1.0;
out[611] = -1.0;
out[614] = 1.0;
out[615] = -1.0;
out[618] = 1.0;
out[619] = -1.0;
out[622] = 1.0;
out[623] = -1.0;
out[626] = 1.0;
out[627] = -1.0;
out[630] = -1.0;
out[633] = -1.0;
out[636] = -1.0;
out[639] = -1.0;
out[642] = -1.0;
out[645] = -1.0;
out[648] = -1.0;
out[651] = -1.0;
out[654] = -1.0;
out[657] = -1.0;
out[660] = -1.0;
out[663] = -1.0;
out[666] = -1.0;
out[669] = -1.0;
out[672] = -1.0;
out[675] = -1.0;
out[678] = -1.0;
out[681] = -1.0;

}

void de_jac_run_xy_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[2220] = p[0]*(2.0*y[8]*u[2] - 2.0*y[14]*u[2] + y[32]*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[9]*y[33] + y[8]*y[32] - y[15]*y[33] - y[14]*y[32])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   -1.0*y[8]*pow(pow(y[9], 2) + pow(y[8], 2), -0.5)/pow(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[2221] = p[0]*(2.0*y[9]*u[2] - 2.0*y[15]*u[2] + y[33]*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[9]*y[33] + y[8]*y[32] - y[15]*y[33] - y[14]*y[32])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   -1.0*y[9]*pow(pow(y[9], 2) + pow(y[8], 2), -0.5)/pow(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[2226] = p[0]*(-2.0*y[8]*u[2] + 2.0*y[14]*u[2] - y[32]*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[2227] = p[0]*(-2.0*y[9]*u[2] + 2.0*y[15]*u[2] - y[33]*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[2244] = p[0]*(y[8] - y[14])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[2245] = p[0]*(y[9] - y[15])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[2289] = p[0]*(2.0*y[10]*u[6] - 2.0*y[14]*u[6] + y[34]*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[11]*y[35] + y[10]*y[34] - y[15]*y[35] - y[14]*y[34])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   -1.0*y[10]*pow(pow(y[11], 2) + pow(y[10], 2), -0.5)/pow(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[2290] = p[0]*(2.0*y[11]*u[6] - 2.0*y[15]*u[6] + y[35]*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[11]*y[35] + y[10]*y[34] - y[15]*y[35] - y[14]*y[34])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   -1.0*y[11]*pow(pow(y[11], 2) + pow(y[10], 2), -0.5)/pow(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[2293] = p[0]*(-2.0*y[10]*u[6] + 2.0*y[14]*u[6] - y[34]*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[2294] = p[0]*(-2.0*y[11]*u[6] + 2.0*y[15]*u[6] - y[35]*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[2313] = p[0]*(y[10] - y[14])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[2314] = p[0]*(y[11] - y[15])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[2358] = p[0]*(2.0*y[12]*u[10] - 2.0*y[14]*u[10] + y[36]*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[13]*y[37] + y[12]*y[36] - y[15]*y[37] - y[14]*y[36])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   -1.0*y[12]*pow(pow(y[13], 2) + pow(y[12], 2), -0.5)/pow(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[2359] = p[0]*(2.0*y[13]*u[10] - 2.0*y[15]*u[10] + y[37]*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[13]*y[37] + y[12]*y[36] - y[15]*y[37] - y[14]*y[36])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   -1.0*y[13]*pow(pow(y[13], 2) + pow(y[12], 2), -0.5)/pow(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[2360] = p[0]*(-2.0*y[12]*u[10] + 2.0*y[14]*u[10] - y[36]*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[2361] = p[0]*(-2.0*y[13]*u[10] + 2.0*y[15]*u[10] - y[37]*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[2382] = p[0]*(y[12] - y[14])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[2383] = p[0]*(y[13] - y[15])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[2421] = p[0]*(-2.0*y[8]*u[3] + 2.0*y[14]*u[3] - y[33]*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[9]*y[32] - y[8]*y[33] - y[15]*y[32] + y[14]*y[33])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   -1.0*y[8]*pow(pow(y[9], 2) + pow(y[8], 2), -0.5)/pow(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[2422] = p[0]*(-2.0*y[9]*u[3] + 2.0*y[15]*u[3] + y[32]*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[9]*y[32] - y[8]*y[33] - y[15]*y[32] + y[14]*y[33])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   -1.0*y[9]*pow(pow(y[9], 2) + pow(y[8], 2), -0.5)/pow(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[2427] = p[0]*(2.0*y[8]*u[3] - 2.0*y[14]*u[3] + y[33]*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[2428] = p[0]*(2.0*y[9]*u[3] - 2.0*y[15]*u[3] - y[32]*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[2445] = p[0]*(y[9] - y[15])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[2446] = p[0]*(-y[8] + y[14])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[2490] = p[0]*(-2.0*y[10]*u[7] + 2.0*y[14]*u[7] - y[35]*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[11]*y[34] - y[10]*y[35] - y[15]*y[34] + y[14]*y[35])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   -1.0*y[10]*pow(pow(y[11], 2) + pow(y[10], 2), -0.5)/pow(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[2491] = p[0]*(-2.0*y[11]*u[7] + 2.0*y[15]*u[7] + y[34]*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[11]*y[34] - y[10]*y[35] - y[15]*y[34] + y[14]*y[35])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   -1.0*y[11]*pow(pow(y[11], 2) + pow(y[10], 2), -0.5)/pow(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[2494] = p[0]*(2.0*y[10]*u[7] - 2.0*y[14]*u[7] + y[35]*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[2495] = p[0]*(2.0*y[11]*u[7] - 2.0*y[15]*u[7] - y[34]*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[2514] = p[0]*(y[11] - y[15])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[2515] = p[0]*(-y[10] + y[14])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[2559] = p[0]*(-2.0*y[12]*u[11] + 2.0*y[14]*u[11] - y[37]*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[13]*y[36] - y[12]*y[37] - y[15]*y[36] + y[14]*y[37])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   -1.0*y[12]*pow(pow(y[13], 2) + pow(y[12], 2), -0.5)/pow(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[2560] = p[0]*(-2.0*y[13]*u[11] + 2.0*y[15]*u[11] + y[36]*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[13]*y[36] - y[12]*y[37] - y[15]*y[36] + y[14]*y[37])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   -1.0*y[13]*pow(pow(y[13], 2) + pow(y[12], 2), -0.5)/pow(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[2561] = p[0]*(2.0*y[12]*u[11] - 2.0*y[14]*u[11] + y[37]*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[2562] = p[0]*(2.0*y[13]*u[11] - 2.0*y[15]*u[11] - y[36]*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[2583] = p[0]*(y[13] - y[15])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[2584] = p[0]*(-y[12] + y[14])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
));

}

void de_jac_run_up_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[0] = -p[17];
out[644] = -u[12];
out[715] = -u[12];
out[779] = -u[12];
out[850] = -u[12];
out[914] = -u[12];
out[985] = -u[12];
out[1049] = -u[12];
out[1120] = -u[12];
out[1180] = u[12];
out[1251] = u[12];
out[1315] = u[12];
out[1386] = u[12];
out[1450] = u[12];
out[1521] = u[12];
out[1585] = u[12];
out[1656] = u[12];
out[2788] = -p[1];
out[2792] = 1.0*p[2];
out[2856] = -p[1];
out[2860] = 1.0*p[2];
out[2924] = -p[1];
out[2928] = 1.0*p[2];
out[2992] = -p[1];
out[2996] = 1.0*p[2];
out[3056] = -1.0*p[2];
out[3060] = -1.0*p[1];
out[3124] = -1.0*p[2];
out[3128] = -1.0*p[1];
out[3192] = -1.0*p[2];
out[3196] = -1.0*p[1];
out[3260] = -1.0*p[2];
out[3264] = -1.0*p[1];
out[3332] = -p[4];
out[3336] = 1.0*p[3];
out[3400] = -p[4];
out[3404] = 1.0*p[3];
out[3468] = -p[4];
out[3472] = 1.0*p[3];
out[3536] = -p[6];
out[3540] = 1.0*p[5];
out[3600] = -1.0*p[3];
out[3604] = -1.0*p[4];
out[3668] = -1.0*p[3];
out[3672] = -1.0*p[4];
out[3736] = -1.0*p[3];
out[3740] = -1.0*p[4];
out[3804] = -1.0*p[5];
out[3808] = -1.0*p[6];
out[3876] = -p[10];
out[3880] = 1.0*p[9];
out[3944] = -p[10];
out[3948] = 1.0*p[9];
out[4012] = -p[10];
out[4016] = 1.0*p[9];
out[4080] = -p[12];
out[4084] = 1.0*p[11];
out[4144] = -1.0*p[9];
out[4148] = -1.0*p[10];
out[4212] = -1.0*p[9];
out[4216] = -1.0*p[10];
out[4280] = -1.0*p[9];
out[4284] = -1.0*p[10];
out[4348] = -1.0*p[11];
out[4352] = -1.0*p[12];
out[4422] = p[16];
out[4487] = -p[15];

}

void de_jac_run_num_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[65] = -1.0;
out[68] = 184.75059629806526;
out[69] = 94.615102344455664;
out[70] = -69.368640085185305;
out[71] = -17.974903844364835;
out[72] = -43.675799641021079;
out[73] = -40.748687841669557;
out[74] = -69.368640085185291;
out[75] = -17.974903844364821;
out[76] = -184.75059629806526;
out[77] = -94.615102344455664;
out[78] = 69.368640085185305;
out[79] = 17.974903844364835;
out[80] = 43.675799641021079;
out[81] = 40.748687841669557;
out[82] = 69.368640085185291;
out[83] = 17.974903844364821;
out[116] = -1.0;
out[135] = -94.615102344455664;
out[136] = 184.75059629806526;
out[137] = 17.974903844364835;
out[138] = -69.368640085185305;
out[139] = 40.748687841669557;
out[140] = -43.675799641021079;
out[141] = 17.974903844364821;
out[142] = -69.368640085185291;
out[143] = 94.615102344455664;
out[144] = -184.75059629806526;
out[145] = -17.974903844364835;
out[146] = 69.368640085185305;
out[147] = -40.748687841669557;
out[148] = 43.675799641021079;
out[149] = -17.974903844364821;
out[150] = 69.368640085185291;
out[187] = -1.0;
out[202] = -69.368640085185291;
out[203] = -17.974903844364825;
out[204] = 184.7505962980652;
out[205] = 94.615102344455636;
out[206] = -69.368640085185319;
out[207] = -17.97490384436481;
out[208] = -43.675799641021079;
out[209] = -40.748687841669565;
out[210] = 69.368640085185291;
out[211] = 17.974903844364825;
out[212] = -184.7505962980652;
out[213] = -94.615102344455636;
out[214] = 69.368640085185319;
out[215] = 17.97490384436481;
out[216] = 43.675799641021079;
out[217] = 40.748687841669565;
out[251] = -1.0;
out[269] = 17.974903844364825;
out[270] = -69.368640085185291;
out[271] = -94.615102344455636;
out[272] = 184.7505962980652;
out[273] = 17.97490384436481;
out[274] = -69.368640085185319;
out[275] = 40.748687841669565;
out[276] = -43.675799641021079;
out[277] = -17.974903844364825;
out[278] = 69.368640085185291;
out[279] = 94.615102344455636;
out[280] = -184.7505962980652;
out[281] = -17.97490384436481;
out[282] = 69.368640085185319;
out[283] = -40.748687841669565;
out[284] = 43.675799641021079;
out[322] = -1.0;
out[336] = -43.675799641021101;
out[337] = -40.748687841669579;
out[338] = -69.368640085185305;
out[339] = -17.974903844364821;
out[340] = 184.75059629806526;
out[341] = 94.61510234445565;
out[342] = -69.368640085185305;
out[343] = -17.974903844364821;
out[344] = 43.675799641021101;
out[345] = 40.748687841669579;
out[346] = 69.368640085185305;
out[347] = 17.974903844364821;
out[348] = -184.75059629806526;
out[349] = -94.61510234445565;
out[350] = 69.368640085185305;
out[351] = 17.974903844364821;
out[386] = -1.0;
out[403] = 40.748687841669579;
out[404] = -43.675799641021101;
out[405] = 17.974903844364821;
out[406] = -69.368640085185305;
out[407] = -94.61510234445565;
out[408] = 184.75059629806526;
out[409] = 17.974903844364821;
out[410] = -69.368640085185305;
out[411] = -40.748687841669579;
out[412] = 43.675799641021101;
out[413] = -17.974903844364821;
out[414] = 69.368640085185305;
out[415] = 94.61510234445565;
out[416] = -184.75059629806526;
out[417] = -17.974903844364821;
out[418] = 69.368640085185305;
out[457] = -1.0;
out[470] = -69.368640085185305;
out[471] = -17.974903844364825;
out[472] = -43.675799641021079;
out[473] = -40.74868784166955;
out[474] = -69.368640085185291;
out[475] = -17.974903844364839;
out[476] = 184.7505962980652;
out[477] = 94.61510234445565;
out[478] = 69.368640085185305;
out[479] = 17.974903844364825;
out[480] = 43.675799641021079;
out[481] = 40.74868784166955;
out[482] = 69.368640085185291;
out[483] = 17.974903844364839;
out[484] = -184.7505962980652;
out[485] = -94.61510234445565;
out[521] = -1.0;
out[537] = 17.974903844364825;
out[538] = -69.368640085185305;
out[539] = 40.74868784166955;
out[540] = -43.675799641021079;
out[541] = 17.974903844364839;
out[542] = -69.368640085185291;
out[543] = -94.61510234445565;
out[544] = 184.7505962980652;
out[545] = -17.974903844364825;
out[546] = 69.368640085185305;
out[547] = -40.74868784166955;
out[548] = 43.675799641021079;
out[549] = -17.974903844364839;
out[550] = 69.368640085185291;
out[551] = 94.61510234445565;
out[552] = -184.7505962980652;
out[592] = -1.0;
out[604] = -184.75059629806526;
out[605] = -94.615102344455664;
out[606] = 69.368640085185305;
out[607] = 17.974903844364835;
out[608] = 43.675799641021079;
out[609] = 40.748687841669557;
out[610] = 69.368640085185291;
out[611] = 17.974903844364821;
out[612] = 184.75059629806526;
out[613] = 94.615102344455664;
out[614] = -69.368640085185305;
out[615] = -17.974903844364835;
out[616] = -43.675799641021079;
out[617] = -40.748687841669557;
out[618] = -69.368640085185291;
out[619] = -17.974903844364821;
out[636] = -1.0;
out[671] = 94.615102344455664;
out[672] = -184.75059629806526;
out[673] = -17.974903844364835;
out[674] = 69.368640085185305;
out[675] = -40.748687841669557;
out[676] = 43.675799641021079;
out[677] = -17.974903844364821;
out[678] = 69.368640085185291;
out[679] = -94.615102344455664;
out[680] = 184.75059629806526;
out[681] = 17.974903844364835;
out[682] = -69.368640085185305;
out[683] = 40.748687841669557;
out[684] = -43.675799641021079;
out[685] = 17.974903844364821;
out[686] = -69.368640085185291;
out[704] = -1.0;
out[738] = 69.368640085185291;
out[739] = 17.974903844364825;
out[740] = -184.7505962980652;
out[741] = -94.615102344455636;
out[742] = 69.368640085185319;
out[743] = 17.97490384436481;
out[744] = 43.675799641021079;
out[745] = 40.748687841669565;
out[746] = -69.368640085185291;
out[747] = -17.974903844364825;
out[748] = 184.7505962980652;
out[749] = 94.615102344455636;
out[750] = -69.368640085185319;
out[751] = -17.97490384436481;
out[752] = -43.675799641021079;
out[753] = -40.748687841669565;
out[772] = -1.0;
out[805] = -17.974903844364825;
out[806] = 69.368640085185291;
out[807] = 94.615102344455636;
out[808] = -184.7505962980652;
out[809] = -17.97490384436481;
out[810] = 69.368640085185319;
out[811] = -40.748687841669565;
out[812] = 43.675799641021079;
out[813] = 17.974903844364825;
out[814] = -69.368640085185291;
out[815] = -94.615102344455636;
out[816] = 184.7505962980652;
out[817] = 17.97490384436481;
out[818] = -69.368640085185319;
out[819] = 40.748687841669565;
out[820] = -43.675799641021079;
out[840] = -1.0;
out[872] = 43.675799641021101;
out[873] = 40.748687841669579;
out[874] = 69.368640085185305;
out[875] = 17.974903844364821;
out[876] = -184.75059629806526;
out[877] = -94.61510234445565;
out[878] = 69.368640085185305;
out[879] = 17.974903844364821;
out[880] = -43.675799641021101;
out[881] = -40.748687841669579;
out[882] = -69.368640085185305;
out[883] = -17.974903844364821;
out[884] = 184.75059629806526;
out[885] = 94.61510234445565;
out[886] = -69.368640085185305;
out[887] = -17.974903844364821;
out[908] = -1.0;
out[939] = -40.748687841669579;
out[940] = 43.675799641021101;
out[941] = -17.974903844364821;
out[942] = 69.368640085185305;
out[943] = 94.61510234445565;
out[944] = -184.75059629806526;
out[945] = -17.974903844364821;
out[946] = 69.368640085185305;
out[947] = 40.748687841669579;
out[948] = -43.675799641021101;
out[949] = 17.974903844364821;
out[950] = -69.368640085185305;
out[951] = -94.61510234445565;
out[952] = 184.75059629806526;
out[953] = 17.974903844364821;
out[954] = -69.368640085185305;
out[976] = -1.0;
out[1006] = 69.368640085185305;
out[1007] = 17.974903844364825;
out[1008] = 43.675799641021079;
out[1009] = 40.74868784166955;
out[1010] = 69.368640085185291;
out[1011] = 17.974903844364839;
out[1012] = -184.7505962980652;
out[1013] = -94.61510234445565;
out[1014] = -69.368640085185305;
out[1015] = -17.974903844364825;
out[1016] = -43.675799641021079;
out[1017] = -40.74868784166955;
out[1018] = -69.368640085185291;
out[1019] = -17.974903844364839;
out[1020] = 184.7505962980652;
out[1021] = 94.61510234445565;
out[1044] = -1.0;
out[1073] = -17.974903844364825;
out[1074] = 69.368640085185305;
out[1075] = -40.74868784166955;
out[1076] = 43.675799641021079;
out[1077] = -17.974903844364839;
out[1078] = 69.368640085185291;
out[1079] = 94.61510234445565;
out[1080] = -184.7505962980652;
out[1081] = 17.974903844364825;
out[1082] = -69.368640085185305;
out[1083] = 40.74868784166955;
out[1084] = -43.675799641021079;
out[1085] = 17.974903844364839;
out[1086] = -69.368640085185291;
out[1087] = -94.61510234445565;
out[1088] = 184.7505962980652;
out[1112] = -1.0;
out[1156] = 184.75059629806526;
out[1157] = 94.615102344455664;
out[1158] = -69.368640085185305;
out[1159] = -17.974903844364835;
out[1160] = -43.675799641021079;
out[1161] = -40.748687841669557;
out[1162] = -69.368640085185291;
out[1163] = -17.974903844364821;
out[1164] = -184.75059629806526;
out[1165] = -94.615102344455664;
out[1166] = 69.368640085185305;
out[1167] = 17.974903844364835;
out[1168] = 43.675799641021079;
out[1169] = 40.748687841669557;
out[1170] = 69.368640085185291;
out[1171] = 17.974903844364821;
out[1223] = -94.615102344455664;
out[1224] = 184.75059629806526;
out[1225] = 17.974903844364835;
out[1226] = -69.368640085185305;
out[1227] = 40.748687841669557;
out[1228] = -43.675799641021079;
out[1229] = 17.974903844364821;
out[1230] = -69.368640085185291;
out[1231] = 94.615102344455664;
out[1232] = -184.75059629806526;
out[1233] = -17.974903844364835;
out[1234] = 69.368640085185305;
out[1235] = -40.748687841669557;
out[1236] = 43.675799641021079;
out[1237] = -17.974903844364821;
out[1238] = 69.368640085185291;
out[1290] = -69.368640085185291;
out[1291] = -17.974903844364825;
out[1292] = 184.7505962980652;
out[1293] = 94.615102344455636;
out[1294] = -69.368640085185319;
out[1295] = -17.97490384436481;
out[1296] = -43.675799641021079;
out[1297] = -40.748687841669565;
out[1298] = 69.368640085185291;
out[1299] = 17.974903844364825;
out[1300] = -184.7505962980652;
out[1301] = -94.615102344455636;
out[1302] = 69.368640085185319;
out[1303] = 17.97490384436481;
out[1304] = 43.675799641021079;
out[1305] = 40.748687841669565;
out[1357] = 17.974903844364825;
out[1358] = -69.368640085185291;
out[1359] = -94.615102344455636;
out[1360] = 184.7505962980652;
out[1361] = 17.97490384436481;
out[1362] = -69.368640085185319;
out[1363] = 40.748687841669565;
out[1364] = -43.675799641021079;
out[1365] = -17.974903844364825;
out[1366] = 69.368640085185291;
out[1367] = 94.615102344455636;
out[1368] = -184.7505962980652;
out[1369] = -17.97490384436481;
out[1370] = 69.368640085185319;
out[1371] = -40.748687841669565;
out[1372] = 43.675799641021079;
out[1424] = -43.675799641021101;
out[1425] = -40.748687841669579;
out[1426] = -69.368640085185305;
out[1427] = -17.974903844364821;
out[1428] = 184.75059629806526;
out[1429] = 94.61510234445565;
out[1430] = -69.368640085185305;
out[1431] = -17.974903844364821;
out[1432] = 43.675799641021101;
out[1433] = 40.748687841669579;
out[1434] = 69.368640085185305;
out[1435] = 17.974903844364821;
out[1436] = -184.75059629806526;
out[1437] = -94.61510234445565;
out[1438] = 69.368640085185305;
out[1439] = 17.974903844364821;
out[1491] = 40.748687841669579;
out[1492] = -43.675799641021101;
out[1493] = 17.974903844364821;
out[1494] = -69.368640085185305;
out[1495] = -94.61510234445565;
out[1496] = 184.75059629806526;
out[1497] = 17.974903844364821;
out[1498] = -69.368640085185305;
out[1499] = -40.748687841669579;
out[1500] = 43.675799641021101;
out[1501] = -17.974903844364821;
out[1502] = 69.368640085185305;
out[1503] = 94.61510234445565;
out[1504] = -184.75059629806526;
out[1505] = -17.974903844364821;
out[1506] = 69.368640085185305;
out[1558] = -69.368640085185305;
out[1559] = -17.974903844364825;
out[1560] = -43.675799641021079;
out[1561] = -40.74868784166955;
out[1562] = -69.368640085185291;
out[1563] = -17.974903844364839;
out[1564] = 184.7505962980652;
out[1565] = 94.61510234445565;
out[1566] = 69.368640085185305;
out[1567] = 17.974903844364825;
out[1568] = 43.675799641021079;
out[1569] = 40.74868784166955;
out[1570] = 69.368640085185291;
out[1571] = 17.974903844364839;
out[1572] = -184.7505962980652;
out[1573] = -94.61510234445565;
out[1625] = 17.974903844364825;
out[1626] = -69.368640085185305;
out[1627] = 40.74868784166955;
out[1628] = -43.675799641021079;
out[1629] = 17.974903844364839;
out[1630] = -69.368640085185291;
out[1631] = -94.61510234445565;
out[1632] = 184.7505962980652;
out[1633] = -17.974903844364825;
out[1634] = 69.368640085185305;
out[1635] = -40.74868784166955;
out[1636] = 43.675799641021079;
out[1637] = -17.974903844364839;
out[1638] = 69.368640085185291;
out[1639] = 94.61510234445565;
out[1640] = -184.7505962980652;
out[1692] = -184.75059629806526;
out[1693] = -94.615102344455664;
out[1694] = 69.368640085185305;
out[1695] = 17.974903844364835;
out[1696] = 43.675799641021079;
out[1697] = 40.748687841669557;
out[1698] = 69.368640085185291;
out[1699] = 17.974903844364821;
out[1700] = 184.75059629806526;
out[1701] = 94.615102344455664;
out[1702] = -69.368640085185305;
out[1703] = -17.974903844364835;
out[1704] = -43.675799641021079;
out[1705] = -40.748687841669557;
out[1706] = -69.368640085185291;
out[1707] = -17.974903844364821;
out[1732] = -1.0;
out[1759] = 94.615102344455664;
out[1760] = -184.75059629806526;
out[1761] = -17.974903844364835;
out[1762] = 69.368640085185305;
out[1763] = -40.748687841669557;
out[1764] = 43.675799641021079;
out[1765] = -17.974903844364821;
out[1766] = 69.368640085185291;
out[1767] = -94.615102344455664;
out[1768] = 184.75059629806526;
out[1769] = 17.974903844364835;
out[1770] = -69.368640085185305;
out[1771] = 40.748687841669557;
out[1772] = -43.675799641021079;
out[1773] = 17.974903844364821;
out[1774] = -69.368640085185291;
out[1803] = -1.0;
out[1826] = 69.368640085185291;
out[1827] = 17.974903844364825;
out[1828] = -184.7505962980652;
out[1829] = -94.615102344455636;
out[1830] = 69.368640085185319;
out[1831] = 17.97490384436481;
out[1832] = 43.675799641021079;
out[1833] = 40.748687841669565;
out[1834] = -69.368640085185291;
out[1835] = -17.974903844364825;
out[1836] = 184.7505962980652;
out[1837] = 94.615102344455636;
out[1838] = -69.368640085185319;
out[1839] = -17.97490384436481;
out[1840] = -43.675799641021079;
out[1841] = -40.748687841669565;
out[1867] = -1.0;
out[1893] = -17.974903844364825;
out[1894] = 69.368640085185291;
out[1895] = 94.615102344455636;
out[1896] = -184.7505962980652;
out[1897] = -17.97490384436481;
out[1898] = 69.368640085185319;
out[1899] = -40.748687841669565;
out[1900] = 43.675799641021079;
out[1901] = 17.974903844364825;
out[1902] = -69.368640085185291;
out[1903] = -94.615102344455636;
out[1904] = 184.7505962980652;
out[1905] = 17.97490384436481;
out[1906] = -69.368640085185319;
out[1907] = 40.748687841669565;
out[1908] = -43.675799641021079;
out[1938] = -1.0;
out[1960] = 43.675799641021101;
out[1961] = 40.748687841669579;
out[1962] = 69.368640085185305;
out[1963] = 17.974903844364821;
out[1964] = -184.75059629806526;
out[1965] = -94.61510234445565;
out[1966] = 69.368640085185305;
out[1967] = 17.974903844364821;
out[1968] = -43.675799641021101;
out[1969] = -40.748687841669579;
out[1970] = -69.368640085185305;
out[1971] = -17.974903844364821;
out[1972] = 184.75059629806526;
out[1973] = 94.61510234445565;
out[1974] = -69.368640085185305;
out[1975] = -17.974903844364821;
out[2002] = -1.0;
out[2027] = -40.748687841669579;
out[2028] = 43.675799641021101;
out[2029] = -17.974903844364821;
out[2030] = 69.368640085185305;
out[2031] = 94.61510234445565;
out[2032] = -184.75059629806526;
out[2033] = -17.974903844364821;
out[2034] = 69.368640085185305;
out[2035] = 40.748687841669579;
out[2036] = -43.675799641021101;
out[2037] = 17.974903844364821;
out[2038] = -69.368640085185305;
out[2039] = -94.61510234445565;
out[2040] = 184.75059629806526;
out[2041] = 17.974903844364821;
out[2042] = -69.368640085185305;
out[2073] = -1.0;
out[2094] = 69.368640085185305;
out[2095] = 17.974903844364825;
out[2096] = 43.675799641021079;
out[2097] = 40.74868784166955;
out[2098] = 69.368640085185291;
out[2099] = 17.974903844364839;
out[2100] = -184.7505962980652;
out[2101] = -94.61510234445565;
out[2102] = -69.368640085185305;
out[2103] = -17.974903844364825;
out[2104] = -43.675799641021079;
out[2105] = -40.74868784166955;
out[2106] = -69.368640085185291;
out[2107] = -17.974903844364839;
out[2108] = 184.7505962980652;
out[2109] = 94.61510234445565;
out[2137] = -1.0;
out[2161] = -17.974903844364825;
out[2162] = 69.368640085185305;
out[2163] = -40.74868784166955;
out[2164] = 43.675799641021079;
out[2165] = -17.974903844364839;
out[2166] = 69.368640085185291;
out[2167] = 94.61510234445565;
out[2168] = -184.7505962980652;
out[2169] = 17.974903844364825;
out[2170] = -69.368640085185305;
out[2171] = 40.74868784166955;
out[2172] = -43.675799641021079;
out[2173] = 17.974903844364839;
out[2174] = -69.368640085185291;
out[2175] = -94.61510234445565;
out[2176] = 184.7505962980652;
out[2208] = -1.0;
out[2646] = 1.0;
out[2648] = 1.0;
out[2650] = 1.0;
out[2652] = 1.0;
out[2714] = 1.0;
out[2716] = 1.0;
out[2718] = 1.0;
out[2720] = 1.0;
out[2756] = 1.0;
out[2764] = -1.0;
out[2825] = 1.0;
out[2833] = -1.0;
out[2894] = 1.0;
out[2902] = -1.0;
out[2963] = 1.0;
out[2971] = -1.0;
out[3025] = 1.0;
out[3033] = -1.0;
out[3094] = 1.0;
out[3102] = -1.0;
out[3163] = 1.0;
out[3171] = -1.0;
out[3232] = 1.0;
out[3240] = -1.0;
out[3284] = -1.0;
out[3353] = -1.0;
out[3422] = -1.0;
out[3491] = -1.0;
out[3553] = -1.0;
out[3622] = -1.0;
out[3691] = -1.0;
out[3760] = -1.0;
out[3844] = -1.0;
out[3913] = -1.0;
out[3982] = -1.0;
out[4051] = -1.0;
out[4113] = -1.0;
out[4182] = -1.0;
out[4251] = -1.0;
out[4320] = -1.0;
out[4420] = -1.0;
out[4488] = -1.0;

}

void sp_jac_run_xy_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[554] = p[0]*(2.0*y[8]*u[2] - 2.0*y[14]*u[2] + y[32]*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[9]*y[33] + y[8]*y[32] - y[15]*y[33] - y[14]*y[32])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   -1.0*y[8]*pow(pow(y[9], 2) + pow(y[8], 2), -0.5)/pow(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[555] = p[0]*(2.0*y[9]*u[2] - 2.0*y[15]*u[2] + y[33]*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[9]*y[33] + y[8]*y[32] - y[15]*y[33] - y[14]*y[32])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   -1.0*y[9]*pow(pow(y[9], 2) + pow(y[8], 2), -0.5)/pow(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[556] = p[0]*(-2.0*y[8]*u[2] + 2.0*y[14]*u[2] - y[32]*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[557] = p[0]*(-2.0*y[9]*u[2] + 2.0*y[15]*u[2] - y[33]*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[558] = p[0]*(y[8] - y[14])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[559] = p[0]*(y[9] - y[15])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[560] = p[0]*(2.0*y[10]*u[6] - 2.0*y[14]*u[6] + y[34]*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[11]*y[35] + y[10]*y[34] - y[15]*y[35] - y[14]*y[34])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   -1.0*y[10]*pow(pow(y[11], 2) + pow(y[10], 2), -0.5)/pow(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[561] = p[0]*(2.0*y[11]*u[6] - 2.0*y[15]*u[6] + y[35]*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[11]*y[35] + y[10]*y[34] - y[15]*y[35] - y[14]*y[34])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   -1.0*y[11]*pow(pow(y[11], 2) + pow(y[10], 2), -0.5)/pow(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[562] = p[0]*(-2.0*y[10]*u[6] + 2.0*y[14]*u[6] - y[34]*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[563] = p[0]*(-2.0*y[11]*u[6] + 2.0*y[15]*u[6] - y[35]*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[564] = p[0]*(y[10] - y[14])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[565] = p[0]*(y[11] - y[15])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[566] = p[0]*(2.0*y[12]*u[10] - 2.0*y[14]*u[10] + y[36]*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[13]*y[37] + y[12]*y[36] - y[15]*y[37] - y[14]*y[36])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   -1.0*y[12]*pow(pow(y[13], 2) + pow(y[12], 2), -0.5)/pow(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[567] = p[0]*(2.0*y[13]*u[10] - 2.0*y[15]*u[10] + y[37]*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[13]*y[37] + y[12]*y[36] - y[15]*y[37] - y[14]*y[36])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   -1.0*y[13]*pow(pow(y[13], 2) + pow(y[12], 2), -0.5)/pow(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[568] = p[0]*(-2.0*y[12]*u[10] + 2.0*y[14]*u[10] - y[36]*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[569] = p[0]*(-2.0*y[13]*u[10] + 2.0*y[15]*u[10] - y[37]*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[570] = p[0]*(y[12] - y[14])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[571] = p[0]*(y[13] - y[15])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[572] = p[0]*(-2.0*y[8]*u[3] + 2.0*y[14]*u[3] - y[33]*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[9]*y[32] - y[8]*y[33] - y[15]*y[32] + y[14]*y[33])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   -1.0*y[8]*pow(pow(y[9], 2) + pow(y[8], 2), -0.5)/pow(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[573] = p[0]*(-2.0*y[9]*u[3] + 2.0*y[15]*u[3] + y[32]*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[9]*y[32] - y[8]*y[33] - y[15]*y[32] + y[14]*y[33])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   -1.0*y[9]*pow(pow(y[9], 2) + pow(y[8], 2), -0.5)/pow(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[574] = p[0]*(2.0*y[8]*u[3] - 2.0*y[14]*u[3] + y[33]*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[575] = p[0]*(2.0*y[9]*u[3] - 2.0*y[15]*u[3] - y[32]*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[576] = p[0]*(y[9] - y[15])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[577] = p[0]*(-y[8] + y[14])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[578] = p[0]*(-2.0*y[10]*u[7] + 2.0*y[14]*u[7] - y[35]*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[11]*y[34] - y[10]*y[35] - y[15]*y[34] + y[14]*y[35])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   -1.0*y[10]*pow(pow(y[11], 2) + pow(y[10], 2), -0.5)/pow(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[579] = p[0]*(-2.0*y[11]*u[7] + 2.0*y[15]*u[7] + y[34]*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[11]*y[34] - y[10]*y[35] - y[15]*y[34] + y[14]*y[35])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   -1.0*y[11]*pow(pow(y[11], 2) + pow(y[10], 2), -0.5)/pow(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[580] = p[0]*(2.0*y[10]*u[7] - 2.0*y[14]*u[7] + y[35]*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[581] = p[0]*(2.0*y[11]*u[7] - 2.0*y[15]*u[7] - y[34]*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[582] = p[0]*(y[11] - y[15])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[583] = p[0]*(-y[10] + y[14])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[584] = p[0]*(-2.0*y[12]*u[11] + 2.0*y[14]*u[11] - y[37]*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[13]*y[36] - y[12]*y[37] - y[15]*y[36] + y[14]*y[37])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   -1.0*y[12]*pow(pow(y[13], 2) + pow(y[12], 2), -0.5)/pow(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[585] = p[0]*(-2.0*y[13]*u[11] + 2.0*y[15]*u[11] + y[36]*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[13]*y[36] - y[12]*y[37] - y[15]*y[36] + y[14]*y[37])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   -1.0*y[13]*pow(pow(y[13], 2) + pow(y[12], 2), -0.5)/pow(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[586] = p[0]*(2.0*y[12]*u[11] - 2.0*y[14]*u[11] + y[37]*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[587] = p[0]*(2.0*y[13]*u[11] - 2.0*y[15]*u[11] - y[36]*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[588] = p[0]*(y[13] - y[15])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[589] = p[0]*(-y[12] + y[14])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
));

}

void sp_jac_run_up_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[0] = -p[17];
out[155] = -u[12];
out[173] = -u[12];
out[191] = -u[12];
out[209] = -u[12];
out[227] = -u[12];
out[245] = -u[12];
out[263] = -u[12];
out[281] = -u[12];
out[298] = u[12];
out[315] = u[12];
out[332] = u[12];
out[349] = u[12];
out[366] = u[12];
out[383] = u[12];
out[400] = u[12];
out[417] = u[12];
out[600] = -p[1];
out[601] = 1.0*p[2];
out[604] = -p[1];
out[605] = 1.0*p[2];
out[608] = -p[1];
out[609] = 1.0*p[2];
out[612] = -p[1];
out[613] = 1.0*p[2];
out[616] = -1.0*p[2];
out[617] = -1.0*p[1];
out[620] = -1.0*p[2];
out[621] = -1.0*p[1];
out[624] = -1.0*p[2];
out[625] = -1.0*p[1];
out[628] = -1.0*p[2];
out[629] = -1.0*p[1];
out[631] = -p[4];
out[632] = 1.0*p[3];
out[634] = -p[4];
out[635] = 1.0*p[3];
out[637] = -p[4];
out[638] = 1.0*p[3];
out[640] = -p[6];
out[641] = 1.0*p[5];
out[643] = -1.0*p[3];
out[644] = -1.0*p[4];
out[646] = -1.0*p[3];
out[647] = -1.0*p[4];
out[649] = -1.0*p[3];
out[650] = -1.0*p[4];
out[652] = -1.0*p[5];
out[653] = -1.0*p[6];
out[655] = -p[10];
out[656] = 1.0*p[9];
out[658] = -p[10];
out[659] = 1.0*p[9];
out[661] = -p[10];
out[662] = 1.0*p[9];
out[664] = -p[12];
out[665] = 1.0*p[11];
out[667] = -1.0*p[9];
out[668] = -1.0*p[10];
out[670] = -1.0*p[9];
out[671] = -1.0*p[10];
out[673] = -1.0*p[9];
out[674] = -1.0*p[10];
out[676] = -1.0*p[11];
out[677] = -1.0*p[12];
out[679] = p[16];
out[680] = -p[15];

}

void sp_jac_run_num_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[1] = -1.0;
out[2] = 184.75059629806526;
out[3] = 94.615102344455664;
out[4] = -69.368640085185305;
out[5] = -17.974903844364835;
out[6] = -43.675799641021079;
out[7] = -40.748687841669557;
out[8] = -69.368640085185291;
out[9] = -17.974903844364821;
out[10] = -184.75059629806526;
out[11] = -94.615102344455664;
out[12] = 69.368640085185305;
out[13] = 17.974903844364835;
out[14] = 43.675799641021079;
out[15] = 40.748687841669557;
out[16] = 69.368640085185291;
out[17] = 17.974903844364821;
out[18] = -1.0;
out[19] = -94.615102344455664;
out[20] = 184.75059629806526;
out[21] = 17.974903844364835;
out[22] = -69.368640085185305;
out[23] = 40.748687841669557;
out[24] = -43.675799641021079;
out[25] = 17.974903844364821;
out[26] = -69.368640085185291;
out[27] = 94.615102344455664;
out[28] = -184.75059629806526;
out[29] = -17.974903844364835;
out[30] = 69.368640085185305;
out[31] = -40.748687841669557;
out[32] = 43.675799641021079;
out[33] = -17.974903844364821;
out[34] = 69.368640085185291;
out[35] = -1.0;
out[36] = -69.368640085185291;
out[37] = -17.974903844364825;
out[38] = 184.7505962980652;
out[39] = 94.615102344455636;
out[40] = -69.368640085185319;
out[41] = -17.97490384436481;
out[42] = -43.675799641021079;
out[43] = -40.748687841669565;
out[44] = 69.368640085185291;
out[45] = 17.974903844364825;
out[46] = -184.7505962980652;
out[47] = -94.615102344455636;
out[48] = 69.368640085185319;
out[49] = 17.97490384436481;
out[50] = 43.675799641021079;
out[51] = 40.748687841669565;
out[52] = -1.0;
out[53] = 17.974903844364825;
out[54] = -69.368640085185291;
out[55] = -94.615102344455636;
out[56] = 184.7505962980652;
out[57] = 17.97490384436481;
out[58] = -69.368640085185319;
out[59] = 40.748687841669565;
out[60] = -43.675799641021079;
out[61] = -17.974903844364825;
out[62] = 69.368640085185291;
out[63] = 94.615102344455636;
out[64] = -184.7505962980652;
out[65] = -17.97490384436481;
out[66] = 69.368640085185319;
out[67] = -40.748687841669565;
out[68] = 43.675799641021079;
out[69] = -1.0;
out[70] = -43.675799641021101;
out[71] = -40.748687841669579;
out[72] = -69.368640085185305;
out[73] = -17.974903844364821;
out[74] = 184.75059629806526;
out[75] = 94.61510234445565;
out[76] = -69.368640085185305;
out[77] = -17.974903844364821;
out[78] = 43.675799641021101;
out[79] = 40.748687841669579;
out[80] = 69.368640085185305;
out[81] = 17.974903844364821;
out[82] = -184.75059629806526;
out[83] = -94.61510234445565;
out[84] = 69.368640085185305;
out[85] = 17.974903844364821;
out[86] = -1.0;
out[87] = 40.748687841669579;
out[88] = -43.675799641021101;
out[89] = 17.974903844364821;
out[90] = -69.368640085185305;
out[91] = -94.61510234445565;
out[92] = 184.75059629806526;
out[93] = 17.974903844364821;
out[94] = -69.368640085185305;
out[95] = -40.748687841669579;
out[96] = 43.675799641021101;
out[97] = -17.974903844364821;
out[98] = 69.368640085185305;
out[99] = 94.61510234445565;
out[100] = -184.75059629806526;
out[101] = -17.974903844364821;
out[102] = 69.368640085185305;
out[103] = -1.0;
out[104] = -69.368640085185305;
out[105] = -17.974903844364825;
out[106] = -43.675799641021079;
out[107] = -40.74868784166955;
out[108] = -69.368640085185291;
out[109] = -17.974903844364839;
out[110] = 184.7505962980652;
out[111] = 94.61510234445565;
out[112] = 69.368640085185305;
out[113] = 17.974903844364825;
out[114] = 43.675799641021079;
out[115] = 40.74868784166955;
out[116] = 69.368640085185291;
out[117] = 17.974903844364839;
out[118] = -184.7505962980652;
out[119] = -94.61510234445565;
out[120] = -1.0;
out[121] = 17.974903844364825;
out[122] = -69.368640085185305;
out[123] = 40.74868784166955;
out[124] = -43.675799641021079;
out[125] = 17.974903844364839;
out[126] = -69.368640085185291;
out[127] = -94.61510234445565;
out[128] = 184.7505962980652;
out[129] = -17.974903844364825;
out[130] = 69.368640085185305;
out[131] = -40.74868784166955;
out[132] = 43.675799641021079;
out[133] = -17.974903844364839;
out[134] = 69.368640085185291;
out[135] = 94.61510234445565;
out[136] = -184.7505962980652;
out[137] = -1.0;
out[138] = -184.75059629806526;
out[139] = -94.615102344455664;
out[140] = 69.368640085185305;
out[141] = 17.974903844364835;
out[142] = 43.675799641021079;
out[143] = 40.748687841669557;
out[144] = 69.368640085185291;
out[145] = 17.974903844364821;
out[146] = 184.75059629806526;
out[147] = 94.615102344455664;
out[148] = -69.368640085185305;
out[149] = -17.974903844364835;
out[150] = -43.675799641021079;
out[151] = -40.748687841669557;
out[152] = -69.368640085185291;
out[153] = -17.974903844364821;
out[154] = -1.0;
out[156] = 94.615102344455664;
out[157] = -184.75059629806526;
out[158] = -17.974903844364835;
out[159] = 69.368640085185305;
out[160] = -40.748687841669557;
out[161] = 43.675799641021079;
out[162] = -17.974903844364821;
out[163] = 69.368640085185291;
out[164] = -94.615102344455664;
out[165] = 184.75059629806526;
out[166] = 17.974903844364835;
out[167] = -69.368640085185305;
out[168] = 40.748687841669557;
out[169] = -43.675799641021079;
out[170] = 17.974903844364821;
out[171] = -69.368640085185291;
out[172] = -1.0;
out[174] = 69.368640085185291;
out[175] = 17.974903844364825;
out[176] = -184.7505962980652;
out[177] = -94.615102344455636;
out[178] = 69.368640085185319;
out[179] = 17.97490384436481;
out[180] = 43.675799641021079;
out[181] = 40.748687841669565;
out[182] = -69.368640085185291;
out[183] = -17.974903844364825;
out[184] = 184.7505962980652;
out[185] = 94.615102344455636;
out[186] = -69.368640085185319;
out[187] = -17.97490384436481;
out[188] = -43.675799641021079;
out[189] = -40.748687841669565;
out[190] = -1.0;
out[192] = -17.974903844364825;
out[193] = 69.368640085185291;
out[194] = 94.615102344455636;
out[195] = -184.7505962980652;
out[196] = -17.97490384436481;
out[197] = 69.368640085185319;
out[198] = -40.748687841669565;
out[199] = 43.675799641021079;
out[200] = 17.974903844364825;
out[201] = -69.368640085185291;
out[202] = -94.615102344455636;
out[203] = 184.7505962980652;
out[204] = 17.97490384436481;
out[205] = -69.368640085185319;
out[206] = 40.748687841669565;
out[207] = -43.675799641021079;
out[208] = -1.0;
out[210] = 43.675799641021101;
out[211] = 40.748687841669579;
out[212] = 69.368640085185305;
out[213] = 17.974903844364821;
out[214] = -184.75059629806526;
out[215] = -94.61510234445565;
out[216] = 69.368640085185305;
out[217] = 17.974903844364821;
out[218] = -43.675799641021101;
out[219] = -40.748687841669579;
out[220] = -69.368640085185305;
out[221] = -17.974903844364821;
out[222] = 184.75059629806526;
out[223] = 94.61510234445565;
out[224] = -69.368640085185305;
out[225] = -17.974903844364821;
out[226] = -1.0;
out[228] = -40.748687841669579;
out[229] = 43.675799641021101;
out[230] = -17.974903844364821;
out[231] = 69.368640085185305;
out[232] = 94.61510234445565;
out[233] = -184.75059629806526;
out[234] = -17.974903844364821;
out[235] = 69.368640085185305;
out[236] = 40.748687841669579;
out[237] = -43.675799641021101;
out[238] = 17.974903844364821;
out[239] = -69.368640085185305;
out[240] = -94.61510234445565;
out[241] = 184.75059629806526;
out[242] = 17.974903844364821;
out[243] = -69.368640085185305;
out[244] = -1.0;
out[246] = 69.368640085185305;
out[247] = 17.974903844364825;
out[248] = 43.675799641021079;
out[249] = 40.74868784166955;
out[250] = 69.368640085185291;
out[251] = 17.974903844364839;
out[252] = -184.7505962980652;
out[253] = -94.61510234445565;
out[254] = -69.368640085185305;
out[255] = -17.974903844364825;
out[256] = -43.675799641021079;
out[257] = -40.74868784166955;
out[258] = -69.368640085185291;
out[259] = -17.974903844364839;
out[260] = 184.7505962980652;
out[261] = 94.61510234445565;
out[262] = -1.0;
out[264] = -17.974903844364825;
out[265] = 69.368640085185305;
out[266] = -40.74868784166955;
out[267] = 43.675799641021079;
out[268] = -17.974903844364839;
out[269] = 69.368640085185291;
out[270] = 94.61510234445565;
out[271] = -184.7505962980652;
out[272] = 17.974903844364825;
out[273] = -69.368640085185305;
out[274] = 40.74868784166955;
out[275] = -43.675799641021079;
out[276] = 17.974903844364839;
out[277] = -69.368640085185291;
out[278] = -94.61510234445565;
out[279] = 184.7505962980652;
out[280] = -1.0;
out[282] = 184.75059629806526;
out[283] = 94.615102344455664;
out[284] = -69.368640085185305;
out[285] = -17.974903844364835;
out[286] = -43.675799641021079;
out[287] = -40.748687841669557;
out[288] = -69.368640085185291;
out[289] = -17.974903844364821;
out[290] = -184.75059629806526;
out[291] = -94.615102344455664;
out[292] = 69.368640085185305;
out[293] = 17.974903844364835;
out[294] = 43.675799641021079;
out[295] = 40.748687841669557;
out[296] = 69.368640085185291;
out[297] = 17.974903844364821;
out[299] = -94.615102344455664;
out[300] = 184.75059629806526;
out[301] = 17.974903844364835;
out[302] = -69.368640085185305;
out[303] = 40.748687841669557;
out[304] = -43.675799641021079;
out[305] = 17.974903844364821;
out[306] = -69.368640085185291;
out[307] = 94.615102344455664;
out[308] = -184.75059629806526;
out[309] = -17.974903844364835;
out[310] = 69.368640085185305;
out[311] = -40.748687841669557;
out[312] = 43.675799641021079;
out[313] = -17.974903844364821;
out[314] = 69.368640085185291;
out[316] = -69.368640085185291;
out[317] = -17.974903844364825;
out[318] = 184.7505962980652;
out[319] = 94.615102344455636;
out[320] = -69.368640085185319;
out[321] = -17.97490384436481;
out[322] = -43.675799641021079;
out[323] = -40.748687841669565;
out[324] = 69.368640085185291;
out[325] = 17.974903844364825;
out[326] = -184.7505962980652;
out[327] = -94.615102344455636;
out[328] = 69.368640085185319;
out[329] = 17.97490384436481;
out[330] = 43.675799641021079;
out[331] = 40.748687841669565;
out[333] = 17.974903844364825;
out[334] = -69.368640085185291;
out[335] = -94.615102344455636;
out[336] = 184.7505962980652;
out[337] = 17.97490384436481;
out[338] = -69.368640085185319;
out[339] = 40.748687841669565;
out[340] = -43.675799641021079;
out[341] = -17.974903844364825;
out[342] = 69.368640085185291;
out[343] = 94.615102344455636;
out[344] = -184.7505962980652;
out[345] = -17.97490384436481;
out[346] = 69.368640085185319;
out[347] = -40.748687841669565;
out[348] = 43.675799641021079;
out[350] = -43.675799641021101;
out[351] = -40.748687841669579;
out[352] = -69.368640085185305;
out[353] = -17.974903844364821;
out[354] = 184.75059629806526;
out[355] = 94.61510234445565;
out[356] = -69.368640085185305;
out[357] = -17.974903844364821;
out[358] = 43.675799641021101;
out[359] = 40.748687841669579;
out[360] = 69.368640085185305;
out[361] = 17.974903844364821;
out[362] = -184.75059629806526;
out[363] = -94.61510234445565;
out[364] = 69.368640085185305;
out[365] = 17.974903844364821;
out[367] = 40.748687841669579;
out[368] = -43.675799641021101;
out[369] = 17.974903844364821;
out[370] = -69.368640085185305;
out[371] = -94.61510234445565;
out[372] = 184.75059629806526;
out[373] = 17.974903844364821;
out[374] = -69.368640085185305;
out[375] = -40.748687841669579;
out[376] = 43.675799641021101;
out[377] = -17.974903844364821;
out[378] = 69.368640085185305;
out[379] = 94.61510234445565;
out[380] = -184.75059629806526;
out[381] = -17.974903844364821;
out[382] = 69.368640085185305;
out[384] = -69.368640085185305;
out[385] = -17.974903844364825;
out[386] = -43.675799641021079;
out[387] = -40.74868784166955;
out[388] = -69.368640085185291;
out[389] = -17.974903844364839;
out[390] = 184.7505962980652;
out[391] = 94.61510234445565;
out[392] = 69.368640085185305;
out[393] = 17.974903844364825;
out[394] = 43.675799641021079;
out[395] = 40.74868784166955;
out[396] = 69.368640085185291;
out[397] = 17.974903844364839;
out[398] = -184.7505962980652;
out[399] = -94.61510234445565;
out[401] = 17.974903844364825;
out[402] = -69.368640085185305;
out[403] = 40.74868784166955;
out[404] = -43.675799641021079;
out[405] = 17.974903844364839;
out[406] = -69.368640085185291;
out[407] = -94.61510234445565;
out[408] = 184.7505962980652;
out[409] = -17.974903844364825;
out[410] = 69.368640085185305;
out[411] = -40.74868784166955;
out[412] = 43.675799641021079;
out[413] = -17.974903844364839;
out[414] = 69.368640085185291;
out[415] = 94.61510234445565;
out[416] = -184.7505962980652;
out[418] = -184.75059629806526;
out[419] = -94.615102344455664;
out[420] = 69.368640085185305;
out[421] = 17.974903844364835;
out[422] = 43.675799641021079;
out[423] = 40.748687841669557;
out[424] = 69.368640085185291;
out[425] = 17.974903844364821;
out[426] = 184.75059629806526;
out[427] = 94.615102344455664;
out[428] = -69.368640085185305;
out[429] = -17.974903844364835;
out[430] = -43.675799641021079;
out[431] = -40.748687841669557;
out[432] = -69.368640085185291;
out[433] = -17.974903844364821;
out[434] = -1.0;
out[435] = 94.615102344455664;
out[436] = -184.75059629806526;
out[437] = -17.974903844364835;
out[438] = 69.368640085185305;
out[439] = -40.748687841669557;
out[440] = 43.675799641021079;
out[441] = -17.974903844364821;
out[442] = 69.368640085185291;
out[443] = -94.615102344455664;
out[444] = 184.75059629806526;
out[445] = 17.974903844364835;
out[446] = -69.368640085185305;
out[447] = 40.748687841669557;
out[448] = -43.675799641021079;
out[449] = 17.974903844364821;
out[450] = -69.368640085185291;
out[451] = -1.0;
out[452] = 69.368640085185291;
out[453] = 17.974903844364825;
out[454] = -184.7505962980652;
out[455] = -94.615102344455636;
out[456] = 69.368640085185319;
out[457] = 17.97490384436481;
out[458] = 43.675799641021079;
out[459] = 40.748687841669565;
out[460] = -69.368640085185291;
out[461] = -17.974903844364825;
out[462] = 184.7505962980652;
out[463] = 94.615102344455636;
out[464] = -69.368640085185319;
out[465] = -17.97490384436481;
out[466] = -43.675799641021079;
out[467] = -40.748687841669565;
out[468] = -1.0;
out[469] = -17.974903844364825;
out[470] = 69.368640085185291;
out[471] = 94.615102344455636;
out[472] = -184.7505962980652;
out[473] = -17.97490384436481;
out[474] = 69.368640085185319;
out[475] = -40.748687841669565;
out[476] = 43.675799641021079;
out[477] = 17.974903844364825;
out[478] = -69.368640085185291;
out[479] = -94.615102344455636;
out[480] = 184.7505962980652;
out[481] = 17.97490384436481;
out[482] = -69.368640085185319;
out[483] = 40.748687841669565;
out[484] = -43.675799641021079;
out[485] = -1.0;
out[486] = 43.675799641021101;
out[487] = 40.748687841669579;
out[488] = 69.368640085185305;
out[489] = 17.974903844364821;
out[490] = -184.75059629806526;
out[491] = -94.61510234445565;
out[492] = 69.368640085185305;
out[493] = 17.974903844364821;
out[494] = -43.675799641021101;
out[495] = -40.748687841669579;
out[496] = -69.368640085185305;
out[497] = -17.974903844364821;
out[498] = 184.75059629806526;
out[499] = 94.61510234445565;
out[500] = -69.368640085185305;
out[501] = -17.974903844364821;
out[502] = -1.0;
out[503] = -40.748687841669579;
out[504] = 43.675799641021101;
out[505] = -17.974903844364821;
out[506] = 69.368640085185305;
out[507] = 94.61510234445565;
out[508] = -184.75059629806526;
out[509] = -17.974903844364821;
out[510] = 69.368640085185305;
out[511] = 40.748687841669579;
out[512] = -43.675799641021101;
out[513] = 17.974903844364821;
out[514] = -69.368640085185305;
out[515] = -94.61510234445565;
out[516] = 184.75059629806526;
out[517] = 17.974903844364821;
out[518] = -69.368640085185305;
out[519] = -1.0;
out[520] = 69.368640085185305;
out[521] = 17.974903844364825;
out[522] = 43.675799641021079;
out[523] = 40.74868784166955;
out[524] = 69.368640085185291;
out[525] = 17.974903844364839;
out[526] = -184.7505962980652;
out[527] = -94.61510234445565;
out[528] = -69.368640085185305;
out[529] = -17.974903844364825;
out[530] = -43.675799641021079;
out[531] = -40.74868784166955;
out[532] = -69.368640085185291;
out[533] = -17.974903844364839;
out[534] = 184.7505962980652;
out[535] = 94.61510234445565;
out[536] = -1.0;
out[537] = -17.974903844364825;
out[538] = 69.368640085185305;
out[539] = -40.74868784166955;
out[540] = 43.675799641021079;
out[541] = -17.974903844364839;
out[542] = 69.368640085185291;
out[543] = 94.61510234445565;
out[544] = -184.7505962980652;
out[545] = 17.974903844364825;
out[546] = -69.368640085185305;
out[547] = 40.74868784166955;
out[548] = -43.675799641021079;
out[549] = 17.974903844364839;
out[550] = -69.368640085185291;
out[551] = -94.61510234445565;
out[552] = 184.7505962980652;
out[553] = -1.0;
out[590] = 1.0;
out[591] = 1.0;
out[592] = 1.0;
out[593] = 1.0;
out[594] = 1.0;
out[595] = 1.0;
out[596] = 1.0;
out[597] = 1.0;
out[598] = 1.0;
out[599] = -1.0;
out[602] = 1.0;
out[603] = -1.0;
out[606] = 1.0;
out[607] = -1.0;
out[610] = 1.0;
out[611] = -1.0;
out[614] = 1.0;
out[615] = -1.0;
out[618] = 1.0;
out[619] = -1.0;
out[622] = 1.0;
out[623] = -1.0;
out[626] = 1.0;
out[627] = -1.0;
out[630] = -1.0;
out[633] = -1.0;
out[636] = -1.0;
out[639] = -1.0;
out[642] = -1.0;
out[645] = -1.0;
out[648] = -1.0;
out[651] = -1.0;
out[654] = -1.0;
out[657] = -1.0;
out[660] = -1.0;
out[663] = -1.0;
out[666] = -1.0;
out[669] = -1.0;
out[672] = -1.0;
out[675] = -1.0;
out[678] = -1.0;
out[681] = -1.0;

}

void de_jac_trap_xy_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[2220] = p[0]*(2.0*y[8]*u[2] - 2.0*y[14]*u[2] + y[32]*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[9]*y[33] + y[8]*y[32] - y[15]*y[33] - y[14]*y[32])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   -1.0*y[8]*pow(pow(y[9], 2) + pow(y[8], 2), -0.5)/pow(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[2221] = p[0]*(2.0*y[9]*u[2] - 2.0*y[15]*u[2] + y[33]*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[9]*y[33] + y[8]*y[32] - y[15]*y[33] - y[14]*y[32])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   -1.0*y[9]*pow(pow(y[9], 2) + pow(y[8], 2), -0.5)/pow(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[2226] = p[0]*(-2.0*y[8]*u[2] + 2.0*y[14]*u[2] - y[32]*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[2227] = p[0]*(-2.0*y[9]*u[2] + 2.0*y[15]*u[2] - y[33]*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[2244] = p[0]*(y[8] - y[14])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[2245] = p[0]*(y[9] - y[15])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[2289] = p[0]*(2.0*y[10]*u[6] - 2.0*y[14]*u[6] + y[34]*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[11]*y[35] + y[10]*y[34] - y[15]*y[35] - y[14]*y[34])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   -1.0*y[10]*pow(pow(y[11], 2) + pow(y[10], 2), -0.5)/pow(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[2290] = p[0]*(2.0*y[11]*u[6] - 2.0*y[15]*u[6] + y[35]*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[11]*y[35] + y[10]*y[34] - y[15]*y[35] - y[14]*y[34])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   -1.0*y[11]*pow(pow(y[11], 2) + pow(y[10], 2), -0.5)/pow(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[2293] = p[0]*(-2.0*y[10]*u[6] + 2.0*y[14]*u[6] - y[34]*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[2294] = p[0]*(-2.0*y[11]*u[6] + 2.0*y[15]*u[6] - y[35]*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[2313] = p[0]*(y[10] - y[14])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[2314] = p[0]*(y[11] - y[15])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[2358] = p[0]*(2.0*y[12]*u[10] - 2.0*y[14]*u[10] + y[36]*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[13]*y[37] + y[12]*y[36] - y[15]*y[37] - y[14]*y[36])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   -1.0*y[12]*pow(pow(y[13], 2) + pow(y[12], 2), -0.5)/pow(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[2359] = p[0]*(2.0*y[13]*u[10] - 2.0*y[15]*u[10] + y[37]*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[13]*y[37] + y[12]*y[36] - y[15]*y[37] - y[14]*y[36])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   -1.0*y[13]*pow(pow(y[13], 2) + pow(y[12], 2), -0.5)/pow(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[2360] = p[0]*(-2.0*y[12]*u[10] + 2.0*y[14]*u[10] - y[36]*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[2361] = p[0]*(-2.0*y[13]*u[10] + 2.0*y[15]*u[10] - y[37]*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[2382] = p[0]*(y[12] - y[14])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[2383] = p[0]*(y[13] - y[15])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[2421] = p[0]*(-2.0*y[8]*u[3] + 2.0*y[14]*u[3] - y[33]*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[9]*y[32] - y[8]*y[33] - y[15]*y[32] + y[14]*y[33])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   -1.0*y[8]*pow(pow(y[9], 2) + pow(y[8], 2), -0.5)/pow(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[2422] = p[0]*(-2.0*y[9]*u[3] + 2.0*y[15]*u[3] + y[32]*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[9]*y[32] - y[8]*y[33] - y[15]*y[32] + y[14]*y[33])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   -1.0*y[9]*pow(pow(y[9], 2) + pow(y[8], 2), -0.5)/pow(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[2427] = p[0]*(2.0*y[8]*u[3] - 2.0*y[14]*u[3] + y[33]*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[2428] = p[0]*(2.0*y[9]*u[3] - 2.0*y[15]*u[3] - y[32]*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[2445] = p[0]*(y[9] - y[15])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[2446] = p[0]*(-y[8] + y[14])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[2490] = p[0]*(-2.0*y[10]*u[7] + 2.0*y[14]*u[7] - y[35]*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[11]*y[34] - y[10]*y[35] - y[15]*y[34] + y[14]*y[35])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   -1.0*y[10]*pow(pow(y[11], 2) + pow(y[10], 2), -0.5)/pow(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[2491] = p[0]*(-2.0*y[11]*u[7] + 2.0*y[15]*u[7] + y[34]*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[11]*y[34] - y[10]*y[35] - y[15]*y[34] + y[14]*y[35])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   -1.0*y[11]*pow(pow(y[11], 2) + pow(y[10], 2), -0.5)/pow(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[2494] = p[0]*(2.0*y[10]*u[7] - 2.0*y[14]*u[7] + y[35]*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[2495] = p[0]*(2.0*y[11]*u[7] - 2.0*y[15]*u[7] - y[34]*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[2514] = p[0]*(y[11] - y[15])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[2515] = p[0]*(-y[10] + y[14])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[2559] = p[0]*(-2.0*y[12]*u[11] + 2.0*y[14]*u[11] - y[37]*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[13]*y[36] - y[12]*y[37] - y[15]*y[36] + y[14]*y[37])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   -1.0*y[12]*pow(pow(y[13], 2) + pow(y[12], 2), -0.5)/pow(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[2560] = p[0]*(-2.0*y[13]*u[11] + 2.0*y[15]*u[11] + y[36]*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[13]*y[36] - y[12]*y[37] - y[15]*y[36] + y[14]*y[37])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   -1.0*y[13]*pow(pow(y[13], 2) + pow(y[12], 2), -0.5)/pow(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[2561] = p[0]*(2.0*y[12]*u[11] - 2.0*y[14]*u[11] + y[37]*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[2562] = p[0]*(2.0*y[13]*u[11] - 2.0*y[15]*u[11] - y[36]*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[2583] = p[0]*(y[13] - y[15])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[2584] = p[0]*(-y[12] + y[14])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
));

}

void de_jac_trap_up_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[0] = 0.5*Dt*p[17] + 1.0;
out[65] = 0.5*Dt;
out[644] = -u[12];
out[715] = -u[12];
out[779] = -u[12];
out[850] = -u[12];
out[914] = -u[12];
out[985] = -u[12];
out[1049] = -u[12];
out[1120] = -u[12];
out[1180] = u[12];
out[1251] = u[12];
out[1315] = u[12];
out[1386] = u[12];
out[1450] = u[12];
out[1521] = u[12];
out[1585] = u[12];
out[1656] = u[12];
out[2788] = -p[1];
out[2792] = 1.0*p[2];
out[2856] = -p[1];
out[2860] = 1.0*p[2];
out[2924] = -p[1];
out[2928] = 1.0*p[2];
out[2992] = -p[1];
out[2996] = 1.0*p[2];
out[3056] = -1.0*p[2];
out[3060] = -1.0*p[1];
out[3124] = -1.0*p[2];
out[3128] = -1.0*p[1];
out[3192] = -1.0*p[2];
out[3196] = -1.0*p[1];
out[3260] = -1.0*p[2];
out[3264] = -1.0*p[1];
out[3332] = -p[4];
out[3336] = 1.0*p[3];
out[3400] = -p[4];
out[3404] = 1.0*p[3];
out[3468] = -p[4];
out[3472] = 1.0*p[3];
out[3536] = -p[6];
out[3540] = 1.0*p[5];
out[3600] = -1.0*p[3];
out[3604] = -1.0*p[4];
out[3668] = -1.0*p[3];
out[3672] = -1.0*p[4];
out[3736] = -1.0*p[3];
out[3740] = -1.0*p[4];
out[3804] = -1.0*p[5];
out[3808] = -1.0*p[6];
out[3876] = -p[10];
out[3880] = 1.0*p[9];
out[3944] = -p[10];
out[3948] = 1.0*p[9];
out[4012] = -p[10];
out[4016] = 1.0*p[9];
out[4080] = -p[12];
out[4084] = 1.0*p[11];
out[4144] = -1.0*p[9];
out[4148] = -1.0*p[10];
out[4212] = -1.0*p[9];
out[4216] = -1.0*p[10];
out[4280] = -1.0*p[9];
out[4284] = -1.0*p[10];
out[4348] = -1.0*p[11];
out[4352] = -1.0*p[12];
out[4422] = p[16];
out[4487] = -p[15];

}

void de_jac_trap_num_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[68] = 184.75059629806526;
out[69] = 94.615102344455664;
out[70] = -69.368640085185305;
out[71] = -17.974903844364835;
out[72] = -43.675799641021079;
out[73] = -40.748687841669557;
out[74] = -69.368640085185291;
out[75] = -17.974903844364821;
out[76] = -184.75059629806526;
out[77] = -94.615102344455664;
out[78] = 69.368640085185305;
out[79] = 17.974903844364835;
out[80] = 43.675799641021079;
out[81] = 40.748687841669557;
out[82] = 69.368640085185291;
out[83] = 17.974903844364821;
out[116] = -1.0;
out[135] = -94.615102344455664;
out[136] = 184.75059629806526;
out[137] = 17.974903844364835;
out[138] = -69.368640085185305;
out[139] = 40.748687841669557;
out[140] = -43.675799641021079;
out[141] = 17.974903844364821;
out[142] = -69.368640085185291;
out[143] = 94.615102344455664;
out[144] = -184.75059629806526;
out[145] = -17.974903844364835;
out[146] = 69.368640085185305;
out[147] = -40.748687841669557;
out[148] = 43.675799641021079;
out[149] = -17.974903844364821;
out[150] = 69.368640085185291;
out[187] = -1.0;
out[202] = -69.368640085185291;
out[203] = -17.974903844364825;
out[204] = 184.7505962980652;
out[205] = 94.615102344455636;
out[206] = -69.368640085185319;
out[207] = -17.97490384436481;
out[208] = -43.675799641021079;
out[209] = -40.748687841669565;
out[210] = 69.368640085185291;
out[211] = 17.974903844364825;
out[212] = -184.7505962980652;
out[213] = -94.615102344455636;
out[214] = 69.368640085185319;
out[215] = 17.97490384436481;
out[216] = 43.675799641021079;
out[217] = 40.748687841669565;
out[251] = -1.0;
out[269] = 17.974903844364825;
out[270] = -69.368640085185291;
out[271] = -94.615102344455636;
out[272] = 184.7505962980652;
out[273] = 17.97490384436481;
out[274] = -69.368640085185319;
out[275] = 40.748687841669565;
out[276] = -43.675799641021079;
out[277] = -17.974903844364825;
out[278] = 69.368640085185291;
out[279] = 94.615102344455636;
out[280] = -184.7505962980652;
out[281] = -17.97490384436481;
out[282] = 69.368640085185319;
out[283] = -40.748687841669565;
out[284] = 43.675799641021079;
out[322] = -1.0;
out[336] = -43.675799641021101;
out[337] = -40.748687841669579;
out[338] = -69.368640085185305;
out[339] = -17.974903844364821;
out[340] = 184.75059629806526;
out[341] = 94.61510234445565;
out[342] = -69.368640085185305;
out[343] = -17.974903844364821;
out[344] = 43.675799641021101;
out[345] = 40.748687841669579;
out[346] = 69.368640085185305;
out[347] = 17.974903844364821;
out[348] = -184.75059629806526;
out[349] = -94.61510234445565;
out[350] = 69.368640085185305;
out[351] = 17.974903844364821;
out[386] = -1.0;
out[403] = 40.748687841669579;
out[404] = -43.675799641021101;
out[405] = 17.974903844364821;
out[406] = -69.368640085185305;
out[407] = -94.61510234445565;
out[408] = 184.75059629806526;
out[409] = 17.974903844364821;
out[410] = -69.368640085185305;
out[411] = -40.748687841669579;
out[412] = 43.675799641021101;
out[413] = -17.974903844364821;
out[414] = 69.368640085185305;
out[415] = 94.61510234445565;
out[416] = -184.75059629806526;
out[417] = -17.974903844364821;
out[418] = 69.368640085185305;
out[457] = -1.0;
out[470] = -69.368640085185305;
out[471] = -17.974903844364825;
out[472] = -43.675799641021079;
out[473] = -40.74868784166955;
out[474] = -69.368640085185291;
out[475] = -17.974903844364839;
out[476] = 184.7505962980652;
out[477] = 94.61510234445565;
out[478] = 69.368640085185305;
out[479] = 17.974903844364825;
out[480] = 43.675799641021079;
out[481] = 40.74868784166955;
out[482] = 69.368640085185291;
out[483] = 17.974903844364839;
out[484] = -184.7505962980652;
out[485] = -94.61510234445565;
out[521] = -1.0;
out[537] = 17.974903844364825;
out[538] = -69.368640085185305;
out[539] = 40.74868784166955;
out[540] = -43.675799641021079;
out[541] = 17.974903844364839;
out[542] = -69.368640085185291;
out[543] = -94.61510234445565;
out[544] = 184.7505962980652;
out[545] = -17.974903844364825;
out[546] = 69.368640085185305;
out[547] = -40.74868784166955;
out[548] = 43.675799641021079;
out[549] = -17.974903844364839;
out[550] = 69.368640085185291;
out[551] = 94.61510234445565;
out[552] = -184.7505962980652;
out[592] = -1.0;
out[604] = -184.75059629806526;
out[605] = -94.615102344455664;
out[606] = 69.368640085185305;
out[607] = 17.974903844364835;
out[608] = 43.675799641021079;
out[609] = 40.748687841669557;
out[610] = 69.368640085185291;
out[611] = 17.974903844364821;
out[612] = 184.75059629806526;
out[613] = 94.615102344455664;
out[614] = -69.368640085185305;
out[615] = -17.974903844364835;
out[616] = -43.675799641021079;
out[617] = -40.748687841669557;
out[618] = -69.368640085185291;
out[619] = -17.974903844364821;
out[636] = -1.0;
out[671] = 94.615102344455664;
out[672] = -184.75059629806526;
out[673] = -17.974903844364835;
out[674] = 69.368640085185305;
out[675] = -40.748687841669557;
out[676] = 43.675799641021079;
out[677] = -17.974903844364821;
out[678] = 69.368640085185291;
out[679] = -94.615102344455664;
out[680] = 184.75059629806526;
out[681] = 17.974903844364835;
out[682] = -69.368640085185305;
out[683] = 40.748687841669557;
out[684] = -43.675799641021079;
out[685] = 17.974903844364821;
out[686] = -69.368640085185291;
out[704] = -1.0;
out[738] = 69.368640085185291;
out[739] = 17.974903844364825;
out[740] = -184.7505962980652;
out[741] = -94.615102344455636;
out[742] = 69.368640085185319;
out[743] = 17.97490384436481;
out[744] = 43.675799641021079;
out[745] = 40.748687841669565;
out[746] = -69.368640085185291;
out[747] = -17.974903844364825;
out[748] = 184.7505962980652;
out[749] = 94.615102344455636;
out[750] = -69.368640085185319;
out[751] = -17.97490384436481;
out[752] = -43.675799641021079;
out[753] = -40.748687841669565;
out[772] = -1.0;
out[805] = -17.974903844364825;
out[806] = 69.368640085185291;
out[807] = 94.615102344455636;
out[808] = -184.7505962980652;
out[809] = -17.97490384436481;
out[810] = 69.368640085185319;
out[811] = -40.748687841669565;
out[812] = 43.675799641021079;
out[813] = 17.974903844364825;
out[814] = -69.368640085185291;
out[815] = -94.615102344455636;
out[816] = 184.7505962980652;
out[817] = 17.97490384436481;
out[818] = -69.368640085185319;
out[819] = 40.748687841669565;
out[820] = -43.675799641021079;
out[840] = -1.0;
out[872] = 43.675799641021101;
out[873] = 40.748687841669579;
out[874] = 69.368640085185305;
out[875] = 17.974903844364821;
out[876] = -184.75059629806526;
out[877] = -94.61510234445565;
out[878] = 69.368640085185305;
out[879] = 17.974903844364821;
out[880] = -43.675799641021101;
out[881] = -40.748687841669579;
out[882] = -69.368640085185305;
out[883] = -17.974903844364821;
out[884] = 184.75059629806526;
out[885] = 94.61510234445565;
out[886] = -69.368640085185305;
out[887] = -17.974903844364821;
out[908] = -1.0;
out[939] = -40.748687841669579;
out[940] = 43.675799641021101;
out[941] = -17.974903844364821;
out[942] = 69.368640085185305;
out[943] = 94.61510234445565;
out[944] = -184.75059629806526;
out[945] = -17.974903844364821;
out[946] = 69.368640085185305;
out[947] = 40.748687841669579;
out[948] = -43.675799641021101;
out[949] = 17.974903844364821;
out[950] = -69.368640085185305;
out[951] = -94.61510234445565;
out[952] = 184.75059629806526;
out[953] = 17.974903844364821;
out[954] = -69.368640085185305;
out[976] = -1.0;
out[1006] = 69.368640085185305;
out[1007] = 17.974903844364825;
out[1008] = 43.675799641021079;
out[1009] = 40.74868784166955;
out[1010] = 69.368640085185291;
out[1011] = 17.974903844364839;
out[1012] = -184.7505962980652;
out[1013] = -94.61510234445565;
out[1014] = -69.368640085185305;
out[1015] = -17.974903844364825;
out[1016] = -43.675799641021079;
out[1017] = -40.74868784166955;
out[1018] = -69.368640085185291;
out[1019] = -17.974903844364839;
out[1020] = 184.7505962980652;
out[1021] = 94.61510234445565;
out[1044] = -1.0;
out[1073] = -17.974903844364825;
out[1074] = 69.368640085185305;
out[1075] = -40.74868784166955;
out[1076] = 43.675799641021079;
out[1077] = -17.974903844364839;
out[1078] = 69.368640085185291;
out[1079] = 94.61510234445565;
out[1080] = -184.7505962980652;
out[1081] = 17.974903844364825;
out[1082] = -69.368640085185305;
out[1083] = 40.74868784166955;
out[1084] = -43.675799641021079;
out[1085] = 17.974903844364839;
out[1086] = -69.368640085185291;
out[1087] = -94.61510234445565;
out[1088] = 184.7505962980652;
out[1112] = -1.0;
out[1156] = 184.75059629806526;
out[1157] = 94.615102344455664;
out[1158] = -69.368640085185305;
out[1159] = -17.974903844364835;
out[1160] = -43.675799641021079;
out[1161] = -40.748687841669557;
out[1162] = -69.368640085185291;
out[1163] = -17.974903844364821;
out[1164] = -184.75059629806526;
out[1165] = -94.615102344455664;
out[1166] = 69.368640085185305;
out[1167] = 17.974903844364835;
out[1168] = 43.675799641021079;
out[1169] = 40.748687841669557;
out[1170] = 69.368640085185291;
out[1171] = 17.974903844364821;
out[1223] = -94.615102344455664;
out[1224] = 184.75059629806526;
out[1225] = 17.974903844364835;
out[1226] = -69.368640085185305;
out[1227] = 40.748687841669557;
out[1228] = -43.675799641021079;
out[1229] = 17.974903844364821;
out[1230] = -69.368640085185291;
out[1231] = 94.615102344455664;
out[1232] = -184.75059629806526;
out[1233] = -17.974903844364835;
out[1234] = 69.368640085185305;
out[1235] = -40.748687841669557;
out[1236] = 43.675799641021079;
out[1237] = -17.974903844364821;
out[1238] = 69.368640085185291;
out[1290] = -69.368640085185291;
out[1291] = -17.974903844364825;
out[1292] = 184.7505962980652;
out[1293] = 94.615102344455636;
out[1294] = -69.368640085185319;
out[1295] = -17.97490384436481;
out[1296] = -43.675799641021079;
out[1297] = -40.748687841669565;
out[1298] = 69.368640085185291;
out[1299] = 17.974903844364825;
out[1300] = -184.7505962980652;
out[1301] = -94.615102344455636;
out[1302] = 69.368640085185319;
out[1303] = 17.97490384436481;
out[1304] = 43.675799641021079;
out[1305] = 40.748687841669565;
out[1357] = 17.974903844364825;
out[1358] = -69.368640085185291;
out[1359] = -94.615102344455636;
out[1360] = 184.7505962980652;
out[1361] = 17.97490384436481;
out[1362] = -69.368640085185319;
out[1363] = 40.748687841669565;
out[1364] = -43.675799641021079;
out[1365] = -17.974903844364825;
out[1366] = 69.368640085185291;
out[1367] = 94.615102344455636;
out[1368] = -184.7505962980652;
out[1369] = -17.97490384436481;
out[1370] = 69.368640085185319;
out[1371] = -40.748687841669565;
out[1372] = 43.675799641021079;
out[1424] = -43.675799641021101;
out[1425] = -40.748687841669579;
out[1426] = -69.368640085185305;
out[1427] = -17.974903844364821;
out[1428] = 184.75059629806526;
out[1429] = 94.61510234445565;
out[1430] = -69.368640085185305;
out[1431] = -17.974903844364821;
out[1432] = 43.675799641021101;
out[1433] = 40.748687841669579;
out[1434] = 69.368640085185305;
out[1435] = 17.974903844364821;
out[1436] = -184.75059629806526;
out[1437] = -94.61510234445565;
out[1438] = 69.368640085185305;
out[1439] = 17.974903844364821;
out[1491] = 40.748687841669579;
out[1492] = -43.675799641021101;
out[1493] = 17.974903844364821;
out[1494] = -69.368640085185305;
out[1495] = -94.61510234445565;
out[1496] = 184.75059629806526;
out[1497] = 17.974903844364821;
out[1498] = -69.368640085185305;
out[1499] = -40.748687841669579;
out[1500] = 43.675799641021101;
out[1501] = -17.974903844364821;
out[1502] = 69.368640085185305;
out[1503] = 94.61510234445565;
out[1504] = -184.75059629806526;
out[1505] = -17.974903844364821;
out[1506] = 69.368640085185305;
out[1558] = -69.368640085185305;
out[1559] = -17.974903844364825;
out[1560] = -43.675799641021079;
out[1561] = -40.74868784166955;
out[1562] = -69.368640085185291;
out[1563] = -17.974903844364839;
out[1564] = 184.7505962980652;
out[1565] = 94.61510234445565;
out[1566] = 69.368640085185305;
out[1567] = 17.974903844364825;
out[1568] = 43.675799641021079;
out[1569] = 40.74868784166955;
out[1570] = 69.368640085185291;
out[1571] = 17.974903844364839;
out[1572] = -184.7505962980652;
out[1573] = -94.61510234445565;
out[1625] = 17.974903844364825;
out[1626] = -69.368640085185305;
out[1627] = 40.74868784166955;
out[1628] = -43.675799641021079;
out[1629] = 17.974903844364839;
out[1630] = -69.368640085185291;
out[1631] = -94.61510234445565;
out[1632] = 184.7505962980652;
out[1633] = -17.974903844364825;
out[1634] = 69.368640085185305;
out[1635] = -40.74868784166955;
out[1636] = 43.675799641021079;
out[1637] = -17.974903844364839;
out[1638] = 69.368640085185291;
out[1639] = 94.61510234445565;
out[1640] = -184.7505962980652;
out[1692] = -184.75059629806526;
out[1693] = -94.615102344455664;
out[1694] = 69.368640085185305;
out[1695] = 17.974903844364835;
out[1696] = 43.675799641021079;
out[1697] = 40.748687841669557;
out[1698] = 69.368640085185291;
out[1699] = 17.974903844364821;
out[1700] = 184.75059629806526;
out[1701] = 94.615102344455664;
out[1702] = -69.368640085185305;
out[1703] = -17.974903844364835;
out[1704] = -43.675799641021079;
out[1705] = -40.748687841669557;
out[1706] = -69.368640085185291;
out[1707] = -17.974903844364821;
out[1732] = -1.0;
out[1759] = 94.615102344455664;
out[1760] = -184.75059629806526;
out[1761] = -17.974903844364835;
out[1762] = 69.368640085185305;
out[1763] = -40.748687841669557;
out[1764] = 43.675799641021079;
out[1765] = -17.974903844364821;
out[1766] = 69.368640085185291;
out[1767] = -94.615102344455664;
out[1768] = 184.75059629806526;
out[1769] = 17.974903844364835;
out[1770] = -69.368640085185305;
out[1771] = 40.748687841669557;
out[1772] = -43.675799641021079;
out[1773] = 17.974903844364821;
out[1774] = -69.368640085185291;
out[1803] = -1.0;
out[1826] = 69.368640085185291;
out[1827] = 17.974903844364825;
out[1828] = -184.7505962980652;
out[1829] = -94.615102344455636;
out[1830] = 69.368640085185319;
out[1831] = 17.97490384436481;
out[1832] = 43.675799641021079;
out[1833] = 40.748687841669565;
out[1834] = -69.368640085185291;
out[1835] = -17.974903844364825;
out[1836] = 184.7505962980652;
out[1837] = 94.615102344455636;
out[1838] = -69.368640085185319;
out[1839] = -17.97490384436481;
out[1840] = -43.675799641021079;
out[1841] = -40.748687841669565;
out[1867] = -1.0;
out[1893] = -17.974903844364825;
out[1894] = 69.368640085185291;
out[1895] = 94.615102344455636;
out[1896] = -184.7505962980652;
out[1897] = -17.97490384436481;
out[1898] = 69.368640085185319;
out[1899] = -40.748687841669565;
out[1900] = 43.675799641021079;
out[1901] = 17.974903844364825;
out[1902] = -69.368640085185291;
out[1903] = -94.615102344455636;
out[1904] = 184.7505962980652;
out[1905] = 17.97490384436481;
out[1906] = -69.368640085185319;
out[1907] = 40.748687841669565;
out[1908] = -43.675799641021079;
out[1938] = -1.0;
out[1960] = 43.675799641021101;
out[1961] = 40.748687841669579;
out[1962] = 69.368640085185305;
out[1963] = 17.974903844364821;
out[1964] = -184.75059629806526;
out[1965] = -94.61510234445565;
out[1966] = 69.368640085185305;
out[1967] = 17.974903844364821;
out[1968] = -43.675799641021101;
out[1969] = -40.748687841669579;
out[1970] = -69.368640085185305;
out[1971] = -17.974903844364821;
out[1972] = 184.75059629806526;
out[1973] = 94.61510234445565;
out[1974] = -69.368640085185305;
out[1975] = -17.974903844364821;
out[2002] = -1.0;
out[2027] = -40.748687841669579;
out[2028] = 43.675799641021101;
out[2029] = -17.974903844364821;
out[2030] = 69.368640085185305;
out[2031] = 94.61510234445565;
out[2032] = -184.75059629806526;
out[2033] = -17.974903844364821;
out[2034] = 69.368640085185305;
out[2035] = 40.748687841669579;
out[2036] = -43.675799641021101;
out[2037] = 17.974903844364821;
out[2038] = -69.368640085185305;
out[2039] = -94.61510234445565;
out[2040] = 184.75059629806526;
out[2041] = 17.974903844364821;
out[2042] = -69.368640085185305;
out[2073] = -1.0;
out[2094] = 69.368640085185305;
out[2095] = 17.974903844364825;
out[2096] = 43.675799641021079;
out[2097] = 40.74868784166955;
out[2098] = 69.368640085185291;
out[2099] = 17.974903844364839;
out[2100] = -184.7505962980652;
out[2101] = -94.61510234445565;
out[2102] = -69.368640085185305;
out[2103] = -17.974903844364825;
out[2104] = -43.675799641021079;
out[2105] = -40.74868784166955;
out[2106] = -69.368640085185291;
out[2107] = -17.974903844364839;
out[2108] = 184.7505962980652;
out[2109] = 94.61510234445565;
out[2137] = -1.0;
out[2161] = -17.974903844364825;
out[2162] = 69.368640085185305;
out[2163] = -40.74868784166955;
out[2164] = 43.675799641021079;
out[2165] = -17.974903844364839;
out[2166] = 69.368640085185291;
out[2167] = 94.61510234445565;
out[2168] = -184.7505962980652;
out[2169] = 17.974903844364825;
out[2170] = -69.368640085185305;
out[2171] = 40.74868784166955;
out[2172] = -43.675799641021079;
out[2173] = 17.974903844364839;
out[2174] = -69.368640085185291;
out[2175] = -94.61510234445565;
out[2176] = 184.7505962980652;
out[2208] = -1.0;
out[2646] = 1.0;
out[2648] = 1.0;
out[2650] = 1.0;
out[2652] = 1.0;
out[2714] = 1.0;
out[2716] = 1.0;
out[2718] = 1.0;
out[2720] = 1.0;
out[2756] = 1.0;
out[2764] = -1.0;
out[2825] = 1.0;
out[2833] = -1.0;
out[2894] = 1.0;
out[2902] = -1.0;
out[2963] = 1.0;
out[2971] = -1.0;
out[3025] = 1.0;
out[3033] = -1.0;
out[3094] = 1.0;
out[3102] = -1.0;
out[3163] = 1.0;
out[3171] = -1.0;
out[3232] = 1.0;
out[3240] = -1.0;
out[3284] = -1.0;
out[3353] = -1.0;
out[3422] = -1.0;
out[3491] = -1.0;
out[3553] = -1.0;
out[3622] = -1.0;
out[3691] = -1.0;
out[3760] = -1.0;
out[3844] = -1.0;
out[3913] = -1.0;
out[3982] = -1.0;
out[4051] = -1.0;
out[4113] = -1.0;
out[4182] = -1.0;
out[4251] = -1.0;
out[4320] = -1.0;
out[4420] = -1.0;
out[4488] = -1.0;

}

void sp_jac_trap_xy_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[554] = p[0]*(2.0*y[8]*u[2] - 2.0*y[14]*u[2] + y[32]*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[9]*y[33] + y[8]*y[32] - y[15]*y[33] - y[14]*y[32])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   -1.0*y[8]*pow(pow(y[9], 2) + pow(y[8], 2), -0.5)/pow(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[555] = p[0]*(2.0*y[9]*u[2] - 2.0*y[15]*u[2] + y[33]*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[9]*y[33] + y[8]*y[32] - y[15]*y[33] - y[14]*y[32])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   -1.0*y[9]*pow(pow(y[9], 2) + pow(y[8], 2), -0.5)/pow(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[556] = p[0]*(-2.0*y[8]*u[2] + 2.0*y[14]*u[2] - y[32]*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[557] = p[0]*(-2.0*y[9]*u[2] + 2.0*y[15]*u[2] - y[33]*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[558] = p[0]*(y[8] - y[14])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[559] = p[0]*(y[9] - y[15])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[560] = p[0]*(2.0*y[10]*u[6] - 2.0*y[14]*u[6] + y[34]*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[11]*y[35] + y[10]*y[34] - y[15]*y[35] - y[14]*y[34])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   -1.0*y[10]*pow(pow(y[11], 2) + pow(y[10], 2), -0.5)/pow(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[561] = p[0]*(2.0*y[11]*u[6] - 2.0*y[15]*u[6] + y[35]*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[11]*y[35] + y[10]*y[34] - y[15]*y[35] - y[14]*y[34])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   -1.0*y[11]*pow(pow(y[11], 2) + pow(y[10], 2), -0.5)/pow(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[562] = p[0]*(-2.0*y[10]*u[6] + 2.0*y[14]*u[6] - y[34]*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[563] = p[0]*(-2.0*y[11]*u[6] + 2.0*y[15]*u[6] - y[35]*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[564] = p[0]*(y[10] - y[14])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[565] = p[0]*(y[11] - y[15])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[566] = p[0]*(2.0*y[12]*u[10] - 2.0*y[14]*u[10] + y[36]*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[13]*y[37] + y[12]*y[36] - y[15]*y[37] - y[14]*y[36])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   -1.0*y[12]*pow(pow(y[13], 2) + pow(y[12], 2), -0.5)/pow(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[567] = p[0]*(2.0*y[13]*u[10] - 2.0*y[15]*u[10] + y[37]*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[13]*y[37] + y[12]*y[36] - y[15]*y[37] - y[14]*y[36])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   -1.0*y[13]*pow(pow(y[13], 2) + pow(y[12], 2), -0.5)/pow(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[568] = p[0]*(-2.0*y[12]*u[10] + 2.0*y[14]*u[10] - y[36]*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[569] = p[0]*(-2.0*y[13]*u[10] + 2.0*y[15]*u[10] - y[37]*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[570] = p[0]*(y[12] - y[14])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[571] = p[0]*(y[13] - y[15])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[572] = p[0]*(-2.0*y[8]*u[3] + 2.0*y[14]*u[3] - y[33]*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[9]*y[32] - y[8]*y[33] - y[15]*y[32] + y[14]*y[33])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   -1.0*y[8]*pow(pow(y[9], 2) + pow(y[8], 2), -0.5)/pow(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[573] = p[0]*(-2.0*y[9]*u[3] + 2.0*y[15]*u[3] + y[32]*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[9]*y[32] - y[8]*y[33] - y[15]*y[32] + y[14]*y[33])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   -1.0*y[9]*pow(pow(y[9], 2) + pow(y[8], 2), -0.5)/pow(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[574] = p[0]*(2.0*y[8]*u[3] - 2.0*y[14]*u[3] + y[33]*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[575] = p[0]*(2.0*y[9]*u[3] - 2.0*y[15]*u[3] - y[32]*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[576] = p[0]*(y[9] - y[15])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[577] = p[0]*(-y[8] + y[14])*((sqrt(pow(y[9], 2) + pow(y[8], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[9], 2) + pow(y[8], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[578] = p[0]*(-2.0*y[10]*u[7] + 2.0*y[14]*u[7] - y[35]*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[11]*y[34] - y[10]*y[35] - y[15]*y[34] + y[14]*y[35])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   -1.0*y[10]*pow(pow(y[11], 2) + pow(y[10], 2), -0.5)/pow(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[579] = p[0]*(-2.0*y[11]*u[7] + 2.0*y[15]*u[7] + y[34]*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[11]*y[34] - y[10]*y[35] - y[15]*y[34] + y[14]*y[35])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   -1.0*y[11]*pow(pow(y[11], 2) + pow(y[10], 2), -0.5)/pow(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[580] = p[0]*(2.0*y[10]*u[7] - 2.0*y[14]*u[7] + y[35]*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[581] = p[0]*(2.0*y[11]*u[7] - 2.0*y[15]*u[7] - y[34]*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[582] = p[0]*(y[11] - y[15])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[583] = p[0]*(-y[10] + y[14])*((sqrt(pow(y[11], 2) + pow(y[10], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[11], 2) + pow(y[10], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[584] = p[0]*(-2.0*y[12]*u[11] + 2.0*y[14]*u[11] - y[37]*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[13]*y[36] - y[12]*y[37] - y[15]*y[36] + y[14]*y[37])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   -1.0*y[12]*pow(pow(y[13], 2) + pow(y[12], 2), -0.5)/pow(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[585] = p[0]*(-2.0*y[13]*u[11] + 2.0*y[15]*u[11] + y[36]*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)) + (y[13]*y[36] - y[12]*y[37] - y[15]*y[36] + y[14]*y[37])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   -1.0*y[13]*pow(pow(y[13], 2) + pow(y[12], 2), -0.5)/pow(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999, 2)
)
: (
   0
)));
out[586] = p[0]*(2.0*y[12]*u[11] - 2.0*y[14]*u[11] + y[37]*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[587] = p[0]*(2.0*y[13]*u[11] - 2.0*y[15]*u[11] - y[36]*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
)));
out[588] = p[0]*(y[13] - y[15])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
));
out[589] = p[0]*(-y[12] + y[14])*((sqrt(pow(y[13], 2) + pow(y[12], 2)) < 0.69999999999999996) ? (
   1.0/(sqrt(pow(y[13], 2) + pow(y[12], 2)) + 0.29999999999999999)
)
: (
   1.0
));

}

void sp_jac_trap_up_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[0] = 0.5*Dt*p[17] + 1.0;
out[1] = 0.5*Dt;
out[155] = -u[12];
out[173] = -u[12];
out[191] = -u[12];
out[209] = -u[12];
out[227] = -u[12];
out[245] = -u[12];
out[263] = -u[12];
out[281] = -u[12];
out[298] = u[12];
out[315] = u[12];
out[332] = u[12];
out[349] = u[12];
out[366] = u[12];
out[383] = u[12];
out[400] = u[12];
out[417] = u[12];
out[600] = -p[1];
out[601] = 1.0*p[2];
out[604] = -p[1];
out[605] = 1.0*p[2];
out[608] = -p[1];
out[609] = 1.0*p[2];
out[612] = -p[1];
out[613] = 1.0*p[2];
out[616] = -1.0*p[2];
out[617] = -1.0*p[1];
out[620] = -1.0*p[2];
out[621] = -1.0*p[1];
out[624] = -1.0*p[2];
out[625] = -1.0*p[1];
out[628] = -1.0*p[2];
out[629] = -1.0*p[1];
out[631] = -p[4];
out[632] = 1.0*p[3];
out[634] = -p[4];
out[635] = 1.0*p[3];
out[637] = -p[4];
out[638] = 1.0*p[3];
out[640] = -p[6];
out[641] = 1.0*p[5];
out[643] = -1.0*p[3];
out[644] = -1.0*p[4];
out[646] = -1.0*p[3];
out[647] = -1.0*p[4];
out[649] = -1.0*p[3];
out[650] = -1.0*p[4];
out[652] = -1.0*p[5];
out[653] = -1.0*p[6];
out[655] = -p[10];
out[656] = 1.0*p[9];
out[658] = -p[10];
out[659] = 1.0*p[9];
out[661] = -p[10];
out[662] = 1.0*p[9];
out[664] = -p[12];
out[665] = 1.0*p[11];
out[667] = -1.0*p[9];
out[668] = -1.0*p[10];
out[670] = -1.0*p[9];
out[671] = -1.0*p[10];
out[673] = -1.0*p[9];
out[674] = -1.0*p[10];
out[676] = -1.0*p[11];
out[677] = -1.0*p[12];
out[679] = p[16];
out[680] = -p[15];

}

void sp_jac_trap_num_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[2] = 184.75059629806526;
out[3] = 94.615102344455664;
out[4] = -69.368640085185305;
out[5] = -17.974903844364835;
out[6] = -43.675799641021079;
out[7] = -40.748687841669557;
out[8] = -69.368640085185291;
out[9] = -17.974903844364821;
out[10] = -184.75059629806526;
out[11] = -94.615102344455664;
out[12] = 69.368640085185305;
out[13] = 17.974903844364835;
out[14] = 43.675799641021079;
out[15] = 40.748687841669557;
out[16] = 69.368640085185291;
out[17] = 17.974903844364821;
out[18] = -1.0;
out[19] = -94.615102344455664;
out[20] = 184.75059629806526;
out[21] = 17.974903844364835;
out[22] = -69.368640085185305;
out[23] = 40.748687841669557;
out[24] = -43.675799641021079;
out[25] = 17.974903844364821;
out[26] = -69.368640085185291;
out[27] = 94.615102344455664;
out[28] = -184.75059629806526;
out[29] = -17.974903844364835;
out[30] = 69.368640085185305;
out[31] = -40.748687841669557;
out[32] = 43.675799641021079;
out[33] = -17.974903844364821;
out[34] = 69.368640085185291;
out[35] = -1.0;
out[36] = -69.368640085185291;
out[37] = -17.974903844364825;
out[38] = 184.7505962980652;
out[39] = 94.615102344455636;
out[40] = -69.368640085185319;
out[41] = -17.97490384436481;
out[42] = -43.675799641021079;
out[43] = -40.748687841669565;
out[44] = 69.368640085185291;
out[45] = 17.974903844364825;
out[46] = -184.7505962980652;
out[47] = -94.615102344455636;
out[48] = 69.368640085185319;
out[49] = 17.97490384436481;
out[50] = 43.675799641021079;
out[51] = 40.748687841669565;
out[52] = -1.0;
out[53] = 17.974903844364825;
out[54] = -69.368640085185291;
out[55] = -94.615102344455636;
out[56] = 184.7505962980652;
out[57] = 17.97490384436481;
out[58] = -69.368640085185319;
out[59] = 40.748687841669565;
out[60] = -43.675799641021079;
out[61] = -17.974903844364825;
out[62] = 69.368640085185291;
out[63] = 94.615102344455636;
out[64] = -184.7505962980652;
out[65] = -17.97490384436481;
out[66] = 69.368640085185319;
out[67] = -40.748687841669565;
out[68] = 43.675799641021079;
out[69] = -1.0;
out[70] = -43.675799641021101;
out[71] = -40.748687841669579;
out[72] = -69.368640085185305;
out[73] = -17.974903844364821;
out[74] = 184.75059629806526;
out[75] = 94.61510234445565;
out[76] = -69.368640085185305;
out[77] = -17.974903844364821;
out[78] = 43.675799641021101;
out[79] = 40.748687841669579;
out[80] = 69.368640085185305;
out[81] = 17.974903844364821;
out[82] = -184.75059629806526;
out[83] = -94.61510234445565;
out[84] = 69.368640085185305;
out[85] = 17.974903844364821;
out[86] = -1.0;
out[87] = 40.748687841669579;
out[88] = -43.675799641021101;
out[89] = 17.974903844364821;
out[90] = -69.368640085185305;
out[91] = -94.61510234445565;
out[92] = 184.75059629806526;
out[93] = 17.974903844364821;
out[94] = -69.368640085185305;
out[95] = -40.748687841669579;
out[96] = 43.675799641021101;
out[97] = -17.974903844364821;
out[98] = 69.368640085185305;
out[99] = 94.61510234445565;
out[100] = -184.75059629806526;
out[101] = -17.974903844364821;
out[102] = 69.368640085185305;
out[103] = -1.0;
out[104] = -69.368640085185305;
out[105] = -17.974903844364825;
out[106] = -43.675799641021079;
out[107] = -40.74868784166955;
out[108] = -69.368640085185291;
out[109] = -17.974903844364839;
out[110] = 184.7505962980652;
out[111] = 94.61510234445565;
out[112] = 69.368640085185305;
out[113] = 17.974903844364825;
out[114] = 43.675799641021079;
out[115] = 40.74868784166955;
out[116] = 69.368640085185291;
out[117] = 17.974903844364839;
out[118] = -184.7505962980652;
out[119] = -94.61510234445565;
out[120] = -1.0;
out[121] = 17.974903844364825;
out[122] = -69.368640085185305;
out[123] = 40.74868784166955;
out[124] = -43.675799641021079;
out[125] = 17.974903844364839;
out[126] = -69.368640085185291;
out[127] = -94.61510234445565;
out[128] = 184.7505962980652;
out[129] = -17.974903844364825;
out[130] = 69.368640085185305;
out[131] = -40.74868784166955;
out[132] = 43.675799641021079;
out[133] = -17.974903844364839;
out[134] = 69.368640085185291;
out[135] = 94.61510234445565;
out[136] = -184.7505962980652;
out[137] = -1.0;
out[138] = -184.75059629806526;
out[139] = -94.615102344455664;
out[140] = 69.368640085185305;
out[141] = 17.974903844364835;
out[142] = 43.675799641021079;
out[143] = 40.748687841669557;
out[144] = 69.368640085185291;
out[145] = 17.974903844364821;
out[146] = 184.75059629806526;
out[147] = 94.615102344455664;
out[148] = -69.368640085185305;
out[149] = -17.974903844364835;
out[150] = -43.675799641021079;
out[151] = -40.748687841669557;
out[152] = -69.368640085185291;
out[153] = -17.974903844364821;
out[154] = -1.0;
out[156] = 94.615102344455664;
out[157] = -184.75059629806526;
out[158] = -17.974903844364835;
out[159] = 69.368640085185305;
out[160] = -40.748687841669557;
out[161] = 43.675799641021079;
out[162] = -17.974903844364821;
out[163] = 69.368640085185291;
out[164] = -94.615102344455664;
out[165] = 184.75059629806526;
out[166] = 17.974903844364835;
out[167] = -69.368640085185305;
out[168] = 40.748687841669557;
out[169] = -43.675799641021079;
out[170] = 17.974903844364821;
out[171] = -69.368640085185291;
out[172] = -1.0;
out[174] = 69.368640085185291;
out[175] = 17.974903844364825;
out[176] = -184.7505962980652;
out[177] = -94.615102344455636;
out[178] = 69.368640085185319;
out[179] = 17.97490384436481;
out[180] = 43.675799641021079;
out[181] = 40.748687841669565;
out[182] = -69.368640085185291;
out[183] = -17.974903844364825;
out[184] = 184.7505962980652;
out[185] = 94.615102344455636;
out[186] = -69.368640085185319;
out[187] = -17.97490384436481;
out[188] = -43.675799641021079;
out[189] = -40.748687841669565;
out[190] = -1.0;
out[192] = -17.974903844364825;
out[193] = 69.368640085185291;
out[194] = 94.615102344455636;
out[195] = -184.7505962980652;
out[196] = -17.97490384436481;
out[197] = 69.368640085185319;
out[198] = -40.748687841669565;
out[199] = 43.675799641021079;
out[200] = 17.974903844364825;
out[201] = -69.368640085185291;
out[202] = -94.615102344455636;
out[203] = 184.7505962980652;
out[204] = 17.97490384436481;
out[205] = -69.368640085185319;
out[206] = 40.748687841669565;
out[207] = -43.675799641021079;
out[208] = -1.0;
out[210] = 43.675799641021101;
out[211] = 40.748687841669579;
out[212] = 69.368640085185305;
out[213] = 17.974903844364821;
out[214] = -184.75059629806526;
out[215] = -94.61510234445565;
out[216] = 69.368640085185305;
out[217] = 17.974903844364821;
out[218] = -43.675799641021101;
out[219] = -40.748687841669579;
out[220] = -69.368640085185305;
out[221] = -17.974903844364821;
out[222] = 184.75059629806526;
out[223] = 94.61510234445565;
out[224] = -69.368640085185305;
out[225] = -17.974903844364821;
out[226] = -1.0;
out[228] = -40.748687841669579;
out[229] = 43.675799641021101;
out[230] = -17.974903844364821;
out[231] = 69.368640085185305;
out[232] = 94.61510234445565;
out[233] = -184.75059629806526;
out[234] = -17.974903844364821;
out[235] = 69.368640085185305;
out[236] = 40.748687841669579;
out[237] = -43.675799641021101;
out[238] = 17.974903844364821;
out[239] = -69.368640085185305;
out[240] = -94.61510234445565;
out[241] = 184.75059629806526;
out[242] = 17.974903844364821;
out[243] = -69.368640085185305;
out[244] = -1.0;
out[246] = 69.368640085185305;
out[247] = 17.974903844364825;
out[248] = 43.675799641021079;
out[249] = 40.74868784166955;
out[250] = 69.368640085185291;
out[251] = 17.974903844364839;
out[252] = -184.7505962980652;
out[253] = -94.61510234445565;
out[254] = -69.368640085185305;
out[255] = -17.974903844364825;
out[256] = -43.675799641021079;
out[257] = -40.74868784166955;
out[258] = -69.368640085185291;
out[259] = -17.974903844364839;
out[260] = 184.7505962980652;
out[261] = 94.61510234445565;
out[262] = -1.0;
out[264] = -17.974903844364825;
out[265] = 69.368640085185305;
out[266] = -40.74868784166955;
out[267] = 43.675799641021079;
out[268] = -17.974903844364839;
out[269] = 69.368640085185291;
out[270] = 94.61510234445565;
out[271] = -184.7505962980652;
out[272] = 17.974903844364825;
out[273] = -69.368640085185305;
out[274] = 40.74868784166955;
out[275] = -43.675799641021079;
out[276] = 17.974903844364839;
out[277] = -69.368640085185291;
out[278] = -94.61510234445565;
out[279] = 184.7505962980652;
out[280] = -1.0;
out[282] = 184.75059629806526;
out[283] = 94.615102344455664;
out[284] = -69.368640085185305;
out[285] = -17.974903844364835;
out[286] = -43.675799641021079;
out[287] = -40.748687841669557;
out[288] = -69.368640085185291;
out[289] = -17.974903844364821;
out[290] = -184.75059629806526;
out[291] = -94.615102344455664;
out[292] = 69.368640085185305;
out[293] = 17.974903844364835;
out[294] = 43.675799641021079;
out[295] = 40.748687841669557;
out[296] = 69.368640085185291;
out[297] = 17.974903844364821;
out[299] = -94.615102344455664;
out[300] = 184.75059629806526;
out[301] = 17.974903844364835;
out[302] = -69.368640085185305;
out[303] = 40.748687841669557;
out[304] = -43.675799641021079;
out[305] = 17.974903844364821;
out[306] = -69.368640085185291;
out[307] = 94.615102344455664;
out[308] = -184.75059629806526;
out[309] = -17.974903844364835;
out[310] = 69.368640085185305;
out[311] = -40.748687841669557;
out[312] = 43.675799641021079;
out[313] = -17.974903844364821;
out[314] = 69.368640085185291;
out[316] = -69.368640085185291;
out[317] = -17.974903844364825;
out[318] = 184.7505962980652;
out[319] = 94.615102344455636;
out[320] = -69.368640085185319;
out[321] = -17.97490384436481;
out[322] = -43.675799641021079;
out[323] = -40.748687841669565;
out[324] = 69.368640085185291;
out[325] = 17.974903844364825;
out[326] = -184.7505962980652;
out[327] = -94.615102344455636;
out[328] = 69.368640085185319;
out[329] = 17.97490384436481;
out[330] = 43.675799641021079;
out[331] = 40.748687841669565;
out[333] = 17.974903844364825;
out[334] = -69.368640085185291;
out[335] = -94.615102344455636;
out[336] = 184.7505962980652;
out[337] = 17.97490384436481;
out[338] = -69.368640085185319;
out[339] = 40.748687841669565;
out[340] = -43.675799641021079;
out[341] = -17.974903844364825;
out[342] = 69.368640085185291;
out[343] = 94.615102344455636;
out[344] = -184.7505962980652;
out[345] = -17.97490384436481;
out[346] = 69.368640085185319;
out[347] = -40.748687841669565;
out[348] = 43.675799641021079;
out[350] = -43.675799641021101;
out[351] = -40.748687841669579;
out[352] = -69.368640085185305;
out[353] = -17.974903844364821;
out[354] = 184.75059629806526;
out[355] = 94.61510234445565;
out[356] = -69.368640085185305;
out[357] = -17.974903844364821;
out[358] = 43.675799641021101;
out[359] = 40.748687841669579;
out[360] = 69.368640085185305;
out[361] = 17.974903844364821;
out[362] = -184.75059629806526;
out[363] = -94.61510234445565;
out[364] = 69.368640085185305;
out[365] = 17.974903844364821;
out[367] = 40.748687841669579;
out[368] = -43.675799641021101;
out[369] = 17.974903844364821;
out[370] = -69.368640085185305;
out[371] = -94.61510234445565;
out[372] = 184.75059629806526;
out[373] = 17.974903844364821;
out[374] = -69.368640085185305;
out[375] = -40.748687841669579;
out[376] = 43.675799641021101;
out[377] = -17.974903844364821;
out[378] = 69.368640085185305;
out[379] = 94.61510234445565;
out[380] = -184.75059629806526;
out[381] = -17.974903844364821;
out[382] = 69.368640085185305;
out[384] = -69.368640085185305;
out[385] = -17.974903844364825;
out[386] = -43.675799641021079;
out[387] = -40.74868784166955;
out[388] = -69.368640085185291;
out[389] = -17.974903844364839;
out[390] = 184.7505962980652;
out[391] = 94.61510234445565;
out[392] = 69.368640085185305;
out[393] = 17.974903844364825;
out[394] = 43.675799641021079;
out[395] = 40.74868784166955;
out[396] = 69.368640085185291;
out[397] = 17.974903844364839;
out[398] = -184.7505962980652;
out[399] = -94.61510234445565;
out[401] = 17.974903844364825;
out[402] = -69.368640085185305;
out[403] = 40.74868784166955;
out[404] = -43.675799641021079;
out[405] = 17.974903844364839;
out[406] = -69.368640085185291;
out[407] = -94.61510234445565;
out[408] = 184.7505962980652;
out[409] = -17.974903844364825;
out[410] = 69.368640085185305;
out[411] = -40.74868784166955;
out[412] = 43.675799641021079;
out[413] = -17.974903844364839;
out[414] = 69.368640085185291;
out[415] = 94.61510234445565;
out[416] = -184.7505962980652;
out[418] = -184.75059629806526;
out[419] = -94.615102344455664;
out[420] = 69.368640085185305;
out[421] = 17.974903844364835;
out[422] = 43.675799641021079;
out[423] = 40.748687841669557;
out[424] = 69.368640085185291;
out[425] = 17.974903844364821;
out[426] = 184.75059629806526;
out[427] = 94.615102344455664;
out[428] = -69.368640085185305;
out[429] = -17.974903844364835;
out[430] = -43.675799641021079;
out[431] = -40.748687841669557;
out[432] = -69.368640085185291;
out[433] = -17.974903844364821;
out[434] = -1.0;
out[435] = 94.615102344455664;
out[436] = -184.75059629806526;
out[437] = -17.974903844364835;
out[438] = 69.368640085185305;
out[439] = -40.748687841669557;
out[440] = 43.675799641021079;
out[441] = -17.974903844364821;
out[442] = 69.368640085185291;
out[443] = -94.615102344455664;
out[444] = 184.75059629806526;
out[445] = 17.974903844364835;
out[446] = -69.368640085185305;
out[447] = 40.748687841669557;
out[448] = -43.675799641021079;
out[449] = 17.974903844364821;
out[450] = -69.368640085185291;
out[451] = -1.0;
out[452] = 69.368640085185291;
out[453] = 17.974903844364825;
out[454] = -184.7505962980652;
out[455] = -94.615102344455636;
out[456] = 69.368640085185319;
out[457] = 17.97490384436481;
out[458] = 43.675799641021079;
out[459] = 40.748687841669565;
out[460] = -69.368640085185291;
out[461] = -17.974903844364825;
out[462] = 184.7505962980652;
out[463] = 94.615102344455636;
out[464] = -69.368640085185319;
out[465] = -17.97490384436481;
out[466] = -43.675799641021079;
out[467] = -40.748687841669565;
out[468] = -1.0;
out[469] = -17.974903844364825;
out[470] = 69.368640085185291;
out[471] = 94.615102344455636;
out[472] = -184.7505962980652;
out[473] = -17.97490384436481;
out[474] = 69.368640085185319;
out[475] = -40.748687841669565;
out[476] = 43.675799641021079;
out[477] = 17.974903844364825;
out[478] = -69.368640085185291;
out[479] = -94.615102344455636;
out[480] = 184.7505962980652;
out[481] = 17.97490384436481;
out[482] = -69.368640085185319;
out[483] = 40.748687841669565;
out[484] = -43.675799641021079;
out[485] = -1.0;
out[486] = 43.675799641021101;
out[487] = 40.748687841669579;
out[488] = 69.368640085185305;
out[489] = 17.974903844364821;
out[490] = -184.75059629806526;
out[491] = -94.61510234445565;
out[492] = 69.368640085185305;
out[493] = 17.974903844364821;
out[494] = -43.675799641021101;
out[495] = -40.748687841669579;
out[496] = -69.368640085185305;
out[497] = -17.974903844364821;
out[498] = 184.75059629806526;
out[499] = 94.61510234445565;
out[500] = -69.368640085185305;
out[501] = -17.974903844364821;
out[502] = -1.0;
out[503] = -40.748687841669579;
out[504] = 43.675799641021101;
out[505] = -17.974903844364821;
out[506] = 69.368640085185305;
out[507] = 94.61510234445565;
out[508] = -184.75059629806526;
out[509] = -17.974903844364821;
out[510] = 69.368640085185305;
out[511] = 40.748687841669579;
out[512] = -43.675799641021101;
out[513] = 17.974903844364821;
out[514] = -69.368640085185305;
out[515] = -94.61510234445565;
out[516] = 184.75059629806526;
out[517] = 17.974903844364821;
out[518] = -69.368640085185305;
out[519] = -1.0;
out[520] = 69.368640085185305;
out[521] = 17.974903844364825;
out[522] = 43.675799641021079;
out[523] = 40.74868784166955;
out[524] = 69.368640085185291;
out[525] = 17.974903844364839;
out[526] = -184.7505962980652;
out[527] = -94.61510234445565;
out[528] = -69.368640085185305;
out[529] = -17.974903844364825;
out[530] = -43.675799641021079;
out[531] = -40.74868784166955;
out[532] = -69.368640085185291;
out[533] = -17.974903844364839;
out[534] = 184.7505962980652;
out[535] = 94.61510234445565;
out[536] = -1.0;
out[537] = -17.974903844364825;
out[538] = 69.368640085185305;
out[539] = -40.74868784166955;
out[540] = 43.675799641021079;
out[541] = -17.974903844364839;
out[542] = 69.368640085185291;
out[543] = 94.61510234445565;
out[544] = -184.7505962980652;
out[545] = 17.974903844364825;
out[546] = -69.368640085185305;
out[547] = 40.74868784166955;
out[548] = -43.675799641021079;
out[549] = 17.974903844364839;
out[550] = -69.368640085185291;
out[551] = -94.61510234445565;
out[552] = 184.7505962980652;
out[553] = -1.0;
out[590] = 1.0;
out[591] = 1.0;
out[592] = 1.0;
out[593] = 1.0;
out[594] = 1.0;
out[595] = 1.0;
out[596] = 1.0;
out[597] = 1.0;
out[598] = 1.0;
out[599] = -1.0;
out[602] = 1.0;
out[603] = -1.0;
out[606] = 1.0;
out[607] = -1.0;
out[610] = 1.0;
out[611] = -1.0;
out[614] = 1.0;
out[615] = -1.0;
out[618] = 1.0;
out[619] = -1.0;
out[622] = 1.0;
out[623] = -1.0;
out[626] = 1.0;
out[627] = -1.0;
out[630] = -1.0;
out[633] = -1.0;
out[636] = -1.0;
out[639] = -1.0;
out[642] = -1.0;
out[645] = -1.0;
out[648] = -1.0;
out[651] = -1.0;
out[654] = -1.0;
out[657] = -1.0;
out[660] = -1.0;
out[663] = -1.0;
out[666] = -1.0;
out[669] = -1.0;
out[672] = -1.0;
out[675] = -1.0;
out[678] = -1.0;
out[681] = -1.0;

}

void de_Fu_run_xy_eval(double *out,double *x,double *y,double *u,double *p,double Dt){


}

void de_Fu_run_up_eval(double *out,double *x,double *y,double *u,double *p,double Dt){


}

void de_Fu_run_num_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[23] = 1.0;

}

void de_Gu_run_xy_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[204] = -y[40];
out[228] = -y[44];
out[252] = -y[41];
out[276] = -y[45];
out[300] = -y[42];
out[324] = -y[46];
out[348] = -y[43];
out[372] = -y[47];
out[396] = y[40];
out[420] = y[44];
out[444] = y[41];
out[468] = y[45];
out[492] = y[42];
out[516] = y[46];
out[540] = y[43];
out[564] = y[47];
out[770] = p[0]*(pow(y[9], 2) - 2.0*y[9]*y[15] + pow(y[8], 2) - 2.0*y[8]*y[14] + pow(y[15], 2) + pow(y[14], 2));
out[798] = p[0]*(pow(y[11], 2) - 2.0*y[11]*y[15] + pow(y[10], 2) - 2.0*y[10]*y[14] + pow(y[15], 2) + pow(y[14], 2));
out[826] = p[0]*(pow(y[13], 2) - 2.0*y[13]*y[15] + pow(y[12], 2) - 2.0*y[12]*y[14] + pow(y[15], 2) + pow(y[14], 2));
out[843] = p[0]*(-1.0*pow(y[9], 2) + 2.0*y[9]*y[15] - 1.0*pow(y[8], 2) + 2.0*y[8]*y[14] - 1.0*pow(y[15], 2) - 1.0*pow(y[14], 2));
out[871] = p[0]*(-1.0*pow(y[11], 2) + 2.0*y[11]*y[15] - 1.0*pow(y[10], 2) + 2.0*y[10]*y[14] - 1.0*pow(y[15], 2) - 1.0*pow(y[14], 2));
out[899] = p[0]*(-1.0*pow(y[13], 2) + 2.0*y[13]*y[15] - 1.0*pow(y[12], 2) + 2.0*y[12]*y[14] - 1.0*pow(y[15], 2) - 1.0*pow(y[14], 2));

}

void de_Gu_run_up_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[768] = p[0];
out[796] = p[0];
out[824] = p[0];
out[841] = p[0];
out[869] = p[0];
out[897] = p[0];
out[1165] = cos(u[17]);
out[1169] = -u[13]*sin(u[17]);
out[1190] = cos(u[17] - 2.0943951023931953);
out[1193] = -u[14]*sin(u[17] - 2.0943951023931953);
out[1215] = cos(u[17] - 4.1887902047863905);
out[1217] = -u[15]*sin(u[17] - 4.1887902047863905);
out[1261] = 1.0*sin(u[17]);
out[1265] = 1.0*u[13]*cos(u[17]);
out[1286] = 1.0*sin(u[17] - 2.0943951023931953);
out[1289] = 1.0*u[14]*cos(u[17] - 2.0943951023931953);
out[1311] = 1.0*sin(u[17] - 4.1887902047863905);
out[1313] = 1.0*u[15]*cos(u[17] - 4.1887902047863905);
out[1362] = cos(u[22]);
out[1366] = -u[18]*sin(u[22]);
out[1387] = cos(u[22] - 2.0943951023931953);
out[1390] = -u[19]*sin(u[22] - 2.0943951023931953);
out[1412] = cos(u[22] - 4.1887902047863905);
out[1414] = -u[20]*sin(u[22] - 4.1887902047863905);
out[1458] = 1.0*sin(u[22]);
out[1462] = 1.0*u[18]*cos(u[22]);
out[1483] = 1.0*sin(u[22] - 2.0943951023931953);
out[1486] = 1.0*u[19]*cos(u[22] - 2.0943951023931953);
out[1508] = 1.0*sin(u[22] - 4.1887902047863905);
out[1510] = 1.0*u[20]*cos(u[22] - 4.1887902047863905);

}

void de_Gu_run_num_eval(double *out,double *x,double *y,double *u,double *p,double Dt){


}

void de_Hx_run_xy_eval(double *out,double *x,double *y,double *u,double *p,double Dt){


}

void de_Hx_run_up_eval(double *out,double *x,double *y,double *u,double *p,double Dt){


}

void de_Hx_run_num_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[36] = 1.0;

}

void de_Hy_run_xy_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[1096] = y[40]*pow(1.0*pow(y[44], 2) + pow(y[40], 2), -0.5);
out[1100] = 1.0*y[44]*pow(1.0*pow(y[44], 2) + pow(y[40], 2), -0.5);
out[1163] = y[41]*pow(1.0*pow(y[45], 2) + pow(y[41], 2), -0.5);
out[1167] = 1.0*y[45]*pow(1.0*pow(y[45], 2) + pow(y[41], 2), -0.5);
out[1230] = y[42]*pow(1.0*pow(y[46], 2) + pow(y[42], 2), -0.5);
out[1234] = 1.0*y[46]*pow(1.0*pow(y[46], 2) + pow(y[42], 2), -0.5);
out[1297] = y[43]*pow(1.0*pow(y[47], 2) + pow(y[43], 2), -0.5);
out[1301] = 1.0*y[47]*pow(1.0*pow(y[47], 2) + pow(y[43], 2), -0.5);
out[1360] = y[40]*pow(1.0*pow(y[44], 2) + pow(y[40], 2), -0.5);
out[1364] = 1.0*y[44]*pow(1.0*pow(y[44], 2) + pow(y[40], 2), -0.5);
out[1427] = y[41]*pow(1.0*pow(y[45], 2) + pow(y[41], 2), -0.5);
out[1431] = 1.0*y[45]*pow(1.0*pow(y[45], 2) + pow(y[41], 2), -0.5);
out[1494] = y[42]*pow(1.0*pow(y[46], 2) + pow(y[42], 2), -0.5);
out[1498] = 1.0*y[46]*pow(1.0*pow(y[46], 2) + pow(y[42], 2), -0.5);
out[1561] = y[43]*pow(1.0*pow(y[47], 2) + pow(y[43], 2), -0.5);
out[1565] = 1.0*y[47]*pow(1.0*pow(y[47], 2) + pow(y[43], 2), -0.5);
out[1632] = y[48]*pow(1.0*pow(y[52], 2) + pow(y[48], 2), -0.5);
out[1636] = 1.0*y[52]*pow(1.0*pow(y[52], 2) + pow(y[48], 2), -0.5);
out[1699] = y[49]*pow(1.0*pow(y[53], 2) + pow(y[49], 2), -0.5);
out[1703] = 1.0*y[53]*pow(1.0*pow(y[53], 2) + pow(y[49], 2), -0.5);
out[1766] = y[50]*pow(1.0*pow(y[54], 2) + pow(y[50], 2), -0.5);
out[1770] = 1.0*y[54]*pow(1.0*pow(y[54], 2) + pow(y[50], 2), -0.5);
out[1833] = y[51]*pow(1.0*pow(y[55], 2) + pow(y[51], 2), -0.5);
out[1837] = 1.0*y[55]*pow(1.0*pow(y[55], 2) + pow(y[51], 2), -0.5);
out[1848] = y[48];
out[1849] = 1.0*y[52];
out[1850] = y[49];
out[1851] = 1.0*y[53];
out[1852] = y[50];
out[1853] = 1.0*y[54];
out[1896] = y[0];
out[1897] = y[2];
out[1898] = y[4];
out[1900] = 1.0*y[1];
out[1901] = 1.0*y[3];
out[1902] = 1.0*y[5];
out[1914] = -1.0*y[52];
out[1915] = 1.0*y[48];
out[1916] = -1.0*y[53];
out[1917] = 1.0*y[49];
out[1918] = -1.0*y[54];
out[1919] = 1.0*y[50];
out[1962] = 1.0*y[1];
out[1963] = 1.0*y[3];
out[1964] = 1.0*y[5];
out[1966] = -1.0*y[0];
out[1967] = -1.0*y[2];
out[1968] = -1.0*y[4];
out[2036] = y[56]*pow(1.0*pow(y[60], 2) + pow(y[56], 2), -0.5);
out[2040] = 1.0*y[60]*pow(1.0*pow(y[60], 2) + pow(y[56], 2), -0.5);
out[2103] = y[57]*pow(1.0*pow(y[61], 2) + pow(y[57], 2), -0.5);
out[2107] = 1.0*y[61]*pow(1.0*pow(y[61], 2) + pow(y[57], 2), -0.5);
out[2170] = y[58]*pow(1.0*pow(y[62], 2) + pow(y[58], 2), -0.5);
out[2174] = 1.0*y[62]*pow(1.0*pow(y[62], 2) + pow(y[58], 2), -0.5);
out[2237] = y[59]*pow(1.0*pow(y[63], 2) + pow(y[59], 2), -0.5);
out[2241] = 1.0*y[63]*pow(1.0*pow(y[63], 2) + pow(y[59], 2), -0.5);
out[2268] = y[56];
out[2269] = 1.0*y[60];
out[2270] = y[57];
out[2271] = 1.0*y[61];
out[2272] = y[58];
out[2273] = 1.0*y[62];
out[2300] = y[24];
out[2301] = y[26];
out[2302] = y[28];
out[2304] = 1.0*y[25];
out[2305] = 1.0*y[27];
out[2306] = 1.0*y[29];
out[2334] = -1.0*y[60];
out[2335] = 1.0*y[56];
out[2336] = -1.0*y[61];
out[2337] = 1.0*y[57];
out[2338] = -1.0*y[62];
out[2339] = 1.0*y[58];
out[2366] = 1.0*y[25];
out[2367] = 1.0*y[27];
out[2368] = 1.0*y[29];
out[2370] = -1.0*y[24];
out[2371] = -1.0*y[26];
out[2372] = -1.0*y[28];

}

void de_Hy_run_up_eval(double *out,double *x,double *y,double *u,double *p,double Dt){


}

void de_Hy_run_num_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[0] = 184.75059629806526;
out[1] = 94.615102344455664;
out[2] = -69.368640085185305;
out[3] = -17.974903844364835;
out[4] = -43.675799641021079;
out[5] = -40.748687841669557;
out[6] = -69.368640085185291;
out[7] = -17.974903844364821;
out[8] = -184.75059629806526;
out[9] = -94.615102344455664;
out[10] = 69.368640085185305;
out[11] = 17.974903844364835;
out[12] = 43.675799641021079;
out[13] = 40.748687841669557;
out[14] = 69.368640085185291;
out[15] = 17.974903844364821;
out[66] = -94.615102344455664;
out[67] = 184.75059629806526;
out[68] = 17.974903844364835;
out[69] = -69.368640085185305;
out[70] = 40.748687841669557;
out[71] = -43.675799641021079;
out[72] = 17.974903844364821;
out[73] = -69.368640085185291;
out[74] = 94.615102344455664;
out[75] = -184.75059629806526;
out[76] = -17.974903844364835;
out[77] = 69.368640085185305;
out[78] = -40.748687841669557;
out[79] = 43.675799641021079;
out[80] = -17.974903844364821;
out[81] = 69.368640085185291;
out[132] = -69.368640085185291;
out[133] = -17.974903844364825;
out[134] = 184.7505962980652;
out[135] = 94.615102344455636;
out[136] = -69.368640085185319;
out[137] = -17.97490384436481;
out[138] = -43.675799641021079;
out[139] = -40.748687841669565;
out[140] = 69.368640085185291;
out[141] = 17.974903844364825;
out[142] = -184.7505962980652;
out[143] = -94.615102344455636;
out[144] = 69.368640085185319;
out[145] = 17.97490384436481;
out[146] = 43.675799641021079;
out[147] = 40.748687841669565;
out[198] = 17.974903844364825;
out[199] = -69.368640085185291;
out[200] = -94.615102344455636;
out[201] = 184.7505962980652;
out[202] = 17.97490384436481;
out[203] = -69.368640085185319;
out[204] = 40.748687841669565;
out[205] = -43.675799641021079;
out[206] = -17.974903844364825;
out[207] = 69.368640085185291;
out[208] = 94.615102344455636;
out[209] = -184.7505962980652;
out[210] = -17.97490384436481;
out[211] = 69.368640085185319;
out[212] = -40.748687841669565;
out[213] = 43.675799641021079;
out[264] = -43.675799641021101;
out[265] = -40.748687841669579;
out[266] = -69.368640085185305;
out[267] = -17.974903844364821;
out[268] = 184.75059629806526;
out[269] = 94.61510234445565;
out[270] = -69.368640085185305;
out[271] = -17.974903844364821;
out[272] = 43.675799641021101;
out[273] = 40.748687841669579;
out[274] = 69.368640085185305;
out[275] = 17.974903844364821;
out[276] = -184.75059629806526;
out[277] = -94.61510234445565;
out[278] = 69.368640085185305;
out[279] = 17.974903844364821;
out[330] = 40.748687841669579;
out[331] = -43.675799641021101;
out[332] = 17.974903844364821;
out[333] = -69.368640085185305;
out[334] = -94.61510234445565;
out[335] = 184.75059629806526;
out[336] = 17.974903844364821;
out[337] = -69.368640085185305;
out[338] = -40.748687841669579;
out[339] = 43.675799641021101;
out[340] = -17.974903844364821;
out[341] = 69.368640085185305;
out[342] = 94.61510234445565;
out[343] = -184.75059629806526;
out[344] = -17.974903844364821;
out[345] = 69.368640085185305;
out[396] = -69.368640085185305;
out[397] = -17.974903844364825;
out[398] = -43.675799641021079;
out[399] = -40.74868784166955;
out[400] = -69.368640085185291;
out[401] = -17.974903844364839;
out[402] = 184.7505962980652;
out[403] = 94.61510234445565;
out[404] = 69.368640085185305;
out[405] = 17.974903844364825;
out[406] = 43.675799641021079;
out[407] = 40.74868784166955;
out[408] = 69.368640085185291;
out[409] = 17.974903844364839;
out[410] = -184.7505962980652;
out[411] = -94.61510234445565;
out[462] = 17.974903844364825;
out[463] = -69.368640085185305;
out[464] = 40.74868784166955;
out[465] = -43.675799641021079;
out[466] = 17.974903844364839;
out[467] = -69.368640085185291;
out[468] = -94.61510234445565;
out[469] = 184.7505962980652;
out[470] = -17.974903844364825;
out[471] = 69.368640085185305;
out[472] = -40.74868784166955;
out[473] = 43.675799641021079;
out[474] = -17.974903844364839;
out[475] = 69.368640085185291;
out[476] = 94.61510234445565;
out[477] = -184.7505962980652;
out[544] = 184.75059629806526;
out[545] = 94.615102344455664;
out[546] = -69.368640085185305;
out[547] = -17.974903844364835;
out[548] = -43.675799641021079;
out[549] = -40.748687841669557;
out[550] = -69.368640085185291;
out[551] = -17.974903844364821;
out[552] = -184.75059629806526;
out[553] = -94.615102344455664;
out[554] = 69.368640085185305;
out[555] = 17.974903844364835;
out[556] = 43.675799641021079;
out[557] = 40.748687841669557;
out[558] = 69.368640085185291;
out[559] = 17.974903844364821;
out[610] = -94.615102344455664;
out[611] = 184.75059629806526;
out[612] = 17.974903844364835;
out[613] = -69.368640085185305;
out[614] = 40.748687841669557;
out[615] = -43.675799641021079;
out[616] = 17.974903844364821;
out[617] = -69.368640085185291;
out[618] = 94.615102344455664;
out[619] = -184.75059629806526;
out[620] = -17.974903844364835;
out[621] = 69.368640085185305;
out[622] = -40.748687841669557;
out[623] = 43.675799641021079;
out[624] = -17.974903844364821;
out[625] = 69.368640085185291;
out[676] = -69.368640085185291;
out[677] = -17.974903844364825;
out[678] = 184.7505962980652;
out[679] = 94.615102344455636;
out[680] = -69.368640085185319;
out[681] = -17.97490384436481;
out[682] = -43.675799641021079;
out[683] = -40.748687841669565;
out[684] = 69.368640085185291;
out[685] = 17.974903844364825;
out[686] = -184.7505962980652;
out[687] = -94.615102344455636;
out[688] = 69.368640085185319;
out[689] = 17.97490384436481;
out[690] = 43.675799641021079;
out[691] = 40.748687841669565;
out[742] = 17.974903844364825;
out[743] = -69.368640085185291;
out[744] = -94.615102344455636;
out[745] = 184.7505962980652;
out[746] = 17.97490384436481;
out[747] = -69.368640085185319;
out[748] = 40.748687841669565;
out[749] = -43.675799641021079;
out[750] = -17.974903844364825;
out[751] = 69.368640085185291;
out[752] = 94.615102344455636;
out[753] = -184.7505962980652;
out[754] = -17.97490384436481;
out[755] = 69.368640085185319;
out[756] = -40.748687841669565;
out[757] = 43.675799641021079;
out[808] = -43.675799641021101;
out[809] = -40.748687841669579;
out[810] = -69.368640085185305;
out[811] = -17.974903844364821;
out[812] = 184.75059629806526;
out[813] = 94.61510234445565;
out[814] = -69.368640085185305;
out[815] = -17.974903844364821;
out[816] = 43.675799641021101;
out[817] = 40.748687841669579;
out[818] = 69.368640085185305;
out[819] = 17.974903844364821;
out[820] = -184.75059629806526;
out[821] = -94.61510234445565;
out[822] = 69.368640085185305;
out[823] = 17.974903844364821;
out[874] = 40.748687841669579;
out[875] = -43.675799641021101;
out[876] = 17.974903844364821;
out[877] = -69.368640085185305;
out[878] = -94.61510234445565;
out[879] = 184.75059629806526;
out[880] = 17.974903844364821;
out[881] = -69.368640085185305;
out[882] = -40.748687841669579;
out[883] = 43.675799641021101;
out[884] = -17.974903844364821;
out[885] = 69.368640085185305;
out[886] = 94.61510234445565;
out[887] = -184.75059629806526;
out[888] = -17.974903844364821;
out[889] = 69.368640085185305;
out[940] = -69.368640085185305;
out[941] = -17.974903844364825;
out[942] = -43.675799641021079;
out[943] = -40.74868784166955;
out[944] = -69.368640085185291;
out[945] = -17.974903844364839;
out[946] = 184.7505962980652;
out[947] = 94.61510234445565;
out[948] = 69.368640085185305;
out[949] = 17.974903844364825;
out[950] = 43.675799641021079;
out[951] = 40.74868784166955;
out[952] = 69.368640085185291;
out[953] = 17.974903844364839;
out[954] = -184.7505962980652;
out[955] = -94.61510234445565;
out[1006] = 17.974903844364825;
out[1007] = -69.368640085185305;
out[1008] = 40.74868784166955;
out[1009] = -43.675799641021079;
out[1010] = 17.974903844364839;
out[1011] = -69.368640085185291;
out[1012] = -94.61510234445565;
out[1013] = 184.7505962980652;
out[1014] = -17.974903844364825;
out[1015] = 69.368640085185305;
out[1016] = -40.74868784166955;
out[1017] = 43.675799641021079;
out[1018] = -17.974903844364839;
out[1019] = 69.368640085185291;
out[1020] = 94.61510234445565;
out[1021] = -184.7505962980652;

}

void de_Hu_run_xy_eval(double *out,double *x,double *y,double *u,double *p,double Dt){


}

void de_Hu_run_up_eval(double *out,double *x,double *y,double *u,double *p,double Dt){


}

void de_Hu_run_num_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[911] = 1.0;

}

void sp_Fu_run_xy_eval(double *out,double *x,double *y,double *u,double *p,double Dt){


}

void sp_Fu_run_up_eval(double *out,double *x,double *y,double *u,double *p,double Dt){


}

void sp_Fu_run_num_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[0] = 1.0;

}

void sp_Gu_run_xy_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[0] = -y[40];
out[1] = -y[44];
out[2] = -y[41];
out[3] = -y[45];
out[4] = -y[42];
out[5] = -y[46];
out[6] = -y[43];
out[7] = -y[47];
out[8] = y[40];
out[9] = y[44];
out[10] = y[41];
out[11] = y[45];
out[12] = y[42];
out[13] = y[46];
out[14] = y[43];
out[15] = y[47];
out[17] = p[0]*(pow(y[9], 2) - 2.0*y[9]*y[15] + pow(y[8], 2) - 2.0*y[8]*y[14] + pow(y[15], 2) + pow(y[14], 2));
out[19] = p[0]*(pow(y[11], 2) - 2.0*y[11]*y[15] + pow(y[10], 2) - 2.0*y[10]*y[14] + pow(y[15], 2) + pow(y[14], 2));
out[21] = p[0]*(pow(y[13], 2) - 2.0*y[13]*y[15] + pow(y[12], 2) - 2.0*y[12]*y[14] + pow(y[15], 2) + pow(y[14], 2));
out[23] = p[0]*(-1.0*pow(y[9], 2) + 2.0*y[9]*y[15] - 1.0*pow(y[8], 2) + 2.0*y[8]*y[14] - 1.0*pow(y[15], 2) - 1.0*pow(y[14], 2));
out[25] = p[0]*(-1.0*pow(y[11], 2) + 2.0*y[11]*y[15] - 1.0*pow(y[10], 2) + 2.0*y[10]*y[14] - 1.0*pow(y[15], 2) - 1.0*pow(y[14], 2));
out[27] = p[0]*(-1.0*pow(y[13], 2) + 2.0*y[13]*y[15] - 1.0*pow(y[12], 2) + 2.0*y[12]*y[14] - 1.0*pow(y[15], 2) - 1.0*pow(y[14], 2));

}

void sp_Gu_run_up_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[16] = p[0];
out[18] = p[0];
out[20] = p[0];
out[22] = p[0];
out[24] = p[0];
out[26] = p[0];
out[28] = cos(u[17]);
out[29] = -u[13]*sin(u[17]);
out[30] = cos(u[17] - 2.0943951023931953);
out[31] = -u[14]*sin(u[17] - 2.0943951023931953);
out[32] = cos(u[17] - 4.1887902047863905);
out[33] = -u[15]*sin(u[17] - 4.1887902047863905);
out[34] = 1.0*sin(u[17]);
out[35] = 1.0*u[13]*cos(u[17]);
out[36] = 1.0*sin(u[17] - 2.0943951023931953);
out[37] = 1.0*u[14]*cos(u[17] - 2.0943951023931953);
out[38] = 1.0*sin(u[17] - 4.1887902047863905);
out[39] = 1.0*u[15]*cos(u[17] - 4.1887902047863905);
out[40] = cos(u[22]);
out[41] = -u[18]*sin(u[22]);
out[42] = cos(u[22] - 2.0943951023931953);
out[43] = -u[19]*sin(u[22] - 2.0943951023931953);
out[44] = cos(u[22] - 4.1887902047863905);
out[45] = -u[20]*sin(u[22] - 4.1887902047863905);
out[46] = 1.0*sin(u[22]);
out[47] = 1.0*u[18]*cos(u[22]);
out[48] = 1.0*sin(u[22] - 2.0943951023931953);
out[49] = 1.0*u[19]*cos(u[22] - 2.0943951023931953);
out[50] = 1.0*sin(u[22] - 4.1887902047863905);
out[51] = 1.0*u[20]*cos(u[22] - 4.1887902047863905);

}

void sp_Gu_run_num_eval(double *out,double *x,double *y,double *u,double *p,double Dt){


}

void sp_Hx_run_xy_eval(double *out,double *x,double *y,double *u,double *p,double Dt){


}

void sp_Hx_run_up_eval(double *out,double *x,double *y,double *u,double *p,double Dt){


}

void sp_Hx_run_num_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[0] = 1.0;

}

void sp_Hy_run_xy_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[256] = y[40]*pow(1.0*pow(y[44], 2) + pow(y[40], 2), -0.5);
out[257] = 1.0*y[44]*pow(1.0*pow(y[44], 2) + pow(y[40], 2), -0.5);
out[258] = y[41]*pow(1.0*pow(y[45], 2) + pow(y[41], 2), -0.5);
out[259] = 1.0*y[45]*pow(1.0*pow(y[45], 2) + pow(y[41], 2), -0.5);
out[260] = y[42]*pow(1.0*pow(y[46], 2) + pow(y[42], 2), -0.5);
out[261] = 1.0*y[46]*pow(1.0*pow(y[46], 2) + pow(y[42], 2), -0.5);
out[262] = y[43]*pow(1.0*pow(y[47], 2) + pow(y[43], 2), -0.5);
out[263] = 1.0*y[47]*pow(1.0*pow(y[47], 2) + pow(y[43], 2), -0.5);
out[264] = y[40]*pow(1.0*pow(y[44], 2) + pow(y[40], 2), -0.5);
out[265] = 1.0*y[44]*pow(1.0*pow(y[44], 2) + pow(y[40], 2), -0.5);
out[266] = y[41]*pow(1.0*pow(y[45], 2) + pow(y[41], 2), -0.5);
out[267] = 1.0*y[45]*pow(1.0*pow(y[45], 2) + pow(y[41], 2), -0.5);
out[268] = y[42]*pow(1.0*pow(y[46], 2) + pow(y[42], 2), -0.5);
out[269] = 1.0*y[46]*pow(1.0*pow(y[46], 2) + pow(y[42], 2), -0.5);
out[270] = y[43]*pow(1.0*pow(y[47], 2) + pow(y[43], 2), -0.5);
out[271] = 1.0*y[47]*pow(1.0*pow(y[47], 2) + pow(y[43], 2), -0.5);
out[272] = y[48]*pow(1.0*pow(y[52], 2) + pow(y[48], 2), -0.5);
out[273] = 1.0*y[52]*pow(1.0*pow(y[52], 2) + pow(y[48], 2), -0.5);
out[274] = y[49]*pow(1.0*pow(y[53], 2) + pow(y[49], 2), -0.5);
out[275] = 1.0*y[53]*pow(1.0*pow(y[53], 2) + pow(y[49], 2), -0.5);
out[276] = y[50]*pow(1.0*pow(y[54], 2) + pow(y[50], 2), -0.5);
out[277] = 1.0*y[54]*pow(1.0*pow(y[54], 2) + pow(y[50], 2), -0.5);
out[278] = y[51]*pow(1.0*pow(y[55], 2) + pow(y[51], 2), -0.5);
out[279] = 1.0*y[55]*pow(1.0*pow(y[55], 2) + pow(y[51], 2), -0.5);
out[280] = y[48];
out[281] = 1.0*y[52];
out[282] = y[49];
out[283] = 1.0*y[53];
out[284] = y[50];
out[285] = 1.0*y[54];
out[286] = y[0];
out[287] = y[2];
out[288] = y[4];
out[289] = 1.0*y[1];
out[290] = 1.0*y[3];
out[291] = 1.0*y[5];
out[292] = -1.0*y[52];
out[293] = 1.0*y[48];
out[294] = -1.0*y[53];
out[295] = 1.0*y[49];
out[296] = -1.0*y[54];
out[297] = 1.0*y[50];
out[298] = 1.0*y[1];
out[299] = 1.0*y[3];
out[300] = 1.0*y[5];
out[301] = -1.0*y[0];
out[302] = -1.0*y[2];
out[303] = -1.0*y[4];
out[304] = y[56]*pow(1.0*pow(y[60], 2) + pow(y[56], 2), -0.5);
out[305] = 1.0*y[60]*pow(1.0*pow(y[60], 2) + pow(y[56], 2), -0.5);
out[306] = y[57]*pow(1.0*pow(y[61], 2) + pow(y[57], 2), -0.5);
out[307] = 1.0*y[61]*pow(1.0*pow(y[61], 2) + pow(y[57], 2), -0.5);
out[308] = y[58]*pow(1.0*pow(y[62], 2) + pow(y[58], 2), -0.5);
out[309] = 1.0*y[62]*pow(1.0*pow(y[62], 2) + pow(y[58], 2), -0.5);
out[310] = y[59]*pow(1.0*pow(y[63], 2) + pow(y[59], 2), -0.5);
out[311] = 1.0*y[63]*pow(1.0*pow(y[63], 2) + pow(y[59], 2), -0.5);
out[312] = y[56];
out[313] = 1.0*y[60];
out[314] = y[57];
out[315] = 1.0*y[61];
out[316] = y[58];
out[317] = 1.0*y[62];
out[318] = y[24];
out[319] = y[26];
out[320] = y[28];
out[321] = 1.0*y[25];
out[322] = 1.0*y[27];
out[323] = 1.0*y[29];
out[324] = -1.0*y[60];
out[325] = 1.0*y[56];
out[326] = -1.0*y[61];
out[327] = 1.0*y[57];
out[328] = -1.0*y[62];
out[329] = 1.0*y[58];
out[330] = 1.0*y[25];
out[331] = 1.0*y[27];
out[332] = 1.0*y[29];
out[333] = -1.0*y[24];
out[334] = -1.0*y[26];
out[335] = -1.0*y[28];

}

void sp_Hy_run_up_eval(double *out,double *x,double *y,double *u,double *p,double Dt){


}

void sp_Hy_run_num_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[0] = 184.75059629806526;
out[1] = 94.615102344455664;
out[2] = -69.368640085185305;
out[3] = -17.974903844364835;
out[4] = -43.675799641021079;
out[5] = -40.748687841669557;
out[6] = -69.368640085185291;
out[7] = -17.974903844364821;
out[8] = -184.75059629806526;
out[9] = -94.615102344455664;
out[10] = 69.368640085185305;
out[11] = 17.974903844364835;
out[12] = 43.675799641021079;
out[13] = 40.748687841669557;
out[14] = 69.368640085185291;
out[15] = 17.974903844364821;
out[16] = -94.615102344455664;
out[17] = 184.75059629806526;
out[18] = 17.974903844364835;
out[19] = -69.368640085185305;
out[20] = 40.748687841669557;
out[21] = -43.675799641021079;
out[22] = 17.974903844364821;
out[23] = -69.368640085185291;
out[24] = 94.615102344455664;
out[25] = -184.75059629806526;
out[26] = -17.974903844364835;
out[27] = 69.368640085185305;
out[28] = -40.748687841669557;
out[29] = 43.675799641021079;
out[30] = -17.974903844364821;
out[31] = 69.368640085185291;
out[32] = -69.368640085185291;
out[33] = -17.974903844364825;
out[34] = 184.7505962980652;
out[35] = 94.615102344455636;
out[36] = -69.368640085185319;
out[37] = -17.97490384436481;
out[38] = -43.675799641021079;
out[39] = -40.748687841669565;
out[40] = 69.368640085185291;
out[41] = 17.974903844364825;
out[42] = -184.7505962980652;
out[43] = -94.615102344455636;
out[44] = 69.368640085185319;
out[45] = 17.97490384436481;
out[46] = 43.675799641021079;
out[47] = 40.748687841669565;
out[48] = 17.974903844364825;
out[49] = -69.368640085185291;
out[50] = -94.615102344455636;
out[51] = 184.7505962980652;
out[52] = 17.97490384436481;
out[53] = -69.368640085185319;
out[54] = 40.748687841669565;
out[55] = -43.675799641021079;
out[56] = -17.974903844364825;
out[57] = 69.368640085185291;
out[58] = 94.615102344455636;
out[59] = -184.7505962980652;
out[60] = -17.97490384436481;
out[61] = 69.368640085185319;
out[62] = -40.748687841669565;
out[63] = 43.675799641021079;
out[64] = -43.675799641021101;
out[65] = -40.748687841669579;
out[66] = -69.368640085185305;
out[67] = -17.974903844364821;
out[68] = 184.75059629806526;
out[69] = 94.61510234445565;
out[70] = -69.368640085185305;
out[71] = -17.974903844364821;
out[72] = 43.675799641021101;
out[73] = 40.748687841669579;
out[74] = 69.368640085185305;
out[75] = 17.974903844364821;
out[76] = -184.75059629806526;
out[77] = -94.61510234445565;
out[78] = 69.368640085185305;
out[79] = 17.974903844364821;
out[80] = 40.748687841669579;
out[81] = -43.675799641021101;
out[82] = 17.974903844364821;
out[83] = -69.368640085185305;
out[84] = -94.61510234445565;
out[85] = 184.75059629806526;
out[86] = 17.974903844364821;
out[87] = -69.368640085185305;
out[88] = -40.748687841669579;
out[89] = 43.675799641021101;
out[90] = -17.974903844364821;
out[91] = 69.368640085185305;
out[92] = 94.61510234445565;
out[93] = -184.75059629806526;
out[94] = -17.974903844364821;
out[95] = 69.368640085185305;
out[96] = -69.368640085185305;
out[97] = -17.974903844364825;
out[98] = -43.675799641021079;
out[99] = -40.74868784166955;
out[100] = -69.368640085185291;
out[101] = -17.974903844364839;
out[102] = 184.7505962980652;
out[103] = 94.61510234445565;
out[104] = 69.368640085185305;
out[105] = 17.974903844364825;
out[106] = 43.675799641021079;
out[107] = 40.74868784166955;
out[108] = 69.368640085185291;
out[109] = 17.974903844364839;
out[110] = -184.7505962980652;
out[111] = -94.61510234445565;
out[112] = 17.974903844364825;
out[113] = -69.368640085185305;
out[114] = 40.74868784166955;
out[115] = -43.675799641021079;
out[116] = 17.974903844364839;
out[117] = -69.368640085185291;
out[118] = -94.61510234445565;
out[119] = 184.7505962980652;
out[120] = -17.974903844364825;
out[121] = 69.368640085185305;
out[122] = -40.74868784166955;
out[123] = 43.675799641021079;
out[124] = -17.974903844364839;
out[125] = 69.368640085185291;
out[126] = 94.61510234445565;
out[127] = -184.7505962980652;
out[128] = 184.75059629806526;
out[129] = 94.615102344455664;
out[130] = -69.368640085185305;
out[131] = -17.974903844364835;
out[132] = -43.675799641021079;
out[133] = -40.748687841669557;
out[134] = -69.368640085185291;
out[135] = -17.974903844364821;
out[136] = -184.75059629806526;
out[137] = -94.615102344455664;
out[138] = 69.368640085185305;
out[139] = 17.974903844364835;
out[140] = 43.675799641021079;
out[141] = 40.748687841669557;
out[142] = 69.368640085185291;
out[143] = 17.974903844364821;
out[144] = -94.615102344455664;
out[145] = 184.75059629806526;
out[146] = 17.974903844364835;
out[147] = -69.368640085185305;
out[148] = 40.748687841669557;
out[149] = -43.675799641021079;
out[150] = 17.974903844364821;
out[151] = -69.368640085185291;
out[152] = 94.615102344455664;
out[153] = -184.75059629806526;
out[154] = -17.974903844364835;
out[155] = 69.368640085185305;
out[156] = -40.748687841669557;
out[157] = 43.675799641021079;
out[158] = -17.974903844364821;
out[159] = 69.368640085185291;
out[160] = -69.368640085185291;
out[161] = -17.974903844364825;
out[162] = 184.7505962980652;
out[163] = 94.615102344455636;
out[164] = -69.368640085185319;
out[165] = -17.97490384436481;
out[166] = -43.675799641021079;
out[167] = -40.748687841669565;
out[168] = 69.368640085185291;
out[169] = 17.974903844364825;
out[170] = -184.7505962980652;
out[171] = -94.615102344455636;
out[172] = 69.368640085185319;
out[173] = 17.97490384436481;
out[174] = 43.675799641021079;
out[175] = 40.748687841669565;
out[176] = 17.974903844364825;
out[177] = -69.368640085185291;
out[178] = -94.615102344455636;
out[179] = 184.7505962980652;
out[180] = 17.97490384436481;
out[181] = -69.368640085185319;
out[182] = 40.748687841669565;
out[183] = -43.675799641021079;
out[184] = -17.974903844364825;
out[185] = 69.368640085185291;
out[186] = 94.615102344455636;
out[187] = -184.7505962980652;
out[188] = -17.97490384436481;
out[189] = 69.368640085185319;
out[190] = -40.748687841669565;
out[191] = 43.675799641021079;
out[192] = -43.675799641021101;
out[193] = -40.748687841669579;
out[194] = -69.368640085185305;
out[195] = -17.974903844364821;
out[196] = 184.75059629806526;
out[197] = 94.61510234445565;
out[198] = -69.368640085185305;
out[199] = -17.974903844364821;
out[200] = 43.675799641021101;
out[201] = 40.748687841669579;
out[202] = 69.368640085185305;
out[203] = 17.974903844364821;
out[204] = -184.75059629806526;
out[205] = -94.61510234445565;
out[206] = 69.368640085185305;
out[207] = 17.974903844364821;
out[208] = 40.748687841669579;
out[209] = -43.675799641021101;
out[210] = 17.974903844364821;
out[211] = -69.368640085185305;
out[212] = -94.61510234445565;
out[213] = 184.75059629806526;
out[214] = 17.974903844364821;
out[215] = -69.368640085185305;
out[216] = -40.748687841669579;
out[217] = 43.675799641021101;
out[218] = -17.974903844364821;
out[219] = 69.368640085185305;
out[220] = 94.61510234445565;
out[221] = -184.75059629806526;
out[222] = -17.974903844364821;
out[223] = 69.368640085185305;
out[224] = -69.368640085185305;
out[225] = -17.974903844364825;
out[226] = -43.675799641021079;
out[227] = -40.74868784166955;
out[228] = -69.368640085185291;
out[229] = -17.974903844364839;
out[230] = 184.7505962980652;
out[231] = 94.61510234445565;
out[232] = 69.368640085185305;
out[233] = 17.974903844364825;
out[234] = 43.675799641021079;
out[235] = 40.74868784166955;
out[236] = 69.368640085185291;
out[237] = 17.974903844364839;
out[238] = -184.7505962980652;
out[239] = -94.61510234445565;
out[240] = 17.974903844364825;
out[241] = -69.368640085185305;
out[242] = 40.74868784166955;
out[243] = -43.675799641021079;
out[244] = 17.974903844364839;
out[245] = -69.368640085185291;
out[246] = -94.61510234445565;
out[247] = 184.7505962980652;
out[248] = -17.974903844364825;
out[249] = 69.368640085185305;
out[250] = -40.74868784166955;
out[251] = 43.675799641021079;
out[252] = -17.974903844364839;
out[253] = 69.368640085185291;
out[254] = 94.61510234445565;
out[255] = -184.7505962980652;

}

void sp_Hu_run_xy_eval(double *out,double *x,double *y,double *u,double *p,double Dt){


}

void sp_Hu_run_up_eval(double *out,double *x,double *y,double *u,double *p,double Dt){


}

void sp_Hu_run_num_eval(double *out,double *x,double *y,double *u,double *p,double Dt){

out[0] = 1.0;

}



/************************************************************/

static void *_cffi_types[] = {
/*  0 */ _CFFI_OP(_CFFI_OP_FUNCTION, 8), // void()(double *, double *, double *, double *, double *, double)
/*  1 */ _CFFI_OP(_CFFI_OP_POINTER, 6), // double *
/*  2 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/*  3 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/*  4 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/*  5 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/*  6 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 14), // double
/*  7 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/*  8 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 0), // void
};

static void _cffi_d_de_Fu_run_num_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_Fu_run_num_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_Fu_run_num_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_Fu_run_num_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_Fu_run_num_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_Fu_run_num_eval _cffi_d_de_Fu_run_num_eval
#endif

static void _cffi_d_de_Fu_run_up_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_Fu_run_up_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_Fu_run_up_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_Fu_run_up_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_Fu_run_up_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_Fu_run_up_eval _cffi_d_de_Fu_run_up_eval
#endif

static void _cffi_d_de_Fu_run_xy_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_Fu_run_xy_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_Fu_run_xy_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_Fu_run_xy_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_Fu_run_xy_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_Fu_run_xy_eval _cffi_d_de_Fu_run_xy_eval
#endif

static void _cffi_d_de_Gu_run_num_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_Gu_run_num_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_Gu_run_num_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_Gu_run_num_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_Gu_run_num_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_Gu_run_num_eval _cffi_d_de_Gu_run_num_eval
#endif

static void _cffi_d_de_Gu_run_up_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_Gu_run_up_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_Gu_run_up_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_Gu_run_up_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_Gu_run_up_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_Gu_run_up_eval _cffi_d_de_Gu_run_up_eval
#endif

static void _cffi_d_de_Gu_run_xy_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_Gu_run_xy_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_Gu_run_xy_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_Gu_run_xy_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_Gu_run_xy_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_Gu_run_xy_eval _cffi_d_de_Gu_run_xy_eval
#endif

static void _cffi_d_de_Hu_run_num_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_Hu_run_num_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_Hu_run_num_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_Hu_run_num_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_Hu_run_num_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_Hu_run_num_eval _cffi_d_de_Hu_run_num_eval
#endif

static void _cffi_d_de_Hu_run_up_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_Hu_run_up_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_Hu_run_up_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_Hu_run_up_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_Hu_run_up_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_Hu_run_up_eval _cffi_d_de_Hu_run_up_eval
#endif

static void _cffi_d_de_Hu_run_xy_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_Hu_run_xy_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_Hu_run_xy_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_Hu_run_xy_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_Hu_run_xy_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_Hu_run_xy_eval _cffi_d_de_Hu_run_xy_eval
#endif

static void _cffi_d_de_Hx_run_num_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_Hx_run_num_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_Hx_run_num_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_Hx_run_num_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_Hx_run_num_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_Hx_run_num_eval _cffi_d_de_Hx_run_num_eval
#endif

static void _cffi_d_de_Hx_run_up_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_Hx_run_up_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_Hx_run_up_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_Hx_run_up_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_Hx_run_up_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_Hx_run_up_eval _cffi_d_de_Hx_run_up_eval
#endif

static void _cffi_d_de_Hx_run_xy_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_Hx_run_xy_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_Hx_run_xy_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_Hx_run_xy_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_Hx_run_xy_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_Hx_run_xy_eval _cffi_d_de_Hx_run_xy_eval
#endif

static void _cffi_d_de_Hy_run_num_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_Hy_run_num_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_Hy_run_num_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_Hy_run_num_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_Hy_run_num_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_Hy_run_num_eval _cffi_d_de_Hy_run_num_eval
#endif

static void _cffi_d_de_Hy_run_up_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_Hy_run_up_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_Hy_run_up_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_Hy_run_up_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_Hy_run_up_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_Hy_run_up_eval _cffi_d_de_Hy_run_up_eval
#endif

static void _cffi_d_de_Hy_run_xy_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_Hy_run_xy_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_Hy_run_xy_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_Hy_run_xy_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_Hy_run_xy_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_Hy_run_xy_eval _cffi_d_de_Hy_run_xy_eval
#endif

static void _cffi_d_de_jac_ini_num_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_jac_ini_num_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_jac_ini_num_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_jac_ini_num_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_jac_ini_num_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_jac_ini_num_eval _cffi_d_de_jac_ini_num_eval
#endif

static void _cffi_d_de_jac_ini_up_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_jac_ini_up_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_jac_ini_up_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_jac_ini_up_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_jac_ini_up_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_jac_ini_up_eval _cffi_d_de_jac_ini_up_eval
#endif

static void _cffi_d_de_jac_ini_xy_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_jac_ini_xy_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_jac_ini_xy_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_jac_ini_xy_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_jac_ini_xy_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_jac_ini_xy_eval _cffi_d_de_jac_ini_xy_eval
#endif

static void _cffi_d_de_jac_run_num_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_jac_run_num_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_jac_run_num_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_jac_run_num_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_jac_run_num_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_jac_run_num_eval _cffi_d_de_jac_run_num_eval
#endif

static void _cffi_d_de_jac_run_up_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_jac_run_up_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_jac_run_up_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_jac_run_up_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_jac_run_up_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_jac_run_up_eval _cffi_d_de_jac_run_up_eval
#endif

static void _cffi_d_de_jac_run_xy_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_jac_run_xy_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_jac_run_xy_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_jac_run_xy_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_jac_run_xy_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_jac_run_xy_eval _cffi_d_de_jac_run_xy_eval
#endif

static void _cffi_d_de_jac_trap_num_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_jac_trap_num_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_jac_trap_num_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_jac_trap_num_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_jac_trap_num_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_jac_trap_num_eval _cffi_d_de_jac_trap_num_eval
#endif

static void _cffi_d_de_jac_trap_up_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_jac_trap_up_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_jac_trap_up_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_jac_trap_up_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_jac_trap_up_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_jac_trap_up_eval _cffi_d_de_jac_trap_up_eval
#endif

static void _cffi_d_de_jac_trap_xy_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_jac_trap_xy_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_jac_trap_xy_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_jac_trap_xy_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_jac_trap_xy_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_jac_trap_xy_eval _cffi_d_de_jac_trap_xy_eval
#endif

static void _cffi_d_f_ini_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  f_ini_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_f_ini_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "f_ini_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { f_ini_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_f_ini_eval _cffi_d_f_ini_eval
#endif

static void _cffi_d_f_run_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  f_run_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_f_run_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "f_run_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { f_run_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_f_run_eval _cffi_d_f_run_eval
#endif

static void _cffi_d_g_ini_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  g_ini_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_g_ini_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "g_ini_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { g_ini_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_g_ini_eval _cffi_d_g_ini_eval
#endif

static void _cffi_d_g_run_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  g_run_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_g_run_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "g_run_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { g_run_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_g_run_eval _cffi_d_g_run_eval
#endif

static void _cffi_d_h_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  h_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_h_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "h_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { h_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_h_eval _cffi_d_h_eval
#endif

static void _cffi_d_sp_Fu_run_num_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  sp_Fu_run_num_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_sp_Fu_run_num_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "sp_Fu_run_num_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { sp_Fu_run_num_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_sp_Fu_run_num_eval _cffi_d_sp_Fu_run_num_eval
#endif

static void _cffi_d_sp_Fu_run_up_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  sp_Fu_run_up_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_sp_Fu_run_up_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "sp_Fu_run_up_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { sp_Fu_run_up_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_sp_Fu_run_up_eval _cffi_d_sp_Fu_run_up_eval
#endif

static void _cffi_d_sp_Fu_run_xy_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  sp_Fu_run_xy_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_sp_Fu_run_xy_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "sp_Fu_run_xy_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { sp_Fu_run_xy_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_sp_Fu_run_xy_eval _cffi_d_sp_Fu_run_xy_eval
#endif

static void _cffi_d_sp_Gu_run_num_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  sp_Gu_run_num_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_sp_Gu_run_num_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "sp_Gu_run_num_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { sp_Gu_run_num_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_sp_Gu_run_num_eval _cffi_d_sp_Gu_run_num_eval
#endif

static void _cffi_d_sp_Gu_run_up_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  sp_Gu_run_up_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_sp_Gu_run_up_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "sp_Gu_run_up_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { sp_Gu_run_up_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_sp_Gu_run_up_eval _cffi_d_sp_Gu_run_up_eval
#endif

static void _cffi_d_sp_Gu_run_xy_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  sp_Gu_run_xy_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_sp_Gu_run_xy_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "sp_Gu_run_xy_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { sp_Gu_run_xy_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_sp_Gu_run_xy_eval _cffi_d_sp_Gu_run_xy_eval
#endif

static void _cffi_d_sp_Hu_run_num_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  sp_Hu_run_num_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_sp_Hu_run_num_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "sp_Hu_run_num_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { sp_Hu_run_num_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_sp_Hu_run_num_eval _cffi_d_sp_Hu_run_num_eval
#endif

static void _cffi_d_sp_Hu_run_up_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  sp_Hu_run_up_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_sp_Hu_run_up_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "sp_Hu_run_up_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { sp_Hu_run_up_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_sp_Hu_run_up_eval _cffi_d_sp_Hu_run_up_eval
#endif

static void _cffi_d_sp_Hu_run_xy_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  sp_Hu_run_xy_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_sp_Hu_run_xy_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "sp_Hu_run_xy_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { sp_Hu_run_xy_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_sp_Hu_run_xy_eval _cffi_d_sp_Hu_run_xy_eval
#endif

static void _cffi_d_sp_Hx_run_num_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  sp_Hx_run_num_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_sp_Hx_run_num_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "sp_Hx_run_num_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { sp_Hx_run_num_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_sp_Hx_run_num_eval _cffi_d_sp_Hx_run_num_eval
#endif

static void _cffi_d_sp_Hx_run_up_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  sp_Hx_run_up_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_sp_Hx_run_up_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "sp_Hx_run_up_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { sp_Hx_run_up_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_sp_Hx_run_up_eval _cffi_d_sp_Hx_run_up_eval
#endif

static void _cffi_d_sp_Hx_run_xy_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  sp_Hx_run_xy_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_sp_Hx_run_xy_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "sp_Hx_run_xy_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { sp_Hx_run_xy_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_sp_Hx_run_xy_eval _cffi_d_sp_Hx_run_xy_eval
#endif

static void _cffi_d_sp_Hy_run_num_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  sp_Hy_run_num_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_sp_Hy_run_num_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "sp_Hy_run_num_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { sp_Hy_run_num_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_sp_Hy_run_num_eval _cffi_d_sp_Hy_run_num_eval
#endif

static void _cffi_d_sp_Hy_run_up_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  sp_Hy_run_up_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_sp_Hy_run_up_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "sp_Hy_run_up_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { sp_Hy_run_up_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_sp_Hy_run_up_eval _cffi_d_sp_Hy_run_up_eval
#endif

static void _cffi_d_sp_Hy_run_xy_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  sp_Hy_run_xy_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_sp_Hy_run_xy_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "sp_Hy_run_xy_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { sp_Hy_run_xy_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_sp_Hy_run_xy_eval _cffi_d_sp_Hy_run_xy_eval
#endif

static void _cffi_d_sp_jac_ini_num_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  sp_jac_ini_num_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_sp_jac_ini_num_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "sp_jac_ini_num_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { sp_jac_ini_num_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_sp_jac_ini_num_eval _cffi_d_sp_jac_ini_num_eval
#endif

static void _cffi_d_sp_jac_ini_up_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  sp_jac_ini_up_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_sp_jac_ini_up_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "sp_jac_ini_up_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { sp_jac_ini_up_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_sp_jac_ini_up_eval _cffi_d_sp_jac_ini_up_eval
#endif

static void _cffi_d_sp_jac_ini_xy_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  sp_jac_ini_xy_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_sp_jac_ini_xy_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "sp_jac_ini_xy_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { sp_jac_ini_xy_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_sp_jac_ini_xy_eval _cffi_d_sp_jac_ini_xy_eval
#endif

static void _cffi_d_sp_jac_run_num_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  sp_jac_run_num_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_sp_jac_run_num_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "sp_jac_run_num_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { sp_jac_run_num_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_sp_jac_run_num_eval _cffi_d_sp_jac_run_num_eval
#endif

static void _cffi_d_sp_jac_run_up_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  sp_jac_run_up_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_sp_jac_run_up_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "sp_jac_run_up_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { sp_jac_run_up_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_sp_jac_run_up_eval _cffi_d_sp_jac_run_up_eval
#endif

static void _cffi_d_sp_jac_run_xy_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  sp_jac_run_xy_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_sp_jac_run_xy_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "sp_jac_run_xy_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { sp_jac_run_xy_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_sp_jac_run_xy_eval _cffi_d_sp_jac_run_xy_eval
#endif

static void _cffi_d_sp_jac_trap_num_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  sp_jac_trap_num_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_sp_jac_trap_num_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "sp_jac_trap_num_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { sp_jac_trap_num_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_sp_jac_trap_num_eval _cffi_d_sp_jac_trap_num_eval
#endif

static void _cffi_d_sp_jac_trap_up_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  sp_jac_trap_up_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_sp_jac_trap_up_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "sp_jac_trap_up_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { sp_jac_trap_up_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_sp_jac_trap_up_eval _cffi_d_sp_jac_trap_up_eval
#endif

static void _cffi_d_sp_jac_trap_xy_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  sp_jac_trap_xy_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_sp_jac_trap_xy_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "sp_jac_trap_xy_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { sp_jac_trap_xy_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_sp_jac_trap_xy_eval _cffi_d_sp_jac_trap_xy_eval
#endif

static const struct _cffi_global_s _cffi_globals[] = {
  { "de_Fu_run_num_eval", (void *)_cffi_f_de_Fu_run_num_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_Fu_run_num_eval },
  { "de_Fu_run_up_eval", (void *)_cffi_f_de_Fu_run_up_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_Fu_run_up_eval },
  { "de_Fu_run_xy_eval", (void *)_cffi_f_de_Fu_run_xy_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_Fu_run_xy_eval },
  { "de_Gu_run_num_eval", (void *)_cffi_f_de_Gu_run_num_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_Gu_run_num_eval },
  { "de_Gu_run_up_eval", (void *)_cffi_f_de_Gu_run_up_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_Gu_run_up_eval },
  { "de_Gu_run_xy_eval", (void *)_cffi_f_de_Gu_run_xy_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_Gu_run_xy_eval },
  { "de_Hu_run_num_eval", (void *)_cffi_f_de_Hu_run_num_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_Hu_run_num_eval },
  { "de_Hu_run_up_eval", (void *)_cffi_f_de_Hu_run_up_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_Hu_run_up_eval },
  { "de_Hu_run_xy_eval", (void *)_cffi_f_de_Hu_run_xy_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_Hu_run_xy_eval },
  { "de_Hx_run_num_eval", (void *)_cffi_f_de_Hx_run_num_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_Hx_run_num_eval },
  { "de_Hx_run_up_eval", (void *)_cffi_f_de_Hx_run_up_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_Hx_run_up_eval },
  { "de_Hx_run_xy_eval", (void *)_cffi_f_de_Hx_run_xy_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_Hx_run_xy_eval },
  { "de_Hy_run_num_eval", (void *)_cffi_f_de_Hy_run_num_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_Hy_run_num_eval },
  { "de_Hy_run_up_eval", (void *)_cffi_f_de_Hy_run_up_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_Hy_run_up_eval },
  { "de_Hy_run_xy_eval", (void *)_cffi_f_de_Hy_run_xy_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_Hy_run_xy_eval },
  { "de_jac_ini_num_eval", (void *)_cffi_f_de_jac_ini_num_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_jac_ini_num_eval },
  { "de_jac_ini_up_eval", (void *)_cffi_f_de_jac_ini_up_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_jac_ini_up_eval },
  { "de_jac_ini_xy_eval", (void *)_cffi_f_de_jac_ini_xy_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_jac_ini_xy_eval },
  { "de_jac_run_num_eval", (void *)_cffi_f_de_jac_run_num_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_jac_run_num_eval },
  { "de_jac_run_up_eval", (void *)_cffi_f_de_jac_run_up_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_jac_run_up_eval },
  { "de_jac_run_xy_eval", (void *)_cffi_f_de_jac_run_xy_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_jac_run_xy_eval },
  { "de_jac_trap_num_eval", (void *)_cffi_f_de_jac_trap_num_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_jac_trap_num_eval },
  { "de_jac_trap_up_eval", (void *)_cffi_f_de_jac_trap_up_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_jac_trap_up_eval },
  { "de_jac_trap_xy_eval", (void *)_cffi_f_de_jac_trap_xy_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_jac_trap_xy_eval },
  { "f_ini_eval", (void *)_cffi_f_f_ini_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_f_ini_eval },
  { "f_run_eval", (void *)_cffi_f_f_run_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_f_run_eval },
  { "g_ini_eval", (void *)_cffi_f_g_ini_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_g_ini_eval },
  { "g_run_eval", (void *)_cffi_f_g_run_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_g_run_eval },
  { "h_eval", (void *)_cffi_f_h_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_h_eval },
  { "sp_Fu_run_num_eval", (void *)_cffi_f_sp_Fu_run_num_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_sp_Fu_run_num_eval },
  { "sp_Fu_run_up_eval", (void *)_cffi_f_sp_Fu_run_up_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_sp_Fu_run_up_eval },
  { "sp_Fu_run_xy_eval", (void *)_cffi_f_sp_Fu_run_xy_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_sp_Fu_run_xy_eval },
  { "sp_Gu_run_num_eval", (void *)_cffi_f_sp_Gu_run_num_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_sp_Gu_run_num_eval },
  { "sp_Gu_run_up_eval", (void *)_cffi_f_sp_Gu_run_up_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_sp_Gu_run_up_eval },
  { "sp_Gu_run_xy_eval", (void *)_cffi_f_sp_Gu_run_xy_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_sp_Gu_run_xy_eval },
  { "sp_Hu_run_num_eval", (void *)_cffi_f_sp_Hu_run_num_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_sp_Hu_run_num_eval },
  { "sp_Hu_run_up_eval", (void *)_cffi_f_sp_Hu_run_up_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_sp_Hu_run_up_eval },
  { "sp_Hu_run_xy_eval", (void *)_cffi_f_sp_Hu_run_xy_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_sp_Hu_run_xy_eval },
  { "sp_Hx_run_num_eval", (void *)_cffi_f_sp_Hx_run_num_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_sp_Hx_run_num_eval },
  { "sp_Hx_run_up_eval", (void *)_cffi_f_sp_Hx_run_up_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_sp_Hx_run_up_eval },
  { "sp_Hx_run_xy_eval", (void *)_cffi_f_sp_Hx_run_xy_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_sp_Hx_run_xy_eval },
  { "sp_Hy_run_num_eval", (void *)_cffi_f_sp_Hy_run_num_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_sp_Hy_run_num_eval },
  { "sp_Hy_run_up_eval", (void *)_cffi_f_sp_Hy_run_up_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_sp_Hy_run_up_eval },
  { "sp_Hy_run_xy_eval", (void *)_cffi_f_sp_Hy_run_xy_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_sp_Hy_run_xy_eval },
  { "sp_jac_ini_num_eval", (void *)_cffi_f_sp_jac_ini_num_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_sp_jac_ini_num_eval },
  { "sp_jac_ini_up_eval", (void *)_cffi_f_sp_jac_ini_up_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_sp_jac_ini_up_eval },
  { "sp_jac_ini_xy_eval", (void *)_cffi_f_sp_jac_ini_xy_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_sp_jac_ini_xy_eval },
  { "sp_jac_run_num_eval", (void *)_cffi_f_sp_jac_run_num_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_sp_jac_run_num_eval },
  { "sp_jac_run_up_eval", (void *)_cffi_f_sp_jac_run_up_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_sp_jac_run_up_eval },
  { "sp_jac_run_xy_eval", (void *)_cffi_f_sp_jac_run_xy_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_sp_jac_run_xy_eval },
  { "sp_jac_trap_num_eval", (void *)_cffi_f_sp_jac_trap_num_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_sp_jac_trap_num_eval },
  { "sp_jac_trap_up_eval", (void *)_cffi_f_sp_jac_trap_up_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_sp_jac_trap_up_eval },
  { "sp_jac_trap_xy_eval", (void *)_cffi_f_sp_jac_trap_xy_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_sp_jac_trap_xy_eval },
};

static const struct _cffi_type_context_s _cffi_type_context = {
  _cffi_types,
  _cffi_globals,
  NULL,  /* no fields */
  NULL,  /* no struct_unions */
  NULL,  /* no enums */
  NULL,  /* no typenames */
  53,  /* num_globals */
  0,  /* num_struct_unions */
  0,  /* num_enums */
  0,  /* num_typenames */
  NULL,  /* no includes */
  9,  /* num_types */
  0,  /* flags */
};

#ifdef __GNUC__
#  pragma GCC visibility push(default)  /* for -fvisibility= */
#endif

#ifdef PYPY_VERSION
PyMODINIT_FUNC
_cffi_pypyinit_temp_cffi(const void *p[])
{
    p[0] = (const void *)0x2601;
    p[1] = &_cffi_type_context;
#if PY_MAJOR_VERSION >= 3
    return NULL;
#endif
}
#  ifdef _MSC_VER
     PyMODINIT_FUNC
#  if PY_MAJOR_VERSION >= 3
     PyInit_temp_cffi(void) { return NULL; }
#  else
     inittemp_cffi(void) { }
#  endif
#  endif
#elif PY_MAJOR_VERSION >= 3
PyMODINIT_FUNC
PyInit_temp_cffi(void)
{
  return _cffi_init("temp_cffi", 0x2601, &_cffi_type_context);
}
#else
PyMODINIT_FUNC
inittemp_cffi(void)
{
  _cffi_init("temp_cffi", 0x2601, &_cffi_type_context);
}
#endif

#ifdef __GNUC__
#  pragma GCC visibility pop
#endif
