#define _CFFI_

/* We try to define Py_LIMITED_API before including Python.h.

   Mess: we can only define it if Py_DEBUG, Py_TRACE_REFS and
   Py_REF_DEBUG are not defined.  This is a best-effort approximation:
   we can learn about Py_DEBUG from pyconfig.h, but it is unclear if
   the same works for the other two macros.  Py_DEBUG implies them,
   but not the other way around.

   The implementation is messy (issue #350): on Windows, with _MSC_VER,
   we have to define Py_LIMITED_API even before including pyconfig.h.
   In that case, we guess what pyconfig.h will do to the macros above,
   and check our guess after the #include.

   Note that on Windows, with CPython 3.x, you need >= 3.5 and virtualenv
   version >= 16.0.0.  With older versions of either, you don't get a
   copy of PYTHON3.DLL in the virtualenv.  We can't check the version of
   CPython *before* we even include pyconfig.h.  ffi.set_source() puts
   a ``#define _CFFI_NO_LIMITED_API'' at the start of this file if it is
   running on Windows < 3.5, as an attempt at fixing it, but that's
   arguably wrong because it may not be the target version of Python.
   Still better than nothing I guess.  As another workaround, you can
   remove the definition of Py_LIMITED_API here.

   See also 'py_limited_api' in cffi/setuptools_ext.py.
*/
#if !defined(_CFFI_USE_EMBEDDING) && !defined(Py_LIMITED_API)
#  ifdef _MSC_VER
#    if !defined(_DEBUG) && !defined(Py_DEBUG) && !defined(Py_TRACE_REFS) && !defined(Py_REF_DEBUG) && !defined(_CFFI_NO_LIMITED_API)
#      define Py_LIMITED_API
#    endif
#    include <pyconfig.h>
     /* sanity-check: Py_LIMITED_API will cause crashes if any of these
        are also defined.  Normally, the Python file PC/pyconfig.h does not
        cause any of these to be defined, with the exception that _DEBUG
        causes Py_DEBUG.  Double-check that. */
#    ifdef Py_LIMITED_API
#      if defined(Py_DEBUG)
#        error "pyconfig.h unexpectedly defines Py_DEBUG, but Py_LIMITED_API is set"
#      endif
#      if defined(Py_TRACE_REFS)
#        error "pyconfig.h unexpectedly defines Py_TRACE_REFS, but Py_LIMITED_API is set"
#      endif
#      if defined(Py_REF_DEBUG)
#        error "pyconfig.h unexpectedly defines Py_REF_DEBUG, but Py_LIMITED_API is set"
#      endif
#    endif
#  else
#    include <pyconfig.h>
#    if !defined(Py_DEBUG) && !defined(Py_TRACE_REFS) && !defined(Py_REF_DEBUG) && !defined(_CFFI_NO_LIMITED_API)
#      define Py_LIMITED_API
#    endif
#  endif
#endif

#include <Python.h>
#ifdef __cplusplus
extern "C" {
#endif
#include <stddef.h>

/* This part is from file 'cffi/parse_c_type.h'.  It is copied at the
   beginning of C sources generated by CFFI's ffi.set_source(). */

typedef void *_cffi_opcode_t;

#define _CFFI_OP(opcode, arg)   (_cffi_opcode_t)(opcode | (((uintptr_t)(arg)) << 8))
#define _CFFI_GETOP(cffi_opcode)    ((unsigned char)(uintptr_t)cffi_opcode)
#define _CFFI_GETARG(cffi_opcode)   (((intptr_t)cffi_opcode) >> 8)

#define _CFFI_OP_PRIMITIVE       1
#define _CFFI_OP_POINTER         3
#define _CFFI_OP_ARRAY           5
#define _CFFI_OP_OPEN_ARRAY      7
#define _CFFI_OP_STRUCT_UNION    9
#define _CFFI_OP_ENUM           11
#define _CFFI_OP_FUNCTION       13
#define _CFFI_OP_FUNCTION_END   15
#define _CFFI_OP_NOOP           17
#define _CFFI_OP_BITFIELD       19
#define _CFFI_OP_TYPENAME       21
#define _CFFI_OP_CPYTHON_BLTN_V 23   // varargs
#define _CFFI_OP_CPYTHON_BLTN_N 25   // noargs
#define _CFFI_OP_CPYTHON_BLTN_O 27   // O  (i.e. a single arg)
#define _CFFI_OP_CONSTANT       29
#define _CFFI_OP_CONSTANT_INT   31
#define _CFFI_OP_GLOBAL_VAR     33
#define _CFFI_OP_DLOPEN_FUNC    35
#define _CFFI_OP_DLOPEN_CONST   37
#define _CFFI_OP_GLOBAL_VAR_F   39
#define _CFFI_OP_EXTERN_PYTHON  41

#define _CFFI_PRIM_VOID          0
#define _CFFI_PRIM_BOOL          1
#define _CFFI_PRIM_CHAR          2
#define _CFFI_PRIM_SCHAR         3
#define _CFFI_PRIM_UCHAR         4
#define _CFFI_PRIM_SHORT         5
#define _CFFI_PRIM_USHORT        6
#define _CFFI_PRIM_INT           7
#define _CFFI_PRIM_UINT          8
#define _CFFI_PRIM_LONG          9
#define _CFFI_PRIM_ULONG        10
#define _CFFI_PRIM_LONGLONG     11
#define _CFFI_PRIM_ULONGLONG    12
#define _CFFI_PRIM_FLOAT        13
#define _CFFI_PRIM_DOUBLE       14
#define _CFFI_PRIM_LONGDOUBLE   15

#define _CFFI_PRIM_WCHAR        16
#define _CFFI_PRIM_INT8         17
#define _CFFI_PRIM_UINT8        18
#define _CFFI_PRIM_INT16        19
#define _CFFI_PRIM_UINT16       20
#define _CFFI_PRIM_INT32        21
#define _CFFI_PRIM_UINT32       22
#define _CFFI_PRIM_INT64        23
#define _CFFI_PRIM_UINT64       24
#define _CFFI_PRIM_INTPTR       25
#define _CFFI_PRIM_UINTPTR      26
#define _CFFI_PRIM_PTRDIFF      27
#define _CFFI_PRIM_SIZE         28
#define _CFFI_PRIM_SSIZE        29
#define _CFFI_PRIM_INT_LEAST8   30
#define _CFFI_PRIM_UINT_LEAST8  31
#define _CFFI_PRIM_INT_LEAST16  32
#define _CFFI_PRIM_UINT_LEAST16 33
#define _CFFI_PRIM_INT_LEAST32  34
#define _CFFI_PRIM_UINT_LEAST32 35
#define _CFFI_PRIM_INT_LEAST64  36
#define _CFFI_PRIM_UINT_LEAST64 37
#define _CFFI_PRIM_INT_FAST8    38
#define _CFFI_PRIM_UINT_FAST8   39
#define _CFFI_PRIM_INT_FAST16   40
#define _CFFI_PRIM_UINT_FAST16  41
#define _CFFI_PRIM_INT_FAST32   42
#define _CFFI_PRIM_UINT_FAST32  43
#define _CFFI_PRIM_INT_FAST64   44
#define _CFFI_PRIM_UINT_FAST64  45
#define _CFFI_PRIM_INTMAX       46
#define _CFFI_PRIM_UINTMAX      47
#define _CFFI_PRIM_FLOATCOMPLEX 48
#define _CFFI_PRIM_DOUBLECOMPLEX 49
#define _CFFI_PRIM_CHAR16       50
#define _CFFI_PRIM_CHAR32       51

#define _CFFI__NUM_PRIM         52
#define _CFFI__UNKNOWN_PRIM           (-1)
#define _CFFI__UNKNOWN_FLOAT_PRIM     (-2)
#define _CFFI__UNKNOWN_LONG_DOUBLE    (-3)

#define _CFFI__IO_FILE_STRUCT         (-1)


struct _cffi_global_s {
    const char *name;
    void *address;
    _cffi_opcode_t type_op;
    void *size_or_direct_fn;  // OP_GLOBAL_VAR: size, or 0 if unknown
                              // OP_CPYTHON_BLTN_*: addr of direct function
};

struct _cffi_getconst_s {
    unsigned long long value;
    const struct _cffi_type_context_s *ctx;
    int gindex;
};

struct _cffi_struct_union_s {
    const char *name;
    int type_index;          // -> _cffi_types, on a OP_STRUCT_UNION
    int flags;               // _CFFI_F_* flags below
    size_t size;
    int alignment;
    int first_field_index;   // -> _cffi_fields array
    int num_fields;
};
#define _CFFI_F_UNION         0x01   // is a union, not a struct
#define _CFFI_F_CHECK_FIELDS  0x02   // complain if fields are not in the
                                     // "standard layout" or if some are missing
#define _CFFI_F_PACKED        0x04   // for CHECK_FIELDS, assume a packed struct
#define _CFFI_F_EXTERNAL      0x08   // in some other ffi.include()
#define _CFFI_F_OPAQUE        0x10   // opaque

struct _cffi_field_s {
    const char *name;
    size_t field_offset;
    size_t field_size;
    _cffi_opcode_t field_type_op;
};

struct _cffi_enum_s {
    const char *name;
    int type_index;          // -> _cffi_types, on a OP_ENUM
    int type_prim;           // _CFFI_PRIM_xxx
    const char *enumerators; // comma-delimited string
};

struct _cffi_typename_s {
    const char *name;
    int type_index;   /* if opaque, points to a possibly artificial
                         OP_STRUCT which is itself opaque */
};

struct _cffi_type_context_s {
    _cffi_opcode_t *types;
    const struct _cffi_global_s *globals;
    const struct _cffi_field_s *fields;
    const struct _cffi_struct_union_s *struct_unions;
    const struct _cffi_enum_s *enums;
    const struct _cffi_typename_s *typenames;
    int num_globals;
    int num_struct_unions;
    int num_enums;
    int num_typenames;
    const char *const *includes;
    int num_types;
    int flags;      /* future extension */
};

struct _cffi_parse_info_s {
    const struct _cffi_type_context_s *ctx;
    _cffi_opcode_t *output;
    unsigned int output_size;
    size_t error_location;
    const char *error_message;
};

struct _cffi_externpy_s {
    const char *name;
    size_t size_of_result;
    void *reserved1, *reserved2;
};

#ifdef _CFFI_INTERNAL
static int parse_c_type(struct _cffi_parse_info_s *info, const char *input);
static int search_in_globals(const struct _cffi_type_context_s *ctx,
                             const char *search, size_t search_len);
static int search_in_struct_unions(const struct _cffi_type_context_s *ctx,
                                   const char *search, size_t search_len);
#endif

/* this block of #ifs should be kept exactly identical between
   c/_cffi_backend.c, cffi/vengine_cpy.py, cffi/vengine_gen.py
   and cffi/_cffi_include.h */
#if defined(_MSC_VER)
# include <malloc.h>   /* for alloca() */
# if _MSC_VER < 1600   /* MSVC < 2010 */
   typedef __int8 int8_t;
   typedef __int16 int16_t;
   typedef __int32 int32_t;
   typedef __int64 int64_t;
   typedef unsigned __int8 uint8_t;
   typedef unsigned __int16 uint16_t;
   typedef unsigned __int32 uint32_t;
   typedef unsigned __int64 uint64_t;
   typedef __int8 int_least8_t;
   typedef __int16 int_least16_t;
   typedef __int32 int_least32_t;
   typedef __int64 int_least64_t;
   typedef unsigned __int8 uint_least8_t;
   typedef unsigned __int16 uint_least16_t;
   typedef unsigned __int32 uint_least32_t;
   typedef unsigned __int64 uint_least64_t;
   typedef __int8 int_fast8_t;
   typedef __int16 int_fast16_t;
   typedef __int32 int_fast32_t;
   typedef __int64 int_fast64_t;
   typedef unsigned __int8 uint_fast8_t;
   typedef unsigned __int16 uint_fast16_t;
   typedef unsigned __int32 uint_fast32_t;
   typedef unsigned __int64 uint_fast64_t;
   typedef __int64 intmax_t;
   typedef unsigned __int64 uintmax_t;
# else
#  include <stdint.h>
# endif
# if _MSC_VER < 1800   /* MSVC < 2013 */
#  ifndef __cplusplus
    typedef unsigned char _Bool;
#  endif
# endif
#else
# include <stdint.h>
# if (defined (__SVR4) && defined (__sun)) || defined(_AIX) || defined(__hpux)
#  include <alloca.h>
# endif
#endif

#ifdef __GNUC__
# define _CFFI_UNUSED_FN  __attribute__((unused))
#else
# define _CFFI_UNUSED_FN  /* nothing */
#endif

#ifdef __cplusplus
# ifndef _Bool
   typedef bool _Bool;   /* semi-hackish: C++ has no _Bool; bool is builtin */
# endif
#endif

/**********  CPython-specific section  **********/
#ifndef PYPY_VERSION


#if PY_MAJOR_VERSION >= 3
# define PyInt_FromLong PyLong_FromLong
#endif

#define _cffi_from_c_double PyFloat_FromDouble
#define _cffi_from_c_float PyFloat_FromDouble
#define _cffi_from_c_long PyInt_FromLong
#define _cffi_from_c_ulong PyLong_FromUnsignedLong
#define _cffi_from_c_longlong PyLong_FromLongLong
#define _cffi_from_c_ulonglong PyLong_FromUnsignedLongLong
#define _cffi_from_c__Bool PyBool_FromLong

#define _cffi_to_c_double PyFloat_AsDouble
#define _cffi_to_c_float PyFloat_AsDouble

#define _cffi_from_c_int(x, type)                                        \
    (((type)-1) > 0 ? /* unsigned */                                     \
        (sizeof(type) < sizeof(long) ?                                   \
            PyInt_FromLong((long)x) :                                    \
         sizeof(type) == sizeof(long) ?                                  \
            PyLong_FromUnsignedLong((unsigned long)x) :                  \
            PyLong_FromUnsignedLongLong((unsigned long long)x)) :        \
        (sizeof(type) <= sizeof(long) ?                                  \
            PyInt_FromLong((long)x) :                                    \
            PyLong_FromLongLong((long long)x)))

#define _cffi_to_c_int(o, type)                                          \
    ((type)(                                                             \
     sizeof(type) == 1 ? (((type)-1) > 0 ? (type)_cffi_to_c_u8(o)        \
                                         : (type)_cffi_to_c_i8(o)) :     \
     sizeof(type) == 2 ? (((type)-1) > 0 ? (type)_cffi_to_c_u16(o)       \
                                         : (type)_cffi_to_c_i16(o)) :    \
     sizeof(type) == 4 ? (((type)-1) > 0 ? (type)_cffi_to_c_u32(o)       \
                                         : (type)_cffi_to_c_i32(o)) :    \
     sizeof(type) == 8 ? (((type)-1) > 0 ? (type)_cffi_to_c_u64(o)       \
                                         : (type)_cffi_to_c_i64(o)) :    \
     (Py_FatalError("unsupported size for type " #type), (type)0)))

#define _cffi_to_c_i8                                                    \
                 ((int(*)(PyObject *))_cffi_exports[1])
#define _cffi_to_c_u8                                                    \
                 ((int(*)(PyObject *))_cffi_exports[2])
#define _cffi_to_c_i16                                                   \
                 ((int(*)(PyObject *))_cffi_exports[3])
#define _cffi_to_c_u16                                                   \
                 ((int(*)(PyObject *))_cffi_exports[4])
#define _cffi_to_c_i32                                                   \
                 ((int(*)(PyObject *))_cffi_exports[5])
#define _cffi_to_c_u32                                                   \
                 ((unsigned int(*)(PyObject *))_cffi_exports[6])
#define _cffi_to_c_i64                                                   \
                 ((long long(*)(PyObject *))_cffi_exports[7])
#define _cffi_to_c_u64                                                   \
                 ((unsigned long long(*)(PyObject *))_cffi_exports[8])
#define _cffi_to_c_char                                                  \
                 ((int(*)(PyObject *))_cffi_exports[9])
#define _cffi_from_c_pointer                                             \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[10])
#define _cffi_to_c_pointer                                               \
    ((char *(*)(PyObject *, struct _cffi_ctypedescr *))_cffi_exports[11])
#define _cffi_get_struct_layout                                          \
    not used any more
#define _cffi_restore_errno                                              \
    ((void(*)(void))_cffi_exports[13])
#define _cffi_save_errno                                                 \
    ((void(*)(void))_cffi_exports[14])
#define _cffi_from_c_char                                                \
    ((PyObject *(*)(char))_cffi_exports[15])
#define _cffi_from_c_deref                                               \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[16])
#define _cffi_to_c                                                       \
    ((int(*)(char *, struct _cffi_ctypedescr *, PyObject *))_cffi_exports[17])
#define _cffi_from_c_struct                                              \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[18])
#define _cffi_to_c_wchar_t                                               \
    ((_cffi_wchar_t(*)(PyObject *))_cffi_exports[19])
#define _cffi_from_c_wchar_t                                             \
    ((PyObject *(*)(_cffi_wchar_t))_cffi_exports[20])
#define _cffi_to_c_long_double                                           \
    ((long double(*)(PyObject *))_cffi_exports[21])
#define _cffi_to_c__Bool                                                 \
    ((_Bool(*)(PyObject *))_cffi_exports[22])
#define _cffi_prepare_pointer_call_argument                              \
    ((Py_ssize_t(*)(struct _cffi_ctypedescr *,                           \
                    PyObject *, char **))_cffi_exports[23])
#define _cffi_convert_array_from_object                                  \
    ((int(*)(char *, struct _cffi_ctypedescr *, PyObject *))_cffi_exports[24])
#define _CFFI_CPIDX  25
#define _cffi_call_python                                                \
    ((void(*)(struct _cffi_externpy_s *, char *))_cffi_exports[_CFFI_CPIDX])
#define _cffi_to_c_wchar3216_t                                           \
    ((int(*)(PyObject *))_cffi_exports[26])
#define _cffi_from_c_wchar3216_t                                         \
    ((PyObject *(*)(int))_cffi_exports[27])
#define _CFFI_NUM_EXPORTS 28

struct _cffi_ctypedescr;

static void *_cffi_exports[_CFFI_NUM_EXPORTS];

#define _cffi_type(index)   (                           \
    assert((((uintptr_t)_cffi_types[index]) & 1) == 0), \
    (struct _cffi_ctypedescr *)_cffi_types[index])

static PyObject *_cffi_init(const char *module_name, Py_ssize_t version,
                            const struct _cffi_type_context_s *ctx)
{
    PyObject *module, *o_arg, *new_module;
    void *raw[] = {
        (void *)module_name,
        (void *)version,
        (void *)_cffi_exports,
        (void *)ctx,
    };

    module = PyImport_ImportModule("_cffi_backend");
    if (module == NULL)
        goto failure;

    o_arg = PyLong_FromVoidPtr((void *)raw);
    if (o_arg == NULL)
        goto failure;

    new_module = PyObject_CallMethod(
        module, (char *)"_init_cffi_1_0_external_module", (char *)"O", o_arg);

    Py_DECREF(o_arg);
    Py_DECREF(module);
    return new_module;

  failure:
    Py_XDECREF(module);
    return NULL;
}


#ifdef HAVE_WCHAR_H
typedef wchar_t _cffi_wchar_t;
#else
typedef uint16_t _cffi_wchar_t;   /* same random pick as _cffi_backend.c */
#endif

_CFFI_UNUSED_FN static uint16_t _cffi_to_c_char16_t(PyObject *o)
{
    if (sizeof(_cffi_wchar_t) == 2)
        return (uint16_t)_cffi_to_c_wchar_t(o);
    else
        return (uint16_t)_cffi_to_c_wchar3216_t(o);
}

_CFFI_UNUSED_FN static PyObject *_cffi_from_c_char16_t(uint16_t x)
{
    if (sizeof(_cffi_wchar_t) == 2)
        return _cffi_from_c_wchar_t((_cffi_wchar_t)x);
    else
        return _cffi_from_c_wchar3216_t((int)x);
}

_CFFI_UNUSED_FN static int _cffi_to_c_char32_t(PyObject *o)
{
    if (sizeof(_cffi_wchar_t) == 4)
        return (int)_cffi_to_c_wchar_t(o);
    else
        return (int)_cffi_to_c_wchar3216_t(o);
}

_CFFI_UNUSED_FN static PyObject *_cffi_from_c_char32_t(unsigned int x)
{
    if (sizeof(_cffi_wchar_t) == 4)
        return _cffi_from_c_wchar_t((_cffi_wchar_t)x);
    else
        return _cffi_from_c_wchar3216_t((int)x);
}

union _cffi_union_alignment_u {
    unsigned char m_char;
    unsigned short m_short;
    unsigned int m_int;
    unsigned long m_long;
    unsigned long long m_longlong;
    float m_float;
    double m_double;
    long double m_longdouble;
};

struct _cffi_freeme_s {
    struct _cffi_freeme_s *next;
    union _cffi_union_alignment_u alignment;
};

_CFFI_UNUSED_FN static int
_cffi_convert_array_argument(struct _cffi_ctypedescr *ctptr, PyObject *arg,
                             char **output_data, Py_ssize_t datasize,
                             struct _cffi_freeme_s **freeme)
{
    char *p;
    if (datasize < 0)
        return -1;

    p = *output_data;
    if (p == NULL) {
        struct _cffi_freeme_s *fp = (struct _cffi_freeme_s *)PyObject_Malloc(
            offsetof(struct _cffi_freeme_s, alignment) + (size_t)datasize);
        if (fp == NULL)
            return -1;
        fp->next = *freeme;
        *freeme = fp;
        p = *output_data = (char *)&fp->alignment;
    }
    memset((void *)p, 0, (size_t)datasize);
    return _cffi_convert_array_from_object(p, ctptr, arg);
}

_CFFI_UNUSED_FN static void
_cffi_free_array_arguments(struct _cffi_freeme_s *freeme)
{
    do {
        void *p = (void *)freeme;
        freeme = freeme->next;
        PyObject_Free(p);
    } while (freeme != NULL);
}

/**********  end CPython-specific section  **********/
#else
_CFFI_UNUSED_FN
static void (*_cffi_call_python_org)(struct _cffi_externpy_s *, char *);
# define _cffi_call_python  _cffi_call_python_org
#endif


#define _cffi_array_len(array)   (sizeof(array) / sizeof((array)[0]))

#define _cffi_prim_int(size, sign)                                      \
    ((size) == 1 ? ((sign) ? _CFFI_PRIM_INT8  : _CFFI_PRIM_UINT8)  :    \
     (size) == 2 ? ((sign) ? _CFFI_PRIM_INT16 : _CFFI_PRIM_UINT16) :    \
     (size) == 4 ? ((sign) ? _CFFI_PRIM_INT32 : _CFFI_PRIM_UINT32) :    \
     (size) == 8 ? ((sign) ? _CFFI_PRIM_INT64 : _CFFI_PRIM_UINT64) :    \
     _CFFI__UNKNOWN_PRIM)

#define _cffi_prim_float(size)                                          \
    ((size) == sizeof(float) ? _CFFI_PRIM_FLOAT :                       \
     (size) == sizeof(double) ? _CFFI_PRIM_DOUBLE :                     \
     (size) == sizeof(long double) ? _CFFI__UNKNOWN_LONG_DOUBLE :       \
     _CFFI__UNKNOWN_FLOAT_PRIM)

#define _cffi_check_int(got, got_nonpos, expected)      \
    ((got_nonpos) == (expected <= 0) &&                 \
     (got) == (unsigned long long)expected)

#ifdef MS_WIN32
# define _cffi_stdcall  __stdcall
#else
# define _cffi_stdcall  /* nothing */
#endif

#ifdef __cplusplus
}
#endif

/************************************************************/

void f_ini_eval(double *data,double *x,double *y,double *u,double *p,double Dt){

data[0] = -p[20]*x[0] + 314.15926535897933*x[1] - 314.15926535897933*y[42]; 
data[1] = (1.0/2.0)*(-p[18]*(x[1] - 1.0) - x[3] + y[27])/p[17]; 
data[2] = -x[2]*(1 - ((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11]) || 0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0
)
: (
   1
))) - 9.9999999999999995e-7*x[2] + (u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])*((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11]) || 0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0
)
: (
   1
)); 
data[3] = (-x[3] + (0.33333333333333331*y[1]*y[21] - y[1]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) - y[1]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333331*y[0]*y[16] + y[0]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[0]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) + 0.33333333333333326*y[3]*y[22] - 0.16666666666666652*y[3]*y[23] + 0.28867513459481281*y[3]*y[18] + 0.33333333333333326*y[2]*y[17] - 0.28867513459481281*y[2]*y[23] - 0.16666666666666652*y[2]*y[18] - 0.16666666666666652*y[5]*y[22] - 0.28867513459481281*y[5]*y[17] + 0.33333333333333326*y[5]*y[23] + 0.28867513459481281*y[4]*y[22] - 0.16666666666666652*y[4]*y[17] + 0.33333333333333326*y[4]*y[18] + 1.0*y[21]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[21]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]) + 1.0*y[16]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) + 1.0*y[16]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]))/p[11])/p[13]; 
data[4] = (u[14] - x[4])/p[14]; 
data[5] = (-x[5] + u[6] + ((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   -0.050000000000000003*p[12]
)
: ((0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0.050000000000000003*p[12]
)
: (
   p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])
))))/p[15]; 
data[6] = (-x[6] + u[7] + ((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   -0.050000000000000003*p[12]
)
: ((0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0.050000000000000003*p[12]
)
: (
   p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])
))))/p[15]; 
data[7] = (-x[7] + u[8] + ((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   -0.050000000000000003*p[12]
)
: ((0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0.050000000000000003*p[12]
)
: (
   p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])
))))/p[15]; 
data[8] = (-x[8] + u[9])/p[15]; 
data[9] = -p[32]*x[9] - y[42] + u[24] + 1; 

}

void g_ini_eval(double *data,double *x,double *y,double *u,double *p,double Dt){

data[0] = 94.615102344455565*y[1] + 184.75059629806532*y[0] - 17.974903844364775*y[3] - 69.368640085185334*y[2] - 40.74868784166955*y[5] - 43.675799641021136*y[4] - 17.974903844364817*y[7] - 69.368640085185348*y[6] - 94.615102344455565*y[9] - 184.75059629806532*y[8] + 17.974903844364775*y[11] + 69.368640085185334*y[10] + 40.74868784166955*y[13] + 43.675799641021136*y[12] + 17.974903844364817*y[15] + 69.368640085185348*y[14] - y[16]; 
data[1] = 184.75059629806532*y[1] - 94.615102344455565*y[0] - 69.368640085185334*y[3] + 17.974903844364775*y[2] - 43.675799641021136*y[5] + 40.74868784166955*y[4] - 69.368640085185348*y[7] + 17.974903844364817*y[6] - 184.75059629806532*y[9] + 94.615102344455565*y[8] + 69.368640085185334*y[11] - 17.974903844364775*y[10] + 43.675799641021136*y[13] - 40.74868784166955*y[12] + 69.368640085185348*y[15] - 17.974903844364817*y[14] - y[21]; 
data[2] = -17.974903844364796*y[1] - 69.368640085185319*y[0] + 94.615102344455607*y[3] + 184.75059629806523*y[2] - 17.974903844364814*y[5] - 69.368640085185291*y[4] - 40.748687841669586*y[7] - 43.675799641021079*y[6] + 17.974903844364796*y[9] + 69.368640085185319*y[8] - 94.615102344455607*y[11] - 184.75059629806523*y[10] + 17.974903844364814*y[13] + 69.368640085185291*y[12] + 40.748687841669586*y[15] + 43.675799641021079*y[14] - y[17]; 
data[3] = -69.368640085185319*y[1] + 17.974903844364796*y[0] + 184.75059629806523*y[3] - 94.615102344455607*y[2] - 69.368640085185291*y[5] + 17.974903844364814*y[4] - 43.675799641021079*y[7] + 40.748687841669586*y[6] + 69.368640085185319*y[9] - 17.974903844364796*y[8] - 184.75059629806523*y[11] + 94.615102344455607*y[10] + 69.368640085185291*y[13] - 17.974903844364814*y[12] + 43.675799641021079*y[15] - 40.748687841669586*y[14] - y[22]; 
data[4] = -40.748687841669557*y[1] - 43.675799641021122*y[0] - 17.97490384436481*y[3] - 69.368640085185277*y[2] + 94.615102344455636*y[5] + 184.75059629806526*y[4] - 17.974903844364817*y[7] - 69.368640085185277*y[6] + 40.748687841669557*y[9] + 43.675799641021122*y[8] + 17.97490384436481*y[11] + 69.368640085185277*y[10] - 94.615102344455636*y[13] - 184.75059629806526*y[12] + 17.974903844364817*y[15] + 69.368640085185277*y[14] - y[18]; 
data[5] = -43.675799641021122*y[1] + 40.748687841669557*y[0] - 69.368640085185277*y[3] + 17.97490384436481*y[2] + 184.75059629806526*y[5] - 94.615102344455636*y[4] - 69.368640085185277*y[7] + 17.974903844364817*y[6] + 43.675799641021122*y[9] - 40.748687841669557*y[8] + 69.368640085185277*y[11] - 17.97490384436481*y[10] - 184.75059629806526*y[13] + 94.615102344455636*y[12] + 69.368640085185277*y[15] - 17.974903844364817*y[14] - y[23]; 
data[6] = -17.9749038443648*y[1] - 69.368640085185334*y[0] - 40.748687841669586*y[3] - 43.675799641021086*y[2] - 17.974903844364821*y[5] - 69.368640085185291*y[4] + 94.61510234445565*y[7] + 184.75059629806523*y[6] + 17.9749038443648*y[9] + 69.368640085185334*y[8] + 40.748687841669586*y[11] + 43.675799641021086*y[10] + 17.974903844364821*y[13] + 69.368640085185291*y[12] - 94.61510234445565*y[15] - 184.75059629806523*y[14] - y[19]; 
data[7] = -69.368640085185334*y[1] + 17.9749038443648*y[0] - 43.675799641021086*y[3] + 40.748687841669586*y[2] - 69.368640085185291*y[5] + 17.974903844364821*y[4] + 184.75059629806523*y[7] - 94.61510234445565*y[6] + 69.368640085185334*y[9] - 17.9749038443648*y[8] + 43.675799641021086*y[11] - 40.748687841669586*y[10] + 69.368640085185291*y[13] - 17.974903844364821*y[12] - 184.75059629806523*y[15] + 94.61510234445565*y[14] - y[24]; 
data[8] = -94.615102344455565*y[1] - 184.75059629806532*y[0] + 17.974903844364775*y[3] + 69.368640085185334*y[2] + 40.74868784166955*y[5] + 43.675799641021136*y[4] + 17.974903844364817*y[7] + 69.368640085185348*y[6] + 94.615102344455565*y[9] + 184.75059629806532*y[8] - 17.974903844364775*y[11] - 69.368640085185334*y[10] - 40.74868784166955*y[13] - 43.675799641021136*y[12] - 17.974903844364817*y[15] - 69.368640085185348*y[14] - y[34]; 
data[9] = -184.75059629806532*y[1] + 94.615102344455565*y[0] + 69.368640085185334*y[3] - 17.974903844364775*y[2] + 43.675799641021136*y[5] - 40.74868784166955*y[4] + 69.368640085185348*y[7] - 17.974903844364817*y[6] + 184.75059629806532*y[9] - 94.615102344455565*y[8] - 69.368640085185334*y[11] + 17.974903844364775*y[10] - 43.675799641021136*y[13] + 40.74868784166955*y[12] - 69.368640085185348*y[15] + 17.974903844364817*y[14] - y[38]; 
data[10] = 17.974903844364796*y[1] + 69.368640085185319*y[0] - 94.615102344455607*y[3] - 184.75059629806523*y[2] + 17.974903844364814*y[5] + 69.368640085185291*y[4] + 40.748687841669586*y[7] + 43.675799641021079*y[6] - 17.974903844364796*y[9] - 69.368640085185319*y[8] + 94.615102344455607*y[11] + 184.75059629806523*y[10] - 17.974903844364814*y[13] - 69.368640085185291*y[12] - 40.748687841669586*y[15] - 43.675799641021079*y[14] - y[35]; 
data[11] = 69.368640085185319*y[1] - 17.974903844364796*y[0] - 184.75059629806523*y[3] + 94.615102344455607*y[2] + 69.368640085185291*y[5] - 17.974903844364814*y[4] + 43.675799641021079*y[7] - 40.748687841669586*y[6] - 69.368640085185319*y[9] + 17.974903844364796*y[8] + 184.75059629806523*y[11] - 94.615102344455607*y[10] - 69.368640085185291*y[13] + 17.974903844364814*y[12] - 43.675799641021079*y[15] + 40.748687841669586*y[14] - y[39]; 
data[12] = 40.748687841669557*y[1] + 43.675799641021122*y[0] + 17.97490384436481*y[3] + 69.368640085185277*y[2] - 94.615102344455636*y[5] - 184.75059629806526*y[4] + 17.974903844364817*y[7] + 69.368640085185277*y[6] - 40.748687841669557*y[9] - 43.675799641021122*y[8] - 17.97490384436481*y[11] - 69.368640085185277*y[10] + 94.615102344455636*y[13] + 184.75059629806526*y[12] - 17.974903844364817*y[15] - 69.368640085185277*y[14] - y[36]; 
data[13] = 43.675799641021122*y[1] - 40.748687841669557*y[0] + 69.368640085185277*y[3] - 17.97490384436481*y[2] - 184.75059629806526*y[5] + 94.615102344455636*y[4] + 69.368640085185277*y[7] - 17.974903844364817*y[6] - 43.675799641021122*y[9] + 40.748687841669557*y[8] - 69.368640085185277*y[11] + 17.97490384436481*y[10] + 184.75059629806526*y[13] - 94.615102344455636*y[12] - 69.368640085185277*y[15] + 17.974903844364817*y[14] - y[40]; 
data[14] = 17.9749038443648*y[1] + 69.368640085185334*y[0] + 40.748687841669586*y[3] + 43.675799641021086*y[2] + 17.974903844364821*y[5] + 69.368640085185291*y[4] - 94.61510234445565*y[7] - 184.75059629806523*y[6] - 17.9749038443648*y[9] - 69.368640085185334*y[8] - 40.748687841669586*y[11] - 43.675799641021086*y[10] - 17.974903844364821*y[13] - 69.368640085185291*y[12] + 94.61510234445565*y[15] + 184.75059629806523*y[14] - y[37]; 
data[15] = 69.368640085185334*y[1] - 17.9749038443648*y[0] + 43.675799641021086*y[3] - 40.748687841669586*y[2] + 69.368640085185291*y[5] - 17.974903844364821*y[4] - 184.75059629806523*y[7] + 94.61510234445565*y[6] - 69.368640085185334*y[9] + 17.9749038443648*y[8] - 43.675799641021086*y[11] + 40.748687841669586*y[10] - 69.368640085185291*y[13] + 17.974903844364821*y[12] + 184.75059629806523*y[15] - 94.61510234445565*y[14] - y[41]; 
data[16] = -p[1]*y[16] - y[0] + 1.0*p[0]*y[21] + y[20] + y[28]; 
data[17] = -p[1]*y[17] - y[2] + 1.0*p[0]*y[22] + y[20] + y[29]; 
data[18] = -p[1]*y[18] - y[4] + 1.0*p[0]*y[23] + y[20] + y[30]; 
data[19] = -p[3]*y[19] - y[6] + 1.0*p[2]*y[24] + y[20] + u[0]; 
data[20] = p[5]*y[20]/(pow(p[5], 2) + 1.0*pow(p[4], 2)) + 1.0*p[4]*y[25]/(pow(p[5], 2) + 1.0*pow(p[4], 2)) + y[16] + y[17] + y[18] + y[19]; 
data[21] = -1.0*p[1]*y[21] - 1.0*y[1] - 1.0*p[0]*y[16] + 1.0*y[25] + 1.0*y[31]; 
data[22] = -1.0*p[1]*y[22] - 1.0*y[3] - 1.0*p[0]*y[17] + 1.0*y[25] + 1.0*y[32]; 
data[23] = -1.0*p[1]*y[23] - 1.0*y[5] - 1.0*p[0]*y[18] + 1.0*y[25] + 1.0*y[33]; 
data[24] = -1.0*p[3]*y[24] - 1.0*y[7] - 1.0*p[2]*y[19] + 1.0*y[25] + 1.0*u[1]; 
data[25] = 1.0*p[5]*y[25]/(pow(p[5], 2) + 1.0*pow(p[4], 2)) - 1.0*p[4]*y[20]/(pow(p[5], 2) + 1.0*pow(p[4], 2)) + 1.0*y[21] + 1.0*y[22] + 1.0*y[23] + 1.0*y[24]; 
data[26] = p[6]*(pow(y[21], 2) + pow(y[16], 2) + 9.9999999999999995e-7) + p[6]*(pow(y[22], 2) + pow(y[17], 2) + 9.9999999999999995e-7) + p[6]*(pow(y[23], 2) + pow(y[18], 2) + 9.9999999999999995e-7) + p[6]*(pow(y[24], 2) + pow(y[19], 2) + 9.9999999999999995e-7) + p[7]*sqrt(pow(y[21], 2) + pow(y[16], 2) + 9.9999999999999995e-7) + p[7]*sqrt(pow(y[22], 2) + pow(y[17], 2) + 9.9999999999999995e-7) + p[7]*sqrt(pow(y[23], 2) + pow(y[18], 2) + 9.9999999999999995e-7) + p[7]*sqrt(pow(y[24], 2) + pow(y[19], 2) + 9.9999999999999995e-7) + 4*p[8] + 1.0*y[21]*y[31] + y[16]*y[28] + 1.0*y[22]*y[32] + y[17]*y[29] + 1.0*y[23]*y[33] + 1.0*y[23]*u[1] + y[18]*y[30] + y[18]*u[0] - y[26]; 
data[27] = p[19]*x[9] + p[23]*u[17] + x[4] - y[27] - (x[1] - u[15])/p[16]; 
data[28] = -p[10]*y[16] + 1.0*p[9]*y[21] - y[28] + (x[5] + u[2])*cos(x[0] + u[10]); 
data[29] = -p[10]*y[17] + 1.0*p[9]*y[22] - y[29] + (x[6] + u[3])*cos(x[0] + u[11] - 2.0943951023931953); 
data[30] = -p[10]*y[18] + 1.0*p[9]*y[23] - y[30] + (x[7] + u[4])*cos(x[0] + u[12] - 4.1887902047863905); 
data[31] = -1.0*p[10]*y[21] - 1.0*p[9]*y[16] - 1.0*y[31] + 1.0*(x[5] + u[2])*sin(x[0] + u[10]); 
data[32] = -1.0*p[10]*y[22] - 1.0*p[9]*y[17] - 1.0*y[32] + 1.0*(x[6] + u[3])*sin(x[0] + u[11] - 2.0943951023931953); 
data[33] = -1.0*p[10]*y[23] - 1.0*p[9]*y[18] - 1.0*y[33] + 1.0*(x[7] + u[4])*sin(x[0] + u[12] - 4.1887902047863905); 
data[34] = -p[25]*y[34] - y[8] + 1.0*p[24]*y[38] + u[19]*cos(u[23]); 
data[35] = -p[25]*y[35] - y[10] + 1.0*p[24]*y[39] + u[20]*cos(u[23] - 2.0943951023931953); 
data[36] = -p[25]*y[36] - y[12] + 1.0*p[24]*y[40] + u[21]*cos(u[23] - 4.1887902047863905); 
data[37] = -p[27]*y[37] - y[14] + 1.0*p[26]*y[41]; 
data[38] = -1.0*p[25]*y[38] - 1.0*y[9] - 1.0*p[24]*y[34] + 1.0*u[19]*sin(u[23]); 
data[39] = -1.0*p[25]*y[39] - 1.0*y[11] - 1.0*p[24]*y[35] + 1.0*u[20]*sin(u[23] - 2.0943951023931953); 
data[40] = -1.0*p[25]*y[40] - 1.0*y[13] - 1.0*p[24]*y[36] + 1.0*u[21]*sin(u[23] - 4.1887902047863905); 
data[41] = -1.0*p[27]*y[41] - 1.0*y[15] - 1.0*p[26]*y[37]; 
data[42] = -y[42] + (p[11]*x[1] + 1000.0)/(p[11] + 1000.0); 
data[43] = p[31]*x[9] + p[30]*(1 - y[42]) - y[43]; 

}

void h_eval(double *data,double *x,double *y,double *u,double *p,double Dt){

data[0] = 94.615102344455565*y[1] + 184.75059629806532*y[0] - 17.974903844364775*y[3] - 69.368640085185334*y[2] - 40.74868784166955*y[5] - 43.675799641021136*y[4] - 17.974903844364817*y[7] - 69.368640085185348*y[6] - 94.615102344455565*y[9] - 184.75059629806532*y[8] + 17.974903844364775*y[11] + 69.368640085185334*y[10] + 40.74868784166955*y[13] + 43.675799641021136*y[12] + 17.974903844364817*y[15] + 69.368640085185348*y[14]; 
data[1] = 184.75059629806532*y[1] - 94.615102344455565*y[0] - 69.368640085185334*y[3] + 17.974903844364775*y[2] - 43.675799641021136*y[5] + 40.74868784166955*y[4] - 69.368640085185348*y[7] + 17.974903844364817*y[6] - 184.75059629806532*y[9] + 94.615102344455565*y[8] + 69.368640085185334*y[11] - 17.974903844364775*y[10] + 43.675799641021136*y[13] - 40.74868784166955*y[12] + 69.368640085185348*y[15] - 17.974903844364817*y[14]; 
data[2] = -17.974903844364796*y[1] - 69.368640085185319*y[0] + 94.615102344455607*y[3] + 184.75059629806523*y[2] - 17.974903844364814*y[5] - 69.368640085185291*y[4] - 40.748687841669586*y[7] - 43.675799641021079*y[6] + 17.974903844364796*y[9] + 69.368640085185319*y[8] - 94.615102344455607*y[11] - 184.75059629806523*y[10] + 17.974903844364814*y[13] + 69.368640085185291*y[12] + 40.748687841669586*y[15] + 43.675799641021079*y[14]; 
data[3] = -69.368640085185319*y[1] + 17.974903844364796*y[0] + 184.75059629806523*y[3] - 94.615102344455607*y[2] - 69.368640085185291*y[5] + 17.974903844364814*y[4] - 43.675799641021079*y[7] + 40.748687841669586*y[6] + 69.368640085185319*y[9] - 17.974903844364796*y[8] - 184.75059629806523*y[11] + 94.615102344455607*y[10] + 69.368640085185291*y[13] - 17.974903844364814*y[12] + 43.675799641021079*y[15] - 40.748687841669586*y[14]; 
data[4] = -40.748687841669557*y[1] - 43.675799641021122*y[0] - 17.97490384436481*y[3] - 69.368640085185277*y[2] + 94.615102344455636*y[5] + 184.75059629806526*y[4] - 17.974903844364817*y[7] - 69.368640085185277*y[6] + 40.748687841669557*y[9] + 43.675799641021122*y[8] + 17.97490384436481*y[11] + 69.368640085185277*y[10] - 94.615102344455636*y[13] - 184.75059629806526*y[12] + 17.974903844364817*y[15] + 69.368640085185277*y[14]; 
data[5] = -43.675799641021122*y[1] + 40.748687841669557*y[0] - 69.368640085185277*y[3] + 17.97490384436481*y[2] + 184.75059629806526*y[5] - 94.615102344455636*y[4] - 69.368640085185277*y[7] + 17.974903844364817*y[6] + 43.675799641021122*y[9] - 40.748687841669557*y[8] + 69.368640085185277*y[11] - 17.97490384436481*y[10] - 184.75059629806526*y[13] + 94.615102344455636*y[12] + 69.368640085185277*y[15] - 17.974903844364817*y[14]; 
data[6] = -17.9749038443648*y[1] - 69.368640085185334*y[0] - 40.748687841669586*y[3] - 43.675799641021086*y[2] - 17.974903844364821*y[5] - 69.368640085185291*y[4] + 94.61510234445565*y[7] + 184.75059629806523*y[6] + 17.9749038443648*y[9] + 69.368640085185334*y[8] + 40.748687841669586*y[11] + 43.675799641021086*y[10] + 17.974903844364821*y[13] + 69.368640085185291*y[12] - 94.61510234445565*y[15] - 184.75059629806523*y[14]; 
data[7] = -69.368640085185334*y[1] + 17.9749038443648*y[0] - 43.675799641021086*y[3] + 40.748687841669586*y[2] - 69.368640085185291*y[5] + 17.974903844364821*y[4] + 184.75059629806523*y[7] - 94.61510234445565*y[6] + 69.368640085185334*y[9] - 17.9749038443648*y[8] + 43.675799641021086*y[11] - 40.748687841669586*y[10] + 69.368640085185291*y[13] - 17.974903844364821*y[12] - 184.75059629806523*y[15] + 94.61510234445565*y[14]; 
data[8] = sqrt(1.0*pow(y[21], 2) + pow(y[16], 2)); 
data[9] = sqrt(1.0*pow(y[22], 2) + pow(y[17], 2)); 
data[10] = sqrt(1.0*pow(y[23], 2) + pow(y[18], 2)); 
data[11] = sqrt(1.0*pow(y[24], 2) + pow(y[19], 2)); 
data[12] = 1.0*y[1]*y[21] + y[0]*y[16]; 
data[13] = 1.0*y[1]*y[16] - 1.0*y[0]*y[21]; 
data[14] = 1.0*y[3]*y[22] + y[2]*y[17]; 
data[15] = 1.0*y[3]*y[17] - 1.0*y[2]*y[22]; 
data[16] = 1.0*y[5]*y[23] + y[4]*y[18]; 
data[17] = 1.0*y[5]*y[18] - 1.0*y[4]*y[23]; 
data[18] = 0.33333333333333331*y[1]*y[21] - y[1]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) - y[1]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333331*y[0]*y[16] + y[0]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[0]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) + 0.33333333333333326*y[3]*y[22] - 0.16666666666666652*y[3]*y[23] + 0.28867513459481281*y[3]*y[18] + 0.33333333333333326*y[2]*y[17] - 0.28867513459481281*y[2]*y[23] - 0.16666666666666652*y[2]*y[18] - 0.16666666666666652*y[5]*y[22] - 0.28867513459481281*y[5]*y[17] + 0.33333333333333326*y[5]*y[23] + 0.28867513459481281*y[4]*y[22] - 0.16666666666666652*y[4]*y[17] + 0.33333333333333326*y[4]*y[18] + 1.0*y[21]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[21]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]) + 1.0*y[16]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) + 1.0*y[16]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]); 
data[19] = 0.33333333333333331*y[1]*y[21] - y[1]*(0.16666666666666677*y[22] + 0.28867513459481275*y[17]) - y[1]*(0.16666666666666657*y[23] - 0.28867513459481287*y[18]) + 0.33333333333333331*y[0]*y[16] + y[0]*(0.28867513459481275*y[22] - 0.16666666666666677*y[17]) + y[0]*(-0.28867513459481287*y[23] - 0.16666666666666657*y[18]) + 0.33333333333333326*y[3]*y[22] - 0.16666666666666652*y[3]*y[23] - 0.28867513459481281*y[3]*y[18] + 0.33333333333333326*y[2]*y[17] + 0.28867513459481281*y[2]*y[23] - 0.16666666666666652*y[2]*y[18] - 0.16666666666666652*y[5]*y[22] + 0.28867513459481281*y[5]*y[17] + 0.33333333333333326*y[5]*y[23] - 0.28867513459481281*y[4]*y[22] - 0.16666666666666652*y[4]*y[17] + 0.33333333333333326*y[4]*y[18] + 1.0*y[21]*(-0.16666666666666677*y[3] - 0.28867513459481275*y[2]) + 1.0*y[21]*(-0.16666666666666657*y[5] + 0.28867513459481287*y[4]) + 1.0*y[16]*(0.28867513459481275*y[3] - 0.16666666666666677*y[2]) + 1.0*y[16]*(-0.28867513459481287*y[5] - 0.16666666666666657*y[4]); 
data[20] = 0.33333333333333331*y[1]*y[21] + 0.33333333333333331*y[1]*y[22] + 0.33333333333333331*y[1]*y[23] + 0.33333333333333331*y[0]*y[16] + 0.33333333333333331*y[0]*y[17] + 0.33333333333333331*y[0]*y[18] + 0.33333333333333331*y[3]*y[21] + 0.33333333333333331*y[3]*y[22] + 0.33333333333333331*y[3]*y[23] + 0.33333333333333331*y[2]*y[16] + 0.33333333333333331*y[2]*y[17] + 0.33333333333333331*y[2]*y[18] + 0.33333333333333331*y[5]*y[21] + 0.33333333333333331*y[5]*y[22] + 0.33333333333333331*y[5]*y[23] + 0.33333333333333331*y[4]*y[16] + 0.33333333333333331*y[4]*y[17] + 0.33333333333333331*y[4]*y[18]; 
data[21] = 0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]); 
data[22] = u[2]; 
data[23] = u[3]; 
data[24] = u[4]; 
data[25] = u[6]; 
data[26] = u[7]; 
data[27] = u[8]; 
data[28] = u[14]; 
data[29] = u[15]; 
data[30] = x[0]; 
data[31] = sqrt(1.0*pow(y[38], 2) + pow(y[34], 2)); 
data[32] = sqrt(1.0*pow(y[39], 2) + pow(y[35], 2)); 
data[33] = sqrt(1.0*pow(y[40], 2) + pow(y[36], 2)); 
data[34] = sqrt(1.0*pow(y[41], 2) + pow(y[37], 2)); 
data[35] = 1.0*y[9]*y[38] + y[8]*y[34] + 1.0*y[11]*y[39] + y[10]*y[35] + 1.0*y[13]*y[40] + y[12]*y[36]; 
data[36] = 1.0*y[9]*y[34] - 1.0*y[8]*y[38] + 1.0*y[11]*y[35] - 1.0*y[10]*y[39] + 1.0*y[13]*y[36] - 1.0*y[12]*y[40]; 
data[37] = x[9]; 
data[38] = u[24]; 

}

void de_jac_ini_up_eval(double *data,double *x,double *y,double *u,double *p,double Dt){

data[0] = -p[20]; 
data[55] = -1.0/2.0*p[18]/p[17]; 
data[57] = -(1.0/2.0)/p[17]; 
data[91] = (1.0/2.0)/p[17]; 
data[165] = -1/p[13]; 
data[220] = -1/p[14]; 
data[275] = -1/p[15]; 
data[330] = -1/p[15]; 
data[385] = -1/p[15]; 
data[440] = -1/p[15]; 
data[495] = -p[32]; 
data[1430] = -p[1]; 
data[1435] = 1.0*p[0]; 
data[1485] = -p[1]; 
data[1490] = 1.0*p[0]; 
data[1540] = -p[1]; 
data[1545] = 1.0*p[0]; 
data[1595] = -p[3]; 
data[1600] = 1.0*p[2]; 
data[1650] = p[5]/(pow(p[5], 2) + 1.0*pow(p[4], 2)); 
data[1655] = 1.0*p[4]/(pow(p[5], 2) + 1.0*pow(p[4], 2)); 
data[1700] = -1.0*p[0]; 
data[1705] = -1.0*p[1]; 
data[1755] = -1.0*p[0]; 
data[1760] = -1.0*p[1]; 
data[1810] = -1.0*p[0]; 
data[1815] = -1.0*p[1]; 
data[1865] = -1.0*p[2]; 
data[1870] = -1.0*p[3]; 
data[1920] = -1.0*p[4]/(pow(p[5], 2) + 1.0*pow(p[4], 2)); 
data[1925] = 1.0*p[5]/(pow(p[5], 2) + 1.0*pow(p[4], 2)); 
data[1999] = -1/p[16]; 
data[2007] = p[19]; 
data[2078] = -p[10]; 
data[2083] = 1.0*p[9]; 
data[2133] = -p[10]; 
data[2138] = 1.0*p[9]; 
data[2188] = -p[10]; 
data[2193] = 1.0*p[9]; 
data[2240] = -1.0*p[9]; 
data[2245] = -1.0*p[10]; 
data[2295] = -1.0*p[9]; 
data[2300] = -1.0*p[10]; 
data[2350] = -1.0*p[9]; 
data[2355] = -1.0*p[10]; 
data[2420] = -p[25]; 
data[2424] = 1.0*p[24]; 
data[2475] = -p[25]; 
data[2479] = 1.0*p[24]; 
data[2530] = -p[25]; 
data[2534] = 1.0*p[24]; 
data[2585] = -p[27]; 
data[2589] = 1.0*p[26]; 
data[2636] = -1.0*p[24]; 
data[2640] = -1.0*p[25]; 
data[2691] = -1.0*p[24]; 
data[2695] = -1.0*p[25]; 
data[2746] = -1.0*p[24]; 
data[2750] = -1.0*p[25]; 
data[2801] = -1.0*p[26]; 
data[2805] = -1.0*p[27]; 
data[2809] = p[11]/(p[11] + 1000.0); 
data[2871] = p[31]; 
data[2914] = -p[30]; 

}

void de_jac_ini_xy_eval(double *data,double *x,double *y,double *u,double *p,double Dt){

data[110] = ((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11]) || 0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0
)
: (
   1
)) - 1.0000009999999999; 
data[118] = -(-0.33333333333333331*y[21] + 0.16666666666666657*y[22] - 0.28867513459481287*y[17] + 0.16666666666666677*y[23] + 0.28867513459481275*y[18])*((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11]) || 0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0
)
: (
   1
))/p[11]; 
data[119] = -(0.33333333333333331*y[16] - 0.28867513459481287*y[22] - 0.16666666666666657*y[17] + 0.28867513459481275*y[23] - 0.16666666666666677*y[18])*((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11]) || 0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0
)
: (
   1
))/p[11]; 
data[120] = -(0.16666666666666657*y[21] + 0.28867513459481287*y[16] - 0.33333333333333326*y[22] + 0.16666666666666652*y[23] - 0.28867513459481281*y[18])*((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11]) || 0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0
)
: (
   1
))/p[11]; 
data[121] = -(0.28867513459481287*y[21] - 0.16666666666666657*y[16] + 0.33333333333333326*y[17] - 0.28867513459481281*y[23] - 0.16666666666666652*y[18])*((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11]) || 0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0
)
: (
   1
))/p[11]; 
data[122] = -(0.16666666666666677*y[21] - 0.28867513459481275*y[16] + 0.16666666666666652*y[22] + 0.28867513459481281*y[17] - 0.33333333333333326*y[23])*((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11]) || 0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0
)
: (
   1
))/p[11]; 
data[123] = -(-0.28867513459481275*y[21] - 0.16666666666666677*y[16] + 0.28867513459481281*y[22] - 0.16666666666666652*y[17] + 0.33333333333333326*y[18])*((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11]) || 0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0
)
: (
   1
))/p[11]; 
data[134] = -(0.33333333333333331*y[1] - 0.16666666666666657*y[3] + 0.28867513459481287*y[2] - 0.16666666666666677*y[5] - 0.28867513459481275*y[4])*((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11]) || 0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0
)
: (
   1
))/p[11]; 
data[135] = -(-0.16666666666666657*y[1] - 0.28867513459481287*y[0] + 0.33333333333333326*y[3] - 0.16666666666666652*y[5] + 0.28867513459481281*y[4])*((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11]) || 0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0
)
: (
   1
))/p[11]; 
data[136] = -(-0.16666666666666677*y[1] + 0.28867513459481275*y[0] - 0.16666666666666652*y[3] - 0.28867513459481281*y[2] + 0.33333333333333326*y[5])*((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11]) || 0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0
)
: (
   1
))/p[11]; 
data[139] = -(-0.33333333333333331*y[0] + 0.28867513459481287*y[3] + 0.16666666666666657*y[2] - 0.28867513459481275*y[5] + 0.16666666666666677*y[4])*((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11]) || 0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0
)
: (
   1
))/p[11]; 
data[140] = -(-0.28867513459481287*y[1] + 0.16666666666666657*y[0] - 0.33333333333333326*y[2] + 0.28867513459481281*y[5] + 0.16666666666666652*y[4])*((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11]) || 0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0
)
: (
   1
))/p[11]; 
data[141] = -(0.28867513459481275*y[1] + 0.16666666666666677*y[0] - 0.28867513459481281*y[3] + 0.16666666666666652*y[2] - 0.33333333333333326*y[4])*((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11]) || 0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0
)
: (
   1
))/p[11]; 
data[172] = (0.33333333333333331*y[16] - 0.28867513459481287*y[22] - 0.16666666666666657*y[17] + 0.28867513459481275*y[23] - 0.16666666666666677*y[18])/(p[11]*p[13]); 
data[173] = (0.33333333333333331*y[21] - 0.16666666666666657*y[22] + 0.28867513459481287*y[17] - 0.16666666666666677*y[23] - 0.28867513459481275*y[18])/(p[11]*p[13]); 
data[174] = (0.28867513459481287*y[21] - 0.16666666666666657*y[16] + 0.33333333333333326*y[17] - 0.28867513459481281*y[23] - 0.16666666666666652*y[18])/(p[11]*p[13]); 
data[175] = (-0.16666666666666657*y[21] - 0.28867513459481287*y[16] + 0.33333333333333326*y[22] - 0.16666666666666652*y[23] + 0.28867513459481281*y[18])/(p[11]*p[13]); 
data[176] = (-0.28867513459481275*y[21] - 0.16666666666666677*y[16] + 0.28867513459481281*y[22] - 0.16666666666666652*y[17] + 0.33333333333333326*y[18])/(p[11]*p[13]); 
data[177] = (-0.16666666666666677*y[21] + 0.28867513459481275*y[16] - 0.16666666666666652*y[22] - 0.28867513459481281*y[17] + 0.33333333333333326*y[23])/(p[11]*p[13]); 
data[188] = (0.33333333333333331*y[0] - 0.28867513459481287*y[3] - 0.16666666666666657*y[2] + 0.28867513459481275*y[5] - 0.16666666666666677*y[4])/(p[11]*p[13]); 
data[189] = (0.28867513459481287*y[1] - 0.16666666666666657*y[0] + 0.33333333333333326*y[2] - 0.28867513459481281*y[5] - 0.16666666666666652*y[4])/(p[11]*p[13]); 
data[190] = (-0.28867513459481275*y[1] - 0.16666666666666677*y[0] + 0.28867513459481281*y[3] - 0.16666666666666652*y[2] + 0.33333333333333326*y[4])/(p[11]*p[13]); 
data[193] = (0.33333333333333331*y[1] - 0.16666666666666657*y[3] + 0.28867513459481287*y[2] - 0.16666666666666677*y[5] - 0.28867513459481275*y[4])/(p[11]*p[13]); 
data[194] = (-0.16666666666666657*y[1] - 0.28867513459481287*y[0] + 0.33333333333333326*y[3] - 0.16666666666666652*y[5] + 0.28867513459481281*y[4])/(p[11]*p[13]); 
data[195] = (-0.16666666666666677*y[1] + 0.28867513459481275*y[0] - 0.16666666666666652*y[3] - 0.28867513459481281*y[2] + 0.33333333333333326*y[5])/(p[11]*p[13]); 
data[272] = ((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11]) || 0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0
)
: (
   p[22]
))/p[15]; 
data[280] = ((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11]) || 0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0
)
: (
   -p[21]*(-0.33333333333333331*y[21] + 0.16666666666666657*y[22] - 0.28867513459481287*y[17] + 0.16666666666666677*y[23] + 0.28867513459481275*y[18])/p[11]
))/p[15]; 
data[281] = ((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11]) || 0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0
)
: (
   -p[21]*(0.33333333333333331*y[16] - 0.28867513459481287*y[22] - 0.16666666666666657*y[17] + 0.28867513459481275*y[23] - 0.16666666666666677*y[18])/p[11]
))/p[15]; 
data[282] = ((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11]) || 0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0
)
: (
   -p[21]*(0.16666666666666657*y[21] + 0.28867513459481287*y[16] - 0.33333333333333326*y[22] + 0.16666666666666652*y[23] - 0.28867513459481281*y[18])/p[11]
))/p[15]; 
data[283] = ((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11]) || 0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0
)
: (
   -p[21]*(0.28867513459481287*y[21] - 0.16666666666666657*y[16] + 0.33333333333333326*y[17] - 0.28867513459481281*y[23] - 0.16666666666666652*y[18])/p[11]
))/p[15]; 
data[284] = ((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11]) || 0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0
)
: (
   -p[21]*(0.16666666666666677*y[21] - 0.28867513459481275*y[16] + 0.16666666666666652*y[22] + 0.28867513459481281*y[17] - 0.33333333333333326*y[23])/p[11]
))/p[15]; 
data[285] = ((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11]) || 0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0
)
: (
   -p[21]*(-0.28867513459481275*y[21] - 0.16666666666666677*y[16] + 0.28867513459481281*y[22] - 0.16666666666666652*y[17] + 0.33333333333333326*y[18])/p[11]
))/p[15]; 
data[296] = ((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11]) || 0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0
)
: (
   -p[21]*(0.33333333333333331*y[1] - 0.16666666666666657*y[3] + 0.28867513459481287*y[2] - 0.16666666666666677*y[5] - 0.28867513459481275*y[4])/p[11]
))/p[15]; 
data[297] = ((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11]) || 0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0
)
: (
   -p[21]*(-0.16666666666666657*y[1] - 0.28867513459481287*y[0] + 0.33333333333333326*y[3] - 0.16666666666666652*y[5] + 0.28867513459481281*y[4])/p[11]
))/p[15]; 
data[298] = ((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11]) || 0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0
)
: (
   -p[21]*(-0.16666666666666677*y[1] + 0.28867513459481275*y[0] - 0.16666666666666652*y[3] - 0.28867513459481281*y[2] + 0.33333333333333326*y[5])/p[11]
))/p[15]; 
data[301] = ((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11]) || 0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0
)
: (
   -p[21]*(-0.33333333333333331*y[0] + 0.28867513459481287*y[3] + 0.16666666666666657*y[2] - 0.28867513459481275*y[5] + 0.16666666666666677*y[4])/p[11]
))/p[15]; 
data[302] = ((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11]) || 0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0
)
: (
   -p[21]*(-0.28867513459481287*y[1] + 0.16666666666666657*y[0] - 0.33333333333333326*y[2] + 0.28867513459481281*y[5] + 0.16666666666666652*y[4])/p[11]
))/p[15]; 
data[303] = ((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11]) || 0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0
)
: (
   -p[21]*(0.28867513459481275*y[1] + 0.16666666666666677*y[0] - 0.28867513459481281*y[3] + 0.16666666666666652*y[2] - 0.33333333333333326*y[4])/p[11]
))/p[15]; 
data[326] = ((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11]) || 0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0
)
: (
   p[22]
))/p[15]; 
data[334] = ((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11]) || 0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0
)
: (
   -p[21]*(-0.33333333333333331*y[21] + 0.16666666666666657*y[22] - 0.28867513459481287*y[17] + 0.16666666666666677*y[23] + 0.28867513459481275*y[18])/p[11]
))/p[15]; 
data[335] = ((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11]) || 0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0
)
: (
   -p[21]*(0.33333333333333331*y[16] - 0.28867513459481287*y[22] - 0.16666666666666657*y[17] + 0.28867513459481275*y[23] - 0.16666666666666677*y[18])/p[11]
))/p[15]; 
data[336] = ((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11]) || 0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0
)
: (
   -p[21]*(0.16666666666666657*y[21] + 0.28867513459481287*y[16] - 0.33333333333333326*y[22] + 0.16666666666666652*y[23] - 0.28867513459481281*y[18])/p[11]
))/p[15]; 
data[337] = ((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11]) || 0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0
)
: (
   -p[21]*(0.28867513459481287*y[21] - 0.16666666666666657*y[16] + 0.33333333333333326*y[17] - 0.28867513459481281*y[23] - 0.16666666666666652*y[18])/p[11]
))/p[15]; 
data[338] = ((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11]) || 0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0
)
: (
   -p[21]*(0.16666666666666677*y[21] - 0.28867513459481275*y[16] + 0.16666666666666652*y[22] + 0.28867513459481281*y[17] - 0.33333333333333326*y[23])/p[11]
))/p[15]; 
data[339] = ((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11]) || 0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0
)
: (
   -p[21]*(-0.28867513459481275*y[21] - 0.16666666666666677*y[16] + 0.28867513459481281*y[22] - 0.16666666666666652*y[17] + 0.33333333333333326*y[18])/p[11]
))/p[15]; 
data[350] = ((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11]) || 0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0
)
: (
   -p[21]*(0.33333333333333331*y[1] - 0.16666666666666657*y[3] + 0.28867513459481287*y[2] - 0.16666666666666677*y[5] - 0.28867513459481275*y[4])/p[11]
))/p[15]; 
data[351] = ((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11]) || 0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0
)
: (
   -p[21]*(-0.16666666666666657*y[1] - 0.28867513459481287*y[0] + 0.33333333333333326*y[3] - 0.16666666666666652*y[5] + 0.28867513459481281*y[4])/p[11]
))/p[15]; 
data[352] = ((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11]) || 0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0
)
: (
   -p[21]*(-0.16666666666666677*y[1] + 0.28867513459481275*y[0] - 0.16666666666666652*y[3] - 0.28867513459481281*y[2] + 0.33333333333333326*y[5])/p[11]
))/p[15]; 
data[355] = ((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11]) || 0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0
)
: (
   -p[21]*(-0.33333333333333331*y[0] + 0.28867513459481287*y[3] + 0.16666666666666657*y[2] - 0.28867513459481275*y[5] + 0.16666666666666677*y[4])/p[11]
))/p[15]; 
data[356] = ((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11]) || 0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0
)
: (
   -p[21]*(-0.28867513459481287*y[1] + 0.16666666666666657*y[0] - 0.33333333333333326*y[2] + 0.28867513459481281*y[5] + 0.16666666666666652*y[4])/p[11]
))/p[15]; 
data[357] = ((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11]) || 0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0
)
: (
   -p[21]*(0.28867513459481275*y[1] + 0.16666666666666677*y[0] - 0.28867513459481281*y[3] + 0.16666666666666652*y[2] - 0.33333333333333326*y[4])/p[11]
))/p[15]; 
data[380] = ((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11]) || 0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0
)
: (
   p[22]
))/p[15]; 
data[388] = ((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11]) || 0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0
)
: (
   -p[21]*(-0.33333333333333331*y[21] + 0.16666666666666657*y[22] - 0.28867513459481287*y[17] + 0.16666666666666677*y[23] + 0.28867513459481275*y[18])/p[11]
))/p[15]; 
data[389] = ((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11]) || 0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0
)
: (
   -p[21]*(0.33333333333333331*y[16] - 0.28867513459481287*y[22] - 0.16666666666666657*y[17] + 0.28867513459481275*y[23] - 0.16666666666666677*y[18])/p[11]
))/p[15]; 
data[390] = ((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11]) || 0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0
)
: (
   -p[21]*(0.16666666666666657*y[21] + 0.28867513459481287*y[16] - 0.33333333333333326*y[22] + 0.16666666666666652*y[23] - 0.28867513459481281*y[18])/p[11]
))/p[15]; 
data[391] = ((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11]) || 0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0
)
: (
   -p[21]*(0.28867513459481287*y[21] - 0.16666666666666657*y[16] + 0.33333333333333326*y[17] - 0.28867513459481281*y[23] - 0.16666666666666652*y[18])/p[11]
))/p[15]; 
data[392] = ((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11]) || 0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0
)
: (
   -p[21]*(0.16666666666666677*y[21] - 0.28867513459481275*y[16] + 0.16666666666666652*y[22] + 0.28867513459481281*y[17] - 0.33333333333333326*y[23])/p[11]
))/p[15]; 
data[393] = ((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11]) || 0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0
)
: (
   -p[21]*(-0.28867513459481275*y[21] - 0.16666666666666677*y[16] + 0.28867513459481281*y[22] - 0.16666666666666652*y[17] + 0.33333333333333326*y[18])/p[11]
))/p[15]; 
data[404] = ((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11]) || 0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0
)
: (
   -p[21]*(0.33333333333333331*y[1] - 0.16666666666666657*y[3] + 0.28867513459481287*y[2] - 0.16666666666666677*y[5] - 0.28867513459481275*y[4])/p[11]
))/p[15]; 
data[405] = ((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11]) || 0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0
)
: (
   -p[21]*(-0.16666666666666657*y[1] - 0.28867513459481287*y[0] + 0.33333333333333326*y[3] - 0.16666666666666652*y[5] + 0.28867513459481281*y[4])/p[11]
))/p[15]; 
data[406] = ((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11]) || 0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0
)
: (
   -p[21]*(-0.16666666666666677*y[1] + 0.28867513459481275*y[0] - 0.16666666666666652*y[3] - 0.28867513459481281*y[2] + 0.33333333333333326*y[5])/p[11]
))/p[15]; 
data[409] = ((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11]) || 0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0
)
: (
   -p[21]*(-0.33333333333333331*y[0] + 0.28867513459481287*y[3] + 0.16666666666666657*y[2] - 0.28867513459481275*y[5] + 0.16666666666666677*y[4])/p[11]
))/p[15]; 
data[410] = ((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11]) || 0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0
)
: (
   -p[21]*(-0.28867513459481287*y[1] + 0.16666666666666657*y[0] - 0.33333333333333326*y[2] + 0.28867513459481281*y[5] + 0.16666666666666652*y[4])/p[11]
))/p[15]; 
data[411] = ((0.050000000000000003*p[12] < -p[22]*x[2] - p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11]) || 0.050000000000000003*p[12] < p[22]*x[2] + p[21]*(u[16] - (0.33333333333333331*y[1]*y[16] + y[1]*(-0.28867513459481287*y[22] - 0.16666666666666657*y[17]) + y[1]*(0.28867513459481275*y[23] - 0.16666666666666677*y[18]) - 0.33333333333333331*y[0]*y[21] + y[0]*(0.16666666666666657*y[22] - 0.28867513459481287*y[17]) + y[0]*(0.16666666666666677*y[23] + 0.28867513459481275*y[18]) + 0.33333333333333326*y[3]*y[17] - 0.28867513459481281*y[3]*y[23] - 0.16666666666666652*y[3]*y[18] - 0.33333333333333326*y[2]*y[22] + 0.16666666666666652*y[2]*y[23] - 0.28867513459481281*y[2]*y[18] + 0.28867513459481281*y[5]*y[22] - 0.16666666666666652*y[5]*y[17] + 0.33333333333333326*y[5]*y[18] + 0.16666666666666652*y[4]*y[22] + 0.28867513459481281*y[4]*y[17] - 0.33333333333333326*y[4]*y[23] - 1.0*y[21]*(-0.28867513459481287*y[3] - 0.16666666666666657*y[2]) - 1.0*y[21]*(0.28867513459481275*y[5] - 0.16666666666666677*y[4]) + 1.0*y[16]*(-0.16666666666666657*y[3] + 0.28867513459481287*y[2]) + 1.0*y[16]*(-0.16666666666666677*y[5] - 0.28867513459481275*y[4]))/p[11])) ? (
   0
)
: (
   -p[21]*(0.28867513459481275*y[1] + 0.16666666666666677*y[0] - 0.28867513459481281*y[3] + 0.16666666666666652*y[2] - 0.33333333333333326*y[4])/p[11]
))/p[15]; 
data[1970] = 2*p[6]*y[16] + p[7]*y[16]/sqrt(pow(y[21], 2) + pow(y[16], 2) + 9.9999999999999995e-7) + y[28]; 
data[1971] = 2*p[6]*y[17] + p[7]*y[17]/sqrt(pow(y[22], 2) + pow(y[17], 2) + 9.9999999999999995e-7) + y[29]; 
data[1972] = 2*p[6]*y[18] + p[7]*y[18]/sqrt(pow(y[23], 2) + pow(y[18], 2) + 9.9999999999999995e-7) + y[30] + u[0]; 
data[1973] = 2*p[6]*y[19] + p[7]*y[19]/sqrt(pow(y[24], 2) + pow(y[19], 2) + 9.9999999999999995e-7); 
data[1975] = 2*p[6]*y[21] + p[7]*y[21]/sqrt(pow(y[21], 2) + pow(y[16], 2) + 9.9999999999999995e-7) + 1.0*y[31]; 
data[1976] = 2*p[6]*y[22] + p[7]*y[22]/sqrt(pow(y[22], 2) + pow(y[17], 2) + 9.9999999999999995e-7) + 1.0*y[32]; 
data[1977] = 2*p[6]*y[23] + p[7]*y[23]/sqrt(pow(y[23], 2) + pow(y[18], 2) + 9.9999999999999995e-7) + 1.0*y[33] + 1.0*u[1]; 
data[1978] = 2*p[6]*y[24] + p[7]*y[24]/sqrt(pow(y[24], 2) + pow(y[19], 2) + 9.9999999999999995e-7); 
data[1982] = y[16]; 
data[1983] = y[17]; 
data[1984] = y[18]; 
data[1985] = 1.0*y[21]; 
data[1986] = 1.0*y[22]; 
data[1987] = 1.0*y[23]; 
data[2052] = -(x[5] + u[2])*sin(x[0] + u[10]); 
data[2057] = cos(x[0] + u[10]); 
data[2106] = -(x[6] + u[3])*sin(x[0] + u[11] - 2.0943951023931953); 
data[2112] = cos(x[0] + u[11] - 2.0943951023931953); 
data[2160] = -(x[7] + u[4])*sin(x[0] + u[12] - 4.1887902047863905); 
data[2167] = cos(x[0] + u[12] - 4.1887902047863905); 
data[2214] = 1.0*(x[5] + u[2])*cos(x[0] + u[10]); 
data[2219] = 1.0*sin(x[0] + u[10]); 
data[2268] = 1.0*(x[6] + u[3])*cos(x[0] + u[11] - 2.0943951023931953); 
data[2274] = 1.0*sin(x[0] + u[11] - 2.0943951023931953); 
data[2322] = 1.0*(x[7] + u[4])*cos(x[0] + u[12] - 4.1887902047863905); 
data[2329] = 1.0*sin(x[0] + u[12] - 4.1887902047863905); 

}

void de_jac_ini_num_eval(double *data,double *x,double *y,double *u,double *p,double Dt){

data[1] = 314.15926535897933; 
data[52] = -314.15926535897933; 
data[538] = -1; 
data[550] = 184.75059629806532; 
data[551] = 94.615102344455565; 
data[552] = -69.368640085185334; 
data[553] = -17.974903844364775; 
data[554] = -43.675799641021136; 
data[555] = -40.74868784166955; 
data[556] = -69.368640085185348; 
data[557] = -17.974903844364817; 
data[558] = -184.75059629806532; 
data[559] = -94.615102344455565; 
data[560] = 69.368640085185334; 
data[561] = 17.974903844364775; 
data[562] = 43.675799641021136; 
data[563] = 40.74868784166955; 
data[564] = 69.368640085185348; 
data[565] = 17.974903844364817; 
data[566] = -1; 
data[604] = -94.615102344455565; 
data[605] = 184.75059629806532; 
data[606] = 17.974903844364775; 
data[607] = -69.368640085185334; 
data[608] = 40.74868784166955; 
data[609] = -43.675799641021136; 
data[610] = 17.974903844364817; 
data[611] = -69.368640085185348; 
data[612] = 94.615102344455565; 
data[613] = -184.75059629806532; 
data[614] = -17.974903844364775; 
data[615] = 69.368640085185334; 
data[616] = -40.74868784166955; 
data[617] = 43.675799641021136; 
data[618] = -17.974903844364817; 
data[619] = 69.368640085185348; 
data[625] = -1; 
data[658] = -69.368640085185319; 
data[659] = -17.974903844364796; 
data[660] = 184.75059629806523; 
data[661] = 94.615102344455607; 
data[662] = -69.368640085185291; 
data[663] = -17.974903844364814; 
data[664] = -43.675799641021079; 
data[665] = -40.748687841669586; 
data[666] = 69.368640085185319; 
data[667] = 17.974903844364796; 
data[668] = -184.75059629806523; 
data[669] = -94.615102344455607; 
data[670] = 69.368640085185291; 
data[671] = 17.974903844364814; 
data[672] = 43.675799641021079; 
data[673] = 40.748687841669586; 
data[675] = -1; 
data[712] = 17.974903844364796; 
data[713] = -69.368640085185319; 
data[714] = -94.615102344455607; 
data[715] = 184.75059629806523; 
data[716] = 17.974903844364814; 
data[717] = -69.368640085185291; 
data[718] = 40.748687841669586; 
data[719] = -43.675799641021079; 
data[720] = -17.974903844364796; 
data[721] = 69.368640085185319; 
data[722] = 94.615102344455607; 
data[723] = -184.75059629806523; 
data[724] = -17.974903844364814; 
data[725] = 69.368640085185291; 
data[726] = -40.748687841669586; 
data[727] = 43.675799641021079; 
data[734] = -1; 
data[766] = -43.675799641021122; 
data[767] = -40.748687841669557; 
data[768] = -69.368640085185277; 
data[769] = -17.97490384436481; 
data[770] = 184.75059629806526; 
data[771] = 94.615102344455636; 
data[772] = -69.368640085185277; 
data[773] = -17.974903844364817; 
data[774] = 43.675799641021122; 
data[775] = 40.748687841669557; 
data[776] = 69.368640085185277; 
data[777] = 17.97490384436481; 
data[778] = -184.75059629806526; 
data[779] = -94.615102344455636; 
data[780] = 69.368640085185277; 
data[781] = 17.974903844364817; 
data[784] = -1; 
data[820] = 40.748687841669557; 
data[821] = -43.675799641021122; 
data[822] = 17.97490384436481; 
data[823] = -69.368640085185277; 
data[824] = -94.615102344455636; 
data[825] = 184.75059629806526; 
data[826] = 17.974903844364817; 
data[827] = -69.368640085185277; 
data[828] = -40.748687841669557; 
data[829] = 43.675799641021122; 
data[830] = -17.97490384436481; 
data[831] = 69.368640085185277; 
data[832] = 94.615102344455636; 
data[833] = -184.75059629806526; 
data[834] = -17.974903844364817; 
data[835] = 69.368640085185277; 
data[843] = -1; 
data[874] = -69.368640085185334; 
data[875] = -17.9749038443648; 
data[876] = -43.675799641021086; 
data[877] = -40.748687841669586; 
data[878] = -69.368640085185291; 
data[879] = -17.974903844364821; 
data[880] = 184.75059629806523; 
data[881] = 94.61510234445565; 
data[882] = 69.368640085185334; 
data[883] = 17.9749038443648; 
data[884] = 43.675799641021086; 
data[885] = 40.748687841669586; 
data[886] = 69.368640085185291; 
data[887] = 17.974903844364821; 
data[888] = -184.75059629806523; 
data[889] = -94.61510234445565; 
data[893] = -1; 
data[928] = 17.9749038443648; 
data[929] = -69.368640085185334; 
data[930] = 40.748687841669586; 
data[931] = -43.675799641021086; 
data[932] = 17.974903844364821; 
data[933] = -69.368640085185291; 
data[934] = -94.61510234445565; 
data[935] = 184.75059629806523; 
data[936] = -17.9749038443648; 
data[937] = 69.368640085185334; 
data[938] = -40.748687841669586; 
data[939] = 43.675799641021086; 
data[940] = -17.974903844364821; 
data[941] = 69.368640085185291; 
data[942] = 94.61510234445565; 
data[943] = -184.75059629806523; 
data[952] = -1; 
data[982] = -184.75059629806532; 
data[983] = -94.615102344455565; 
data[984] = 69.368640085185334; 
data[985] = 17.974903844364775; 
data[986] = 43.675799641021136; 
data[987] = 40.74868784166955; 
data[988] = 69.368640085185348; 
data[989] = 17.974903844364817; 
data[990] = 184.75059629806532; 
data[991] = 94.615102344455565; 
data[992] = -69.368640085185334; 
data[993] = -17.974903844364775; 
data[994] = -43.675799641021136; 
data[995] = -40.74868784166955; 
data[996] = -69.368640085185348; 
data[997] = -17.974903844364817; 
data[1016] = -1; 
data[1036] = 94.615102344455565; 
data[1037] = -184.75059629806532; 
data[1038] = -17.974903844364775; 
data[1039] = 69.368640085185334; 
data[1040] = -40.74868784166955; 
data[1041] = 43.675799641021136; 
data[1042] = -17.974903844364817; 
data[1043] = 69.368640085185348; 
data[1044] = -94.615102344455565; 
data[1045] = 184.75059629806532; 
data[1046] = 17.974903844364775; 
data[1047] = -69.368640085185334; 
data[1048] = 40.74868784166955; 
data[1049] = -43.675799641021136; 
data[1050] = 17.974903844364817; 
data[1051] = -69.368640085185348; 
data[1074] = -1; 
data[1090] = 69.368640085185319; 
data[1091] = 17.974903844364796; 
data[1092] = -184.75059629806523; 
data[1093] = -94.615102344455607; 
data[1094] = 69.368640085185291; 
data[1095] = 17.974903844364814; 
data[1096] = 43.675799641021079; 
data[1097] = 40.748687841669586; 
data[1098] = -69.368640085185319; 
data[1099] = -17.974903844364796; 
data[1100] = 184.75059629806523; 
data[1101] = 94.615102344455607; 
data[1102] = -69.368640085185291; 
data[1103] = -17.974903844364814; 
data[1104] = -43.675799641021079; 
data[1105] = -40.748687841669586; 
data[1125] = -1; 
data[1144] = -17.974903844364796; 
data[1145] = 69.368640085185319; 
data[1146] = 94.615102344455607; 
data[1147] = -184.75059629806523; 
data[1148] = -17.974903844364814; 
data[1149] = 69.368640085185291; 
data[1150] = -40.748687841669586; 
data[1151] = 43.675799641021079; 
data[1152] = 17.974903844364796; 
data[1153] = -69.368640085185319; 
data[1154] = -94.615102344455607; 
data[1155] = 184.75059629806523; 
data[1156] = 17.974903844364814; 
data[1157] = -69.368640085185291; 
data[1158] = 40.748687841669586; 
data[1159] = -43.675799641021079; 
data[1183] = -1; 
data[1198] = 43.675799641021122; 
data[1199] = 40.748687841669557; 
data[1200] = 69.368640085185277; 
data[1201] = 17.97490384436481; 
data[1202] = -184.75059629806526; 
data[1203] = -94.615102344455636; 
data[1204] = 69.368640085185277; 
data[1205] = 17.974903844364817; 
data[1206] = -43.675799641021122; 
data[1207] = -40.748687841669557; 
data[1208] = -69.368640085185277; 
data[1209] = -17.97490384436481; 
data[1210] = 184.75059629806526; 
data[1211] = 94.615102344455636; 
data[1212] = -69.368640085185277; 
data[1213] = -17.974903844364817; 
data[1234] = -1; 
data[1252] = -40.748687841669557; 
data[1253] = 43.675799641021122; 
data[1254] = -17.97490384436481; 
data[1255] = 69.368640085185277; 
data[1256] = 94.615102344455636; 
data[1257] = -184.75059629806526; 
data[1258] = -17.974903844364817; 
data[1259] = 69.368640085185277; 
data[1260] = 40.748687841669557; 
data[1261] = -43.675799641021122; 
data[1262] = 17.97490384436481; 
data[1263] = -69.368640085185277; 
data[1264] = -94.615102344455636; 
data[1265] = 184.75059629806526; 
data[1266] = 17.974903844364817; 
data[1267] = -69.368640085185277; 
data[1292] = -1; 
data[1306] = 69.368640085185334; 
data[1307] = 17.9749038443648; 
data[1308] = 43.675799641021086; 
data[1309] = 40.748687841669586; 
data[1310] = 69.368640085185291; 
data[1311] = 17.974903844364821; 
data[1312] = -184.75059629806523; 
data[1313] = -94.61510234445565; 
data[1314] = -69.368640085185334; 
data[1315] = -17.9749038443648; 
data[1316] = -43.675799641021086; 
data[1317] = -40.748687841669586; 
data[1318] = -69.368640085185291; 
data[1319] = -17.974903844364821; 
data[1320] = 184.75059629806523; 
data[1321] = 94.61510234445565; 
data[1343] = -1; 
data[1360] = -17.9749038443648; 
data[1361] = 69.368640085185334; 
data[1362] = -40.748687841669586; 
data[1363] = 43.675799641021086; 
data[1364] = -17.974903844364821; 
data[1365] = 69.368640085185291; 
data[1366] = 94.61510234445565; 
data[1367] = -184.75059629806523; 
data[1368] = 17.9749038443648; 
data[1369] = -69.368640085185334; 
data[1370] = 40.748687841669586; 
data[1371] = -43.675799641021086; 
data[1372] = 17.974903844364821; 
data[1373] = -69.368640085185291; 
data[1374] = -94.61510234445565; 
data[1375] = 184.75059629806523; 
data[1401] = -1; 
data[1414] = -1; 
data[1434] = 1; 
data[1442] = 1; 
data[1470] = -1; 
data[1488] = 1; 
data[1497] = 1; 
data[1526] = -1; 
data[1542] = 1; 
data[1552] = 1; 
data[1582] = -1; 
data[1596] = 1; 
data[1646] = 1; 
data[1647] = 1; 
data[1648] = 1; 
data[1649] = 1; 
data[1685] = -1.0; 
data[1709] = 1.0; 
data[1715] = 1.0; 
data[1741] = -1.0; 
data[1763] = 1.0; 
data[1770] = 1.0; 
data[1797] = -1.0; 
data[1817] = 1.0; 
data[1825] = 1.0; 
data[1853] = -1.0; 
data[1871] = 1.0; 
data[1921] = 1.0; 
data[1922] = 1.0; 
data[1923] = 1.0; 
data[1924] = 1.0; 
data[1980] = -1; 
data[2002] = 1; 
data[2035] = -1; 
data[2090] = -1; 
data[2145] = -1; 
data[2200] = -1; 
data[2255] = -1.0; 
data[2310] = -1.0; 
data[2365] = -1.0; 
data[2394] = -1; 
data[2450] = -1; 
data[2506] = -1; 
data[2562] = -1; 
data[2611] = -1.0; 
data[2667] = -1.0; 
data[2723] = -1.0; 
data[2779] = -1.0; 
data[2860] = -1; 
data[2915] = -1; 

}



/************************************************************/

static void *_cffi_types[] = {
/*  0 */ _CFFI_OP(_CFFI_OP_FUNCTION, 8), // void()(double *, double *, double *, double *, double *, double)
/*  1 */ _CFFI_OP(_CFFI_OP_POINTER, 6), // double *
/*  2 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/*  3 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/*  4 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/*  5 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/*  6 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 14), // double
/*  7 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/*  8 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 0), // void
};

static void _cffi_d_de_jac_ini_num_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_jac_ini_num_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_jac_ini_num_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_jac_ini_num_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_jac_ini_num_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_jac_ini_num_eval _cffi_d_de_jac_ini_num_eval
#endif

static void _cffi_d_de_jac_ini_up_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_jac_ini_up_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_jac_ini_up_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_jac_ini_up_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_jac_ini_up_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_jac_ini_up_eval _cffi_d_de_jac_ini_up_eval
#endif

static void _cffi_d_de_jac_ini_xy_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_jac_ini_xy_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_jac_ini_xy_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_jac_ini_xy_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_jac_ini_xy_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_jac_ini_xy_eval _cffi_d_de_jac_ini_xy_eval
#endif

static void _cffi_d_f_ini_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  f_ini_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_f_ini_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "f_ini_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { f_ini_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_f_ini_eval _cffi_d_f_ini_eval
#endif

static void _cffi_d_g_ini_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  g_ini_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_g_ini_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "g_ini_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { g_ini_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_g_ini_eval _cffi_d_g_ini_eval
#endif

static void _cffi_d_h_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  h_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_h_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "h_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { h_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_h_eval _cffi_d_h_eval
#endif

static const struct _cffi_global_s _cffi_globals[] = {
  { "de_jac_ini_num_eval", (void *)_cffi_f_de_jac_ini_num_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_jac_ini_num_eval },
  { "de_jac_ini_up_eval", (void *)_cffi_f_de_jac_ini_up_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_jac_ini_up_eval },
  { "de_jac_ini_xy_eval", (void *)_cffi_f_de_jac_ini_xy_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_jac_ini_xy_eval },
  { "f_ini_eval", (void *)_cffi_f_f_ini_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_f_ini_eval },
  { "g_ini_eval", (void *)_cffi_f_g_ini_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_g_ini_eval },
  { "h_eval", (void *)_cffi_f_h_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_h_eval },
};

static const struct _cffi_type_context_s _cffi_type_context = {
  _cffi_types,
  _cffi_globals,
  NULL,  /* no fields */
  NULL,  /* no struct_unions */
  NULL,  /* no enums */
  NULL,  /* no typenames */
  6,  /* num_globals */
  0,  /* num_struct_unions */
  0,  /* num_enums */
  0,  /* num_typenames */
  NULL,  /* no includes */
  9,  /* num_types */
  0,  /* flags */
};

#ifdef __GNUC__
#  pragma GCC visibility push(default)  /* for -fvisibility= */
#endif

#ifdef PYPY_VERSION
PyMODINIT_FUNC
_cffi_pypyinit_temp_ini_cffi(const void *p[])
{
    p[0] = (const void *)0x2601;
    p[1] = &_cffi_type_context;
#if PY_MAJOR_VERSION >= 3
    return NULL;
#endif
}
#  ifdef _MSC_VER
     PyMODINIT_FUNC
#  if PY_MAJOR_VERSION >= 3
     PyInit_temp_ini_cffi(void) { return NULL; }
#  else
     inittemp_ini_cffi(void) { }
#  endif
#  endif
#elif PY_MAJOR_VERSION >= 3
PyMODINIT_FUNC
PyInit_temp_ini_cffi(void)
{
  return _cffi_init("temp_ini_cffi", 0x2601, &_cffi_type_context);
}
#else
PyMODINIT_FUNC
inittemp_ini_cffi(void)
{
  _cffi_init("temp_ini_cffi", 0x2601, &_cffi_type_context);
}
#endif

#ifdef __GNUC__
#  pragma GCC visibility pop
#endif
